/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File Name:       mdl_auxiliary_Log.cpp                                    */
/* Description:     log class                                                */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_AUXILIARY_LOG_CPP_
#define MDL_AUXILIARY_LOG_CPP_

namespace mdl {
namespace auxiliary {

	/****************************
	 *		Public members
	 ****************************/

	inline
	Log :: Log() :
	stream_ (),
	path_ (NULL),
	message_ (),
	initialized_ (false)
	{
		for (int i = 0; i < MESSAGE_COUNT; ++ i) {
			message_ [i] = NULL;
		}
	}
	Log :: ~ Log() {
		destroy();
	}

	inline bool
	Log :: init (const char* path)
	{
		if (initialized_) {
			return false;
		}
		path_ = new String (path);
		for (int i = 0; i < MESSAGE_COUNT; ++ i) {
			message_ [i] = new String (INITIAL_MESSAGE_CAPACITY);
		}
		initialized_ = true;
		return true;
	}
	inline bool
	Log :: init (const String& path)
	{
		if (initialized_) {
			return false;
		}
		path_ = new String (path);
		for (int i = 0; i < MESSAGE_COUNT; ++ i) {
			message_ [i] = new String (INITIAL_MESSAGE_CAPACITY);
		}
		initialized_ = true;
		return true;
	}
	inline bool
	Log :: init ()
	{
		if (initialized_) {
			return false;
		}
		for (int i = 0; i < MESSAGE_COUNT; ++ i) {
			message_ [i] = new String (INITIAL_MESSAGE_CAPACITY);
		}
		initialized_ = true;
		return true;
	}
	inline void
	Log :: destroy()
	{
		if (!initialized_) {
			return;
		}
		for (int i = 0; i < MESSAGE_COUNT; ++ i) {
			delete message_ [i];
			message_ [i] = NULL;
		}
		delete path_;
		path_ = NULL;
		initialized_ = false;
	}

	inline String&
	Log :: message (const int i) {
		return *message_ [i];
	}

	inline void
	Log :: show (const int i) const
	{
		if (!initialized_) {
			return;
		}
		if (SHOW_HEADER) {
			std :: cerr << "LOG:" << std :: endl;
		}
		std :: cout << *message_ [i] << std :: flush;
		message_ [i]->clear();
	}
	inline void
	Log :: write (const int i) const
	{
		if (!initialized_) {
			return;
		}
		if (path_ == NULL) {
			show (i);
		} else {
			if (message_ [i]->isEmpty()) {
				return;
			}
			open();
			if (SHOW_HEADER) {
				stream_ << "LOG:" << std :: endl;
			}
			stream_ << *message_ [i];
			message_ [i]->clear();
			close();
		}
	}
	inline void
	Log :: clear()
	{
		if (!initialized_) {
			return;
		}
		for (int i = 0; i < MESSAGE_COUNT; ++ i) {
			message_ [i]->clear();
		}
	}

using manipulator :: space;
using manipulator :: tab;
using manipulator :: endLine;

	// object :: Object implementation
	inline void
	Log :: commitSuicide() {
		delete this;
	}
	inline Size_t
	Log :: getVolume() const
	{
		Size_t volume = 0;
		for (int i = 0; i < MESSAGE_COUNT; ++ i) {
			volume += getAggregateVolume (message_ [i]);
		}
		volume += getAggregateVolume (path_);
		return volume;
	}
	inline Size_t
	Log :: getSizeOf() const {
		return sizeof (Log);
	}
	inline void
	Log :: show (String& string) const
	{
		if (!initialized_) {
			return;
		}
		if (SHOW_HEADER) {
			string << "LOG:" << endLine;
		}
		for (int i = 0; i < MESSAGE_COUNT; ++ i) {
			string << *message_ [i] << endLine;
			message_ [i]->clear();
		}
	}

	/****************************
	 *		Private members
	 ****************************/

	inline void
	Log :: open () const
	{
		const char* const
			outputPath = path_->c_str();
		stream_.open (outputPath, std :: ios :: binary);
		if (!stream_.is_open()) {
			Error* error = new Error (Error :: OUTPUT);
			error->message() << "cannot open log file: " << *path_ << " for output" << endLine;
			throw error;
		}
	}
	inline void
	Log :: close() const
	{
		stream_.flush();
		stream_.close();
	}
}
}

#endif /*MDL_AUXILIARY_LOG_CPP_*/
