/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File Name:       mdl_auxiliary_Log.hpp                                    */
/* Description:     log class                                                */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_AUXILIARY_LOG_HPP_
#define MDL_AUXILIARY_LOG_HPP_

#include "mdl/interface/object/mdl_interface_object.hpp"

namespace mdl {
namespace auxiliary {

class Log : public object :: Object {
public :
	Log();
	virtual ~ Log();

	bool init (const char*);
	bool init (const String&);
	bool init ();
	void destroy();

	String& message (const int = 0);
	void show (const int = 0) const;
	void write (const int = 0) const;
	void clear();

	// object :: Object interface
	virtual void commitSuicide();
	virtual Size_t getVolume() const;
	virtual Size_t getSizeOf() const;
	virtual void show (String&) const;

private :
	enum {
		SHOW_HEADER = false,
		MESSAGE_COUNT = 8,
		INITIAL_MESSAGE_CAPACITY = 1024 * 4
	};
	void open() const;
	void close() const;

	mutable OutputFileStream stream_;
	const String*   path_;
	mutable String* message_ [MESSAGE_COUNT];
	bool initialized_;
};

}
}

#endif /*MDL_AUXILIARY_LOG_HPP_*/
