/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File name:       mdl_auxiliary_Timers.cpp                                 */
/* Description:     class for processing timers                              */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_AUXILIARY_TIMERS_CPP_
#define MDL_AUXILIARY_TIMERS_CPP_

#include "mdl/auxiliary/mdl_auxiliary.hpp"

namespace mdl {
namespace auxiliary {

	/****************************
	 *		Public members
	 ****************************/

using manipulator :: endLine;
using manipulator :: space;
using manipulator :: tab;

	// object :: Object interface
	void
	Timers :: commitSuicide() {
		delete this;
	}
	Size_t
	Timers :: getVolume() const {
		return 0;
	}
	Size_t
	Timers :: getSizeOf() const {
		return sizeof (Timers);
	}
	void
	Timers :: show (String& string) const
	{
		string << "timings:" << endLine;
		string << "-------------------------" << endLine;
		if (init_.isUsed()) {
			string << tab << "init:      " << init_ << endLine;
		}
		if (read_.isUsed()) {
			string << tab << "read:      " << read_ << endLine;
		}
		if (build_.isUsed()) {
			string << tab << "build:     " << build_ << endLine;
		}
		if (check_.isUsed()) {
			string << tab << "check:     " << check_ << endLine;
		}
		if (compress_.isUsed()) {
			string << tab << "compress:  " << compress_ << endLine;
		}
		if (prove_.isUsed()) {
			string << tab << "prove:     " << prove_ << endLine;
		}
		if (translate_.isUsed()) {
			string << tab << "translate: " << translate_ << endLine;
		}
		if (write_.isUsed()) {
			string << tab << "write:     " << write_ << endLine;
		}
		if (release_.isUsed()) {
			string << tab << "release:     " << release_ << endLine;
		}
	}

	/**********************************
	 *		Private static members
	 **********************************/

	Timer Timers :: init_;
	Timer Timers :: release_;
}
}

#endif /*MDL_AUXILIARY_TIMERS_CPP_*/
