/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File name:       mdl_auxiliary_Timers.hpp                                 */
/* Description:     class for processing timers                              */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_AUXILIARY_TIMERS_HPP_
#define MDL_AUXILIARY_TIMERS_HPP_

#include "mdl/interface/object/mdl_interface_object_Object.hpp"
#include "mdl/auxiliary/mdl_auxiliary.dpp"

namespace mdl {
namespace auxiliary {

class Timers :
	public object :: Object,
	public Scalar<Timers> {
public :
	Timers();
	virtual ~ Timers();

	static Timer& init();
	static Timer& release();

	Timer& read();
	Timer& build();
	Timer& check();
	Timer& compress();
	Timer& prove();
	Timer& translate();
	Timer& write();
	Timer& total();

	const Timer& read() const;
	const Timer& build() const;
	const Timer& check() const;
	const Timer& compress() const;
	const Timer& prove() const;
	const Timer& translate() const;
	const Timer& write() const;
	const Timer& total() const;

	void stopAll();

	// nstd :: Object interface
	virtual void commitSuicide();
	virtual Size_t getVolume() const;
	virtual Size_t getSizeOf() const;
	virtual void show (String&) const;

private :
	static Timer init_;
	static Timer release_;

	Timer read_;
	Timer build_;
	Timer check_;
	Timer compress_;
	Timer prove_;
	Timer translate_;
	Timer write_;
	Timer total_;
};

}
}

#include "mdl/auxiliary/mdl_auxiliary_Timers.ipp"

#endif /*MDL_AUXILIARY_TIMERS_HPP_*/
