/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File name:       mdl_auxiliary_Timers.ipp                                 */
/* Description:     class for processing timers                              */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_AUXILIARY_TIMERS_IPP_
#define MDL_AUXILIARY_TIMERS_IPP_

namespace mdl {
namespace auxiliary {

	/****************************
	 *		Public members
	 ****************************/

	inline
	Timers :: Timers() :
	read_ (),
	build_ (),
	check_ (),
	compress_ (),
	prove_ (),
	translate_ (),
	write_ (),
	total_ () {
	}
	Timers :: ~ Timers() {
	}

	inline Timer&
	Timers :: init() {
		return init_;
	}
	inline Timer&
	Timers :: release() {
		return release_;
	}

	inline Timer&
	Timers :: read() {
		return read_;
	}
	inline Timer&
	Timers :: build() {
		return build_;
	}
	inline Timer&
	Timers :: check() {
		return check_;
	}
	inline Timer&
	Timers :: compress() {
		return compress_;
	}
	inline Timer&
	Timers :: prove() {
		return prove_;
	}
	inline Timer&
	Timers :: translate() {
		return translate_;
	}
	inline Timer&
	Timers :: write() {
		return write_;
	}
	inline Timer&
	Timers :: total() {
		return total_;
	}

	inline const Timer&
	Timers :: read() const {
		return read_;
	}
	inline const Timer&
	Timers :: build() const {
		return build_;
	}
	inline const Timer&
	Timers :: check() const {
		return check_;
	}
	inline const Timer&
	Timers :: compress() const {
		return compress_;
	}
	inline const Timer&
	Timers :: prove() const {
		return prove_;
	}
	inline const Timer&
	Timers :: translate() const {
		return translate_;
	}
	inline const Timer&
	Timers :: write() const {
		return write_;
	}
	inline const Timer&
	Timers :: total () const {
		return total_;
	}

	inline void
	Timers :: stopAll()
	{
		init_.stop();
		read_.stop();
		build_.stop();
		check_.stop();
		compress_.stop();
		prove_.stop();
		translate_.stop();
		write_.stop();
		release_.stop();
		total_.stop();
	}
}
}

#endif /*MDL_AUXILIARY_TIMERS_IPP_*/
