/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File name:       mdl_auxiliary_Volume.cpp                                 */
/* Description:     class for program data volume counters                   */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_AUXILIARY_VOLUME_CPP_
#define MDL_AUXILIARY_VOLUME_CPP_

#include "mdl/mdl_Mdl.hpp"

namespace mdl {
namespace auxiliary {

	/****************************
	 *		Public members
	 ****************************/

	Volume :: Volume (const Mdl* const mdl) :
	parser_ (0),
	source_ (0),
	target_ (0),
	sizeof_ (0),
	names_ (0),
	math_ (0),
	prover_ (0),
	statistics_ (0),

	used_ (0),
	allocated_ (0),
	delta_ (0),

	calculated_ (false),
	mdl_ (mdl) {
	}
	Volume :: ~ Volume() {
	}

using manipulator :: endLine;
using manipulator :: space;
using manipulator :: tab;

	// object :: Object implementation
	void
	Volume :: commitSuicide() {
		delete this;
	}
	Size_t
	Volume :: getVolume() const {
		return 0;
	}
	Size_t
	Volume :: getSizeOf() const {
		return sizeof (Volume);
	}
	void
	Volume :: show (String& string) const
	{
		const_cast<Volume* const>(this)->calculate();

		if (Config :: showVolume()) {
			showVolume (string);
		}
		if (Config :: showMemoryBalance() || delta_ != 0) {
			showBalance (string);
		}
		if (Config :: showMemoryAlloc()) {
			Memory :: show
			(
				string,
				Config :: showMemoryDetails(),
				Config :: showMemoryDisposed(),
				Config :: showMemoryFragmentation()
			);
		}
	}

	/****************************
	 *		Private members
	 ****************************/

	void
	Volume :: calculate()
	{
		if (calculated_) {
			return;
		}
		parser_ = getAggregateVolume (mdl_->parser_);
		source_ = getAggregateVolume (mdl_->source_);
		//target_ = getAggregateVolume (mdl_->target_);
		if (mdl_ != NULL) {
			sizeof_ = mdl_->getSizeOf();
		}

		names_ = Table :: getVolume();;
		math_ = Math :: getVolume();
		prover_ = Prover :: getVolume();
		statistics_ = Statistics :: getVolume();

		used_ =
			Memory :: getUsedVolume() +
			getAggregateVolume (mdl_) +
			Config :: getVolume() +
			form :: block :: Proof :: getLog().getVolume() +
			names_ +
			math_ +
			prover_ +
			statistics_;
		allocated_ = Memory :: getAllocatedVolume();
		delta_ =
			static_cast<Ptrdiff_t> (allocated_) -
			static_cast<Ptrdiff_t> (used_);
		calculated_ = true;
	}
	inline void
	Volume :: showVolume (String& string) const
	{
		string << "memory usage summary: " << endLine;
		string << "-------------------------" << endLine;
		string << memory :: UsageMessage (used_, used_, "total volume");
		string << memory :: UsageMessage (used_, math_, "math volume");
		string << memory :: UsageMessage (used_, names_, "names volume");
		string << memory :: UsageMessage (used_, parser_, "parser volume");
		string << memory :: UsageMessage (used_, source_, "source volume");
		//string << memory :: UsageMessage (used_, target_, "target volume");
		string << endLine;
	}
	inline void
	Volume :: showBalance (String& string) const
	{
		string << "memory balance (must be 0):" << endLine;
		string << "-------------------------------" << endLine;
		string << "delta:" << tab << delta_ << " = ";
		string << allocated_ << " (allocated) - " << used_ << " (used) " << endLine;
		string << endLine;
	}
}
}

#endif /*MDL_AUXILIARY_VOLUME_CPP_*/
