/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File name:       mdl_auxiliary_Volume.hpp                                 */
/* Description:     class for program data volume counters                   */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_AUXILIARY_VOLUME_HPP_
#define MDL_AUXILIARY_VOLUME_HPP_

#include "mdl/interface/object/mdl_interface_object_Object.hpp"
#include "mdl/mdl.dpp"

namespace mdl {
namespace auxiliary {

class Volume :
	public object :: Object,
	public Scalar<Volume> {
public :
	Volume (const Mdl* const);
	virtual ~ Volume();

	// object :: Object interface
	virtual void commitSuicide();
	virtual Size_t getVolume() const;
	virtual Size_t getSizeOf() const;
	virtual void show (String&) const;

private :
	void calculate();
	void showVolume (String&) const;
	void showBalance (String&) const;

	Size_t parser_;
	Size_t source_;
	Size_t target_;
	Size_t sizeof_;

	Size_t names_;
	Size_t math_;
	Size_t prover_;
	Size_t statistics_;

	Size_t used_;
	Size_t allocated_;
	Ptrdiff_t delta_;

	bool calculated_;
	const Mdl* const mdl_;
};

}
}

#endif /*MDL_AUXILIARY_VOLUME_HPP_*/
