/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File name:       mdl_auxiliary_statistics_Matrix.hpp                      */
/* Description:     matrix class for statistics data                         */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_AUXILIARY_STATISTICS_MATRIX_HPP_
#define MDL_AUXILIARY_STATISTICS_MATRIX_HPP_

#include "mdl/auxiliary/statistics/mdl_auxiliary_statistics_Vector.hpp"

namespace mdl {
namespace auxiliary {
namespace statistics {

template<class T>
class Matrix :
	public nstd :: Object,
	public Scalar<Matrix<T>, allocator :: Standard> {
private :
	typedef T Value_;
public :
	Matrix
	(
		const Size_t initialVectorCapacity = INITIAL_VECTOR_CAPACITY,
		const Size_t initialMatrixCapacity = INITIAL_MATRIX_CAPACITY,
		const Size_t cycleSize = CYCLE_SIZE
	);
	virtual ~ Matrix();

	void addValue (const Value_);

	// nstd :: Object interface
	virtual void commitSuicide();
	virtual Size_t getVolume() const;
	virtual Size_t getSizeOf() const;
	virtual void show (nstd :: String<>&) const;

private :
	typedef
		Vector<Value_>
		Vector_;
	typedef
		typename vector :: Vector
		<
			Vector_,
			storage :: ByPointer,
			allocator :: Standard
		>
		Matrix_;

	const Size_t initialVectorCapacity_;
	const Size_t initialMatrixCapacity_;
	const Size_t cycleSize_;

	Value_ globalCounter_;
	Value_ cycleCounter_;
	Value_ valueCounter_;

	Vector_ vector_;
	mutable Vector_* matrixRow_;
	Matrix_ matrix_;

	void showVector (nstd :: String<>&) const;
	void showMatrixHeader (nstd :: String<>&) const;
	void showMatrixValues (nstd :: String<>&) const;

	enum {
		INITIAL_VECTOR_CAPACITY = 64,
		INITIAL_MATRIX_CAPACITY = 64,
		CYCLE_SIZE = 256 //1024 * 16
	};
};

}
}
}

#include "mdl/auxiliary/statistics/mdl_auxiliary_statistics_Matrix.ipp"

#endif /*MDL_AUXILIARY_STATISTICS_MATRIX_HPP_*/
