/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File name:       mdl_auxiliary_statistics_Matrix.ipp                      */
/* Description:     matrix class for statistics data                         */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_AUXILIARY_STATISTICS_MATRIX_IPP_
#define MDL_AUXILIARY_STATISTICS_MATRIX_IPP_

#include <math.h>

namespace mdl {
namespace auxiliary {
namespace statistics {

	/****************************
	 *		Public members
	 ****************************/

	template<class T>
	Matrix<T> :: Matrix
	(
		const Size_t initialVectorCapacity,
		const Size_t initialMatrixCapacity,
		const Size_t cycleSize
	) :
	initialVectorCapacity_ (initialVectorCapacity),
	initialMatrixCapacity_ (initialMatrixCapacity),
	cycleSize_ (cycleSize),

	globalCounter_ (0),
	cycleCounter_ (0),
	valueCounter_ (0),

	vector_ (initialVectorCapacity),
	matrixRow_ (NULL),
	matrix_ (initialMatrixCapacity) {
		matrixRow_ = matrix_.push();
	}
	template<class T>
	Matrix<T> :: ~ Matrix() {
	}

	template<class T>
	void
	Matrix<T> :: addValue (const Value_ value)
	{
		vector_.addValue (value);
		matrixRow_->addValue (value);

		valueCounter_ += value;
		globalCounter_ ++;

		cycleCounter_ ++;
		if (cycleCounter_ == cycleSize_) {
			cycleCounter_ = 0;
			matrixRow_ = matrix_.push();
		}
	}

	// nstd :: Object implementation
	template<class T>
	void
	Matrix<T> :: commitSuicide() {
		delete this;
	}
	template<class T>
	Size_t
	Matrix<T> :: getVolume() const
	{
		Size_t result = 0;
		result += vector_.getVolume();
		result += matrix_.getVolume();
		return result;
	}
	template<class T>
	Size_t
	Matrix<T> :: getSizeOf() const {
		return sizeof (Matrix);
	}
	template<class T>
	void
	Matrix<T> :: show (nstd :: String<>& string) const
	{
		showVector (string);
		showMatrixHeader (string);
		showMatrixValues (string);
	}

	/****************************
	 *		Private members
	 ****************************/

	template<class T>
	void
	Matrix<T> :: showVector (nstd :: String<>& string) const
	{
		string << "vector:" <<nstd :: manipulator :: endLine;
		vector_.showHeader (string, vector_.getSize());
		string << nstd :: manipulator :: endLine;

		vector_.showValues (string, vector_.getSize());
		string << nstd :: manipulator :: endLine;
	}

	template<class T>
	void
	Matrix<T> :: showMatrixHeader (nstd :: String<>& string) const
	{
		string << "matrix:" <<nstd :: manipulator :: endLine;
		vector_.showHeader (string, vector_.getSize());
		string << nstd :: manipulator :: endLine;
	}

	template<class T>
	void
	Matrix<T> :: showMatrixValues (nstd :: String<>& string) const
	{
		for (value :: Integer i = 0; i < matrix_.getSize(); i++) {
			matrixRow_ = matrix_.getValue (i);
			matrixRow_->showValues (string, vector_.getSize());
		}
		string << nstd :: manipulator :: endLine;
	}
}
}
}

#endif /*MDL_AUXILIARY_STATISTICS_MATRIX_IPP_*/
