/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File name:       mdl_auxiliary_statistics_ProofGaps.cpp                   */
/* Description:     statistics for proof gaps                                */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_AUXILIARY_STATISTICS_PROOF_GAPS_CPP_
#define MDL_AUXILIARY_STATISTICS_PROOF_GAPS_CPP_

#include "mdl/auxiliary/statistics/mdl_auxiliary_statistics_ProofGaps.hpp"

namespace mdl {
namespace auxiliary {
namespace statistics {

	/****************************
	 *		Public members
	 ****************************/

	ProofGaps :: ProofGaps (const Size_t initialCapacity) :
	theoremVector_ (initialCapacity),
	deltaVector_ (initialCapacity) {
	}
	ProofGaps :: ~ ProofGaps() {
	}

	void
	ProofGaps :: addGap
	(
		const mdl :: Assertion* theorem,
		const index :: Step delta,
		const index :: Step length
	)
	{
		theoremVector_.add (theorem);
		deltaVector_.addValue (delta);
	}

	// nstd :: Object implementation
	void
	ProofGaps :: commitSuicide() {
		delete this;
	}
	Size_t
	ProofGaps :: getVolume() const
	{
		Size_t volume = 0;
		volume += theoremVector_.getVolume();
		volume += deltaVector_.getVolume();
		return volume;
	}
	Size_t
	ProofGaps :: getSizeOf() const {
		return sizeof (ProofGaps);
	}
	void
	ProofGaps :: show (nstd :: String<>&) const {
	}
}
}
}

#endif /*MDL_AUXILIARY_STATISTICS_PROOF_GAPS_CPP_*/
