/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File name:       mdl_auxiliary_statistics_ProofGaps.hpp                   */
/* Description:     statistics for proof gaps                                */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_AUXILIARY_STATISTICS_PROOF_GAPS_HPP_
#define MDL_AUXILIARY_STATISTICS_PROOF_GAPS_HPP_

#include "mdl/types/mdl_types.hpp"
#include "mdl/auxiliary/statistics/mdl_auxiliary_statistics.dpp"

namespace mdl {
namespace auxiliary {
namespace statistics {

class ProofGaps :
	public nstd :: Object,
	public Scalar<ProofGaps, allocator :: Standard> {
public :
	ProofGaps (const Size_t = INITIAL_CAPACITY);
	virtual ~ ProofGaps();

	void addGap
	(
		const mdl :: Assertion*, // theorem
		const index :: Step,     // min unprovable gap width
		const index :: Step		 // max provable gap width
	);

	// nstd :: Object interface
	virtual void commitSuicide();
	virtual Size_t getVolume() const;
	virtual Size_t getSizeOf() const;
	virtual void show (nstd :: String<>&) const;

private :
	typedef
		vector :: Vector
		<
			const mdl :: Assertion*,
			storage :: ByValue,
			allocator :: Standard
		>
		TheoremVector_;
	typedef
		Vector<index :: Step>
		DeltaVector_;

	TheoremVector_ theoremVector_;
	DeltaVector_ deltaVector_;
	enum {
		INITIAL_CAPACITY = 1024 * 16
	};
};

}
}
}

#endif /*MDL_AUXILIARY_STATISTICS_PROOF_GAPS_HPP_*/
