/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File name:       mdl_auxiliary_statistics_Statistics.cpp                  */
/* Description:     statistics management class                              */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_AUXILIARY_STATISTICS_STATISTICS_CPP_
#define MDL_AUXILIARY_STATISTICS_STATISTICS_CPP_

#include "mdl/auxiliary/statistics/mdl_auxiliary_statistics.hpp"

namespace mdl {
namespace auxiliary {
namespace statistics {

	/****************************
	 *		Public members
	 ****************************/

	void
	Statistics :: addProofForking (const long forking) {
		proofForkingMatrix_->addValue (forking);
	}
	void
	Statistics :: addProofLength (const long length) {
		proofLengthMatrix_->addValue (length);
	}
	void
	Statistics :: addProofGap
	(
		const mdl :: Assertion* theorem,
		const index :: Step delta,
		const index :: Step length
	) {
		proofGaps_->addGap (theorem, delta, length);
	}

	void
	Statistics :: release()
	{
		deleteAggregateObject (proofForkingMatrix_);
		deleteAggregateObject (proofLengthMatrix_);
		deleteAggregateObject (proofGaps_);
	}

using manipulator :: endLine;
using manipulator :: tab;
using manipulator :: space;

	// nstd :: Static implementation
	void
	Statistics :: initRaw()
	{
		proofForkingMatrix_ = new ProofForkingMatrix_
		(
			INITIAL_PROOF_FORKING_VECTOR_CAPACITY,
			INITIAL_PROOF_FORKING_MATRIX_CAPACITY,
			PROOF_FORKING_CYCLE_SIZE
		);
		proofLengthMatrix_ = new ProofLengthMatrix_
		(
			INITIAL_PROOF_LENGTH_VECTOR_CAPACITY,
			INITIAL_PROOF_LENGTH_MATRIX_CAPACITY,
			PROOF_LENGTH_CYCLE_SIZE
		);
		proofGaps_ = new ProofGaps(INITIAL_PROOF_GAPS_VECTOR_CAPACITY);
	}
	void
	Statistics :: showRaw (nstd :: String<>& string)
	{
		string << "mdl statistics:" <<endLine;
		string << "------------------------------------" <<endLine;
		string << "proof forking: " <<endLine;
		if (proofForkingMatrix_ != NULL) {
			proofForkingMatrix_->show (string);
		} else {
			string << tab << "NULL" << endLine;
		}

		string << "proof length:" <<endLine;
		if (proofLengthMatrix_ != NULL) {
			proofLengthMatrix_->show (string);
		} else {
			string << tab << "NULL" << endLine;
		}
	}
	Size_t
	Statistics :: getRawVolume()
	{
		Size_t volume = 0;
		volume += getAggregateVolume (proofForkingMatrix_);
		volume += getAggregateVolume (proofLengthMatrix_);
		volume += getAggregateVolume (proofGaps_);
		return volume;
	}

	/************************************
	 *		Private static attributes
	 ************************************/

	Statistics :: ProofForkingMatrix_*
		Statistics :: proofForkingMatrix_ = NULL;
	Statistics :: ProofLengthMatrix_*
		Statistics :: proofLengthMatrix_ = NULL;
	ProofGaps*
		Statistics :: proofGaps_ = NULL;
}
}
}

#endif /*MDL_AUXILIARY_STATISTICS_STATISTICS_CPP_*/
