/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File name:       mdl_auxiliary_statistics_Statistics.hpp                  */
/* Description:     statistics management class                              */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_AUXILIARY_STATISTICS_STATISTICS_HPP_
#define MDL_AUXILIARY_STATISTICS_STATISTICS_HPP_

#include "mdl/types/mdl_types.hpp"
#include "mdl/interface/mdl_interface.dpp"
#include "mdl/auxiliary/statistics/mdl_auxiliary_statistics_ProofGaps.hpp"
#include "mdl/auxiliary/statistics/mdl_auxiliary_statistics_Matrix.hpp"
#include "mdl/auxiliary/statistics/mdl_auxiliary_statistics_Vector.hpp"

namespace mdl {
namespace auxiliary {
namespace statistics {

class Statistics : public nstd :: Static<Statistics> {
public :
	static void addProofForking (const long);
	static void addProofLength (const long);
	static void addProofGap
	(
		const mdl :: Assertion*,
		const index :: Step,
		const index :: Step
	);
	static void release();

	// nstd :: Static interface
	static void initRaw();
	static void showRaw (nstd :: String<>&);
	static Size_t getRawVolume();

private :
	typedef
		Matrix<nstd :: U_int32_t>
		ProofForkingMatrix_;
	typedef
		Matrix<nstd :: U_int32_t>
		ProofLengthMatrix_;

	static ProofForkingMatrix_* proofForkingMatrix_;
	static ProofLengthMatrix_* proofLengthMatrix_;
	static ProofGaps* proofGaps_;

	enum {
		INITIAL_PROOF_FORKING_VECTOR_CAPACITY = 64,
		INITIAL_PROOF_FORKING_MATRIX_CAPACITY = 64,
		PROOF_FORKING_CYCLE_SIZE =  1024 * 16,

		INITIAL_PROOF_LENGTH_VECTOR_CAPACITY = 64,
		INITIAL_PROOF_LENGTH_MATRIX_CAPACITY = 64,
		PROOF_LENGTH_CYCLE_SIZE = 1024,

		INITIAL_PROOF_GAPS_VECTOR_CAPACITY = 1024 * 16
	};
};

}
}
}

#endif /*MDL_AUXILIARY_STATISTICS_STATISTICS_HPP_*/
