/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File name:       mdl_auxiliary_statistics_Vector.hpp                      */
/* Description:     vector class for statistics data                         */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_AUXILIARY_STATISTICS_VECTOR_HPP_
#define MDL_AUXILIARY_STATISTICS_VECTOR_HPP_

#include "mdl/types/mdl_types.hpp"
#include "mdl/auxiliary/statistics/mdl_auxiliary_statistics.dpp"

namespace mdl {
namespace auxiliary {
namespace statistics {

template<class T>
class Vector :
	public nstd :: Object,
	public Scalar<Vector<T>, allocator :: Standard> {
private :
	typedef T Value_;
public :
	Vector (const Size_t initialVectorCapacity = INITIAL_VECTOR_CAPACITY);
	virtual ~ Vector();

	void addValue (const Value_);
	Size_t getSize() const;

	void showHeader (nstd :: String<>&, const value :: Integer outerSize = 0) const;
	void showValues (nstd :: String<>&, const value :: Integer outerSize = 0) const;

	void init();

	// nstd :: Object interface
	virtual void commitSuicide();
	virtual Size_t getVolume() const;
	virtual Size_t getSizeOf() const;
	virtual void show (nstd :: String<>&) const;

private :
	typedef
		typename vector :: Vector
		<
			Value_,
			storage :: ByValue,
			allocator :: Standard
		>
		Vector_;

	const Size_t initialVectorCapacity_;
	Vector_ vector_;

	double getAverage() const;
	double getDispersion() const;
	value :: Integer getFactor (const value :: Integer) const;

	enum {
		INITIAL_VECTOR_CAPACITY = 64,
		MAX_SHOWN_LENGTH = 32
	};
};

}
}
}

#include "mdl/auxiliary/statistics/mdl_auxiliary_statistics_Vector.ipp"

#endif /*MDL_AUXILIARY_STATISTICS_VECTOR_HPP_*/
