/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File name:       mdl_auxiliary_statistics_Vector.ipp                      */
/* Description:     vector class for statistics data                         */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_AUXILIARY_STATISTICS_VECTOR_IPP_
#define MDL_AUXILIARY_STATISTICS_VECTOR_IPP_

#include <math.h>

namespace mdl {
namespace auxiliary {
namespace statistics {

	/****************************
	 *		Public members
	 ****************************/

	template<class T>
	inline
	Vector<T> :: Vector (const Size_t initialVectorCapacity) :
	initialVectorCapacity_ (initialVectorCapacity),
	vector_ () {
	}
	template<class T>
	Vector<T> :: ~ Vector() {
	}

	template<class T>
	void
	Vector<T> :: addValue (const Value_ value)
	{
		while (value >= vector_.getSize()) {
			vector_.push() = 0;
		}
		vector_.getReference (value) ++;
	}
	template<class T>
	Size_t
	Vector<T> :: getSize() const {
		return vector_.getSize();
	}

	template<class T>
	void
	Vector<T> :: showValues (nstd :: String<>& string, const value :: Integer outerSize) const
	{
		T sum = 0;
		value :: Integer i = 0;
		while (i < vector_.getSize()){
			const Value_ value = vector_.getValue (i);
			if (i % getFactor (outerSize) == 0 && i != 0) {
				sum += value;
				string << sum << nstd :: manipulator :: tab;
				sum = 0;
			} else {
				sum += value;
			}
			i++;
		}
		while (i < outerSize) {
			if (i % getFactor (outerSize) == 0) {
				string << nstd :: manipulator :: tab;
			}
			i++;
		}
		const double average = getAverage();
		const double dispersion = getDispersion();
		string << average << nstd :: manipulator :: tab << dispersion;
		string << nstd :: manipulator :: endLine;
	}

	template<class T>
	void
	Vector<T> :: showHeader (nstd :: String<>& string, const value :: Integer outerSize) const
	{
		for (value :: Integer  i = 0; i < vector_.getSize(); i++) {
			if (i % getFactor (outerSize) == 0 && i != 0) {
				string << "value:" << nstd :: manipulator :: space;
				string << i << nstd :: manipulator :: tab;
			}
		}
		string << "average" << nstd :: manipulator :: tab << "sigma";
		string << nstd :: manipulator :: endLine;
	}

	template<class T>
	void
	Vector<T> :: init() {
		vector_.init (initialVectorCapacity_);
	}

	// nstd :: Object implementation
	template<class T>
	void
	Vector<T> :: commitSuicide() {
		delete this;
	}
	template<class T>
	Size_t
	Vector<T> :: getVolume() const {
		return vector_.getVolume();
	}
	template<class T>
	Size_t
	Vector<T> :: getSizeOf() const {
		return sizeof (Vector);
	}
	template<class T>
	void
	Vector<T> :: show (nstd :: String<>& string) const {
		showValues (string);
	}

	/****************************
	 *		Private members
	 ****************************/

	template<class T>
	double
	Vector<T> :: getAverage () const
	{
		value :: Integer n = 0;
		Value_ s = 0;

		for (value :: Integer i = 0; i < vector_.getSize(); i++) {
			const Value_ value = vector_.getValue (i);
			n += value;
			s += value * i;
		}
		return static_cast<double>(s) / n;
	}

	template<class T>
	double
	Vector<T> :: getDispersion () const
	{
		value :: Integer n = 0;
		const double average = getAverage();
		double d = 0;

		for (value :: Integer i = 0; i < vector_.getSize(); i++) {
			double value = vector_.getValue (i);
			double delta = value -  average;
			n += vector_.getValue (i);
			d += delta * delta;
		}
		double sigma = d / n;
		return sqrt (sigma);
	}
	template<class T>
	value :: Integer
	Vector<T> :: getFactor (const value :: Integer outerSize) const
	{
		value :: Integer factor = (outerSize / MAX_SHOWN_LENGTH) + 1;
		return factor;
	}
}
}
}

#endif /*MDL_AUXILIARY_STATISTICS_VECTOR_IPP_*/
