/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File name:       mdl_expression_build_Builder.ipp                         */
/* Description:     expression building routines                             */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_EXPRESSION_BUILD_BUILDER_IPP_
#define MDL_EXPRESSION_BUILD_BUILDER_IPP_

namespace mdl {
namespace expression {
namespace build {

	/****************************
	 *		Public members
	 ****************************/

using manipulator :: endLine;
using manipulator :: iterate;

	template<class E, class T, class V>
	typename Builder<E, T, V> :: Term_1_*
	Builder<E, T, V> :: build
	(
		const Term_2_* const source,
		Expression_* const target,
		Value_ value
	)
	{
		Iterator_1_
			targetBegin = Switch_ :: startTarget (target);
		Iterator_2_
			sourceBegin = Switch_ :: startSource (source);
		Iterator_1_
			targetEnd = targetBegin;
		Iterator_2_
			sourceEnd = sourceBegin;

		const Iterator_2_
			absoluteBegin = Switch_ :: beginSource (source);
		const Iterator_2_
			absoluteEnd = Switch_ :: endSource (source);

		Node_ :: manage (target, targetBegin, sourceEnd, value);

		Term_1_* const
			result = addNode
			(
				target,
				targetBegin,   targetEnd,
				sourceBegin,   sourceEnd,
				absoluteBegin, absoluteEnd,
				Switch_ :: getTerms (sourceEnd),
				false,
				value
			);

		#ifdef DEBUG
			result->verifyIntegrity();
			source->verifyIntegrity();
			if (!source->coinsides (*result)) {
				Error* error = new Error (Error :: SEMANTIC);
				error->message() << "builder error" << endLine;
				error->message() << "source: " << *source << endLine;
				source->showTerms (error->message());
				error->message() << endLine;
				error->message() << "target: " << *result << endLine;
				result->showTerms (error->message());
				//std :: cerr << error->message();

				if (!source->coinsides (*result)) {
					std :: cerr << error->message();
				}

				throw error;
			}
		#endif /*DEBUG*/

		return result;
	}

	/****************************
	 *		Private members
	 ****************************/

	template<class E, class T, class V>
	typename Builder<E, T, V> :: Term_1_*
	Builder<E, T, V> :: addNode
	(
		Expression_* const target,
		const Iterator_1_ targetBegin,
		Iterator_1_& targetEnd,
		const Iterator_2_ sourceBegin,
		Iterator_2_& sourceEnd,
		const Iterator_2_ absoluteBegin,
		const Iterator_2_ absoluteEnd,
		const Terms_2_& sourceTerms,
		const bool initial,
		Value_ value
	)
	{
		Node_ :: manage (target, targetEnd, sourceEnd, value);

		if (initial && !Switch_ :: getTerms (sourceEnd).isEmpty()) {
			addNode
			(
				target,
				targetEnd,     targetEnd,
				sourceEnd,     sourceEnd,
				absoluteBegin, absoluteEnd,
				Switch_ :: getTerms (sourceEnd),
				false,
				value
			);
		}

		typedef
			typename Terms_2_ :: Node_
			Node;

		const Node*
			node = sourceTerms.getFirst();
		Term_1_*
			targetTerm = NULL;
		const Term_2_*
			sourceTerm = NULL;
		while (node != NULL) {
			sourceTerm = node->getValue();
			if (Switch_ :: end (sourceTerm, sourceEnd)) {
				targetTerm = Switch_ :: newTerm
					(
						targetBegin,
						targetEnd,
						sourceBegin,
						sourceEnd,
						sourceTerm
					);
				if (Switch_ :: end (sourceTerm, absoluteEnd)) {
					break;
				}
			}
			node = node->getNext();
		}
		if (sourceTerm != NULL &&
			!Switch_ :: end (sourceTerm, sourceEnd)) {
			Switch_ :: inc (targetEnd, sourceEnd);
			return addNode
			(
				target,
				targetBegin,   targetEnd,
				sourceBegin,   sourceEnd,
				absoluteBegin, absoluteEnd,
				sourceTerms,
				true,
				value
			);
		} else {
			return targetTerm;
		}
	}
}
}
}

#endif /*MDL_EXPRESSION_BUILD_BUILDER_IPP_*/
