/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File name:       mdl_expression.tpp                                       */
/* Description:     typedefs for expression module                           */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_EXPRESSION_TPP_
#define MDL_EXPRESSION_TPP_

#include "mdl/expression/mdl_expression.dpp"
#include "mdl/expression/mdl_expression.hpp"

namespace mdl {
namespace array {

	typedef
		expression :: Array<>
		Expression;
	typedef
		Expression :: Substitution_
		Substitution;
	typedef
		Expression :: Term_
		Term;
}
namespace list {

	typedef
		expression :: List<>
		Expression;
}
namespace tree {

	typedef
		expression :: Tree
		<
			const mdl :: Rule*,
			storage :: ByValue
		>
		Rule;
	typedef
		expression :: Tree
		<
			const mdl :: Proposition*,
			storage :: ByValue,
			expression :: node :: container :: List
		>
		Proposition;
	typedef
		expression :: Tree
		<
			const mdl :: Term*,
			storage :: ByValue
		>
		Term;
}
namespace prover {
	typedef
		expression :: List<allocator :: Prover>
		Expression;
	typedef
		Expression :: Term_
		Term;
	typedef
		expression :: Substitution<Term>
		Substitution;

namespace stack {
	typedef
		expression :: List<allocator :: Stack>
		Expression;
	typedef
		Expression :: Term_
		Term;
	typedef
		expression :: Substitution<Term>
		Substitution;
	typedef
		mdl :: vector :: Vector
		<
			const prover :: Substitution*,
			storage :: ByValue,
			allocator :: Stack
		>
		Substivector;
}
	typedef
		substitution :: prod :: Unifier<Expression>
		MatrixUnifier;

	typedef
		expression :: Tree
		<
			mdl :: prover :: DotEvidence,
			storage :: ByValue,
			expression :: node :: container :: Set,
			allocator :: Stack
		>
		Cell;

namespace vector {

	typedef
		mdl :: vector :: Vector
		<
			prover :: Cell*,
			storage :: ByValue,
			allocator :: Stack
		>
		Cell;
	typedef
		mdl :: vector :: Vector
		<
			mdl :: Symbol,
			storage :: ByValue,
			allocator :: Stack
		>
		Symbol;
}
namespace matrix {
	typedef
		mdl :: vector :: Vector
		<
			vector :: Cell*,
			storage :: ByValue,
			allocator :: Stack
		>
		Cell;
}

namespace tree {
	typedef
		expression :: Tree
		<
			const prover :: Premise*,
			storage :: ByValue,
			expression :: node :: container :: Singleton,
			allocator :: Stack
		>
		Premise;
	typedef
		expression :: Tree
		<
			const prover :: Node*,
			storage :: ByValue,
			expression :: node :: container :: Singleton,
			allocator :: Prover
		>
		Node;
}
}
namespace forest {
	typedef
		expression :: Forest
		<
			const mdl :: Proposition*,
			storage :: ByValue,
			expression :: node :: container :: List
		>
		Proposition;
}
}

#endif /*MDL_EXPRESSION_TPP_*/
