/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File name:       mdl_expression_Forest.ipp                                */
/* Description:     forest expression set class                              */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_EXPRESSION_FOREST_IPP_
#define MDL_EXPRESSION_FOREST_IPP_

namespace mdl {
namespace expression {

	/****************************
	 *		Public members
	 ****************************/

	template
	<
		class V,
		template<class> class H,
		template<class, template<class> class, class> class S,
		class A
	>
	inline
	Forest<V, H, S, A> :: Forest () :
	Ancestor_ (),
	level_ (0) {
	}
	template
	<
		class V,
		template<class> class H,
		template<class, template<class> class, class> class S,
		class A
	>
	inline
	Forest<V, H, S, A> :: ~ Forest() {
	}

	template
	<
		class V,
		template<class> class H,
		template<class, template<class> class, class> class S,
		class A
	>
	template<class E>
	inline typename Forest<V, H, S, A> :: Iterator_
	Forest<V, H, S, A> :: addBranch
	(
		const E* const expression,
		const Value_ value,
		const index :: Step level
	)
	{
		if (level == index :: undefined :: STEP) {
			++ level_;
		} else {
			level_ = level;
		}
		return Ancestor_ :: addBranch (expression, value);
	}
	template
	<
		class V,
		template<class> class H,
		template<class, template<class> class, class> class S,
		class A
	>
	template<class E>
	inline void
	Forest<V, H, S, A> :: addSubBranches
	(
		const E* const expression,
		const Value_ value,
		const index :: Step level
	)
	{
		if (level == index :: undefined :: STEP) {
			++ level_;
		} else {
			level_ = level;
		}
		Ancestor_ :: addSubBranches (expression, value);
	}
	template
	<
		class V,
		template<class> class H,
		template<class, template<class> class, class> class S,
		class A
	>
	template<class E>
	inline void
	Forest<V, H, S, A> :: addFactorBranches
	(
		const E* const expression,
		const Value_ value,
		const index :: Step level
	)
	{
		if (level == index :: undefined :: STEP) {
			++ level_;
		} else {
			level_ = level;
		}
		Ancestor_ :: addFactorBranches (expression, value);
	}
	template
	<
		class V,
		template<class> class H,
		template<class, template<class> class, class> class S,
		class A
	>
	inline index :: Step
	Forest<V, H, S, A> :: getLevel() const {
		return level_;
	}
	template
	<
		class V,
		template<class> class H,
		template<class, template<class> class, class> class S,
		class A
	>
	template<class I>
	inline void
	Forest<V, H, S, A> :: copyNode (Iterator_& target, const I& source) const
	{
		Ancestor_ :: copyNode (target, source);
		target->setLevel (source->getLevel());
	}
	template
	<
		class V,
		template<class> class H,
		template<class, template<class> class, class> class S,
		class A
	>
	template<class I>
	inline bool
	Forest<V, H, S, A> :: findNode (Iterator_& target, const I& source) const {
		return Ancestor_ :: findNode (target, source);
	}
	template
	<
		class V,
		template<class> class H,
		template<class, template<class> class, class> class S,
		class A
	>
	template<class I>
	inline void
	Forest<V, H, S, A> :: addFindNode (Iterator_& target, const I& source) const
	{
		Ancestor_ :: addFindNode (target, source);
		if (target->getLevel() == index :: undefined :: STEP) {
			target->setLevel (level_);
		}
	}

using manipulator :: endLine;

	// object :: Object interface
	template
	<
		class V,
		template<class> class H,
		template<class, template<class> class, class> class S,
		class A
	>
	void
	Forest<V, H, S, A> :: commitSuicide() {
		delete this;
	}
	template
	<
		class V,
		template<class> class H,
		template<class, template<class> class, class> class S,
		class A
	>
	Size_t
	Forest<V, H, S, A> :: getVolume() const {
		return Ancestor_ :: getVolume();
	}
	template
	<
		class V,
		template<class> class H,
		template<class, template<class> class, class> class S,
		class A
	>
	Size_t
	Forest<V, H, S, A> :: getSizeOf() const {
		return sizeof (Forest);
	}
	template
	<
		class V,
		template<class> class H,
		template<class, template<class> class, class> class S,
		class A
	>
	void
	Forest<V, H, S, A> :: show (String& string) const
	{
		string << "forest:" << endLine;
		if (Ancestor_ :: root_ != NULL) {
			Ancestor_ :: root_->showForest (string);
		} else {
			string << tab << "NULL" << endLine;
		}
		string << endLine;
	}
}
}

#endif /*MDL_EXPRESSION_FOREST_HPP_*/
