/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File name:       mdl_expression_Mode.hpp                                  */
/* Description:     class for the expression show mode                       */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_EXPRESSION_MODE_HPP_
#define MDL_EXPRESSION_MODE_HPP_

namespace mdl {
namespace expression {

class Mode {
public :
	enum {
		SHOW_WRITE  = 0,
		SHOW_SYMBOL = 1,
		SHOW_HEADER = 2,
		SHOW_TYPE   = 3,
		SHOW_TERMS  = 4,
		SHOW_TREE   = 5,
		SHOW_KIND   = 6,
		SHOW_REPLACEABILITY = 7,
		SHOW_SUBSTITUTION   = 8,
		CARDINALITY = SHOW_SUBSTITUTION + 1
	};

	Mode ();
	Mode (const String&);
	Mode (const Mode&);

	bool isPlain() const;
	bool getValue (const char) const;
	void setValue (const char, const bool = true);

	void operator = (const Mode&);
	bool operator == (const Mode&) const;
	bool operator != (const Mode&) const;

	Size_t getVolume() const;
	Size_t getSizeOf() const;
	void show (String&) const;

private :
	friend String&
	operator << (String&, const Mode&);
	friend Mode&
	operator << (Mode&, const char);
	friend Mode&
	operator << (Mode&, const String&);

	typedef
		vector :: Bit<CARDINALITY>
		Bits_;

	Bits_ bits_;
};

	String&
	operator << (String&, const Mode&);
	Mode&
	operator << (Mode&, const char);
	Mode&
	operator << (Mode&, const String&);
}
}

#endif /*MDL_EXPRESSION_MODE_HPP_*/
