/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File name:       mdl_expression_Mode.ipp                                  */
/* Description:     class for the expression show mode                       */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_EXPRESSION_MODE_IPP_
#define MDL_EXPRESSION_MODE_IPP_

namespace mdl {
namespace expression {

	/****************************
	 *		Public members
	 ****************************/

	inline
	Mode :: Mode () :
	bits_() {
	}
	inline
	Mode :: Mode (const String&) :
	bits_ (String :: getMode()) {
	}
	inline
	Mode :: Mode (const Mode& mode) :
	bits_ (mode.bits_) {
	}

	inline bool
	Mode :: isPlain() const {
		return (bits_.get() == 0);
	}
	inline bool
	Mode :: getValue (const char index) const {
		return bits_.getValue (index);
	}
	inline void
	Mode :: setValue (const char index, const bool value) {
		bits_.setValue (index, value);
	}

	inline void
	Mode :: operator = (const Mode& mode) {
		bits_ = mode.bits_;
	}
	inline bool
	Mode :: operator == (const Mode& mode) const {
		return (bits_ == mode.bits_);
	}
	inline bool
	Mode :: operator != (const Mode& mode) const {
		return !(operator == (mode));
	}

	inline Size_t
	Mode :: getVolume() const {
		return bits_.getVolume();
	}
	inline Size_t
	Mode :: getSizeOf() const {
		return sizeof (Mode);
	}
	inline void
	Mode :: show (String& string) const {
		bits_.show (string);
	}

	/****************************
	 *		Overloaded operators
	 ****************************/

	inline String&
	operator << (String& string, const Mode& mode) {
		return string << manipulator :: mode << mode.bits_.get();
	}
	inline Mode&
	operator << (Mode& mode, const char m)
	{
		mode.setValue (m);
		return mode;
	}
	inline Mode&
	operator << (Mode& mode, const String& string)
	{
		const Size_t m = String :: getMode();
		mode.bits_.set (m);
		return mode;
	}
}
}

#endif /*MDL_EXPRESSION_MODE_IPP_*/
