/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File name:       mdl_expression_Symbol.cpp                                */
/* Description:     expression symbol                                        */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_EXPRESSION_SYMBOL_CPP_
#define MDL_EXPRESSION_SYMBOL_CPP_

#include "mdl/auxiliary/mdl_auxiliary.hpp"
#include "mdl/math/mdl_math.hpp"
#include "mdl/expression/mdl_expression_Symbol.hpp"
#include "mdl/expression/mdl_expression_Mode.hpp"

namespace mdl {
namespace expression {

	/****************************
	 *		Public members
	 ****************************/

using manipulator :: space;
using manipulator :: underline;

	inline const String&
	Symbol :: getString () const {
		return Table :: literals()->get (literal_);
	}

	inline void
	Symbol :: build (const object :: Typing* const typing, const bool makeFixed)
	{
		type_ = typing->getType (literal_);
		if (type_ == NULL && !(Math :: constants()->contain (literal_))) {
			Error* error = new Error (Error :: SEMANTIC);
			error->message() << "literal \"" << Table :: literals()->get (literal_) << "\"" << space;
			error->message() << "is not declared." << endLine;
			throw error;
		}
		if (type_ == NULL || makeFixed) {
			isReplaceable_ = false;
		}
	}
	inline void
	Symbol :: write (String& string) const
	{
		if (isUndefined()) {
			string << "<undefined>";
		} else if (isLambda()) {
			string << "<lambda>";
		} else {
			if (Config :: translate()) {
				string << *Table :: literals()->getAscii (literal_);
			} else {
				string << Table :: literals()->get (literal_);
			}
		}
	}

	// object :: Object implementation
	inline void
	Symbol :: commitSuicide() {
		delete this;
	}
	inline Size_t
	Symbol :: getVolume () const {
		return 0;
	}
	inline Size_t
	Symbol :: getSizeOf () const {
		return sizeof (Symbol);
	}
	inline void
	Symbol :: show (String& string) const
	{
		Mode mode (string);
		if (isUndefined()) {
			string << "<undefined>";
		} else if (isLambda()) {
			string << "<lambda>";
		} else {
			string << Table :: literals()->get (literal_);
		}
		if (isVariable()) {
			if (mode.getValue (Mode :: SHOW_REPLACEABILITY)) {
				if (!isReplaceable_) {
					if (isUndefined()) {
						string << "#";
					} else {
						string << Table :: literals()->get (literal_);
					}
				}
			}
			if (mode.getValue (Mode :: SHOW_TYPE)) {
				string << underline;
				type_->translateName (string);
			}
		}
	}

	/*************************************
	 *		Static private attributes
	 *************************************/

	const value :: Literal
		Symbol :: lambda_ = value :: undefined :: LITERAL - 1;
}
}

#endif /*MDL_EXPRESSION_SYMBOL_CPP_*/
