/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File name:       mdl_expression_node_Forest.ipp                           */
/* Description:     forest (monotonous tree set) expression node             */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_EXPRESSION_NODE_FOREST_IPP_
#define MDL_EXPRESSION_NODE_FOREST_IPP_

namespace mdl {
namespace expression {
namespace node {

	/****************************
	 *		Public members
	 ****************************/

	template<class A>
	Forest<A> :: Forest
	(
		const Forest* const node,
		const Direction direction
	) :
	Forest_ (node, direction) {
	}
	template<class A>
	Forest<A> :: Forest
	(
		const Symbol& symbol,
		const index :: Step level,
		const Forest* const node,
		const Direction direction
	) :
	Forest_ (symbol, level, node, direction) {
	}
	template<class A>
	Forest<A> :: ~ Forest() {
	}

	template<class A>
	inline const typename Forest<A> :: Node_*
	Forest<A> :: goLeft() const {
		return Forest_ :: goLeft (this);
	}
	template<class A>
	inline const typename Forest<A> :: Node_*
	Forest<A> :: goRight() const {
		return Forest_ :: goRight (this);
	}
	template<class A>
	inline const typename Forest<A> :: Node_*
	Forest<A> :: goUp() const {
		return Forest_ :: goUp ();
	}
	template<class A>
	inline const typename Forest<A> :: Node_*
	Forest<A> :: goDown() const {
		return Forest_ :: goDown ();
	}

	template<class A>
	inline typename Forest<A> :: Node_*
	Forest<A> :: goLeft () {
		return Forest_ :: goLeft (this);
	}
	template<class A>
	inline typename Forest<A> :: Node_*
	Forest<A> :: goRight () {
		return Forest_ :: goRight (this);
	}
	template<class A>
	inline typename Forest<A> :: Node_*
	Forest<A> :: goUp () {
		return Forest_ :: goUp (this);
	}
	template<class A>
	inline typename Forest<A> :: Node_*
	Forest<A> :: goDown () {
		return Forest_ :: goDown (this);
	}

	template<class A>
	inline void
	Forest<A> :: operator = (const Symbol& symbol) {
		Forest_ :: operator = (symbol);
	}

	// object :: Object implementation
	template<class A>
	inline void
	Forest<A> :: commitSuicide() {
		delete this;
	}
	template<class A>
	inline Size_t
	Forest<A> :: getVolume() const {
		return Forest_ :: getVolume();
	}
	template<class A>
	inline Size_t
	Forest<A> :: getSizeOf() const {
		return sizeof (Forest);
	}
	template<class A>
	inline void
	Forest<A> :: show (String& string) const {
		Forest_ :: show (string);
	}
}
}
}

#endif /*MDL_EXPRESSION_NODE_FOREST_IPP_*/
