/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File name:       mdl_expression_node_Tree.ipp                             */
/* Description:     tree (2-dimensional) expression node                     */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_EXPRESSION_NODE_TREE_IPP_
#define MDL_EXPRESSION_NODE_TREE_IPP_

namespace mdl {
namespace expression {
namespace node {

	/****************************
	 *		Public members
	 ****************************/

	template<class A>
	Tree<A> :: Tree
	(
		const Tree* const node,
		const Direction direction
	) :
	Tree_ (node, direction) {
	}
	template<class A>
	Tree<A> :: Tree
	(
		const Symbol& symbol,
		const Tree* const node,
		const Direction direction
	) :
	Tree_ (symbol, node, direction) {
	}
	template<class A>
	Tree<A> :: ~ Tree() {
	}

	template<class A>
	inline const typename Tree<A> :: Node_*
	Tree<A> :: goLeft() const {
		return Tree_ :: goLeft (this);
	}
	template<class A>
	inline const typename Tree<A> :: Node_*
	Tree<A> :: goRight() const {
		return Tree_ :: goRight (this);
	}
	template<class A>
	inline const typename Tree<A> :: Node_*
	Tree<A> :: goUp() const {
		return Tree_ :: goUp ();
	}
	template<class A>
	inline const typename Tree<A> :: Node_*
	Tree<A> :: goDown() const {
		return Tree_ :: goDown ();
	}

	template<class A>
	inline typename Tree<A> :: Node_*
	Tree<A> :: goLeft() {
		return Tree_ :: goLeft (this);
	}
	template<class A>
	inline typename Tree<A> :: Node_*
	Tree<A> :: goRight() {
		return Tree_ :: goRight (this);
	}
	template<class A>
	inline typename Tree<A> :: Node_*
	Tree<A> :: goUp() {
		return Tree_ :: goUp (this);
	}
	template<class A>
	inline typename Tree<A> :: Node_*
	Tree<A> :: goDown() {
		return Tree_ :: goDown (this);
	}

	template<class A>
	inline void
	Tree<A> :: operator = (const Symbol& symbol) {
		Tree_ :: operator = (symbol);
	}

	// object :: Object implementation
	template<class A>
	inline void
	Tree<A> :: commitSuicide() {
		delete this;
	}
	template<class A>
	inline Size_t
	Tree<A> :: getVolume() const {
		return Tree_ :: getVolume();
	}
	template<class A>
	inline Size_t
	Tree<A> :: getSizeOf() const {
		return sizeof (Tree);
	}
	template<class A>
	inline void
	Tree<A> :: show (String& string) const {
		Tree_ :: show (string);
	}
}
}
}

#endif /*MDL_EXPRESSION_NODE_TREE_IPP_*/
