/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File name:       mdl_expression_parse_Parser.hpp                          */
/* Description:     static class: expression parser                          */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_EXPRESSION_PARSE_PARSER_HPP_
#define MDL_EXPRESSION_PARSE_PARSER_HPP_

#include "mdl/interface/mdl_interface.dpp"
#include "mdl/expression/parse/mdl_expression_parse_Unit.hpp"
#include "mdl/expression/parse/mdl_expression_parse_Result.hpp"
#include "mdl/expression/parse/mdl_expression_parse_Tree.hpp"

namespace mdl {
namespace expression {
namespace parse {

template<bool d, bool f>
class Parser {
public :
	enum {
		deep_ = d,
		final_ = f
	};
	typedef
		array :: Expression
		Expression_;
	typedef
		Expression_ :: Term_
		Term_;
	typedef
		Substitution<Term_>
		Substitution_;
	typedef
		tree :: Rule :: const_Iterator_
		tree_Iterator_;
	typedef
		Expression_ :: Iterator_
		expr_Iterator_;
	typedef
		Unit<Parser>
		Unit_;
	typedef
		Tree<Parser>
		Tree_;
	typedef
		Result<Parser>
		Result_;

	static bool proceed (Expression_* const, const mdl :: Type* const);

private :
	template<class>
	friend class Unit;

	static Result_ parseVariable (expr_Iterator_, const mdl :: Type*);
	static Result_ parseTerm (expr_Iterator_, const mdl :: Type*, const Tree_* = NULL);
	static Result_ parseTerm (Unit_, const mdl :: Type*);
	static Result_ parseNode (Unit_, const mdl :: Type*);

	static Result_ tryConstant (Unit_, const mdl :: Type*);
	static Result_ tryVariable (Unit_, const mdl :: Type*);
	static Result_ tryType (Unit_, const mdl :: Type*, const mdl :: Type*);
	static Result_ tryRule (Unit_, const mdl :: Type*, const mdl :: Type*);
	static Result_ tryTerm (Unit_, const mdl :: Type*, const Tree_*);
	static Result_ tryVar (Unit_, const mdl :: Type*, const mdl :: Type*);

	static Result_ parseNext (Unit_, const mdl :: Type*);

	static Expression_* expression_;
};

}
}
}

#endif /*MDL_EXPRESSION_PARSE_PARSER_HPP_*/
