/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File name:       mdl_expression_parse_Result.ipp                          */
/* Description:     parsing result class                                     */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_EXPRESSION_PARSE_RESULT_IPP_
#define MDL_EXPRESSION_PARSE_RESULT_IPP_

namespace mdl {
namespace expression {
namespace parse {

	/****************************
	 *		Public members
	 ****************************/

	template<class P>
	inline
	Result<P> :: Result () :
	list_() {
	}
	template<class P>
	inline
	Result<P> :: Result (const Result& result) :
	list_ (result.list_) {
	}
	template<class P>
	inline
	Result<P> :: Result (const Tree_* tree) :
	list_() {
		list_.addLast (tree);
	}

	template<class P>
	inline void
	Result<P> :: operator = (const Result& result) {
		list_ = result.list_;
	}

	template<class P>
	inline bool
	Result<P> :: isObtained() const {
		return !list_.isEmpty();
	}
	template<class P>
	inline void
	Result<P> :: join (const Result& result) {
		list_.join (result.list_);
	}

using manipulator :: endLine;
using manipulator :: tab;

	template<class P>
	inline bool
	Result<P> :: confirm() const
	{
		if (list_.isEmpty()) {
			return confirmEmpty();
		}
		if (list_.isSingleton()) {
			return confirmSingleton();
		} else {
			return confirmList();
		}
	}
	template<class P>
	inline const typename Result<P> :: List_&
	Result<P> :: getList() const {
		return list_;
	}

	/****************************
	 *		Private members
	 ****************************/

	template<class P>
	inline bool
	Result<P> :: confirmEmpty() const
	{
		if (final_) {
			Error* error = new Error (Error :: SEMANTIC);
			error->message() << "expression fails parsing:" << endLine;
			error->message() << tab << "does not match grammar." << endLine;
			throw error;
		} else {
			return false;
		}
	}
	template<class P>
	inline bool
	Result<P> :: confirmSingleton() const
	{
		const Node_* node = list_.getFirst();
		const Tree_* tree = node->getValue();
		if (!tree->isTop()) {
			if (final_) {
				Error* error = new Error (Error :: SEMANTIC);
				error->message() << "expression fails parsing:" << endLine;
				error->message() << tab << "does not match grammar." << endLine;
				throw error;
			} else {
				return false;
			}
		} else {
			tree->confirm();
			return true;
		}
	}
	template<class P>
	inline bool
	Result<P> :: confirmList() const
	{
		const Node_* node = list_.getFirst();
		const Tree_* tree = NULL;
		while (node != NULL) {
			const Tree_* otherTree = node->getValue();
			if (otherTree->isTop()) {
				if (tree == NULL) {
					tree = otherTree;
				} else {
					if (final_) {
						Error* error = new Error (Error :: SEMANTIC);
						error->message() << "expression fails parsing:" << endLine;
						error->message() << tab << "is ambiguous in grammar." << endLine;
						error->message() << tab << "obtained variants:" << endLine;
						tree->show (error->message());
						error->message() << endLine;
						otherTree->show (error->message());
						error->message() << endLine;
						throw error;
					} else {
						return false;
					}
				}
			}
			node = node->getNext();
		}
		tree->confirm();
		return true;
	}
}
}
}

#endif /*MDL_EXPRESSION_PARSE_RESULT_IPP_*/
