/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File name:       mdl_expression_parse_Unit.hpp                            */
/* Description:     expression parser unit: pair of iterators& substitutiuon */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_EXPRESSION_PARSE_UNIT_HPP_
#define MDL_EXPRESSION_PARSE_UNIT_HPP_

#include "mdl/expression/iterator/mdl_expression_iterator.hpp"

namespace mdl {
namespace expression {
namespace parse {

template<class P>
class Unit : public
	iterator :: modifier :: Pair
	<
		typename P :: expr_Iterator_,
		typename P :: tree_Iterator_
	> {
public :
	typedef P Parser_;
	typedef
		typename Parser_ :: expr_Iterator_
		expr_Iterator_;
	typedef
		typename Parser_ :: tree_Iterator_
		tree_Iterator_;
	typedef
		typename Parser_ :: Substitution_
		Substitution_;
	typedef
		typename Parser_ :: Term_
		Term_;
	typedef
		typename Parser_ :: Tree_
		Tree_;
	typedef
		typename iterator :: modifier :: Pair
		<
			expr_Iterator_,
			tree_Iterator_
		>
		Pair_;

	Unit
	(
		const expr_Iterator_&,
		const tree_Iterator_&,
		const Tree_* = NULL
	);
	Unit (const Unit&);

	bool isConstant() const;
	bool isVariable() const;

	enum State {
		DEADLOCK,
		FINAL,
		TERMINAL,
		TRANSIT
	};
	State getState() const;

	void operator ++ ();
	void operator + ();
	void operator - ();

	const Symbol& getTreeSymbol() const;
	const mdl :: Type* getTreeType() const;
	const mdl :: Type* getExprType() const;
	const mdl :: Rule* getTreeRule() const;

	expr_Iterator_ getBegin() const;
	expr_Iterator_ getEnd() const;
	const Tree_* getTree() const;
	void setEnd (const expr_Iterator_);
	void setTree (const Tree_*);

	const Substitution_* getSubstitution() const;
	void newSubstitution (const Term_*);

	const Tree_* newRuleTerm (const mdl :: Type*, const bool);
	const Tree_* newVarTerm (const mdl :: Type*);

	value :: Integer getExpressionLength() const;
	value :: Integer getTreeLength() const;

	void show (String&) const;
	void show() const;

private :
	void showExpression (String&) const;
	void showTreeBranch (String&) const;

	expr_Iterator_ begin_;
	tree_Iterator_ root_;
	Substitution_* substitution_;
	const Tree_*   tree_;
};

	template<class P>
	String&
	operator << (String&, const Unit<P>);
	template<class P>
	std :: ostream&
	operator << (std :: ostream&, const Unit<P>);
}
}
}

#endif /*MDL_EXPRESSION_PARSE_UNIT_HPP_*/
