/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File name:       mdl_expression_unify_sub_Finaly.ipp                      */
/* Description:     final procedure of a substitution unification            */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_EXPRESSION_UNIFY_SUB_FINALY_IPP_
#define MDL_EXPRESSION_UNIFY_SUB_FINALY_IPP_

namespace mdl {
namespace expression {
namespace unify {
namespace sub {

	/****************************
	 *		Public members
	 ****************************/

	template<class S>
	inline void
	Finaly<S> :: addPropEvidence
	(
		const index :: Arity arity,
		prover :: Evidence* evidenceSub,
		const prover :: Evidence* evidenceUp[]
	)
	{
		prover :: evidence :: Prop* evidenceProp = new prover :: evidence :: Prop
		(
			evidenceSub->getNode(),
			evidenceSub,
			arity
		);
		for (index :: Arity i = 0; i < arity; ++ i) {
			evidenceProp->addUp (i, evidenceUp [i]);
		}
		dynamic_cast<prover :: evidence :: Sub*>(evidenceSub)->addUp (evidenceProp);
	}

	template<class S>
	inline prover :: Evidence*
	Finaly<S> :: supplySubEvidence
	(
		const value :: Integer boundary,
		const Substitution_* sb,
		const Substitution_* dl,
		const Substitution_* th,
		prover :: evidence :: Vector& proofs
	)
	{
		prover :: Evidence* evidenceSub = NULL;
		Memory :: stack().push(); {
			Substitution_*
				delta = Substitution_ :: create (dl);
			Substitution_*
				theta = Substitution_ :: create (th);

			if (!Substitution_ :: unifyCommonVariables (delta, theta)) {
				Memory :: stack().pop();
				return NULL;
			}
			delta = delta->compose (theta, false);
			theta = theta->compose (delta);

			if (!Substitution_ :: compositionClosure (boundary, theta)) {
				Memory :: stack().pop();
				return NULL;
			}

			Substitution_*
				sub = Substitution_ :: create (sb);

			if (!Substitution_ :: unifyCommonVariables (sub, theta)) {
				Memory :: stack().pop();
				return NULL;
			}
			sub = sub->compose (theta);

			/*if (!StackSubstitution_ :: compositionClosure (unit.getBoundary(), sub)) {
				Memory :: stack().pop();
				return;
			}*/

			evidenceSub = proofs.supply (sub);
		}
		Memory :: stack().pop();
		return evidenceSub;
	}
}
}
}
}

#endif /*MDL_EXPRESSION_UNIFY_SUB_FINALY_IPP_*/
