/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File Name:       mdl_expression_unify_sub_prod_Multyindex.hpp             */
/* Description:     vector of proof indexes                                  */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_EXPRESSION_UNIFY_SUB_PROD_MULTYINDEX_HPP_
#define MDL_EXPRESSION_UNIFY_SUB_PROD_MULTYINDEX_HPP_

#include "mdl/interface/prover/mdl_interface_prover.dpp"

namespace mdl {
namespace expression {
namespace unify {
namespace sub {
namespace prod {

template<class S>
class Multyindex {
public :
	typedef S Substitution_;
	typedef
		vector :: Vector
		<
			const Substitution_*,
			storage :: ByValue,
			allocator :: Stack
		>
		Substivector_;

	Multyindex
	(
		const index :: Arity,
		const prover :: proof :: Matrix&
	);

	const prover :: Evidence* getValue (const index :: Arity) const;
	bool operator ++();
	bool isOld() const;
	bool isNew() const;
	void assign (Substivector_&) const;
	bool fitsDepth() const;
	bool fitsComplexity() const;

	value :: Integer getSearchVolume() const;
	value :: Integer getWholeVolume() const;
	value :: Integer getObservedVolume() const;

	template<class S1>
	void show (S1&) const;

private :
	typedef
		mdl :: vector :: Vector
		<
			value :: Integer,
			storage :: ByValue,
			allocator :: Stack
		>
		IndexVector_;

	bool increment (const index :: Arity = 0);

	const index :: Arity dimension_;
	const prover :: proof :: Matrix& matrix_;
	IndexVector_ indexVector_;
};

	template<class S1, class S2>
	S1&
	operator << (S1&, const Multyindex<S2>&);
	template<class S>
	std :: ostream&
	operator << (std :: ostream&, const Multyindex<S>&);
}
}
}
}
}

#endif /*MDL_EXPRESSION_UNIFY_SUB_PROD_MULTYINDEX_HPP_*/
