/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File name:       mdl_expression_unify_sub_prod_Unifier.hpp                */
/* Description:     scalar substitution unifier module                       */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_EXPRESSION_UNIFY_SUB_PROD_UNIFIER_HPP_
#define MDL_EXPRESSION_UNIFY_SUB_PROD_UNIFIER_HPP_

#include "mdl/expression/unify/sub/prod/mdl_expression_unify_sub_prod_Multyindex.hpp"
#include "mdl/expression/unify/sub/prod/mdl_expression_unify_sub_prod_Matrix.hpp"

namespace mdl {
namespace expression {
namespace unify {
namespace sub {
namespace prod {

template
<
	class E,
	int P = policy :: SYMMETRIC,
	int K = kind :: COMMON,
	class A = allocator :: Stack
>
class Unifier : public Scalar<A> {
public :
	typedef Matrix<E, P, K, A> Matrix_;
	typedef A Allocator_;
	typedef E Expression_;
	enum { policy_ = P };
	enum { kind_ = K };
	typedef
		Substitution<typename Expression_ :: Term_>
		Substitution_;
	typedef
		vector :: Vector
		<
			const Substitution_*,
			storage :: ByValue,
			Allocator_
		>
		Substivector_;
	typedef
		typename prod :: Multyindex<Substitution_>
		Multyindex_;

	Unifier
	(
		const index :: Arity,
		const Substitution_*,
		prover :: evidence :: Vector&
	);

	void operator() (const prover :: proof :: Matrix&);
	void show (String&) const;

private :
	typedef
		List<Allocator_>
		InnerExpression_;
	typedef
		typename InnerExpression_ :: Term_
		InnerTerm_;
	typedef
		Substitution<InnerTerm_>
		InnerSubstitution_;
	typedef
		vector :: Vector
		<
			InnerExpression_*,
			storage :: ByValue,
			Allocator_
		>
		InnerExprevector_;

	void tryMultyindex (const Substivector_&, const Multyindex_&);
	void finalize (const Multyindex_&);
	InnerSubstitution_* createTheta();

	const index :: Arity dimension_;
	const Substitution_* substitutionInt_;
	prover :: evidence :: Vector& proofs_;
	Matrix_* matrix_;
};

}
}
}
}
}

#endif /*MDL_EXPRESSION_UNIFY_SUB_PROD_UNIFIER_HPP_*/
