/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File name:       mdl_expression_unify_sub_prod_Unifier.ipp                */
/* Description:     scalar substitution unifier module                       */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_EXPRESSION_UNIFY_SUB_PROD_UNIFIER_IPP_
#define MDL_EXPRESSION_UNIFY_SUB_PROD_UNIFIER_IPP_

namespace mdl {
namespace expression {
namespace unify {
namespace sub {
namespace prod {

	/****************************
	 *		Public members
	 ****************************/

	template<class E, int P, int K, class A>
	inline
	Unifier<E, P, K, A> :: Unifier
	(
		const index :: Arity dimension,
		const Substitution_* substitutionInt,
		prover :: evidence :: Vector& proofs
	) :
	dimension_ (dimension),
	substitutionInt_ (substitutionInt),
	proofs_ (proofs),
	matrix_ (NULL) {
	}

	template<class E, int P, int K, class A>
	inline void
	Unifier<E, P, K, A> :: operator() (const prover :: proof :: Matrix& matrix)
	{
		Memory :: stack().push();
		Substivector_ substivector (dimension_);
		Multyindex_ multyindex (dimension_, matrix);

		while (true) {
			Prover :: refresh ("unifying prod matrix");
			const bool goodMultyindex =
				multyindex.isNew() &&
				multyindex.fitsDepth() &&
				multyindex.fitsComplexity();
			if (goodMultyindex) {
				multyindex.assign (substivector);
				Memory :: stack().push();
				tryMultyindex (substivector, multyindex);
				Memory :: stack().pop();
			}
			if (++ multyindex) {
				break;
			}
		}
		Memory :: stack().pop();
	}

using manipulator :: endLine;
using manipulator :: space;
using manipulator :: tab;

	template<class E, int P, int K, class A>
	void
	Unifier<E, P, K, A> :: show (String& string) const {
		matrix_->show (string);
	}

	/****************************
	 *		Private members
	 ****************************/

	template<class E, int P, int K, class A>
	inline void
	Unifier<E, P, K, A> :: tryMultyindex
	(
		const Substivector_& substivector,
		const Multyindex_& multyindex
	)
	{
		matrix_ = new Matrix_ (substivector);
		if (!matrix_->unify()) {
			return;
		}
		finalize (multyindex);
	}
	template<class E, int P, int K, class A>
	void
	Unifier<E, P, K, A> :: finalize (const Multyindex_& multyindex)
	{
		prover :: Evidence* evidenceSub = NULL;
		Memory :: stack().push(); {
			InnerSubstitution_* theta = createTheta();
			evidenceSub = sub :: template Finaly<InnerSubstitution_> :: supplySubEvidence
			(
				matrix_->variables_.getSize(),
				substitutionInt_->template castAway<allocator :: Stack>(),
				matrix_->delta_,
				theta,
				proofs_
			);
		}
		Memory :: stack().pop();
		if (evidenceSub == NULL) {
			return;
		}
		const prover :: Evidence* evidenceUp [dimension_];
		for (index :: Arity i = 0; i < dimension_; ++ i) {
			evidenceUp [i] = multyindex.getValue (i);
		}
		sub :: template Finaly<InnerSubstitution_> :: addPropEvidence
		(
			dimension_,
			evidenceSub,
			evidenceUp
		);
	}
	template<class E, int P, int K, class A>
	typename Unifier<E, P, K, A> :: InnerSubstitution_*
	Unifier<E, P, K, A> :: createTheta()
	{
		InnerSubstitution_* theta = NULL;
		for (index :: Arity i = 0; i < matrix_->dimensionY_; ++ i) {
			const InnerExpression_*
				innerExpression = NULL;
			const InnerExprevector_*
				exprevector = matrix_->exprematrix_.getValue (i);
			for (index :: Arity j = 0; j < matrix_->dimensionX_; ++ j) {
				innerExpression = exprevector->getValue (j);
				if (innerExpression != NULL) {
					break;
				}
			}
			if (innerExpression != NULL) {
				const InnerTerm_* const
					term = InnerExpression_ :: createTerm (innerExpression->getTerm());
				const Symbol&
					variable = matrix_->variables_.getValue (i);
				if (*term == variable) {
					continue;
				}
				theta = new InnerSubstitution_
				(
					matrix_->variables_.getValue (i),
					term, theta, true
				);
			}
		}
		return theta;
	}
}
}
}
}
}

#endif /*MDL_EXPRESSION_UNIFY_SUB_PROD_UNIFIER_IPP_*/
