/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File name:       mdl_expression_unify_sub_tree_Finalize.hpp               */
/* Description:     final procedure of a matrix unification                  */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_EXPRESSION_UNIFY_SUB_TREE_FINALIZE_IPP_
#define MDL_EXPRESSION_UNIFY_SUB_TREE_FINALIZE_IPP_

#include "mdl/expression/unify/sub/mdl_expression_unify_sub_Finaly.hpp"

namespace mdl {
namespace expression {
namespace unify {
namespace sub {
namespace tree {

	/****************************
	 *		Public members
	 ****************************/

	template<class U>
	inline
	Finalize<U> :: Finalize
	(
		const Substitution_* substitutionInt,
		prover :: evidence :: Vector& proofs,
		const value :: Integer boundary
	) :
	substitutionInt_ (substitutionInt),
	proofs_ (proofs),
	boundary_ (boundary) {
	}

	template<class U>
	inline void
	Finalize<U> :: operator () (const Unit_& unit)
	{
		prover :: Evidence* evidenceSub = NULL;
		Memory :: stack().push(); {
			const Substitution_* theta = createTheta (unit);
			evidenceSub = sub :: template Finaly<Substitution_> :: supplySubEvidence
			(
				boundary_,
				substitutionInt_,
				unit.getSubstitution(),
				theta,
				proofs_
			);
		}
		Memory :: stack().pop();
		if (evidenceSub == NULL) {
			return;
		}
		const index :: Arity arity = unit.getDimension();
		const prover :: Evidence* evidenceUp [arity];
		for (index :: Arity i = 0; i < arity; ++ i) {
			evidenceUp [i] = unit.getValue (i).getEvidence();
		}
		sub :: template Finaly<Substitution_> :: addPropEvidence
		(
			arity,
			evidenceSub,
			evidenceUp
		);
	}

	/****************************
	 *		Private members
	 ****************************/

	template<class U>
	inline const typename Finalize<U> :: Substitution_*
	Finalize<U> :: createTheta (const Unit_& unit) const
	{
		Substitution_*
			delta = Substitution_ :: create (unit.getSubstitution());
		Substitution_* theta = NULL;
		for (index :: Arity i = 0; i < unit.getDimension(); ++ i) {
			const prover :: Evidence*
				evidence = unit.getValue (i).getEvidence();
			const Substitution_*
				substitution = Substitution_ :: create (evidence->getSubstitution());
			theta = theta->unite (substitution);
		}
		theta = theta->compose (delta);
		return theta;
	}
}
}
}
}
}

#endif /*MDL_EXPRESSION_UNIFY_SUB_TREE_FINALIZE_IPP_*/
