/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File Name:       mdl_form_block_Assertion.hpp                             */
/* Description:     assertion class                                          */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_FORM_BLOCK_ASSERTION_HPP_
#define MDL_FORM_BLOCK_ASSERTION_HPP_

#include "mdl/interface/mdl_interface.hpp"
#include "mdl/form/mdl_form_Identifiable.hpp"

namespace mdl {
namespace form {
namespace block {

template<class H, class P>
class Assertion :
	public form :: Identifiable,
	virtual public mdl :: Assertion {
protected :
	typedef
		typename form :: Identifiable
		Ancestor_;
	typedef H Hypothesis_;
	typedef P Proposition_;
	typedef
		vector :: Vector
		<
			const Proposition_,
			storage :: ByPointer,
			allocator :: Heap
		>
		PropositionVector_;
	typedef
		vector :: Vector
		<
			const Proposition_*,
			storage :: ByValue,
			allocator :: Heap
		>
		ParserPropositionVector_;
	typedef
		vector :: Vector
		<
			const Hypothesis_,
			storage :: ByPointer,
			allocator :: Heap
		>
		HypothesisVector_;
	typedef
		vector :: Vector
		<
			const Hypothesis_*,
			storage :: ByValue,
			allocator :: Heap
		>
		ParserHypothesisVector_;

public :
	Assertion
	(
		const Location&,
		const mdl :: Comments* const,
		const lexer :: Indent,
		const value :: Name,
		const mdl :: Variables* const,
		const mdl :: Disjointeds* const,
		const ParserHypothesisVector_&,
		const ParserPropositionVector_&
	);
	virtual ~ Assertion();

	// mdl :: Assertion interface
	virtual value :: Kind getKind() const = 0;
	virtual index :: Assertion getIndex() const;

	// general mdl :: Assertion interface part
	virtual const mdl :: Variables* getVariables() const;
	virtual const mdl :: Disjointeds* getDisjointeds() const;
	virtual index :: Arity getHypArity() const;
	virtual index :: Arity getPropArity() const;
	virtual const mdl :: Hypothesis* getHypothesis (const index :: Arity) const;
	virtual const mdl :: Proposition* getProposition (const index :: Arity) const;
	virtual void completeWithHypotheses (prover :: tree :: Premise*) const;

	// object :: Identifiable interface
	virtual const mdl :: Theory* getTheory() const;
	virtual value :: Name getName() const;
	virtual void translateName (String&, const index :: Arity = 0) const;
	virtual void replicateName (String&) const;

	// object :: Verifiable interface
	virtual void check() const = 0;
	virtual void compress() = 0;
	virtual void prove() = 0;
	virtual bool isVerified() const = 0;

	// object :: Buildable interface
	virtual void buildTheory (mdl :: Theory* const);

	// object :: Writable interface
	virtual void translate (String&) const;
	virtual void replicate (String&) const;

	// object :: Object interface
	virtual void commitSuicide() = 0;
	virtual Size_t getSizeOf() const;
	virtual Size_t getVolume() const;

protected :
	void replicateHeader (String&) const;
	void replicateHypothesis (String&) const;
	void replicateBar (String&) const;
	void replicatePropositions (String&) const;

	void buildHeader (mdl :: Theory* const);
	void buildHypothesis (mdl :: Theory* const);
	void buildPropositions (mdl :: Theory* const);
	template<class T>
	void add (const T*, mdl :: Theory* const);

	const mdl :: Variables* variables_;
	const mdl :: Disjointeds* disjointeds_;
	const HypothesisVector_ hypothesisVector_;
	const PropositionVector_ propositionVector_;

private :
	const index :: Assertion index_;
	const lexer :: Indent barIndent_;
};

}
}
}

#endif /*MDL_FORM_BLOCK_ASSERTION_HPP_*/
