/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File Name:       mdl_form_block_Axiom.cpp                                 */
/* Description:     axiom class                                              */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_FORM_BLOCK_AXIOM_CPP_
#define MDL_FORM_BLOCK_AXIOM_CPP_

#include "mdl/form/block/mdl_form_block_Axiom.hpp"

namespace mdl {
namespace form {
namespace block {

	/****************************
	 *		Public members
	 ****************************/

	Axiom :: Axiom
	(
		const Location& location,
		const mdl :: Comments* const comments,
		const lexer :: Indent barIndent,
		const value :: Name name,
		const mdl :: Variables* const variables,
		const mdl :: Disjointeds* const disjointeds,
		const vector :: parser :: Hypothesis& hypothesisVector,
		const vector :: parser :: Axiomatic& propositionVector
	) :
	Ancestor_
	(
		location,
		comments,
		barIndent,
		name,
		variables,
		disjointeds,
		hypothesisVector,
		propositionVector
	) {
	}
	Axiom :: ~ Axiom() {
	}

	// mdl :: Assertion interface
	value :: Kind
	Axiom :: getKind() const {
		return Token :: AXIOM;
	}

	// object :: Verifiable implementation
	void
	Axiom :: check() const {
	}
	void
	Axiom :: compress() {
	}
	void
	Axiom :: prove() {
	}
	bool
	Axiom :: isVerified() const {
		return true;
	}

	// object :: Buildable implementation
	void
	Axiom :: buildTheory (mdl :: Theory* const theory)
	{
		Ancestor_ :: buildTheory (theory);
		Ancestor_ :: add (this, theory);
	}

using manipulator :: space;
using manipulator :: endLine;

	// object :: Writable implementation
	void
	Axiom :: replicate (String& string) const
	{
		if (!Config :: removeComments() && (comments_ != NULL)) {
			string << location_.indent();
			comments_->replicate (string);
			string << endLine;
		}
		string << location_.indent() << Token :: axiom_ << space;
		Ancestor_ :: replicate (string);
		string << endLine;
	}

	// object :: Object implementation
	void
	Axiom :: commitSuicide() {
		delete this;
	}
}
}
}

#endif /*MDL_FORM_BLOCK_AXIOM_CPP_*/
