/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File Name:       mdl_form_block_Constant.cpp                              */
/* Description:     constant class                                           */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_FORM_BLOCK_CONSTANT_CPP_
#define MDL_FORM_BLOCK_CONSTANT_CPP_

#include "mdl/math/mdl_math.hpp"
#include "mdl/form/mdl_form.hpp"

#include "smm/lexer/smm_lexer_Token.hpp"

namespace mdl {
namespace form {
namespace block {

	/****************************
	 *		Public members
	 ****************************/

	inline
	Constant :: Constant
	(
		const Location& location,
		const mdl :: Comments* const comments,
		const String* const symbol,
		const String* const ascii,
		const String* const latex
	) :
	location_ (location),
	comments_ (comments),
	symbol_ (symbol),
	ascii_ (ascii),
	latex_ (latex),
	literal_ (Table :: literals()->add (symbol, ascii, latex)) {
	}
	Constant :: ~ Constant()
	{
		deleteAggregateObject (comments_);
		deleteAggregateObject (symbol_);
		deleteAggregateObject (ascii_);
		deleteAggregateObject (latex_);
	}

	// mdl :: Constant implementation
	index :: Literal
	Constant :: getSize() const {
		return 1;
	}
	value :: Literal
	Constant :: getValue (const index :: Literal index) const {
		return literal_;
	}

	// object :: Verifiable implementation
	void
	Constant :: check() const {
	}
	void
	Constant :: compress() {
	}
	void
	Constant :: prove() {
	}
	bool
	Constant :: isVerified() const {
		return true;
	}

	// object :: Buildable implementation
	void
	Constant :: buildTheory (mdl :: Theory* const)
	{
		try {
			Math :: constants()->add (this);
		} catch (Error* error) {
			error->add (location_);
			throw error;
		}
	}

using nstd :: manipulator :: endLine;
using nstd :: manipulator :: space;
using nstd :: manipulator :: tab;

	// object :: Writable implementation
	void
	Constant :: translate (String& string) const
	{
		if (!Config :: removeComments() && (comments_ != NULL)) {
			string << location_.indent();
			comments_->translate (string);
			string << endLine;
		}
		string << location_.indent();
		string << :: smm :: Token :: constant_ << space;
		string << *Table :: literals()->getAscii (literal_) << space;
		string << :: smm :: Token :: end_ << endLine;
		string << endLine;
	}
	void
	Constant :: replicate (String& string) const
	{
		if (!Config :: removeComments() && (comments_ != NULL)) {
			string << location_.indent();
			comments_->replicate (string);
			string << endLine;
		}
		string << location_.indent() << Token :: constant_ << space << endLine;
		string << location_.indent() << Token :: openBrace_ << space << endLine;

		string << location_.indent() << tab << Token :: symbol_ << space;
		string << *symbol_ << space << Token :: semicolon_ << endLine;;

		if (ascii_ != NULL) {
			string << location_.indent() << tab << Token :: ascii_ << space;
			string << *ascii_ << space << Token :: semicolon_ << endLine;;
		}
		if (latex_ != NULL) {
			string << location_.indent() << tab << Token :: latex_ << space;
			string << *latex_ << space << Token :: semicolon_ << endLine;;
		}
		string << location_.indent() << Token :: closeBrace_ << space << endLine;
		string << endLine;
	}

	// object :: Object implementation
	void
	Constant :: commitSuicide() {
		delete this;
	}
	Size_t
	Constant :: getVolume() const
	{
		Size_t volume = 0;
		volume += getAggregateVolume (comments_);
		volume += getAggregateVolume (symbol_);
		volume += getAggregateVolume (ascii_);
		volume += getAggregateVolume (latex_);
		return volume;
	}
	Size_t
	Constant :: getSizeOf() const {
		return sizeof (Constant);
	}
}
}
}

#endif /*MDL_FORM_BLOCK_CONSTANT_CPP_*/
