/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File Name:       mdl_form_block_Constant.hpp                              */
/* Description:     constant class                                           */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_FORM_BLOCK_CONSTANT_HPP_
#define MDL_FORM_BLOCK_CONSTANT_HPP_

#include "mdl/interface/mdl_interface.hpp"

namespace mdl {
namespace form {
namespace block {

class Constant :
	public mdl :: Constant,
	public Scalar<Constant> {
public :
	Constant
	(
		const Location&,
		const mdl :: Comments* const,
		const String* const,
		const String* const,
		const String* const
	);
	virtual ~ Constant();

	// mdl :: Constant interface
	virtual index :: Literal getSize() const;
	virtual value :: Literal getValue (const index :: Literal) const;

	// object :: Verifiable interface
	virtual void check() const;
	virtual void compress();
	virtual void prove();
	virtual bool isVerified() const;

	// object :: Buildable interface
	virtual void buildTheory (mdl :: Theory* const);

	// object :: Writable interface
	virtual void translate (String&) const;
	virtual void replicate (String&) const;

	// object :: Object interface
	virtual void commitSuicide();
	virtual Size_t getVolume() const;
	virtual Size_t getSizeOf() const;

private :
	const Location location_;
	const mdl :: Comments* const comments_;
	const String* const symbol_;
	const String* const ascii_;
	const String* const latex_;
	const value :: Literal literal_;
};

}
}
}

#endif /*MDL_FORM_BLOCK_CONSTANT_HPP_*/
