/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File Name:       mdl_form_block_Problem.cpp                               */
/* Description:     problem class                                            */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_FORM_BLOCK_PROBLEM_CPP_
#define MDL_FORM_BLOCK_PROBLEM_CPP_

#include "mdl/math/mdl_math.hpp"
#include "mdl/form/block/mdl_form_block_Problem.hpp"

namespace mdl {
namespace form {
namespace block {

	/****************************
	 *		Public members
	 ****************************/

	inline
	Problem :: Problem
	(
		const Location& location,
		const mdl :: Comments* const comments,
		const lexer :: Indent barIndent,
		const value :: Name name,
		const mdl :: Variables* const variables,
		const mdl :: Disjointeds* const disjointeds,
		const vector :: parser :: Hypothesis& hypothesisVector,
		const vector :: parser :: Provable& propositionVector,
		const mdl :: Proof* const proof
	) :
	theorem_
	(
		location,
		comments,
		barIndent,
		name,
		variables,
		disjointeds,
		hypothesisVector,
		propositionVector,
		proof
	) {
	}
	Problem :: ~ Problem() {
	}

	const mdl :: Proof*
	Problem :: getProof() const {
		return theorem_.getProof();
	}

	// mdl :: Assertion implementation
	inline value :: Kind
	Problem :: getKind() const {
		return Token :: PROBLEM;
	}
	inline index :: Assertion
	Problem :: getIndex() const {
		return theorem_.getIndex();
	}

	// general mdl :: Assertion implementation
	inline const mdl :: Variables*
	Problem :: getVariables() const {
		return theorem_.getVariables();
	}
	inline const mdl :: Disjointeds*
	Problem :: getDisjointeds() const {
		return theorem_.getDisjointeds();
	}
	inline index :: Arity
	Problem :: getHypArity() const {
		return theorem_.getHypArity();
	}
	inline index :: Arity
	Problem :: getPropArity() const {
		return theorem_.getPropArity();
	}
	inline const mdl :: Hypothesis*
	Problem :: getHypothesis (const index :: Arity i) const {
		return theorem_.getHypothesis (i);
	}
	inline const mdl :: Proposition*
	Problem :: getProposition (const index :: Arity i) const {
		return theorem_.getProposition (i);
	}
	inline void
	Problem :: completeWithHypotheses (prover :: tree :: Premise* premises) const {
		theorem_.completeWithHypotheses (premises);
	}

using manipulator :: space;
using manipulator :: endLine;
using manipulator :: underline;

	// object :: Identifiable implementation
	inline const mdl :: Theory*
	Problem :: getTheory() const {
		return theorem_.getTheory();
	}
	inline value :: Name
	Problem :: getName() const {
		return theorem_.getName();
	}
	inline const mdl :: Identificator*
	Problem :: getIdentificator() const {
		return theorem_.getIdentificator();
	}
	inline void
	Problem :: translateName (String& string, const index :: Arity index) const {
		theorem_.translateName (string, index);
	}
	inline void
	Problem :: replicateName (String& string) const {
		theorem_.replicateName (string);
	}

	// object :: Verifiable implementation
	inline void
	Problem :: check() const {
		theorem_.check();
	}
	inline void
	Problem :: compress() {
		theorem_.compress();
	}
	inline void
	Problem :: prove()
	{
		const_cast<mdl :: Proof* const>(theorem_.proof_)->prove();
		theorem_.check();
	}
	inline bool
	Problem :: isVerified() const {
		return theorem_.isVerified();
	}

	// object :: Buildable implementation
	inline void
	Problem :: buildTheory (mdl :: Theory* const theory)
	{
		theorem_.scope_ = new proof :: scope :: Assertion (this);
		theorem_.buildComponents (theory);
		theorem_.add (this, theory);
	}

	// object :: Writable implementation
	inline void
	Problem :: translate (String& string) const {
		theorem_.translate (string);
	}
	inline void
	Problem :: replicate (String& string) const
	{
		theorem_.replicateHeader (string, false);
		theorem_.replicateBody (string);
	}

	// object :: Object implementation
	inline void
	Problem :: commitSuicide() {
		delete this;
	}
	inline Size_t
	Problem :: getSizeOf() const {
		return sizeof (Problem);
	}
	inline Size_t
	Problem :: getVolume() const {
		return theorem_.getVolume();
	}
}
}
}

#endif /*MDL_FORM_BLOCK_PROBLEM_CPP_*/
