/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File Name:       mdl_form_block_Problem.hpp                               */
/* Description:     problem class                                            */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_FORM_BLOCK_PROBLEM_HPP_
#define MDL_FORM_BLOCK_PROBLEM_HPP_

#include "mdl/interface/mdl_interface.hpp"
#include "mdl/form/mdl_form_Identifiable.hpp"
#include "mdl/form/block/mdl_form_block_Theorem.hpp"

namespace mdl {
namespace form {
namespace block {

class Problem :
	public mdl :: Assertion,
	public Scalar<Problem> {
public :
	Problem
	(
		const Location&,
		const mdl :: Comments* const,
		const lexer :: Indent,
		const value :: Name,
		const mdl :: Variables* const,
		const mdl :: Disjointeds* const,
		const vector :: parser :: Hypothesis&,
		const vector :: parser :: Provable&,
		const mdl :: Proof* const
	);
	virtual ~ Problem();

	const mdl :: Proof* getProof() const;

	// mdl :: Assertion interface
	virtual value :: Kind getKind() const;
	virtual index :: Assertion getIndex() const;

	// general mdl :: Assertion interface part
	virtual const mdl :: Variables* getVariables() const;
	virtual const mdl :: Disjointeds* getDisjointeds() const;
	virtual index :: Arity getHypArity() const;
	virtual index :: Arity getPropArity() const;
	virtual const mdl :: Hypothesis* getHypothesis (const index :: Arity) const;
	virtual const mdl :: Proposition* getProposition (const index :: Arity) const;
	virtual void completeWithHypotheses (prover :: tree :: Premise*) const;

	// object :: Identifiable interface
	virtual const mdl :: Theory* getTheory() const;
	virtual value :: Name getName() const;
	virtual const mdl :: Identificator* getIdentificator() const;
	virtual void translateName (String&, const index :: Arity = 0) const;
	virtual void replicateName (String&) const;

	// object :: Verifiable interface
	virtual void check() const;
	virtual void compress();
	virtual void prove();
	virtual bool isVerified() const;

	// object :: Buildable interface
	virtual void buildTheory (mdl :: Theory* const);

	// object :: Writable interface
	virtual void translate (String&) const;
	virtual void replicate (String&) const;

	// object :: Object interface
	virtual void commitSuicide();
	virtual Size_t getSizeOf() const;
	virtual Size_t getVolume() const;

private :
	Theorem theorem_;
};

}
}
}

#endif /*MDL_FORM_BLOCK_PROBLEM_HPP_*/
