/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File Name:       mdl_form_block_Proof.hpp                                 */
/* Description:     proof class                                              */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_FORM_BLOCK_PROOF_HPP_
#define MDL_FORM_BLOCK_PROOF_HPP_

#include "mdl/interface/mdl_interface.hpp"

namespace mdl {
namespace form {
namespace block {

class Proof :
	public mdl :: Proof,
	public Scalar<Proof> {
public :
	Proof
	(
		const Location&,
		const mdl :: Comments* const,
		const mdl :: proof :: Line* const,
		const vector :: parser :: Step&
	);
	Proof
	(
		const mdl :: proof :: Step*,
		const index :: Step
	);
	virtual ~ Proof();

	static const Log& getLog();
	static void init();
	static void release();

	// mdl :: Proof interface
	virtual bool isCompressed() const;
	virtual index :: Step getLength() const;
	virtual const mdl :: proof :: Step* getStep (const index :: Step) const;
	virtual const mdl :: proof :: Line* getFirst() const;
	virtual const mdl :: proof :: Line* getLast() const;
	virtual void addStep (const mdl :: proof :: Step*);
	virtual void addVariable (const mdl :: proof :: Variable*);
	virtual void addQed (const mdl :: proof :: Qed*);

	// object :: Scoping
	virtual const mdl :: proof :: Scope* buildScope (const mdl :: proof :: Scope* const);
	virtual const mdl :: proof :: Scope* getScope() const;
	virtual void setScope (const mdl :: proof :: Scope*) const;

	// object :: Verifiable
	virtual void check() const;
	virtual void compress();
	virtual void prove();
	virtual bool isVerified() const;

	// object :: Buildable interface
	virtual void buildTheory (mdl :: Theory* const);

	// object :: Writable interface
	virtual void translate (String& string) const;
	virtual void replicate (String& string) const;

	// object :: Object interface
	virtual void commitSuicide();
	virtual Size_t getSizeOf() const;
	virtual Size_t getVolume() const;

private :
	class Timer;
	class Refiner;
	friend class Timer;
	friend class Refiner;

	void addLine (const mdl :: proof :: Line*);
	void replaceLine (const mdl :: proof :: Line*, const mdl :: proof :: Line*);
	void insertLine (const mdl :: proof :: Line*, const mdl :: proof :: Line*);
	const mdl :: proof :: Line* removeLine (const mdl :: proof :: Line*);

	void questionSelf();
	bool compressRest (const mdl :: proof :: Line*);
	void confirmCompression (const mdl :: proof :: Line*);
	void rejectCompression (const mdl :: proof :: Line*);
	void compressStep (const mdl :: proof :: Line*);

	static void writeCompressionLog (const mdl :: Assertion*);

	const Location location_;
	const mdl :: Comments* const comments_;
	const mdl :: proof :: Line* firstLine_;
	const mdl :: proof :: Line* lastLine_;
	const vector :: proof :: Step stepVector_;
	mutable const mdl :: proof :: Scope* scope_;
	mutable bool isVerified_;
	mutable bool isCompressed_;

	static Log log_;
};

}
}
}

#endif /*MDL_FORM_BLOCK_PROOF_HPP_*/
