/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File Name:       mdl_form_block_Proof_Timer.cpp                           */
/* Description:     timer for proof compression                              */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_FORM_BLOCK_PROOF_TIMER_CPP_
#define MDL_FORM_BLOCK_PROOF_TIMER_CPP_

namespace mdl {
namespace form {
namespace block {

	/****************************
	 *		Public members
	 ****************************/

	void
	Proof :: Timer :: calculateTimeLimit (const Proof* proof)
	{
		const Clock_t clocksPerStep = estimateClocksPerProveAct (proof);
		Prover :: timers().setLimit (clocksPerStep);
	}
	void
	Proof :: Timer :: incrementProveActs (const mdl :: Assertion* const assertion)
	{
		proveActsDone_ += estimateProveActs (assertion);
		//const Clock_t clocksPerStep = estimateClocksPerProveAct (proof);
		//Prover :: timers().setLimit (clocksPerStep);
	}

	inline void
	Proof :: Timer :: init() {
	}
	inline void
	Proof :: Timer :: release() {
	}
	inline Size_t
	Proof :: Timer :: getVolume() {
		return 0;
	}
	inline Size_t
	Proof :: Timer :: getSizeOf() {
		return sizeof (Timer);
	}
	inline void
	Proof :: Timer :: show (String&) {
	}

	/****************************
	 *		Private members
	 ****************************/

	value :: Long
	Proof :: Timer :: estimateProveActsLeft (const Proof* proof)
	{
		value :: Long stepsLeft = 0;
		const index :: Assertion
			index = proof->scope_->getAssertion()->getIndex();
		for (index :: Assertion i = index; i < Math :: assertions()->getSize(); ++ i) {
			const mdl :: Assertion* const
				assertion = Math :: assertions()->get (i);
			if (Config :: compress() && Refiner :: isCompressable (assertion)) {
				stepsLeft += estimateProveActs (assertion);
			}
			if (Config :: prove()) {
				stepsLeft += estimateProveActs (assertion);
			}
		}
		return stepsLeft;
	}
	index :: Step
	Proof :: Timer :: estimateProveActs (const mdl :: Assertion* const assertion)
	{
		if (Config :: compress() && (assertion->getKind() == Token :: THEOREM)) {
			const form :: block :: Theorem* const
				theorem = dynamic_cast<const form :: block :: Theorem*>(assertion);
			const mdl :: Proof* const
				proof = theorem->getProof();
			const index :: Step
				length = proof->getLength();
			return length * (length - 1) / 2;
		}
		if (Config :: prove() && (assertion->getKind() == Token :: PROBLEM)) {
			const form :: block :: Problem* const
				problem = dynamic_cast<const form :: block :: Problem*>(assertion);
			const mdl :: Proof* const
				proof = problem->getProof();
			index :: Step questionCount = 0;
			for (index :: Step i = 0; i < proof->getLength(); ++ i) {
				const mdl :: proof :: Step* const
					step = proof->getStep (i);
				if (step->getClass() == mdl :: proof :: Step :: QUESTION) {
					++ questionCount;
				}
			}
			return questionCount;
		}
		return 0;
	}
	Clock_t
	Proof :: Timer :: estimateClocksPerProveAct (const Proof* proof)
	{
		value :: Long proveActsLeft = estimateProveActsLeft (proof);
		proveActsLeft = (proveActsLeft == 0) ? 1 : proveActsLeft;
		if (proveActsTotal_ == Undefined<value :: Long> :: getValue()) {
			proveActsTotal_ = proveActsLeft;
		}
		const Time_t timeTotal =
			Config :: prove() ?
			Config :: getMaxProveTime() :
			Config :: getMaxCompressTime();
		const Time_t timePassed =
			Config :: prove() ?
			Mdl :: getMdl()->getTimers().prove().getTime() :
			Mdl :: getMdl()->getTimers().compress().getTime();
		const Time_t timeLeft =
			(timeTotal > timePassed) ?
			timeTotal - timePassed :
			0;

		value :: Long clocksLeft = timeLeft;
		clocksLeft *= system :: getClocksPerSecond();
		clocksLeft /= proveActsLeft;
		Clock_t result = clocksLeft;

		if (averageClocksForProve_ == Undefined<Clock_t> :: getValue()) {
			averageClocksForProve_ = result;
		}
		const Clock_t clockMinimal =
			Config :: prove() ?
			Config :: getMinProveTime() :
			Config :: getMinCompressTime();

		result = (result < clockMinimal) ? clockMinimal : result;
		result =
			(result < Config :: getQuantumClock()) ?
			Config :: getQuantumClock() :
			result;

		const value :: Long estimatedClocksLeft =
			(proveActsTotal_ - proveActsDone_) * averageClocksForProve_;
		value :: Long exactClocksLeft = timeLeft;
		exactClocksLeft *= system :: getClocksPerSecond();

		value :: Long clockDelta =
			(exactClocksLeft > estimatedClocksLeft) ?
			exactClocksLeft - estimatedClocksLeft :
			0;

		const value :: Long
			maxClokValue = Undefined<value :: Integer> :: getValue() >> 1 ;

		clockDelta =
			(clockDelta + static_cast<value :: Long>(result) > maxClokValue - 1) ?
			maxClokValue - result - 1:
			clockDelta;

		result += clockDelta;
		return result;
	}


	/************************************
	 *		Private static attributes
	 ************************************/

	value :: Long
		Proof :: Timer :: Timer :: proveActsTotal_ = Undefined<value :: Long> :: getValue();
	value :: Long
		Proof :: Timer :: Timer :: proveActsDone_ = 0;
	Clock_t
		Proof :: Timer :: Timer :: averageClocksForProve_ = Undefined<Clock_t> :: getValue();

	bool
		Proof :: Timer :: initialized_ = false;
	bool
		Proof :: Timer :: counted_ = false;
}
}
}

#endif /*MDL_FORM_BLOCK_PROOF_TIMER_CPP_*/
