/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File Name:       mdl_form_block_Proof_Timer.hpp                           */
/* Description:     timer for proof compression                              */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_FORM_BLOCK_PROOF_TIMER_HPP_
#define MDL_FORM_BLOCK_PROOF_TIMER_HPP_

#include "mdl/interface/mdl_interface.hpp"
#include "mdl/form/block/mdl_form_block.dpp"

namespace mdl {
namespace form {
namespace block {

class Proof :: Timer {
public :
	static void calculateTimeLimit (const Proof*);
	static void incrementProveActs (const mdl :: Assertion* const);

	static void init();
	static void release();
	static Size_t getVolume();
	static Size_t getSizeOf();
	static void show (String&);

private :
	static value :: Long estimateProveActsLeft (const Proof*);
	static index :: Step estimateProveActs (const mdl :: Assertion* const);
	static Clock_t estimateClocksPerProveAct (const Proof*);

	// prove time estimation attributes
	static value :: Long proveActsTotal_;
	static value :: Long proveActsDone_;
	static Clock_t averageClocksForProve_;

	static bool initialized_;
	static bool counted_;
};

}
}
}

#endif /*MDL_FORM_BLOCK_PROOF_TIMER_HPP_*/
