/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File Name:       mdl_form_block_Rule.hpp                                  */
/* Description:     syntax grammar rule class                                */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_FORM_BLOCK_RULE_HPP_
#define MDL_FORM_BLOCK_RULE_HPP_

#include "mdl/interface/mdl_interface.hpp"
#include "mdl/form/mdl_form_Identifiable.hpp"

namespace mdl {
namespace form {
namespace block {

class Rule :
	public mdl :: Rule,
	public form :: Identifiable,
	public Scalar<Rule> {
public :
	Rule
	(
		const Location&,
		const mdl :: Comments* const,
		const value :: Name,
		const mdl :: Variables* const,
		const mdl :: term :: Rule* const
	);
	virtual ~ Rule();

	// mdl :: Rule interface
	virtual const mdl :: Type* getType() const;
	virtual const mdl :: Variables* getVariables() const;
	virtual const mdl :: term :: Rule* getTerm() const;
	virtual void setType (const mdl :: Type*) const;

	// object :: Identifiable interface
	virtual const mdl :: Theory* getTheory() const;
	virtual value :: Name getName() const;
	virtual void translateName (String&, const index :: Arity = 0) const;
	virtual void replicateName (String&) const;

	// object :: Verifiable interface
	virtual void check() const;
	virtual void compress();
	virtual void prove();
	virtual bool isVerified() const;

	// object :: Buildable interface
	virtual void buildTheory (mdl :: Theory* const);

	// object :: Writable interface
	virtual void translate (String&) const;
	virtual void replicate (String&) const;

	// object :: Object interface
	virtual void commitSuicide();
	virtual Size_t getVolume() const;
	virtual Size_t getSizeOf() const;

private :
	mutable const mdl :: Type* type_;
	const mdl :: Variables* variables_;
	const mdl :: term :: Rule* term_;
};

}
}
}

#endif /*MDL_FORM_BLOCK_RULE_HPP_*/
