/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File Name:       mdl_form_block_Super.hpp                                 */
/* Description:     type-supertype relation class                            */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_FORM_BLOCK_SUPER_CPP_
#define MDL_FORM_BLOCK_SUPER_CPP_

#include "mdl/math/mdl_math.hpp"
#include "mdl/form/block/mdl_form_block_Super.hpp"

#include "smm/lexer/smm_lexer_Token.hpp"

namespace mdl {
namespace form {
namespace block {

	/****************************
	 *		Public members
	 ****************************/

	inline
	Super :: Super
	(
		const mdl :: Type* inferType,
		const mdl :: Identificator* identificator
	) :
	header :: Line :: Counter(),
	identificator_ (identificator),
	inferType_ (inferType),
	superType_ (NULL) {
	}
	inline
	Super :: Super
	(
		const mdl :: Type* inferType,
		const mdl :: Type* superType
	) :
	header :: Line :: Counter(),
	identificator_ (NULL),
	inferType_ (inferType),
	superType_ (superType) {
	}
	Super :: ~ Super() {
	}


using nstd :: manipulator :: underline;

	// mdl :: Super implementation
	const mdl :: Type*
	Super :: getType() const {
		return superType_;
	}
	void
	Super :: translateReference (String& string) const
	{
		if (Config :: keepLabels())
		{
			// refernce template:
			// ([header]_)<super_type>_([delimiter]_)<infer_type>

			if (header_[0] != chars :: NULL_CHAR) {
				string << header_ << underline;
			}
			superType_->translateName (string);
			string << underline;
			if (delimiter_[0] != chars :: NULL_CHAR) {
				string << delimiter_ << underline;
			}
			inferType_->translateName (string);
		} else {
			string << :: smm :: Token :: prefix_a_ << Counter :: getIndex();
		}
	}
	void
	Super :: addSuperSuper (vector :: Super& supers) const
	{
		for (index :: Type i = 0; i < superType_->getSuperCount(); ++ i) {
			const mdl :: Type* const
				superSuperType = superType_->getSuper(i)->getType();
			const mdl :: Super* const
				superSuper = new Super (inferType_,superSuperType);
			supers.add (superSuper);
		}
	}

using nstd :: manipulator :: space;
using nstd :: manipulator :: tab;
using nstd :: manipulator :: endLine;

	// object :: Buildable implementation
	void
	Super :: buildTheory (mdl :: Theory* const theory)
	{
		const index :: Type
			index = theory->checkUsed (identificator_);
		superType_ = Math :: types()->get (index);

		if (superType_ == NULL) {
			Error* error = new Error (Error :: SEMANTIC);
			error->message() << "super type ";
			identificator_->replicate (error->message());
			error->message() << "is not declared." << endLine;
			throw error;
		}
		const_cast<mdl :: Type*>(superType_)->addSubtype (inferType_);
	}

	// object :: Writable implementation
	void
	Super :: translate (String& string) const
	{
		// ${
		string << :: smm :: Token :: blockBegin_ << endLine;

		// $v x $.
		string << tab << :: smm :: Token :: variable_ << space;
		string << variable_ << space;
		string << :: smm :: Token :: end_ << endLine;

		// f0 $f type x $.
		string << tab << :: smm :: Token :: prefix_f_ << '0';
		string << space << :: smm :: Token :: floating_ << space;
		inferType_->translateName (string);
		string << space << variable_ << space;
		string << :: smm :: Token :: end_ << endLine;

		// <label> $a super_type x $.
		string << tab;
		translateReference (string);

		string << space << :: smm :: Token :: axiomatic_ << space;
		superType_->translateName (string);
		string << space << variable_ << space;
		string << :: smm :: Token :: end_ << endLine;

		// $}
		string << :: smm :: Token :: blockEnd_ << endLine;
	}
	void
	Super :: replicate (String&) const {
	}

	// object :: Object implementation
	void
	Super :: commitSuicide() {
		delete this;
	}
	Size_t
	Super :: getVolume() const {
		return 0;
	}
	Size_t
	Super :: getSizeOf() const {
		return sizeof (Super);
	}

	/*********************************
	 *		Private static attributes
	 *********************************/

	const char Super :: variable_[] = "x"; //"RULE";
	const char Super :: header_[] = ""; //"RULE";
	const char Super :: delimiter_[] = ""; //"IS_SUPER";
}
}
}

#endif /*MDL_FORM_BLOCK_SUPER_CPP_*/
