/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File Name:       mdl_form_block_Super.hpp                                 */
/* Description:     type-supertype relation class                            */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_FORM_BLOCK_SUPER_HPP_
#define MDL_FORM_BLOCK_SUPER_HPP_

#include "mdl/interface/mdl_interface.hpp"
#include "mdl/form/header/mdl_form_header_Line.hpp"

namespace mdl {
namespace form {
namespace block {

class Super :
	public mdl :: Super,
	public header :: Line :: Counter,
	public Scalar<Super> {
public :
	Super (const mdl :: Type*, const mdl :: Identificator*);
	Super (const mdl :: Type*, const mdl :: Type*);
	virtual ~ Super();

	// mdl :: Super interface
	virtual const mdl :: Type* getType() const;
	virtual void translateReference (String&) const;
	virtual void addSuperSuper (vector :: Super&) const;

	// object :: Buildable interface
	virtual void buildTheory (mdl :: Theory* const);

	// object :: Writable interface
	virtual void translate (String&) const;
	virtual void replicate (String&) const;

	// object :: Object interface
	virtual void commitSuicide();
	virtual Size_t getVolume() const;
	virtual Size_t getSizeOf() const;

private :
	const mdl :: Identificator* const identificator_;
	const mdl :: Type* const inferType_;
	const mdl :: Type* superType_;

	static const char variable_[];
	static const char header_[];
	static const char delimiter_[];
};

}
}
}

#endif /*MDL_FORM_BLOCK_SUPER_HPP_*/
