/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File Name:       mdl_form_block_Theorem.hpp                               */
/* Description:     theorem class                                            */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_FORM_BLOCK_THEOREM_HPP_
#define MDL_FORM_BLOCK_THEOREM_HPP_

#include "mdl/interface/mdl_interface.hpp"
#include "mdl/form/mdl_form_Identifiable.hpp"
#include "mdl/form/block/mdl_form_block_Assertion.hpp"

namespace mdl {
namespace form {
namespace block {

class Theorem :
	public form :: block :: Assertion
	<
		mdl :: Hypothesis,
		mdl :: statement :: Provable
	>,
	public Scalar<Theorem> {
public :
	Theorem
	(
		const Location&,
		const mdl :: Comments* const,
		const lexer :: Indent,
		const value :: Name,
		const mdl :: Variables* const,
		const mdl :: Disjointeds* const,
		const vector :: parser :: Hypothesis&,
		const vector :: parser :: Provable&,
		const mdl :: Proof* const
	);
	virtual ~ Theorem();

	const mdl :: Proof* getProof() const;

	// mdl :: Assertion interface
	virtual value :: Kind getKind() const;

	// object :: Verifiable interface
	virtual void check() const;
	virtual void compress();
	virtual void prove();
	virtual bool isVerified() const;

	// object :: Buildable interface
	virtual void buildTheory (mdl :: Theory* const);

	// object :: Writable interface
	virtual void translate (String&) const;
	virtual void replicate (String&) const;

	// object :: Object interface
	virtual void commitSuicide();
	virtual Size_t getSizeOf() const;
	virtual Size_t getVolume() const;

private :
	typedef
		form :: block :: Assertion
		<
			mdl :: Hypothesis,
			mdl :: statement :: Provable
		>
		Ancestor_;

	friend class Problem;

	void buildComponents (mdl :: Theory* const);
	void buildHypothesis();
	void buildPropositions();
	void replicateHeader (String&, const bool = true) const;
	void replicateBody (String&) const;

	const mdl :: Proof* proof_;
	const mdl :: proof :: Scope* scope_;
};

}
}
}

#endif /*MDL_FORM_BLOCK_THEOREM_HPP_*/
