/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File Name:       mdl_form_block_Type.hpp                                  */
/* Description:     type class                                               */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_FORM_BLOCK_TYPE_HPP_
#define MDL_FORM_BLOCK_TYPE_HPP_

#include "mdl/interface/mdl_interface.hpp"
#include "mdl/form/mdl_form_Identifiable.hpp"

namespace mdl {
namespace form {
namespace block {

class Type :
	public mdl :: Type,
	public Identifiable,
	public Scalar<Type> {
public :
	Type
	(
		const Location&,
		const mdl :: Comments* const,
		const value :: Name,
		const vector :: parser :: Identificator&
	);
	virtual ~ Type();

	// mdl :: Type interface
	virtual const tree :: Rule* getRuleTree() const;
	virtual void addRule (const mdl :: Rule* const);

	virtual bool isSuper (const mdl :: Type* const) const;
	virtual bool isInfer (const mdl :: Type* const) const;

	virtual index :: Type getSuperCount() const;
	virtual const mdl :: Super* getSuper (const index :: Type) const;
	virtual const mdl :: Super* getSuper (const mdl :: Type* const) const;

	virtual index :: Type getSubtypeCount() const;
	virtual const mdl :: Type* getSubtype (const index :: Type) const;
	virtual void addSubtype (const mdl :: Type*);

	// object :: Identifiable interface
	virtual const mdl :: Theory* getTheory() const;
	virtual value :: Name getName() const;
	virtual const mdl :: Identificator* getIdentificator() const;
	virtual void translateName (String&, const index :: Arity = 0) const;
	virtual void replicateName (String&) const;

	// object :: Verifiable interface
	virtual void check() const;
	virtual void compress();
	virtual void prove();
	virtual bool isVerified() const;

	// object :: Buildable interface
	virtual void buildTheory (mdl :: Theory* const);

	// object :: Writable interface
	virtual void translate (String&) const;
	virtual void replicate (String&) const;

	// object :: Object interface
	virtual void commitSuicide();
	virtual Size_t getVolume() const;
	virtual Size_t getSizeOf() const;

private :
	const vector :: Identificator superNames_;
	vector :: Super supers_;
	vector :: Type subTypes_;
	tree :: Rule* ruleTree_;
};

}
}
}

#endif /*MDL_FORM_BLOCK_TYPE_HPP_*/
