/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File Name:       mdl_form_header_Line.cpp                                 */
/* Description:     header line                                              */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_FORM_HEADER_LINE_CPP_
#define MDL_FORM_HEADER_LINE_CPP_

#include "mdl/math/mdl_math.hpp"
#include "mdl/form/header/mdl_form_header_Line.hpp"

namespace mdl {
namespace form {
namespace header {

	/****************************
	 *		Public members
	 ****************************/

	inline
	Line :: Line
	(
		const Location& location,
		const mdl :: Identificator* const identificator,
		const array :: Expression* const expression
	) :
	location_ (location),
	identificator_ (identificator),
	expression_ (expression) {
	}
	Line :: ~ Line()
	{
		deleteAggregateObject (expression_);
		deleteAggregateObject (identificator_);
	}

	// object :: Expressive implementation
	const array :: Expression*
	Line :: getExpression() const {
		return expression_;
	}
	const mdl :: Type*
	Line :: getType() const {
		return type_;
	}

	// object :: Buildable implementation
	void
	Line :: buildTheory (mdl :: Theory* const theory)
	{
		const index :: Type
			index = theory->getIndex (identificator_);
		type_ = Math :: types()->get (index);
		if (type_ == NULL) {
			throw new Error (Error :: SEMANTIC, "undefined type");
		}
	}

	// object :: Object implementation
	Size_t
	Line :: getVolume() const
	{
		Size_t volume = 0;
		volume += getAggregateVolume (expression_);
		volume += getAggregateVolume (identificator_);
		return volume;
	}
	Size_t
	Line :: getSizeOf() const {
		return sizeof (Line);
	}

	inline const Location&
	Line :: getLocation() const {
		return location_;
	}

	inline void
	Line :: Counter :: setIndex() const
	{
		if (index_ == index :: undefined :: ASSERTION) {
			const_cast<index :: Assertion&>(index_) = counter_ ++;
		}
	}
	inline index :: Assertion
	Line :: Counter :: getIndex() const
	{
		if (index_ == index :: undefined :: ASSERTION) {
			const_cast<index :: Assertion&>(index_) = counter_ ++;
		}
		return index_;
	}

	/****************************
	 *		Protected members
	 ****************************/

	inline Line :: Counter :: Counter () :
	index_ (index :: undefined :: ASSERTION) {
	}

	/************************************
	 *		Static private attributes
	 ************************************/

	index :: Assertion
		Line :: Counter :: counter_ = 0;
}
}
}

#endif /*MDL_FORM_HEADER_LINE_CPP_*/
