/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File Name:       mdl_form_header_Line.hpp                                 */
/* Description:     header line                                              */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_FORM_HEADER_LINE_HPP_
#define MDL_FORM_HEADER_LINE_HPP_

#include "mdl/interface/mdl_interface.hpp"

namespace mdl {
namespace form {
namespace header {

class Line : public mdl :: header :: Line {
public :
	Line
	(
		const Location&,
		const mdl :: Identificator* const,
		const array :: Expression* const
	);
	virtual ~ Line();

	// object :: Expressive interface
	virtual const array :: Expression* getExpression() const;
	virtual const mdl :: Type* getType() const;

	// object :: Buildable interface
	virtual void buildTheory (mdl :: Theory* const);

	// object :: Writable interface
	virtual void translate (String&) const = 0;
	virtual void replicate (String&) const = 0;

	// object :: Object interface
	virtual void commitSuicide() = 0;
	virtual Size_t getVolume() const;
	virtual Size_t getSizeOf() const;

	const Location& getLocation() const;

	class Counter {
	public :
		void setIndex() const;
		index :: Assertion getIndex() const;
	protected :
		Counter ();
		const index :: Assertion index_;
	private :
		static index :: Assertion counter_;
	};

protected :
	const Location location_;
	const mdl :: Identificator* identificator_;
	const mdl :: Type* type_;
	mutable const array :: Expression* expression_;
};

}
}
}

#endif /*MDL_FORM_HEADER_LINE_HPP_*/
