/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File Name:       mdl_form_header_Variable.hpp                             */
/* Description:     variable declaration                                     */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_FORM_HEADER_VARIABLE_HPP_
#define MDL_FORM_HEADER_VARIABLE_HPP_

#include "mdl/interface/mdl_interface.hpp"

namespace mdl {
namespace form {
namespace header {

class Variable :
	public mdl :: header :: Variable,
	public Scalar<Variable> {
public :
	Variable
	(
		const Location&,
		const value :: Literal,
		const mdl :: Identificator* const
	);
	Variable
	(
		const Location&,
		const Symbol&
	);
	virtual ~ Variable();

	// header :: Variable interface
	virtual const Symbol& getSymbol() const;
	virtual value :: Literal getLiteral() const;
	virtual const mdl :: Type* getType () const;
	virtual void setIndex (const index :: Arity) const;
	virtual void translateReference (String&) const;

	// object :: Buildable interface
	virtual void buildTheory (mdl :: Theory* const);

	// object :: Writable interface
	virtual void translate (String&) const;
	virtual void replicate (String&) const;

	// object :: Object interface
	virtual void commitSuicide();
	virtual Size_t getVolume() const;
	virtual Size_t getSizeOf() const;

protected :
	const Location location_;
	mutable index :: Arity index_;
	const Symbol symbol_;
	const mdl :: Identificator* identificator_;
};

}
}
}

#endif /*MDL_FORM_HEADER_VARIABLE_HPP_*/
