/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File Name:       mdl_form_header_Variables.hpp                            */
/* Description:     variables declarations                                   */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_FORM_HEADER_VARIABLES_HPP_
#define MDL_FORM_HEADER_VARIABLES_HPP_

#include "mdl/interface/mdl_interface.hpp"

namespace mdl {
namespace form {
namespace header {

class Variables :
	public mdl :: Variables,
	public Scalar<Variables> {
public :
	Variables (const Location&, const vector :: parser :: Variable&);
	virtual ~ Variables();

	static const mdl :: Variables* getCurrent();

	// mdl :: Variables interface
	virtual index :: Arity getSize() const;
	virtual const mdl :: header :: Variable* getVariable (const index :: Arity) const;
	virtual bool contain (const Symbol&) const;

	// object :: Typing interface
	virtual const mdl :: Type* getType (const value :: Literal) const;
	virtual const mdl :: header :: Variable* getDeclaration (const value :: Literal) const;
	virtual const object :: Typing* getPrevious() const;
	virtual void setNext (const object :: Typing*) const;
	virtual void translateVariables (String&) const;
	virtual void translateFloatings (String&, const index :: Arity = 0) const;

	// object :: Buildable interface
	virtual void buildTheory (mdl :: Theory* const);

	// object :: Writable interface
	virtual void translate (String&) const;
	virtual void replicate (String&) const;

	// object :: Object interface
	virtual void commitSuicide();
	virtual Size_t getVolume() const;
	virtual Size_t getSizeOf() const;

private :
	const Location location_;
	const vector :: header :: Variable variableVector_;
	mutable const object :: Typing* next_;

	static const mdl :: Variables* current_;
};

}
}
}

#endif /*MDL_FORM_HEADER_VARIABLES_HPP_*/
