/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File Name:       mdl_form_header_statement_Axiomatic.cpp                  */
/* Description:     axiomatic statement class                                */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_FORM_HEADER_STATEMENT_AXIOMATIC_CPP_
#define MDL_FORM_HEADER_STATEMENT_AXIOMATIC_CPP_

#include "smm/lexer/smm_lexer_Token.hpp"
#include "mdl/form/mdl_form.hpp"

namespace mdl {
namespace form {
namespace header {
namespace statement {

	/****************************
	 *		Public members
	 ****************************/

	inline
	Axiomatic :: Axiomatic
	(
		const Location& location,
		const index :: Arity index,
		const mdl :: Identificator* const identificator,
		const array :: Expression* const expression
	) :
	header :: statement :: Proposition (location, index, identificator, expression) {
	}
	Axiomatic :: ~ Axiomatic() {
	}

	// mdl :: Proposition implementation
	index :: Assertion
	Axiomatic :: getSourceIndex() const {
		return Proposition :: getSourceIndex();
	}
	index :: Assertion
	Axiomatic :: getTargetIndex() const {
		return Proposition :: getTargetIndex();
	}
	prover :: Experience&
	Axiomatic :: getExperience() const {
		return Proposition :: getExperience();
	}

	// mdl :: Statement implementation
	void
	Axiomatic :: setAssertion (const mdl :: Assertion* const assertion) const {
		Proposition :: setAssertion (assertion);
	}
	index :: Arity
	Axiomatic :: getIndex() const {
		return Proposition :: getIndex();
	}
	const mdl :: proof :: Node*
	Axiomatic :: getProofNode() const {
		return Proposition :: proofNode_;
	}

	// object :: Expressive implementation
	const array :: Expression*
	Axiomatic :: getExpression() const {
		return Proposition :: expression_;
	}
	const mdl :: Type*
	Axiomatic :: getType() const {
		return header :: Line :: type_;
	}

	// object :: Buildable implementation
	void
	Axiomatic :: buildTheory (mdl :: Theory* const theory) {
		Proposition :: buildTheory (theory);
	}

using manipulator :: endLine;
using manipulator :: space;
using manipulator :: tab;

	// object :: Writable implementation
	void
	Axiomatic :: translate (String& string) const
	{
		string << tab;
		assertion_->translateName (string, Statement :: index_);
		string << space;
		string << :: smm :: Token :: axiomatic_ << space;
		Proposition :: translate (string);
		string << :: smm :: Token :: end_ << endLine;
	}
	void
	Axiomatic :: replicate (String& string) const {
		Proposition :: replicate (string);
	}

	// object :: Object implementation
	void
	Axiomatic :: commitSuicide() {
		delete this;
	}
	Size_t
	Axiomatic :: getVolume() const {
		return Proposition :: getVolume();
	}
	Size_t
	Axiomatic :: getSizeOf() const {
		return sizeof (Axiomatic);
	}
}
}
}
}

#endif /*MDL_FORM_HEADER_STATEMENT_AXIOMATIC_CPP_*/

