/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File Name:       mdl_form_header_statement_Definition.hpp                 */
/* Description:     definition statement class                               */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_FORM_HEADER_STATEMENT_DEFINITION_HPP_
#define MDL_FORM_HEADER_STATEMENT_DEFINITION_HPP_

#include "mdl/interface/mdl_interface.hpp"
#include "mdl/form/header/statement/mdl_form_header_statement_Proposition.hpp"

namespace mdl {
namespace form {
namespace header {
namespace statement {

class Definition :
	public mdl :: statement :: Definition,
	public Proposition,
	public Scalar<Definition> {
public :
	Definition
	(
		const Location&,
		const index :: Arity,
		const mdl :: Identificator* const,
		const array :: Expression* const
	);
	virtual ~ Definition();

	// statement :: Definition interface
	virtual void setDefiendum (const mdl :: term :: Definition*);
	virtual void setDefiniens (const mdl :: term :: Definition*);

	// mdl :: Proposition interface
	virtual index :: Assertion getSourceIndex() const;
	virtual index :: Assertion getTargetIndex() const;
	virtual prover :: Experience& getExperience() const;

	// mdl :: Statement interface
	virtual void setAssertion (const mdl :: Assertion* const) const;
	virtual index :: Arity getIndex() const;
	virtual const mdl :: proof :: Node* getProofNode() const;

	// object :: Expressive interface
	virtual const array :: Expression* getExpression() const;
	virtual const mdl :: Type* getType() const;

	// object :: Buildable interface
	virtual void buildTheory (mdl :: Theory* const);

	// object :: Writable interface
	virtual void translate (String&) const;
	virtual void replicate (String&) const;

	// object :: Object interface
	virtual void commitSuicide();
	virtual Size_t getVolume() const;
	virtual Size_t getSizeOf() const;

private :
	void insert
	(
		value :: Literal* literals,
		index :: Literal& index,
		const mdl :: term :: Definition*
	) const;
	index :: Literal calculateLength() const;
	void substituteTerms();
	void substituteSymbol
	(
		const index :: Literal i,
		value :: Literal* literals,
		index :: Literal& index,
		bool& defiendumMet,
		bool& definiensMet
	) const;
	void buildOriginal ();

	class MetaVariables_;
	friend class MetaVariables_;

	const array :: Expression* expandedExpression_;
	const array :: Expression* originalExpression_;
	mutable const mdl :: term :: Definition* defiendum_;
	mutable const mdl :: term :: Definition* definiens_;
	const MetaVariables_* metaVariables_;

	// static attributes
	static value :: Literal defiendumKeyword_;
	static value :: Literal definiensKeyword_;
};

}
}
}
}

#endif /*MDL_FORM_HEADER_STATEMENT_DEFINITION_HPP_*/
