/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File Name:       mdl_form_header_statement_Definition_MetaVariables.hpp   */
/* Description:     meta variables in definition proposition                 */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_FORM_HEADER_STATEMENT_DEFINITION_META_VARIABLES_HPP_
#define MDL_FORM_HEADER_STATEMENT_DEFINITION_META_VARIABLES_HPP_

namespace mdl {
namespace form {
namespace header {
namespace statement {

class Definition :: MetaVariables_ :
	public object :: Typing,
	public Scalar<Definition :: MetaVariables_> {
public :
	MetaVariables_
	(
		const object :: Typing*,
		const mdl :: term :: Definition*,
		const mdl :: term :: Definition*
	);
	virtual ~ MetaVariables_();

	// object :: Typing interface
	virtual const mdl :: Type* getType (const value :: Literal) const;
	virtual const mdl :: header :: Variable* getDeclaration (const value :: Literal) const;
	virtual const object :: Typing* getPrevious() const;
	virtual void setNext (const object :: Typing*) const;
	virtual void translateVariables (String&) const;
	virtual void translateFloatings (String&, const index :: Arity = 0) const;

	// object :: Buildable interface
	virtual void buildTheory (mdl :: Theory* const);

	// object :: Writable interface
	virtual void translate (String&) const;
	virtual void replicate (String&) const;

	// object :: Object interface
	virtual void commitSuicide();
	virtual Size_t getVolume() const;
	virtual Size_t getSizeOf() const;

private :
	const object :: Typing* const variables_;
	const mdl :: term :: Definition* const defiendum_;
	const mdl :: term :: Definition* const definiens_;
};

}
}
}
}

#endif /*MDL_FORM_HEADER_STATEMENT_DEFINITION_META_VARIABLES_HPP_*/
