/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File Name:       mdl_form_header_statement_Hypothesis.cpp                 */
/* Description:     hypothesis statement class                               */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_FORM_HEADER_STATEMENT_HYPOTHESIS_CPP_
#define MDL_FORM_HEADER_STATEMENT_HYPOTHESIS_CPP_

#include "mdl/math/mdl_math.hpp"
#include "mdl/form/mdl_form.hpp"

#include "smm/lexer/smm_lexer_Token.hpp"

namespace mdl {
namespace form {
namespace header {
namespace statement {

	/****************************
	 *		Public members
	 ****************************/

	inline
	Hypothesis :: Hypothesis
	(
		const Location& location,
		const index :: Arity index,
		const mdl :: Identificator* const identificator,
		const array :: Expression* const expression
	) :
	form :: header :: statement :: Statement
	(
		location,
		index,
		identificator,
		expression
	) {
	}
	Hypothesis :: ~ Hypothesis() {
	}

	// mdl :: Statement implementation
	const mdl ::Assertion*
	Hypothesis :: getAssertion() const {
		return Statement :: getAssertion();
	}
	void
	Hypothesis :: setAssertion (const mdl :: Assertion* const assertion) const {
		Statement :: setAssertion (assertion);
	}
	index :: Arity
	Hypothesis :: getIndex() const {
		return Statement :: getIndex();
	}
	const mdl :: proof :: Node*
	Hypothesis :: getProofNode() const {
		return Statement :: getProofNode();
	}

	// object :: Expressive implementation
	const array :: Expression*
	Hypothesis :: getExpression() const {
		return Statement :: getExpression();
	}
	const mdl :: Type*
	Hypothesis :: getType() const {
		return Statement :: getType();
	}

	// object :: Buildable implementation
	void
	Hypothesis :: buildTheory (mdl :: Theory* const theory)
	{
		Statement :: buildTheory (theory);
		Statement :: proofNode_ =
			new form :: proof :: node :: Hypothesis
			(
				Statement :: assertion_,
				Statement :: index_,
				Statement :: expression_
			);
	}

using manipulator :: space;
using manipulator :: tab;
using manipulator :: endLine;
using manipulator :: underline;

	// object :: Writable implementation
	void
	Hypothesis :: translate (String& string) const
	{
		string << tab;
		if (Config :: keepLabels()) {
			assertion_->translateName (string);
			string << underline << Token :: hyp_ << underline << (index_ + 1);
		} else {
			string << :: smm :: Token :: prefix_e_ << index_;
		}
		string << space;
		string << :: smm :: Token :: essential_ << space;
		expression_->write (string);
		string << :: smm :: Token :: end_ << endLine;
	}
	void
	Hypothesis :: replicate (String& string) const
	{
		string << location_.indent() << Token :: hyp_ << space;
		if (assertion_->getHypArity() > 1) {
			string << (index_ + 1) << space;
		}
		string << Token :: colon_ << space;
		identificator_->replicate (string);
		string << space << Token :: equality_ << space;
		string << Token :: turnstile_ << space;
		expression_->write (string);
		string << Token :: semicolon_ << endLine;
	}

	// object :: Object implementation
	void
	Hypothesis :: commitSuicide() {
		delete this;
	}
	Size_t
	Hypothesis :: getVolume() const {
		return Statement :: getVolume();
	}
	Size_t
	Hypothesis :: getSizeOf() const {
		return sizeof (Hypothesis);
	}
}
}
}
}

#endif /*MDL_FORM_HEADER_STATEMENT_HYPOTHESIS_CPP_*/
