/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File Name:       mdl_form_header_statement_Proposition.cpp                */
/* Description:     proposition statement class                              */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_FORM_HEADER_STATEMENT_PROPOSITION_CPP_
#define MDL_FORM_HEADER_STATEMENT_PROPOSITION_CPP_

#include "mdl/math/mdl_math.hpp"
#include "mdl/form/mdl_form.hpp"

namespace mdl {
namespace form {
namespace header {
namespace statement {

	/****************************
	 *		Public members
	 ****************************/

	inline
	Proposition :: Proposition
	(
		const Location& location,
		const index :: Arity index,
		const mdl :: Identificator* const identificator,
		const array :: Expression* const expression
	) :
	form :: header :: statement :: Statement
	(
		location,
		index,
		identificator,
		expression
	),
	header :: Line :: Counter(),
	experience_ (),
	sourceIndex_ (counter_ ++) {
	}
	Proposition :: ~ Proposition() {
	}

	// mdl :: Proposition interface
	index :: Assertion
	Proposition :: getSourceIndex() const {
		return sourceIndex_;
	}
	index :: Assertion
	Proposition :: getTargetIndex() const {
		return header :: Line :: Counter :: getIndex();
	}
	mdl :: prover :: Experience&
	Proposition :: getExperience() const {
		return experience_;
	}

	// mdl :: Statement implementation
	const mdl ::Assertion*
	Proposition :: getAssertion() const {
		return Statement :: getAssertion();
	}
	void
	Proposition :: setAssertion (const mdl :: Assertion* const assertion) const {
		Statement :: setAssertion (assertion);
	}
	index :: Arity
	Proposition :: getIndex() const {
		return Statement :: getIndex();
	}
	const mdl :: proof :: Node*
	Proposition :: getProofNode() const {
		return Statement :: getProofNode();
	}

	// object :: Expressive implementation
	const array :: Expression*
	Proposition :: getExpression() const {
		return Statement :: getExpression();
	}
	const mdl :: Type*
	Proposition :: getType() const {
		return Statement :: getType();
	}

	// object :: Buildable implementation
	void
	Proposition :: buildTheory (mdl :: Theory* const theory)
	{
		Statement :: buildTheory (theory);
		Statement :: proofNode_ =
			new form :: proof :: node :: Proposition
			(
				Statement :: assertion_,
				Statement :: index_,
				Statement :: expression_
			);
	}

using manipulator :: tab;
using manipulator :: endLine;
using manipulator :: underline;

	// object :: Writable implementation
	void
	Proposition :: translate (String& string) const {
		expression_->write (string);
	}
	void
	Proposition :: replicate (String& string) const
	{
		string << location_.indent() << Token :: prop_ << space;
		if (assertion_ != NULL && (assertion_->getPropArity() > 1)) {
			string << (Statement :: index_ + 1) << space;
		}
		string << Token :: colon_ << space;
		identificator_->replicate (string);
		string << space << Token :: equality_ << space;
		string << Token :: turnstile_ << space;
		expression_->write (string);
		string << Token :: semicolon_ << endLine;
	}

	// object :: Object implementation
	Size_t
	Proposition :: getVolume() const {
		return Statement :: getVolume();
	}
	Size_t
	Proposition :: getSizeOf() const {
		return sizeof (Proposition);
	}

	/************************************
	 *		Static private attributes
	 ************************************/

	index :: Assertion
		Proposition :: counter_ = 0;
}
}
}
}

#endif /*MDL_FORM_HEADER_STATEMENT_PROPOSITION_CPP_*/
