/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File Name:       mdl_form_header_statement_Proposition.hpp                */
/* Description:     proposition statement class                              */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_FORM_HEADER_STATEMENT_PROPOSITION_HPP_
#define MDL_FORM_HEADER_STATEMENT_PROPOSITION_HPP_

#include "mdl/form/header/statement/mdl_form_header_statement_Statement.hpp"
#include "mdl/form/header/statement/mdl_form_header_statement_Experience.hpp"

namespace mdl {
namespace form {
namespace header {
namespace statement {

class Proposition :
	public Statement,
	public mdl :: Proposition,
	public Line :: Counter {
public :
	Proposition
	(
		const Location&,
		const index :: Arity,
		const mdl :: Identificator* const,
		const array :: Expression* const
	);
	virtual ~ Proposition();

	// mdl :: Proposition interface
	virtual index :: Assertion getSourceIndex() const;
	virtual index :: Assertion getTargetIndex() const;
	virtual mdl :: prover :: Experience& getExperience() const;

	// mdl :: Statement interface
	virtual const mdl :: Assertion* getAssertion() const;
	virtual void setAssertion (const mdl :: Assertion* const) const;
	virtual index :: Arity getIndex() const;
	virtual const mdl :: proof :: Node* getProofNode() const;

	// object :: Expressive interface
	virtual const array :: Expression* getExpression() const;
	virtual const mdl :: Type* getType() const;

	// object :: Buildable interface
	virtual void buildTheory (mdl :: Theory* const);

	// object :: Writable interface
	virtual void translate (String&) const;
	virtual void replicate (String&) const;

	// object :: Object interface
	virtual void commitSuicide() = 0;
	virtual Size_t getVolume() const;
	virtual Size_t getSizeOf() const;

private :
	mutable Experience experience_;
	const index :: Assertion sourceIndex_;
	static index :: Assertion counter_;
};

}
}
}
}

#endif /*MDL_FORM_HEADER_STATEMENT_PROPOSITION_HPP_*/
