/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File Name:       mdl_form_header_statement_Provable.cpp                   */
/* Description:     provable statement class                                 */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_FORM_HEADER_STATEMENT_PROVABLE_CPP_
#define MDL_FORM_HEADER_STATEMENT_PROVABLE_CPP_

#include "smm/lexer/smm_lexer_Token.hpp"
#include "mdl/form/mdl_form.hpp"

namespace mdl {
namespace form {
namespace header {
namespace statement {

	/****************************
	 *		Public members
	 ****************************/

	inline
	Provable :: Provable
	(
		const Location& location,
		const index :: Arity index,
		const mdl :: Identificator* const identificator,
		const array :: Expression* const expression
	) :
	header :: statement :: Proposition
	(
		location,
		index,
		identificator,
		expression
	) {
	}
	Provable :: ~ Provable() {
	}

	// statement :: Provable implementation
	const mdl :: proof :: Provable*
	Provable :: getProof() const {
		return
			dynamic_cast<const mdl :: proof :: Provable*>(Proposition :: proofNode_);
	}

	// mdl :: Proposition interface
	index :: Assertion
	Provable :: getSourceIndex() const {
		return Proposition :: getSourceIndex();
	}
	index :: Assertion
	Provable :: getTargetIndex() const {
		return Proposition :: getTargetIndex();
	}
	prover :: Experience&
	Provable :: getExperience() const {
		return Proposition :: getExperience();
	}

	// mdl :: Statement implementation
	const mdl ::Assertion*
	Provable :: getAssertion() const {
		return Proposition :: getAssertion();
	}
	void
	Provable :: setAssertion (const mdl :: Assertion* const assertion) const {
		Proposition :: setAssertion (assertion);
	}
	index :: Arity
	Provable :: getIndex() const {
		return Proposition :: getIndex();
	}
	const mdl :: proof :: Node*
	Provable :: getProofNode() const {
		return Proposition :: getProofNode();
	}

	// object :: Expressive implementation
	const array :: Expression*
	Provable :: getExpression() const {
		return Proposition :: expression_;
	}
	const mdl :: Type*
	Provable :: getType() const {
		return Proposition :: type_;
	}

	// object :: Buildable implementation
	void
	Provable :: buildTheory (mdl :: Theory* const theory)
	{
		Statement :: buildTheory (theory);
		Statement :: proofNode_ =
			new form :: proof :: node :: Provable
			(
				Statement :: assertion_,
				Statement :: index_,
				Statement :: expression_
			);
	}

using manipulator :: endLine;
using manipulator :: space;
using manipulator :: tab;

	// object :: Writable implementation
	void
	Provable :: translate (String& string) const
	{
		string << tab;
		assertion_->translateName (string, Statement :: index_);
		string << space;
		string << :: smm :: Token :: provable_ << space;
		Proposition :: translate (string);
		string << :: smm :: Token :: equality_<< endLine;
		string << tab;
		proofNode_->translate (string);
		string << :: smm :: Token :: end_<< endLine;
	}
	void
	Provable :: replicate (String& string) const {
		Proposition :: replicate (string);
	}

	// object :: Object implementation
	void
	Provable :: commitSuicide() {
		delete this;
	}
	Size_t
	Provable :: getVolume() const {
		return Proposition :: getVolume();
	}
	Size_t
	Provable :: getSizeOf() const {
		return sizeof (Provable);
	}
}
}
}
}

#endif /*MDL_FORM_HEADER_STATEMENT_PROVABLE_CPP_*/
