/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File Name:       mdl_form_header_statement_Provable.hpp                   */
/* Description:     provable statement class                                 */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_FORM_HEADER_STATEMENT_PROVABLE_HPP_
#define MDL_FORM_HEADER_STATEMENT_PROVABLE_HPP_

#include "mdl/interface/mdl_interface.hpp"
#include "mdl/form/header/statement/mdl_form_header_statement_Statement.hpp"

namespace mdl {
namespace form {
namespace header {
namespace statement {

class Provable :
	public mdl :: statement :: Provable,
	public Proposition,
	public Scalar<Provable> {
public :
	Provable
	(
		const Location&,
		const index :: Arity,
		const mdl :: Identificator* const,
		const array :: Expression* const
	);
	virtual ~ Provable();

	// mdl :: Proposition interface
	virtual index :: Assertion getSourceIndex() const;
	virtual index :: Assertion getTargetIndex() const;
	virtual prover :: Experience& getExperience() const;

	// statement :: Provable interface
	virtual const mdl :: proof :: Provable* getProof() const;

	// mdl :: Statement interface
	virtual const mdl :: Assertion* getAssertion() const;
	virtual void setAssertion (const mdl :: Assertion* const) const;
	virtual index :: Arity getIndex() const;
	virtual const mdl :: proof :: Node* getProofNode() const;

	// object :: Expressive interface
	virtual const array :: Expression* getExpression () const;
	virtual const mdl:: Type* getType() const;

	// object :: Buildable interface
	virtual void buildTheory (mdl :: Theory* const);

	// object :: Writable interface
	virtual void translate (String&) const;
	virtual void replicate (String&) const;

	// object :: Object interface
	virtual void commitSuicide();
	virtual Size_t getVolume() const;
	virtual Size_t getSizeOf() const;
};

}
}
}
}

#endif /*MDL_FORM_HEADER_STATEMENT_PROVABLE_HPP_*/
