/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File Name:       mdl_form_header_statement_Statement.cpp                  */
/* Description:     statement class                                          */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_FORM_HEADER_STATEMENT_STATEMENT_CPP_
#define MDL_FORM_HEADER_STATEMENT_STATEMENT_CPP_

#include "mdl/math/mdl_math.hpp"
#include "mdl/form/mdl_form.hpp"

namespace mdl {
namespace form {
namespace header {
namespace statement {

	/****************************
	 *		Public members
	 ****************************/

	inline
	Statement :: Statement
	(
		const Location& location,
		const index :: Arity index,
		const mdl :: Identificator* const identificator,
		const array :: Expression* const expression
	) :
	header :: Line (location, identificator, expression),
	assertion_ (NULL),
	index_ (index),
	proofNode_ (NULL) {
	}
	Statement :: ~ Statement() {
		deleteAggregateObject (proofNode_);
	}

	// mdl :: Statement implementation
	const mdl :: Assertion*
	Statement :: getAssertion() const {
		return assertion_;
	}
	void
	Statement :: setAssertion (const mdl :: Assertion* const assertion) const {
		assertion_ = assertion;
	}
	index :: Arity
	Statement :: getIndex() const {
		return index_;
	}
	const mdl :: proof :: Node*
	Statement :: getProofNode() const {
		return proofNode_;
	}

	// object :: Expressive implementation
	const array :: Expression*
	Statement :: getExpression() const {
		return expression_;
	}
	const mdl :: Type*
	Statement :: getType() const {
		return type_;
	}

	// object :: Buildable implementation
	void
	Statement :: buildTheory (mdl :: Theory* const theory)
	{
		try {
			header :: Line :: buildTheory (theory);
			const_cast<array :: Expression* const>(expression_)->build (assertion_->getVariables());
			const_cast<array :: Expression* const>(expression_)->parse (type_);
		} catch (Error* error) {
			error->add (location_);
			throw error;
		}
	}

	// object :: Object implementation
	void
	Statement :: commitSuicide() {
		delete this;
	}
	Size_t
	Statement :: getVolume() const
	{
		Size_t volume = 0;
		volume += header :: Line :: getVolume();
		volume += getAggregateVolume (proofNode_);
		return volume;
	}
	Size_t
	Statement :: getSizeOf() const {
		return sizeof (Statement);
	}
}
}
}
}

#endif /*MDL_FORM_HEADER_STATEMENT_STATEMENT_CPP_*/
