/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File Name:       mdl_form_header_statement_Statement.hpp                  */
/* Description:     statement class                                          */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_FORM_HEADER_STATEMENT_STATEMENT_HPP_
#define MDL_FORM_HEADER_STATEMENT_STATEMENT_HPP_

#include "mdl/interface/mdl_interface.hpp"
#include "mdl/form/header/mdl_form_header_Line.hpp"

namespace mdl {
namespace form {
namespace header {
namespace statement {

class Statement :
	public mdl :: Statement,
	public Line {
public :
	Statement
	(
		const Location&,
		const index :: Arity,
		const mdl :: Identificator* const,
		const array :: Expression* const
	);
	virtual ~ Statement();

	// mdl :: Statement interface
	virtual const mdl :: Assertion* getAssertion() const;
	virtual void setAssertion (const mdl :: Assertion* const) const;
	virtual index :: Arity getIndex() const;
	virtual const mdl :: proof :: Node* getProofNode() const;

	// object :: Expressive interface
	virtual const array :: Expression* getExpression() const;
	virtual const mdl :: Type* getType() const;

	// object :: Buildable interface
	virtual void buildTheory (mdl :: Theory* const);

	// object :: Writable interface
	virtual void translate (String&) const = 0;
	virtual void replicate (String&) const = 0;

	// object :: Object interface
	virtual void commitSuicide();
	virtual Size_t getVolume() const;
	virtual Size_t getSizeOf() const;

protected :
	mutable const mdl :: Assertion* assertion_;
	const index :: Arity index_;
	const mdl :: proof :: Node* proofNode_;
};

}
}
}
}

#endif /*MDL_FORM_HEADER_STATEMENT_STATEMENT_HPP_*/
