/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File Name:       mdl_form_header_term_Rule.hpp                            */
/* Description:     syntax rule term class                                   */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_FORM_HEADER_TERM_RULE_HPP_
#define MDL_FORM_HEADER_TERM_RULE_HPP_

#include "mdl/interface/mdl_interface.hpp"
#include "mdl/form/header/mdl_form_header_Line.hpp"

namespace mdl {
namespace form {
namespace header {
namespace term {

class Rule :
	public mdl :: term :: Rule,
	public Line,
	public Line :: Counter,
	public Scalar<Rule> {
public :
	Rule
	(
		const Location&,
		const index :: Arity,
		const mdl :: Identificator* const,
		const array :: Expression* const
	);
	virtual ~ Rule();

	// mdl :: term :: Rule interface
	virtual void setRule (const mdl :: Rule* const) const;
	virtual index :: Assertion getTargetIndex() const;

	// object :: Expressive interface
	virtual const array :: Expression* getExpression() const;
	virtual const mdl :: Type* getType() const;

	// object :: Buildable interface
	virtual void buildTheory (mdl :: Theory* const);

	// object :: Writable interface
	virtual void translate (String&) const;
	virtual void replicate (String&) const;

	// object :: Object interface
	virtual void commitSuicide();
	virtual Size_t getVolume() const;
	virtual Size_t getSizeOf() const;

private :
	void checkSemantics() const;
	void checkVariable (const index :: Literal) const;

	mutable const mdl :: Rule* rule_;
};

}
}
}
}

#endif /*MDL_FORM_HEADER_TERM_RULE_HPP_*/
