/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File Name:       mdl_form_Contents.hpp                                    */
/* Description:     contents of a theory                                     */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/


#ifndef MDL_FORM_CONTENTS_HPP_
#define MDL_FORM_CONTENTS_HPP_

#include "mdl/interface/mdl_interface.hpp"
#include "mdl/form/mdl_form_Identifiable.hpp"

namespace mdl {
namespace form {

class Contents :
	public mdl :: Contents,
	public Identifiable,
	public Scalar<Contents> {
public :
	typedef
		list :: object :: Verifiable
		ComponentList_;

	Contents
	(
		const Location&,
		const mdl :: Comments* const,
		const value :: Name
	);
	virtual ~ Contents();

	// mdl :: Contents interface
	virtual void addComponent (const object :: Verifiable* const);

	// object :: Identifiable interface
	virtual const mdl :: Theory* getTheory() const;
	virtual value :: Name getName() const;
	virtual void translateName (String&) const;
	virtual void replicateName (String&) const;

	// object :: Verifiable interface
	virtual void check() const;
	virtual void compress();
	virtual void prove();
	virtual bool isVerified() const;

	// object :: Buildable interface
	virtual void buildTheory (mdl :: Theory* const);

	// object :: Writable interface
	virtual void translate (String&) const;
	virtual void replicate (String&) const;
	virtual void write (Output&) const;

	// object :: Object interface
	virtual void commitSuicide();
	virtual Size_t getVolume() const;
	virtual Size_t getSizeOf() const;

private :
	ComponentList_ componentList_;
	typedef ComponentList_ :: Node_ Node_;
	mutable bool isVerified_;
};

}
}

#endif /*MDL_FORM_CONTENTS_HPP_*/
