/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File Name:       mdl_form_Identifiable.cpp                                */
/* Description:     base class for identifiable objects                      */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_FORM_IDENTIFIABLE_CPP_
#define MDL_FORM_IDENTIFIABLE_CPP_

#include "mdl/auxiliary/mdl_auxiliary.hpp"
#include "mdl/form/mdl_form.hpp"

namespace mdl {
namespace form {

	/****************************
	 *		Public members
	 ****************************/

	inline
	Identifiable :: Identifiable() :
	location_ (),
	comments_ (NULL),
	name_ (value :: undefined :: NAME),
	identificator_ (NULL),
	theory_ (NULL) {
	}
	inline
	Identifiable :: Identifiable
	(
		const Location& location,
		const mdl :: Comments* comments,
		const value :: Name name
	) :
	location_ (location),
	comments_ (comments),
	name_ (name),
	identificator_ (new form :: Identificator (name)),
	theory_ (NULL) {
	}
	Identifiable :: ~ Identifiable()
	{
		deleteAggregateObject (comments_);
		deleteAggregateObject (identificator_);
	}

	// object :: Identifiable implementation
	const mdl :: Theory*
	Identifiable :: getTheory() const {
		return theory_;
	}
	value :: Name
	Identifiable :: getName() const {
		return name_;
	}
	const mdl :: Identificator*
	Identifiable :: getIdentificator() const {
		return identificator_;
	}

using manipulator :: space;
using manipulator :: underline;

	void
	Identifiable :: translateName (String& string, const index :: Arity) const
	{
		if (theory_ != NULL) {
			if (theory_->getTheory() != NULL) {
				theory_->getPath()->translate (string);
				string << underline;
			}
			string << Table :: names()->get (name_);
		}
	}
	void
	Identifiable :: replicateName (String& string) const
	{
		if (theory_ != NULL) {
			string << Table :: names()->get (name_);
		}
	}

	// object :: Buildable implementation
	void
	Identifiable :: buildTheory (mdl :: Theory* const theory)
	{
		try {
			theory_ = theory;
			theory_->checkUnused (name_);
		} catch (Error* error) {
			error->add (location_);
			throw error;
		}
	}

	// object :: Object implementation
	Size_t
	Identifiable :: getSizeOf() const {
		return sizeof (Identifiable);
	}
	Size_t
	Identifiable :: getVolume() const
	{
		Size_t volume = 0;
		volume += getAggregateVolume (comments_);
		volume += getAggregateVolume (identificator_);
		return volume;
	}
}
}

#endif /*MDL_FORM_IDENTIFIABLE_CPP_*/
