/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File Name:       mdl_form_Identifiable.hpp                                */
/* Description:     base class for identifiable objects                      */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_FORM_IDENTIFIABLE_HPP_
#define MDL_FORM_IDENTIFIABLE_HPP_

#include "mdl/interface/mdl_interface.hpp"

namespace mdl {
namespace form {

class Identifiable : virtual public object :: Identifiable {
public :
	Identifiable ();
	Identifiable
	(
		const Location&,
		const mdl :: Comments* const,
		const value :: Name
	);
	virtual ~ Identifiable();

	// object :: Identifiable interface
	virtual const mdl :: Theory* getTheory() const;
	virtual value :: Name getName() const;
	virtual const mdl :: Identificator* getIdentificator() const;
	virtual void translateName (String&, const index :: Arity = 0) const;
	virtual void replicateName (String&) const;

	// object :: Verifiable interface
	virtual void check() const = 0;
	virtual void prove() = 0;
	virtual bool isVerified() const = 0;

	// object :: Buildable interface
	virtual void buildTheory (mdl :: Theory* const);

	// object :: Writable interface
	virtual void translate (String&) const = 0;
	virtual void replicate (String&) const = 0;

	// object :: Object interface
	virtual void commitSuicide() = 0;
	virtual Size_t getSizeOf() const;
	virtual Size_t getVolume() const;

protected :
	const Location location_;
	const mdl :: Comments* comments_;
	const value :: Name name_;
	const mdl :: Identificator* identificator_;

	const mdl :: Theory* theory_;
};

}
}

#endif /*MDL_FORM_IDENTIFIABLE_HPP_*/
