/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File Name:       mdl_form_Identificator.cpp                               */
/* Description:     identificator                                            */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_FORM_IDENTIFICATOR_CPP_
#define MDL_FORM_IDENTIFICATOR_CPP_

#include "mdl/lexer/mdl_lexer.hpp"
#include "mdl/form/mdl_form_Identificator.hpp"

namespace mdl {
namespace form {

	/****************************
	 *		Public members
	 ****************************/

	inline
	Identificator :: Identificator () :
	name_(value :: undefined :: NAME),
	next_ (NULL) {
	}
	inline
	Identificator :: Identificator (const mdl :: Identificator& identificator) :
	name_(identificator.getName()),
	next_
	(
		(identificator.getNext() == NULL) ?
			NULL :
			new Identificator (identificator.getNext())
	) {
	}
	inline
	Identificator :: Identificator (const mdl :: Identificator* const identificator) :
	name_(identificator->getName()),
	next_
	(
		(identificator->getNext() == NULL) ?
			NULL :
			new Identificator (identificator->getNext())
	) {
	}
	inline
	Identificator :: Identificator (const value :: Name name) :
	name_ (name),
	next_ (NULL) {
	}
	Identificator :: ~ Identificator() {
		deleteAggregateObject (next_);
	}

	// mdl :: Identificator implementation
	value :: Name
	Identificator :: getName() const {
		return name_;
	}
	const mdl :: Identificator*
	Identificator :: getNext() const {
		return next_;
	}
	void
	Identificator :: setNext (const mdl :: Identificator* const next) const {
		next_ = next;
	}

using manipulator :: underline;

	// object :: Writable implementation
	void
	Identificator :: translate (String& string) const
	{
		if (name_ != value :: undefined :: NAME) {
			string << Table :: names()->get (name_);
		}
		if (next_ != NULL) {
			if (name_ != value :: undefined :: NAME) {
				string << underline;
			}
			next_->replicate (string);
		}
	}
	void
	Identificator :: replicate (String& string) const
	{
		if (name_ != value :: undefined :: NAME) {
			string << Table :: names()->get (name_);
		}
		if (next_ != NULL) {
			if (name_ != value :: undefined :: NAME) {
				string << Token :: dot_;
			}
			next_->replicate (string);
		}
	}

	// object :: Object implementation
	void
	Identificator :: commitSuicide() {
		delete this;
	}
	Size_t
	Identificator :: getVolume() const {
		return getAggregateVolume (next_);
	}
	Size_t
	Identificator :: getSizeOf() const {
		return sizeof (Identificator);
	}
}
}

#endif /*MDL_FORM_IDENTIFICATOR_CPP_*/
