/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File Name:       mdl_form_Import.cpp                                      */
/* Description:     import class                                             */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_FORM_IMPORT_CPP_
#define MDL_FORM_IMPORT_CPP_

#include "mdl/lexer/mdl_lexer.hpp"
#include "mdl/form/mdl_form_Identificator.hpp"
#include "mdl/form/mdl_form_Import.hpp"

#include "smm/lexer/smm_lexer_Token.hpp"

namespace mdl {
namespace form {

	/****************************
	 *		Public members
	 ****************************/

	inline
	Import :: Import
	(
		const Location& location,
		const mdl :: Identificator* const pathComponents
	) :
	location_ (location),
	pathComponents_ (pathComponents),
	sourcePath_ (),
	targetPath_ ()
	{
		const mdl :: Identificator*
			pathComponent = pathComponents_;
		while (pathComponent != NULL) {
			const value :: Name
				component = pathComponent->getName();
			const String&
				string = Table :: names()->get (component);
			if (pathComponent->getNext() == NULL) {
				sourcePath_.name() = string;
				targetPath_.name() = string;
			} else {
				sourcePath_.pushDirectory() = string;
				targetPath_.pushDirectory() = string;
			}
			pathComponent = pathComponent->getNext();
		}
		sourcePath_.extension() = "mdl";

		const Config :: Path_& path = *Config :: getTarget();
		if (path.getDepth() > 0) {
			const_cast<Path&> (targetPath_).directory(0) = path.getDirectory (0);
		}
		if (Config :: translate()) {
			targetPath_.extension() = "smm";
		} else {
			targetPath_.extension() = "mdl";
		}
	}
	Import :: ~ Import() {
		deleteAggregateObject (pathComponents_);
	}

	// mdl :: Import implementation
	const Path&
	Import :: getPath() const {
		return sourcePath_;
	}

	// object :: Verifiable implementation
	void
	Import :: check() const {
	}
	void
	Import :: compress() {
	}
	void
	Import :: prove() {
	}
	bool
	Import :: isVerified() const {
		return true;
	}

	// object :: Buildable implementation
	void
	Import :: buildTheory (mdl :: Theory* const) {
	}

using manipulator :: endLine;
using manipulator :: space;

	// object :: Writable implementation
	void
	Import :: translate (String& string) const
	{
		string << location_.indent();
		string << :: smm :: Token :: inclusionBegin_ << space;
		string << targetPath_.getString() << space;
		string << :: smm :: Token :: inclusionEnd_ << endLine;
	}
	void
	Import :: replicate (String& string) const
	{
		string << location_.indent() << Token :: import_ << space;
		string << *pathComponents_ << Token :: semicolon_ << endLine;
	}

	// object :: Object implementation
	void
	Import :: commitSuicide() {
		delete this;
	}
	Size_t
	Import :: getVolume() const {
		Size_t volume = 0;
		volume += getAggregateVolume (pathComponents_);
		volume += sourcePath_.getVolume();
		volume += targetPath_.getVolume();
		return volume;
	}
	Size_t
	Import :: getSizeOf() const {
		return sizeof (Import);
	}

	/****************************
	 *		Private members
	 ****************************/
}
}

#endif /*MDL_FORM_IMPORT_CPP_*/
