/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File Name:       mdl_form_Import.hpp                                      */
/* Description:     import class                                             */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_FORM_IMPORT_HPP_
#define MDL_FORM_IMPORT_HPP_

#include "mdl/interface/mdl_interface.hpp"

namespace mdl {
namespace form {

class Import :
	public mdl :: Import,
	public Scalar<Import> {
public :
	Import (const Location&, const mdl :: Identificator* const);
	virtual ~ Import();

	// mdl :: Import interface
	virtual const Path& getPath() const;

	// object :: Verifiable interface
	virtual void check() const;
	virtual void compress();
	virtual void prove();
	virtual bool isVerified() const;

	// object :: Buildable interface
	virtual void buildTheory (mdl :: Theory* const);

	// object :: Writable interface
	virtual void translate (String&) const;
	virtual void replicate (String&) const;

	// object :: Object interface
	virtual void commitSuicide();
	virtual Size_t getVolume() const;
	virtual Size_t getSizeOf() const;

private :
	const Location location_;
	const mdl :: Identificator* const pathComponents_;
	//mutable Path path_;
	mutable Path sourcePath_;
	mutable Path targetPath_;

};

}
}

#endif /*MDL_FORM_IMPORT_HPP_*/
