/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File Name:       mdl_form_Source.hpp                                      */
/* Description:     source class                                             */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_FORM_SOURCE_HPP_
#define MDL_FORM_SOURCE_HPP_

#include "mdl/interface/mdl_interface.hpp"
#include "mdl/lexer/mdl_lexer.hpp"

namespace mdl {
namespace form {

class Source :
	public mdl :: Source,
	public Scalar<Source> {
public :
	typedef
		list :: object :: Verifiable
		ComponentList_;

	Source
	(
		const bool, const Lexer* const,
		const mdl :: Comments*,
		const Path&, const Path&
	);
	virtual ~ Source();

	// mdl :: Source interface
	virtual void build();
	virtual void addComponent (const object :: Verifiable* const);

	// object :: Verifiable interface
	virtual void check() const;
	virtual void compress();
	virtual void prove();
	virtual bool isVerified() const;

	// object :: Buildable interface
	virtual void buildTheory (mdl :: Theory* const);

	// object :: Writable interface
	virtual void translate (String&) const;
	virtual void replicate (String&) const;
	virtual void write() const;

	// object :: Object interface
	virtual void commitSuicide();
	virtual Size_t getSizeOf() const;
	virtual Size_t getVolume() const;

private :
	typedef ComponentList_ :: Node_ Node_;
	const bool isTop_;
	const Lexer* const lexer_;
	mutable Path sourcePath_;
	mutable Path targetPath_;
	const mdl :: Comments* comments_;
	ComponentList_ componentList_;
	mutable Output* output_;
	mutable bool isVerified_;
};

}
}

#endif /*MDL_FORM_SOURCE_HPP_*/
