/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File Name:       mdl_form_Theory.hpp                                      */
/* Description:     theory class                                             */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_FORM_THEORY_HPP_
#define MDL_FORM_THEORY_HPP_

#include "mdl/interface/mdl_interface.hpp"
#include "mdl/form/mdl_form_Identifiable.hpp"

namespace mdl {
namespace form {

class Theory :
	public mdl :: Theory,
	public Identifiable,
	public Scalar<Theory> {
public :
	Theory ();
	Theory
	(
		const Location&,
		const mdl :: Comments* const,
		const value :: Name
	);
	virtual ~ Theory();

	// mdl :: Theory interface
	virtual void checkUnused (const value :: Name) const;
	virtual index :: Form checkUsed (const mdl :: Identificator* const) const;
	virtual void add (const index :: Form, const value :: Name);
	virtual const mdl :: Identificator* getPath() const;
	virtual index :: Form getIndex (const mdl :: Identificator* const) const;

	// object :: Identifiable interface
	virtual const mdl :: Theory* getTheory() const;
	virtual value :: Name getName() const;
	virtual void translateName (String&) const;
	virtual void replicateName (String&) const;

	// object :: Verifiable interface
	virtual void check() const;
	virtual void compress();
	virtual void prove();
	virtual bool isVerified() const;

	// object :: Buildable interface
	virtual void buildTheory (mdl :: Theory* const);

	// object :: Writable interface
	virtual void translate (String&) const;
	virtual void replicate (String&) const;

	// object :: Object interface
	virtual void commitSuicide();
	virtual Size_t getVolume() const;
	virtual Size_t getSizeOf() const;

private :
	// mdl :: Theory interface
	virtual index :: Form getIndexUp (const mdl :: Identificator* const) const;
	virtual index :: Form getIndexDown (const mdl :: Identificator* const) const;

	typedef
		map :: Scalar
		<
			value :: Name,
			index :: Form,
			allocator :: Heap
		>
		FormMap;

	const mdl :: Identificator* path_;
	FormMap formMap_;
};

}
}

#endif /*MDL_FORM_THEORY_HPP_*/
