/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File Name:       mdl_form_proof_Link.hpp                                  */
/* Description:     proof link class                                         */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_FORM_PROOF_LINK_HPP_
#define MDL_FORM_PROOF_LINK_HPP_

#include "mdl/interface/mdl_interface.hpp"
#include "mdl/lexer/mdl_lexer.hpp"
#include "mdl/form/mdl_form_Identificator.hpp"

namespace mdl {
namespace form {
namespace proof {

class Link :
	public mdl :: proof :: Link,
	public Scalar<Link> {
public :
	Link
	(
		const Location&,
		const value :: Kind,
		const mdl :: Identificator* const,
		const index :: Arity,
		const mdl :: proof :: References* const
	);
	Link
	(
		const Location&,
		const value :: Kind
	);
	Link
	(
		const Location&,
		const mdl :: Proposition* const,
		const mdl :: proof :: References* const
	);
	virtual ~ Link();

	// mdl :: proof :: Link interface
	virtual const mdl :: Assertion* getAssertion() const;
	virtual index :: Arity getPropositionIndex() const;
	virtual const mdl :: Proposition* getProposition() const;
	virtual void buildScope (const mdl :: proof :: Scope* const);
	virtual const mdl :: proof :: References* getReferences() const;
	virtual mdl :: proof :: References* references();

	virtual void translateReferences (String&) const;
	virtual void translateAssertion (String&) const;

	// object :: Buildable interface
	virtual void buildTheory (mdl :: Theory* const);

	// object :: Writable interface
	virtual void translate (String&) const;
	virtual void replicate (String&) const;

	// object :: Object interface
	virtual void commitSuicide();
	virtual Size_t getVolume() const;
	virtual Size_t getSizeOf() const;

private :
	const Location location_;
	const value :: Kind kind_;
	const mdl :: Identificator* identificator_;
	const index :: Arity proposition_;
	const mdl :: proof :: References* const references_;
	const mdl :: Assertion* assertion_;
};

}
}
}

#endif /*MDL_FORM_PROOF_LINK_HPP_*/
