/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File Name:       mdl_form_proof_Qed.hpp                                   */
/* Description:     proof q.e.d. class                                       */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_FORM_PROOF_QED_HPP_
#define MDL_FORM_PROOF_QED_HPP_

#include "mdl/interface/mdl_interface.hpp"
#include "mdl/lexer/mdl_lexer.hpp"

namespace mdl {
namespace form {
namespace proof {

class Qed :
	public mdl :: proof :: Qed,
	public Scalar<Qed> {
public :
	enum {
		PROP = Token :: PROP,
		CLAIM = Token :: CLAIM
	};

	Qed
	(
		const mdl :: proof :: Line* const,
		const Location&,
		const mdl :: Comments* const,
		const value :: Kind,
		const index :: Arity,
		const index :: Step
	);
	Qed
	(
		const mdl :: proof :: Provable* const,
		const mdl :: proof :: Step* const
	);
	virtual ~ Qed();

	// proof :: Qed interface
	virtual const mdl :: proof :: Provable* getProp() const;
	virtual const mdl :: proof :: Node* getStep() const;

	// proof :: Line interface
	virtual Sort getSort() const;
	virtual const mdl :: proof :: Line* getNext() const;
	virtual const mdl :: proof :: Line* getPrev() const;
	virtual void incIndex (const index :: Step) const;
	virtual void decIndex (const index :: Step) const;
	virtual void setNext (const mdl :: proof :: Line* const) const;
	virtual void setPrev (const mdl :: proof :: Line* const) const;
	virtual void completeWithSteps (mdl :: prover :: tree :: Premise*) const;

	// object :: Scoping interface
	virtual const mdl :: proof :: Scope* buildScope (const mdl :: proof :: Scope* const);
	virtual const mdl :: proof :: Scope* getScope() const;
	virtual void setScope (const mdl :: proof :: Scope*) const;

	// object :: Verifiable interface
	virtual void check() const;
	virtual void compress();
	virtual void prove();
	virtual bool isVerified() const;

	// object :: Buildable interface
	virtual void buildTheory (mdl :: Theory* const);

	// object :: Writable interface
	virtual void translate (String&) const;
	virtual void replicate (String&) const;

	// object :: Object interface
	virtual void commitSuicide();
	virtual Size_t getVolume() const;
	virtual Size_t getSizeOf() const;

private :
	const Location location_;
	const mdl :: Comments* const comments_;
	const value :: Kind kind_;
	const index :: Arity indexProp_;
	mutable index :: Step indexStep_;

	mutable const mdl :: proof :: Line* next_;
	mutable const mdl :: proof :: Line* prev_;
	const mdl :: proof :: Provable* prop_;
	mutable const mdl :: proof :: Node* step_;
	mutable const mdl :: proof :: Scope* scope_;
	mutable bool isVerified_;
};

}
}
}

#endif /*MDL_FORM_PROOF_QED_HPP_*/
