/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File Name:       mdl_form_proof_Reference.hpp                             */
/* Description:     reference to the proof node                              */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_FORM_PROOF_REFERENCE_HPP_
#define MDL_FORM_PROOF_REFERENCE_HPP_

#include "mdl/interface/mdl_interface.hpp"
#include "mdl/lexer/mdl_lexer.hpp"

namespace mdl {
namespace form {
namespace proof {

class Reference :
	public mdl :: proof :: Reference,
	public Scalar<Reference> {
public :
	Reference
	(
		const Location&,
		const value :: Kind,
		const index :: Step,
		const mdl :: proof :: Node* = NULL
	);
	Reference (const Reference&);
	Reference (const mdl :: proof :: Reference*);
	virtual ~ Reference();

	// mdl :: proof :: Reference interface
	virtual const mdl :: proof :: Node* getNode() const;
	virtual void build (const mdl :: proof :: Scope* const);

	// object :: Writable interface
	virtual void translate (String&) const;
	virtual void replicate (String&) const;

	// object :: Object interface
	virtual void commitSuicide();
	virtual Size_t getVolume() const;
	virtual Size_t getSizeOf() const;

private :
	const Location location_;
	const value :: Kind kind_;
	mutable index :: Step index_;
	mutable const mdl :: proof :: Node* node_;
};

}
}
}

#endif /*MDL_FORM_PROOF_REFERENCE_HPP_*/

