/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File Name:       mdl_form_proof_References.cpp                            */
/* Description:     vector of references to the proof nodes                  */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_FORM_PROOF_REFERENCES_CPP_
#define MDL_FORM_PROOF_REFERENCES_CPP_

#include "mdl/lexer/mdl_lexer.hpp"
#include "mdl/form/proof/mdl_form_proof_References.hpp"

namespace mdl {
namespace form {
namespace proof {

	/****************************
	 *		Public members
	 ****************************/

	inline
	References :: References (const index :: Arity capacity) :
	referenceVector_ (capacity) {
	}
	inline
	References :: References (const vector :: parser :: Reference& referenceVector) :
	referenceVector_ (referenceVector) {
	}
	References :: ~ References() {
	}

	// proof :: References implementation
	index :: Arity
	References :: getArity() const {
		return referenceVector_.getSize();
	}
	const mdl :: proof :: Reference*
	References :: get (const index :: Arity index) const {
		return referenceVector_.getValue (index);
	}
	void
	References :: build (const mdl :: proof :: Scope* const scope)
	{
		for (index :: Arity i = 0; i < getArity(); ++ i) {
			const mdl :: proof :: Reference* const
				reference = referenceVector_.getValue (i);
			const_cast<mdl :: proof :: Reference* const>(reference)->build (scope);
		}
	}
	void
	References :: add (const mdl :: proof :: Reference* const ref)
	{
		const mdl :: proof :: Reference*
			newReference = ref;
		for (index :: Arity i = 0; i < getArity(); ++ i) {
			const mdl :: proof :: Reference* const
				oldReference = referenceVector_.getValue (i);
			if (oldReference == newReference) {
				const form :: proof :: Reference*
					reference = dynamic_cast<const form :: proof :: Reference*>(newReference);
				newReference = new form :: proof :: Reference (*reference);
				break;
			}
		}
		const_cast<vector :: proof :: Reference&>(referenceVector_).add (newReference);
	}
	index :: Arity
	References :: find (const mdl :: proof :: Node* node) const
	{
		for (index :: Arity i = 0; i < getArity(); ++ i) {
			const mdl :: proof :: Reference* const
				reference = referenceVector_.getValue (i);
			if (reference->getNode() == node) {
				return i;
			}
		}
		return index :: undefined :: ARITY;
	}

using manipulator :: space;

	// object :: Writable implementation
	void
	References :: translate (String& string) const
	{
		for (index :: Arity i = 0; i < getArity(); ++ i) {
			const mdl :: proof :: Reference* const
				reference = referenceVector_.getValue (i);
			reference->translate (string);
		}
	}
	void
	References :: replicate (String& string) const
	{
		string << Token :: openBracket_;
		const index :: Arity
			arity = referenceVector_.getSize();
		for (index :: Arity i = 0; i < arity; ++ i) {
			const mdl :: proof :: Reference* const
				reference = referenceVector_.getValue (i);
			reference->replicate (string);
			if (i < arity - 1) {
				string << Token :: comma_ << space;
			}
		}
		string << Token :: closeBracket_;
	}

	// object :: Object implementation
	void
	References :: commitSuicide() {
		delete this;
	}
	Size_t
	References :: getVolume() const {
		return referenceVector_.getVolume();
	}
	Size_t
	References :: getSizeOf() const {
		return sizeof (References);
	}
}
}
}

#endif /*MDL_FORM_PROOF_REFERENCES_CPP_*/
