/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File Name:       mdl_form_proof_References.hpp                            */
/* Description:     vector of references to the proof nodes                  */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_FORM_PROOF_REFERENCES_HPP_
#define MDL_FORM_PROOF_REFERENCES_HPP_

#include "mdl/interface/mdl_interface.hpp"

namespace mdl {
namespace form {
namespace proof {

class References :
	public mdl :: proof :: References,
	public Scalar<References> {
private :
	enum { INITIAL_REFERENCES_CAPACITY = 8 };
public :
	References (const index :: Arity = INITIAL_REFERENCES_CAPACITY);
	References (const vector :: parser :: Reference&);
	virtual ~ References();

	// proof :: References interface
	virtual index :: Arity getArity() const;
	virtual const mdl :: proof :: Reference* get (const index :: Arity) const;
	virtual void build (const mdl :: proof :: Scope* const);
	virtual void add (const mdl :: proof :: Reference* const);
	virtual index :: Arity find (const mdl :: proof :: Node*) const;

	// object :: Writable interface
	virtual void translate (String&) const;
	virtual void replicate (String&) const;

	// object :: Object interface
	virtual void commitSuicide();
	virtual Size_t getVolume() const;
	virtual Size_t getSizeOf() const;

private :
	const vector :: proof :: Reference referenceVector_;
};

}
}
}

#endif /*MDL_FORM_PROOF_REFERENCES_HPP_*/
