/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File Name:       mdl_form_proof_Step.hpp                                  */
/* Description:     proof step                                               */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_FORM_PROOF_STEP_HPP_
#define MDL_FORM_PROOF_STEP_HPP_

#include "mdl/interface/mdl_interface.hpp"

namespace mdl {
namespace form {
namespace proof {

class Step : virtual public mdl :: proof :: Step {
public :
	Step
	(
		const mdl :: proof :: Line* const,
		const Location&,
		const mdl :: Comments* const,
		const index :: Step,
		const mdl :: Identificator* const,
		const array :: Expression* const,
		const mdl :: proof :: Scope* = NULL
	);
	Step (const mdl :: proof :: Step* const);
	Step
	(
		const mdl :: proof :: Step* const,
		const index :: Step,
		const array :: Expression* const
	);
	virtual ~ Step();

	// proof :: Step interface
	virtual Class getClass() const = 0;
	virtual const mdl :: proof :: Question* questionSelf() const;
	virtual void addReference (const mdl :: proof :: Reference* const) = 0;
	virtual const Location& getLocation() const;

	// proof :: Node interface
	virtual Kind getKind() const;
	virtual index :: Step getIndex() const;
	virtual void replicateReference (String&) const = 0;
	virtual void translateReference (String&) const = 0;

	// proof :: Line interface
	virtual Sort getSort() const;
	virtual const mdl :: proof :: Line* getNext() const;
	virtual const mdl :: proof :: Line* getPrev() const;
	virtual void incIndex (const index :: Step) const;
	virtual void decIndex (const index :: Step) const;
	virtual void setNext (const mdl :: proof :: Line* const) const;
	virtual void setPrev (const mdl :: proof :: Line* const) const;
	virtual void completeWithSteps (mdl :: prover :: tree :: Premise*) const;

	// object :: Scoping interface
	virtual const mdl :: proof :: Scope* buildScope (const mdl :: proof :: Scope* const);
	virtual const mdl :: proof :: Scope* getScope() const;
	virtual void setScope (const mdl :: proof :: Scope*) const;

	// object :: Expressive interface
	virtual const array :: Expression* getExpression() const;
	virtual const mdl :: Type* getType() const;

	// object :: Verifiable interface
	virtual void check() const = 0;
	virtual void compress() = 0;
	virtual void prove() = 0;
	virtual bool isVerified() const;

	// object :: Buildable interface
	virtual void buildTheory (mdl :: Theory* const);

	// object :: Writable interface
	virtual void translate (String&) const = 0;
	virtual void replicate (String&) const = 0;

	// object :: Object interface
	virtual void commitSuicide() = 0;
	virtual Size_t getVolume() const;
	virtual Size_t getSizeOf() const;

protected :
	const Location location_;
	const mdl :: Comments* const comments_;
	mutable index :: Step index_;
	const mdl :: Identificator* const identificator_;
	const mdl :: Type* type_;
	const array :: Expression* const expression_;
	mutable const mdl :: proof :: Line* next_;
	mutable const mdl :: proof :: Line* prev_;
	mutable const mdl :: proof :: Scope* scope_;
	mutable bool isVerified_;
	mutable index :: Step indexGlobal_;
};

}
}
}

#endif /*MDL_FORM_PROOF_STEP_HPP_*/
