/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File Name:       mdl_form_proof_Variable.hpp                              */
/* Description:     variable declaration in proof                            */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_FORM_PROOF_VARIABLE_HPP_
#define MDL_FORM_PROOF_VARIABLE_HPP_

#include "mdl/interface/mdl_interface.hpp"

namespace mdl {
namespace form {
namespace proof {

class Variable :
	public mdl :: proof :: Variable,
	public Scalar<Variable> {
public :
	Variable
	(
		const mdl :: proof :: Line* const,
		const Location&,
		const mdl :: Comments* const,
		const mdl :: header :: Variable* const
	);
	virtual ~ Variable();

	// object :: Typing interface
	virtual const mdl :: Type* getType (const value  :: Literal) const;
	virtual const mdl :: header :: Variable* getDeclaration (const value :: Literal) const;
	virtual const object :: Typing* getPrevious() const;
	virtual void setNext (const object :: Typing*) const;
	virtual void translateVariables (String&) const;
	virtual void translateFloatings (String&, const index :: Arity = 0) const;

	// proof :: Line interface
	virtual Sort getSort() const;
	virtual const mdl :: proof :: Line* getNext() const;
	virtual const mdl :: proof :: Line* getPrev() const;
	virtual void incIndex (const index :: Step) const;
	virtual void decIndex (const index :: Step) const;
	virtual void setNext (const mdl :: proof :: Line* const) const;
	virtual void setPrev (const mdl :: proof :: Line* const) const;
	virtual void completeWithSteps (mdl :: prover :: tree :: Premise*) const;

	// object :: Scoping interface
	virtual const mdl :: proof :: Scope* buildScope (const mdl :: proof :: Scope* const);
	virtual const mdl :: proof :: Scope* getScope() const;
	virtual void setScope (const mdl :: proof :: Scope*) const;

	// object :: Verifiable interface
	virtual void check() const;
	virtual void compress();
	virtual void prove();
	virtual bool isVerified() const;

	// object :: Buildable interface
	virtual void buildTheory (mdl :: Theory* const);

	// object :: Writable interface
	virtual void translate (String&) const;
	virtual void replicate (String&) const;

	// object :: Object interface
	virtual void commitSuicide();
	virtual Size_t getVolume() const;
	virtual Size_t getSizeOf() const;

	class Declaration;

private :
	const Location location_;
	const mdl :: Comments* const comments_;
	const mdl :: header :: Variable* const variable_;
	mutable const mdl :: proof :: Scope* scope_;
	const object :: Typing* previous_;
	mutable const Line* nextLine_;
	mutable const Line* prevLine_;
	mutable const object :: Typing* next_;
};

}
}
}

#include "mdl/form/proof/mdl_form_proof_Variable_Declaration.hpp"

#endif /*MDL_FORM_PROOF_VARIABLE_HPP_*/
