/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File Name:       mdl_form_proof_node_Proposition.cpp                      */
/* Description:     proposition as a proof node                              */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_FORM_PROOF_NODE_PROPOSITION_CPP_
#define MDL_FORM_PROOF_NODE_PROPOSITION_CPP_

#include "mdl/form/mdl_form.hpp"

namespace mdl {
namespace form {
namespace proof {
namespace node {

	/****************************
	 *		Public members
	 ****************************/

	inline
	Proposition :: Proposition
	(
		const mdl :: Assertion* const assertion,
		const index :: Arity index,
		const array :: Expression* const expression
	) :
	Statement (assertion, index, expression) {
	}
	Proposition :: ~ Proposition() {
	}

	// mdl :: proof :: Node implementation
	Proposition :: Kind
	Proposition :: getKind() const {
		return PROP;
	}
	index :: Step
	Proposition :: getIndex() const {
		return Statement :: getIndex();
	}
	void
	Proposition :: replicateReference (String& string) const {
		Statement :: replicateReference (string);
	}
	void
	Proposition :: translateReference (String& string) const {
		Statement :: translateReference (string);
	}

	// object :: Scoping implementation
	const mdl :: proof :: Scope*
	Proposition :: buildScope (const mdl :: proof :: Scope* const scope) {
		return scope;
	}
	const mdl :: proof :: Scope*
	Proposition :: getScope() const {
		return Statement :: getScope();
	}
	void
	Proposition :: setScope (const mdl :: proof :: Scope* scope) const {
		Statement :: setScope (scope);
	}

	// object :: Expressive interface
	const array :: Expression*
	Proposition :: getExpression() const {
		return Statement :: getExpression();
	}
	const mdl :: Type*
	Proposition :: getType() const {
		return Statement :: getType();
	}

	// object :: Writable interface
	void
	Proposition :: translate (String& string) const {
		Statement :: translate (string);
	}
	void
	Proposition :: replicate (String& string) const {
		Statement :: replicate (string);
	}

	// object :: Object interface
	void
	Proposition :: commitSuicide() {
		delete this;
	}
	Size_t
	Proposition :: getVolume() const {
		return Statement :: getVolume ();
	}
	Size_t
	Proposition :: getSizeOf() const {
		return sizeof (Proposition);
	}
}
}
}
}

#endif /*MDL_FORM_PROOF_NODE_PROPOSITION_CPP_*/
