/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File Name:       mdl_form_proof_node_Provable.hpp                         */
/* Description:     provable proof node                                      */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_FORM_PROOF_NODE_PROVABLE_HPP_
#define MDL_FORM_PROOF_NODE_PROVABLE_HPP_

#include "mdl/form/proof/node/mdl_form_proof_node_Statement.hpp"

namespace mdl {
namespace form {
namespace proof {
namespace node {

class Provable :
	virtual public Statement,
	virtual public mdl :: proof :: Provable,
	public Scalar<Provable> {
public :
	Provable
	(
		const mdl :: Assertion* const,
		const index :: Arity,
		const array :: Expression* const
	);
	virtual ~ Provable();

	// proof :: Provable interface
	virtual void setProof (const mdl :: proof :: Node* const) const;

	// proof :: Node interface
	virtual Kind getKind() const;
	virtual index :: Step getIndex() const;
	virtual void replicateReference (String&) const;
	virtual void translateReference (String&) const;

	// object :: Scoping interface
	virtual const mdl :: proof :: Scope* buildScope (const mdl :: proof :: Scope* const);
	virtual const mdl :: proof :: Scope* getScope() const;
	virtual void setScope (const mdl :: proof :: Scope*) const;

	// object :: Expressive interface
	virtual const array :: Expression* getExpression() const;
	virtual const mdl :: Type* getType() const;

	// object :: Writable interface
	virtual void translate (String&) const;
	virtual void replicate (String&) const;

	// object :: Object interface
	virtual void commitSuicide();
	virtual Size_t getVolume() const;
	virtual Size_t getSizeOf() const;

private :
	mutable const mdl :: proof :: Node* proof_;
};

}
}
}
}

#endif /*MDL_FORM_PROOF_NODE_PROVABLE_HPP_*/
