/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File Name:       mdl_form_proof_node_Statement.cpp                        */
/* Description:     statement as a base class for proof node                 */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_FORM_PROOF_NODE_STATEMENT_CPP_
#define MDL_FORM_PROOF_NODE_STATEMENT_CPP_

#include "mdl/form/mdl_form.hpp"

namespace mdl {
namespace form {
namespace proof {
namespace node {

	/****************************
	 *		Public members
	 ****************************/

	inline
	Statement :: Statement
	(
		const mdl :: Assertion* const assertion,
		const index :: Arity index,
		const array :: Expression* const expression
	) :
	assertion_ (assertion),
	index_ (index),
	expression_ (new array :: Expression (*expression, false)),
	scope_ (NULL) {
	}
	Statement :: ~ Statement() {
		deleteAggregateObject (expression_);
	}

using manipulator :: endLine;
using manipulator :: space;
using manipulator :: tab;
using manipulator :: underline;

	// mdl :: proof :: Node implementation
	index :: Step
	Statement :: getIndex() const {
		return index_;
	}
	void
	Statement :: replicateReference (String& string) const
	{
		string << Token :: prop_;
		if (assertion_->getPropArity() > 1) {
			string << space << (index_ + 1) << space;
		} else {
			string << space;
		}
	}
	void
	Statement :: translateReference (String& string) const {
		assertion_->translateName (string, index_);
	}

	// object :: Scoping implementation
	const mdl :: proof :: Scope*
	Statement :: buildScope (const mdl :: proof :: Scope* const scope) {
		return scope;
	}
	const mdl :: proof :: Scope*
	Statement :: getScope() const {
		return scope_;
	}
	void
	Statement :: setScope (const mdl :: proof :: Scope* scope) const {
		scope_ = scope;
	}

	// object :: Expressive interface
	const array :: Expression*
	Statement :: getExpression() const {
		return expression_;
	}
	const mdl :: Type*
	Statement :: getType() const {
		return assertion_->getProposition (index_ - 1)->getType();
	}

	// object :: Writable interface
	void
	Statement :: translate (String& string) const {
		translateReference (string);
	}
	void
	Statement :: replicate (String& string) const {
		replicateReference (string);
	}

	// object :: Object interface
	Size_t
	Statement :: getVolume() const {
		return getAggregateVolume (expression_);
	}
	Size_t
	Statement :: getSizeOf() const {
		return sizeof (Statement);
	}
}
}
}
}

#endif /*MDL_FORM_PROOF_NODE_STATEMENT_CPP_*/
