/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File Name:       mdl_form_proof_notation_Notation.cpp                     */
/* Description:     notation class                                           */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_FORM_PROOF_NOTATION_NOTATION_CPP_
#define MDL_FORM_PROOF_NOTATION_NOTATION_CPP_

#include "mdl/math/mdl_math.hpp"
#include "mdl/form/mdl_form.hpp"

namespace mdl {
namespace form {
namespace proof {
namespace notation {

	/****************************
	 *		Public members
	 ****************************/

	inline
	Notation :: Notation
	(
		const mdl :: proof :: Line* const prev,
		const Location& location,
		const value :: Literal literal,
		const mdl :: Variables* const variables,
		const mdl :: proof :: Term* const abbreviation,
		const mdl :: proof :: Term* const meaning
	) :
	location_ (location),
	literal_ (literal),
	variables_ (variables),
	abbreviation_ (abbreviation),
	meaning_ (meaning),
	next_ (NULL),
	prev_ (prev),
	scope_ (NULL)
	{
		if (prev != NULL) {
			prev->setNext (this);
		}
	}
	Notation :: ~ Notation()
	{
		deleteAggregateObject (variables_);
		deleteAggregateObject (abbreviation_);
		deleteAggregateObject (meaning_);
		deleteAggregateObject (next_);
		deleteAggregateObject (scope_);
	}

	// mdl :: proof :: Notation implementation
	value :: Literal
	Notation :: getLiteral() const {
		return literal_;
	}
	const mdl :: Variables*
	Notation :: getVariables() const {
		return variables_;
	}
	const mdl :: proof :: Term*
	Notation :: getAbbreviation() const {
		return abbreviation_;
	}
	const mdl :: proof :: Term*
	Notation :: getMeaning() const {
		return meaning_;
	}

	// mdl :: proof :: Line implementation
	Notation :: Sort
	Notation :: getSort() const {
		return NOTATION;
	}
	const mdl :: proof :: Line*
	Notation :: getNext() const {
		return next_;
	}
	const mdl :: proof :: Line*
	Notation :: getPrev() const {
		return prev_;
	}
	void
	Notation :: incIndex (const index :: Step) const {
	}
	void
	Notation :: decIndex (const index :: Step) const {
	}
	void
	Notation :: setNext (const mdl :: proof :: Line* const line) const {
		next_ = line;
	}
	void
	Notation :: setPrev (const mdl :: proof :: Line* const line) const {
		prev_ = line;
	}
	void
	Notation :: completeWithSteps (mdl :: prover :: tree :: Premise*) const {
	}

	// object :: Scoping implementation
	const mdl :: proof :: Scope*
	Notation :: buildScope (const mdl :: proof :: Scope* const scope)
	{
		scope_ = new proof :: scope :: Notation (scope, this);
		return scope_;
	}
	const mdl :: proof :: Scope*
	Notation :: getScope() const {
		return scope_;
	}
	void
	Notation :: setScope (const mdl :: proof :: Scope* scope) const {
		scope_ = scope;
	}

	// object :: Verifiable implementation
	void
	Notation :: check() const {
	}
	void
	Notation :: compress() {
	}
	void
	Notation :: prove() {
	}
	bool
	Notation :: isVerified() const {
		return true;
	}

	// object :: Buildable implementation
	void
	Notation :: buildTheory (mdl :: Theory* const theory)
	{
		const_cast<mdl :: Variables* const>(variables_)->buildTheory (theory);
		const_cast<mdl :: proof :: Term* const>(abbreviation_)->buildTheory (theory);
		const_cast<mdl :: proof :: Term* const>(meaning_)->buildTheory (theory);
	}

using manipulator :: space;
using manipulator :: underline;
using manipulator :: endLine;

	// object :: Writable implementation
	void
	Notation :: translate (String&) const {
	}
	void
	Notation :: replicate (String& string) const
	{
		string << location_.indent() << Token :: notation_ << space;
		string << Table :: literals()->get (literal_) << space;
		variables_->replicate (string);
		string << Token :: openBrace_ << endLine;
		abbreviation_->replicate (string);
		meaning_->replicate (string);
		string << Token :: closeBrace_ << endLine;
	}

	// object :: Object implementation
	void
	Notation :: commitSuicide() {
		delete this;
	}
	Size_t
	Notation :: getVolume() const
	{
		Size_t volume = 0;
		volume += getAggregateVolume (variables_);
		volume += getAggregateVolume (abbreviation_);
		volume += getAggregateVolume (meaning_);

		volume += getAggregateVolume (next_);
		volume += getAggregateVolume (scope_);
		return volume;
	}
	Size_t
	Notation :: getSizeOf() const {
		return sizeof (Notation);
	}
}
}
}
}

#endif /*MDL_FORM_PROOF_NOTATION_NOTATION_CPP_*/
