/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File Name:       mdl_form_proof_notation_Notation.hpp                     */
/* Description:     notation class                                           */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_FORM_PROOF_NOTATION_NOTATION_HPP_
#define MDL_FORM_PROOF_NOTATION_NOTATION_HPP_

#include "mdl/interface/mdl_interface.hpp"

namespace mdl {
namespace form {
namespace proof {
namespace notation {

class Notation :
	public mdl :: proof :: Notation,
	public Scalar<Notation> {
public :
	Notation
	(
		const mdl :: proof :: Line* const,
		const Location&,
		const value :: Literal,
		const mdl :: Variables* const,
		const mdl :: proof :: Term* const,
		const mdl :: proof :: Term* const
	);
	virtual ~ Notation();

	// mdl :: proof :: Notation interface
	virtual value :: Literal getLiteral() const;
	virtual const mdl :: Variables* getVariables() const;
	virtual const mdl :: proof :: Term* getAbbreviation() const;
	virtual const mdl :: proof :: Term* getMeaning() const;

	// mdl :: proof :: Line interface
	virtual Sort getSort() const;
	virtual const mdl :: proof :: Line* getNext() const;
	virtual const mdl :: proof :: Line* getPrev() const;
	virtual void incIndex (const index :: Step) const;
	virtual void decIndex (const index :: Step) const;
	virtual void setNext (const mdl :: proof :: Line* const) const;
	virtual void setPrev (const mdl :: proof :: Line* const) const;
	virtual void completeWithSteps (mdl :: prover :: tree :: Premise*) const;

	// object :: Scoping interface
	virtual const mdl :: proof :: Scope* buildScope (const mdl :: proof :: Scope* const);
	virtual const mdl :: proof :: Scope* getScope() const;
	virtual void setScope (const mdl :: proof :: Scope*) const;

	// object :: Verifiable interface
	virtual void check() const;
	virtual void compress();
	virtual void prove();
	virtual bool isVerified() const;

	// object :: Buildable interface
	virtual void buildTheory (mdl :: Theory* const);

	// object :: Writable interface
	virtual void translate (String&) const;
	virtual void replicate (String&) const;

	// object :: Object interface
	virtual void commitSuicide();
	virtual Size_t getVolume() const;
	virtual Size_t getSizeOf() const;

private :
	const Location location_;
	const value :: Literal literal_;
	const mdl :: Variables* const variables_;
	const mdl :: proof :: Term* const abbreviation_;
	const mdl :: proof :: Term* const meaning_;

	mutable const mdl :: proof :: Line* next_;
	mutable const mdl :: proof :: Line* prev_;
	mutable const mdl :: proof :: Scope* scope_;
};

}
}
}
}

#endif /*MDL_FORM_PROOF_NOTATION_NOTATION_HPP_*/
