/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File Name:       mdl_form_proof_notation_Term.hpp                         */
/* Description:     notation term class                                      */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_FORM_PROOF_NOTATION_TERM_HPP_
#define MDL_FORM_PROOF_NOTATION_TERM_HPP_

#include "mdl/interface/mdl_interface.hpp"

namespace mdl {
namespace form {
namespace proof {
namespace notation {

class Term :
	public mdl :: proof :: Term,
	public Scalar<Term> {
public :
	Term
	(
		const Location&,
		const mdl :: Identificator* const,
		const array :: Expression* const
	);
	virtual ~ Term();

	// object :: Expressive interface
	virtual const array :: Expression* getExpression() const;
	virtual const mdl :: Type* getType() const;

	// object :: Buildable interface
	virtual void buildTheory (mdl :: Theory* const);

	// object :: Writable interface
	virtual void translate (String&) const;
	virtual void replicate (String&) const = 0;

	// object :: Object interface
	virtual void commitSuicide() = 0;
	virtual Size_t getVolume() const;
	virtual Size_t getSizeOf() const;

protected :
	const Location location_;
	const mdl :: Identificator* const identificator_;
	const mdl :: Type* type_;
	const array :: Expression* const expression_;
};

}
}
}
}

#endif /*MDL_FORM_PROOF_NOTATION_TERM_HPP_*/
