/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File Name:       mdl_form_proof_scope_Notation.cpp                        */
/* Description:     scope which contains notation                            */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_FORM_PROOF_SCOPE_NOTATION_CPP_
#define MDL_FORM_PROOF_SCOPE_NOTATION_CPP_

#include "mdl/form/mdl_form.hpp"

namespace mdl {
namespace form {
namespace proof {
namespace scope {

	/****************************
	 *		Public members
	 ****************************/

	inline
	Notation :: Notation
	(
		const mdl :: proof :: Scope* const scope,
		const mdl :: proof :: Notation* const notation
	) :
	Ancestor (scope),
	notation_ (notation) {
	}
	Notation :: ~ Notation() {
	}

	// proof :: Scope implementation
	const mdl :: Assertion*
	Notation :: getAssertion () const {
		return scope_->getAssertion();
	}
	const mdl :: proof :: Claim*
	Notation :: getClaim() const {
		return scope_->getClaim();
	}
	const mdl :: proof :: Node*
	Notation :: getHypothesis (const index :: Arity index) const {
		return scope_->getHypothesis (index);
	}
	const mdl :: proof :: Node*
	Notation :: getProposition (const index :: Arity index) const {
		return scope_->getProposition (index);
	}
	const object :: Typing*
	Notation :: getVariables() const {
		return scope_->getVariables();
	}
	const mdl :: proof :: Notation*
	Notation :: getNotation (const value :: Literal literal) const
	{
		if (notation_->getLiteral() == literal) {
			return notation_;
		}
		return scope_->getNotation (literal);
	}
	const mdl :: proof :: Step*
	Notation :: getStep (const index :: Step index) const {
		return scope_->getStep (index);
	}

	// object :: Object implementation
	void
	Notation :: commitSuicide() {
		delete this;
	}
	Size_t
	Notation :: getVolume() const {
		return 0;
	}
	Size_t
	Notation :: getSizeOf() const {
		return sizeof (Notation);
	}
}
}
}
}

#endif /*MDL_FORM_PROOF_SCOPE_NOTATION_CPP_*/
