/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File Name:       mdl_form_proof_step_Claim.hpp                            */
/* Description:     step with claim inference                                */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_FORM_PROOF_STEP_CLAIM_HPP_
#define MDL_FORM_PROOF_STEP_CLAIM_HPP_

#include "mdl/interface/mdl_interface.hpp"
#include "mdl/form/proof/mdl_form_proof_Step.hpp"

namespace mdl {
namespace form {
namespace proof {
namespace step {

class Claim :
	virtual public proof :: Step,
	virtual public mdl :: proof :: Claim,
	public Scalar<Claim> {
	typedef
		proof :: Step
		Ancestor_;
public :
	Claim
	(
		const mdl :: proof :: Line* const,
		const Location&,
		const mdl :: Comments* const,
		const index :: Step,
		const mdl :: Identificator* const,
		const mdl :: proof :: Link* const,
		const array :: Expression* const,
		const mdl :: Proof* const
	);
	Claim
	(
		const mdl :: proof :: Step*,
		const index :: Step
	);
	virtual ~ Claim();

	// proof :: Claim interface
	virtual const mdl :: Proof* getProof() const;
	virtual void completeWithHypotheses (mdl :: prover :: tree :: Premise*) const;
	virtual void addStep (const mdl :: proof :: Step* const);
	virtual void addVariable (const Symbol&);
	virtual const mdl :: proof :: Link* getLink() const;
	virtual mdl :: proof :: Link* link();

	// proof :: Provable interface
	virtual void setProof (const mdl :: proof :: Node* const) const;

	// proof :: Step interface
	virtual Class getClass() const;
	virtual void addReference (const mdl :: proof :: Reference* const);

	// proof :: Node interface
	virtual void replicateReference (String&) const;
	virtual void translateReference (String&) const;

	// proof :: Line interface
	virtual const mdl :: proof :: Line* getNext() const;
	virtual const mdl :: proof :: Line* getPrev() const;
	virtual void incIndex (const index :: Step) const;
	virtual void decIndex (const index :: Step) const;
	virtual void setNext (const mdl :: proof :: Line* const) const;
	virtual void setPrev (const mdl :: proof :: Line* const) const;
	virtual void completeWithSteps (mdl :: prover :: tree :: Premise*) const;

	// object :: Scoping interface
	virtual const mdl :: proof :: Scope* buildScope (const mdl :: proof :: Scope* const);
	virtual const mdl :: proof :: Scope* getScope() const;
	virtual void setScope (const mdl :: proof :: Scope*) const;

	// object :: Expressive interface
	virtual const array :: Expression* getExpression() const;
	virtual const mdl :: Type* getType() const;

	// object :: Verifiable interface
	virtual void check() const;
	virtual void compress();
	virtual void prove();

	// object :: Buildable interface
	virtual void buildTheory (mdl :: Theory* const);

	// object :: Writable interface
	virtual void translate (String&) const;
	virtual void replicate (String&) const ;

	// object :: Object interface
	virtual void commitSuicide();
	virtual Size_t getVolume() const;
	virtual Size_t getSizeOf() const;

private :
	const mdl :: proof :: Link* const link_;
	const mdl :: Proof* const proof_;
	mutable const mdl :: proof :: Node* node_;
	mutable const mdl :: proof :: Scope* scope_;
};

}
}
}
}

#endif /*MDL_FORM_PROOF_STEP_CLAIM_HPP_*/
