/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File Name:       mdl_form_proof_guess_Question.hpp                         */
/* Description:     unknown step inference                                   */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_FORM_PROOF_STEP_QUESTION_HPP_
#define MDL_FORM_PROOF_STEP_QUESTION_HPP_

#include "mdl/interface/mdl_interface.hpp"
#include "mdl/form/proof/mdl_form_proof_Step.hpp"

namespace mdl {
namespace form {
namespace proof {
namespace step {

class Question :
	public mdl :: proof :: Question,
	public proof :: Step,
	public Scalar<Question> {
	typedef proof :: Step Ancestor_;
public :
	Question
	(
		const mdl :: proof :: Line* const,
		const Location&,
		const mdl :: Comments* const,
		const index :: Step,
		const mdl :: Identificator* const,
		const array :: Expression* const,
		const mdl :: proof :: Scope* = NULL
	);
	Question (const proof :: Step*);
	virtual ~ Question();

	// proof :: step :: Question interface
	virtual bool isProved() const;
	virtual void confirmCompression();
	virtual void rejectCompression();
	virtual const mdl :: proof :: Step* getAnswer() const;
	virtual const mdl :: proof :: Step* getGuess() const;

	// proof :: Step interface
	virtual Class getClass() const;
	virtual void addReference (const mdl :: proof :: Reference* const);

	// proof :: Node interface
	virtual void replicateReference (String&) const;
	virtual void translateReference (String&) const;

	// proof :: Line interface
	virtual const mdl :: proof :: Line* getNext() const;
	virtual const mdl :: proof :: Line* getPrev() const;
	virtual void incIndex (const index :: Step) const;
	virtual void decIndex (const index :: Step) const;
	virtual void setNext (const mdl :: proof :: Line* const) const;
	virtual void setPrev (const mdl :: proof :: Line* const) const;
	virtual void completeWithSteps (mdl :: prover :: tree :: Premise*) const;

	// object :: Scoping interface
	virtual const mdl :: proof :: Scope* buildScope (const mdl :: proof :: Scope* const);
	virtual const mdl :: proof :: Scope* getScope() const;
	virtual void setScope (const mdl :: proof :: Scope*) const;

	// object :: Expressive interface
	virtual const array :: Expression* getExpression() const;
	virtual const mdl :: Type* getType() const;

	// object :: Verifiable interface
	virtual void check() const;
	virtual void compress();
	virtual void prove();

	// object :: Buildable interface
	virtual void buildTheory (mdl :: Theory* const);

	// object :: Writable interface
	virtual void translate (String&) const;
	virtual void replicate (String&) const ;

	// object :: Object interface
	virtual void commitSuicide();
	virtual Size_t getVolume() const;
	virtual Size_t getSizeOf() const;

private :
	friend class form :: proof :: Step;
	bool isProved_;
	const mdl :: proof :: Step* answer_;
	const mdl :: proof :: Step* guess_;
};

}
}
}
}

#endif /*MDL_FORM_PROOF_STEP_QUESTION_HPP_*/
