/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File Name:       mdl_prover_interface_Proofs.hpp                          */
/* Description:     interface for a collection of proofs                     */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_INTERFACE_PROVER_PROOFS_HPP_
#define MDL_INTERFACE_PROVER_PROOFS_HPP_

#include "mdl/interface/mdl_interface.dpp"

namespace mdl {
namespace interface {
namespace prover {

class Confirmation : public object :: Object {
public :
	virtual void verifyIntegrity() const = 0;
	virtual bool boundaryMoved() const = 0;
	virtual void markBoundary() = 0;
	virtual value :: Integer getBoundary() const = 0;
	virtual value :: Integer getSize() const = 0;
	virtual value :: Integer getDelta() const = 0;
	virtual Evidence* supply (const mdl :: prover :: stack :: Substitution*) = 0;
	virtual void add (const Evidence*) = 0;
	virtual void replace (const Evidence*, const value :: Integer) = 0;
	virtual const Evidence* get (const value :: Integer) const = 0;
	virtual const Evidence* getNew (const value :: Integer) const = 0;

	virtual Size_t getEvidenceVolume() const = 0;
	virtual Size_t getSubstitutionVolume() const = 0;
	virtual Size_t getEvidenceNodeVolume() const = 0;
	virtual Size_t getGarbageVolume() const = 0;
};

}
}
}

#endif /*MDL_INTERFACE_PROVER_PROOFS_HPP_*/
