/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File Name:       mdl_prover_interface_DotEvidence.hpp                     */
/* Description:     class for dotted evidence: evidence and a boolean flag   */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_INTERFACE_PROVER_DOT_EVIDENCE_HPP_
#define MDL_INTERFACE_PROVER_DOT_EVIDENCE_HPP_

#include "mdl/interface/mdl_interface.dpp"

namespace mdl {
namespace interface {
namespace prover {

class DotEvidence :
	public object :: Object,
	public Scalar<DotEvidence, allocator :: Prover>  {
public :
	DotEvidence ();
	DotEvidence (const DotEvidence&);
	DotEvidence (const mdl :: prover :: Evidence*, const bool);
	virtual ~ DotEvidence();

	bool operator < (const DotEvidence&) const;
	bool operator > (const DotEvidence&) const;
	bool operator == (const DotEvidence&) const;
	bool operator != (const DotEvidence&) const;
	void operator = (const DotEvidence&);

	const Evidence* getEvidence() const;
	bool isNew() const;

	// object :: Object interface
	virtual void commitSuicide();
	virtual Size_t getVolume() const;
	virtual Size_t getSizeOf() const;
	virtual void show (String&) const;

private :
	const Evidence* evidence_;
	bool isNew_;
};

}
}
}

#endif /*MDL_INTERFACE_PROVER_DOT_EVIDENCE_HPP_*/
