/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File Name:       mdl_prover_interface_DotEvidence.ipp                     */
/* Description:     class for dotted evidence: evidence and a boolean flag   */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_INTERFACE_PROVER_DOT_EVIDENCE_IPP_
#define MDL_INTERFACE_PROVER_DOT_EVIDENCE_IPP_

#include "mdl/interface/mdl_interface.dpp"

namespace mdl {
namespace interface {
namespace prover {

	/****************************
	 *		Public members
	 ****************************/

	inline
	DotEvidence :: DotEvidence() :
	evidence_ (NULL),
	isNew_ (false) {
	}
	inline
	DotEvidence :: DotEvidence (const DotEvidence& dot) :
	evidence_ (dot.evidence_),
	isNew_ (dot.isNew_) {
	}
	inline
	DotEvidence :: DotEvidence
	(
		const mdl :: prover :: Evidence* proof,
		const bool isNew
	) :
	evidence_ (proof),
	isNew_ (isNew) {
	}
	DotEvidence :: ~ DotEvidence() {
	}

	inline bool
	DotEvidence :: operator < (const DotEvidence& dot) const {
		return
			static_cast<const void*>(evidence_) < static_cast<const void*>(dot.evidence_);
	}
	inline bool
	DotEvidence :: operator > (const DotEvidence& dot) const {
		return
			static_cast<const void*>(evidence_) > static_cast<const void*>(dot.evidence_);
	}
	inline bool
	DotEvidence :: operator == (const DotEvidence& dot) const
	{
		return
			(evidence_ == dot.evidence_) &&
			(isNew_ == dot.isNew_);
	}
	inline bool
	DotEvidence :: operator != (const DotEvidence& dot) const {
		return !(operator == (dot));
	}
	inline void
	DotEvidence :: operator = (const DotEvidence& dot)
	{
		evidence_ = dot.evidence_;
		isNew_ = dot.isNew_;
	}

	inline const Evidence*
	DotEvidence :: getEvidence () const {
		return evidence_;
	}
	inline bool
	DotEvidence :: isNew () const {
		return isNew_;
	}

using manipulator :: endLine;
using manipulator :: space;
using manipulator :: tab;

	// object :: Object implementation
	void
	DotEvidence :: commitSuicide() {
		delete this;
	}
	Size_t
	DotEvidence :: getVolume() const {
		return 0;
	}
	Size_t
	DotEvidence :: getSizeOf() const {
		return sizeof (DotEvidence);
	}
	void
	DotEvidence :: show (String& string) const
	{
		if (isNew_) {
			string << "new";
		} else {
			string << "old";
		}
		string << space << ":" << space;
		string << evidence_ << tab;
	}
}
}
}

#endif /*MDL_INTERFACE_PROVER_DOT_EVIDENCE_IPP_*/
