/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File Name:       mdl_prover_interface_Evidence.hpp                        */
/* Description:     interface for evidence (semiproof) in variant tree       */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_INTERFACE_PROVER_EVIDENCE_HPP_
#define MDL_INTERFACE_PROVER_EVIDENCE_HPP_

#include "mdl/interface/mdl_interface.dpp"

namespace mdl {
namespace interface {
namespace prover {

class Evidence : public object :: Cloneable<Evidence> {
public :
	enum Kind {
		HYP,
		PROP,
		REF,
		TOP,
		SUB
	};
	virtual value :: Kind getKind() const = 0;
	virtual const Node* getNode() const = 0;
	virtual void verifyIntegrity() const = 0;
	virtual const mdl :: prover :: Substitution* getSubstitution() const = 0;
	virtual index :: Step getDepth() const = 0;
	virtual value :: Integer getCardinality() const = 0;
	virtual const Proof* constructProof (const mdl :: prover :: Substitution* = NULL) const = 0;
	virtual mdl :: prover :: vector :: Proof constructProofs (const mdl :: prover :: Substitution* = NULL) const = 0;
};

}
}
}

#endif /*MDL_INTERFACE_PROVER_EVIDENCE_HPP_*/
