/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File Name:       mdl_prover_interface_Node.hpp                            */
/* Description:     interface for a node of the variant tree                 */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_INTERFACE_PROVER_NODE_HPP_
#define MDL_INTERFACE_PROVER_NODE_HPP_

#include "mdl/interface/prover/mdl_interface_prover_Statistics.hpp"

namespace mdl {
namespace interface {
namespace prover {

class Node :
	public Statistics,
	public object :: Cloneable<Node> {
public :
	virtual const mdl :: prover :: Expression* getExpression() const = 0;
	virtual const mdl :: Proposition* getProposition() const = 0;
	virtual const Confirmation& getConfirmation() const = 0;
	virtual value :: Weight getWeight() const = 0;
	virtual void calculateWeightUp() const = 0;
	virtual void calculateWeightDown() const = 0;
	virtual bool operator < (const Node&) const = 0;
	virtual value :: Rate getRate() const = 0;
	virtual void setRate (const value :: Rate) const = 0;
	virtual void searchProof() = 0;
	virtual bool isProved() const = 0;
	virtual void buildUp (Level* const) = 0;
	virtual void buildDown (const bool = true) = 0;
	virtual bool isRoot() const = 0;
	virtual void addDown (const Node*) = 0;
	virtual bool liesDown (const Node*) const = 0;

	virtual void showNode
	(
		String&,
		const value :: Integer = 0
	) const = 0;
	virtual void showTree
	(
		String&,
		const value :: Integer = 0,
		const value :: Integer = value :: undefined :: INTEGER
	) const = 0;
};

}
}
}

#endif /*MDL_INTERFACE_PROVER_NODE_HPP_*/
