/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File Name:       mdl_prover_interface_Statistics.hpp                      */
/* Description:     interface for a statistics of the variant tree           */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_INTERFACE_PROVER_STATISTICS_HPP_
#define MDL_INTERFACE_PROVER_STATISTICS_HPP_

#include "mdl/interface/mdl_interface.dpp"

namespace mdl {
namespace interface {
namespace prover {

class Statistics {
public :
	// general volume
	virtual Size_t getEvidenceVolume() const = 0;
	virtual Size_t getTreeVolume() const = 0;
	virtual Size_t getExpressionVolume() const = 0;
	virtual Size_t getTreeNodeVolume() const = 0;
	virtual Size_t getEvidenceNodeVolume() const = 0;
	virtual Size_t getEvidenceContainerVolume() const = 0;
	virtual Size_t getGarbageVolume() const = 0;

	// substitution volume
	virtual Size_t getSubstitutionAllVolume() const = 0;
	virtual Size_t getSubstitutionIntVolume() const = 0;
	virtual Size_t getSubstitutionExtVolume() const = 0;
	virtual Size_t getSubstitutionVolume() const = 0;

	// node count
	virtual value :: Integer getTreeNodeCount() const = 0;
	virtual value :: Integer getEvidenceNodeCount() const = 0;
	virtual value :: Integer getExpressionCount() const = 0;
	virtual value :: Integer getHypCount() const = 0;
	virtual value :: Integer getPropCount() const = 0;
	virtual value :: Integer getRefCount() const = 0;
};

}
}
}

#endif /*MDL_INTERFACE_PROVER_STATISTICS_HPP_*/
