/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File Name:       mdl_math_Assertions.cpp                                  */
/* Description:     global container for assertions                          */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_MATH_ASSERTIONS_CPP_
#define MDL_MATH_ASSERTIONS_CPP_

#include "mdl/auxiliary/mdl_auxiliary.hpp"
#include "mdl/math/mdl_math_Assertions.hpp"

namespace mdl {
namespace math {

	/****************************
	 *		Public members
	 ****************************/

	inline
	Assertions :: Assertions () :
	Ancestor_ (INITIAL_ASSERTION_VECTOR_CAPACITY),
	propositionTree_ (new forest :: Proposition()),
	axiomCount_ (0),
	definitionCount_ (0),
	theoremCount_ (0),
	problemCount_ (0) {
	}
	Assertions :: ~ Assertions () {
		deleteAggregateObject (propositionTree_);
	}

using manipulator :: endLine;
using manipulator :: tab;
using manipulator :: space;
	
	// object :: Object implementation
	void
	Assertions :: commitSuicide() {
		delete this;
	}
	Size_t
	Assertions :: getVolume() const
	{
		Size_t volume = 0;
		volume += Ancestor_ :: getVolume();
		volume += getAggregateVolume (propositionTree_);
		return volume;
	}
	Size_t
	Assertions :: getSizeOf() const {
		return sizeof (Assertions);
	}
	void
	Assertions :: show (String& string) const
	{
		string << tab << "axioms: " << tab << axiomCount_ << endLine;
		string << tab << "definitions: " << tab << definitionCount_ << endLine;
		string << tab << "theorems: " << tab<< theoremCount_ << endLine;
		string << tab << "problems: " << tab << problemCount_ << endLine;
	}
}
}

#endif /*MDL_MATH_ASSERTIONS_CPP_*/
