/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File Name:       mdl_math_Assertions.hpp                                  */
/* Description:     global container for assertions                          */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_MATH_ASSERTIONS_HPP_
#define MDL_MATH_ASSERTIONS_HPP_

#include "mdl/math/mdl_math_Container.hpp"

namespace mdl {
namespace math {

class Assertions :
	public Container<mdl :: Assertion, index :: Assertion>,
	public Scalar<Assertions> {
public :
	Assertions ();
	virtual ~ Assertions();
	
	index :: Assertion getSize() const;
	index :: Assertion add (const mdl :: Assertion* const);
	const mdl :: Assertion* get (const index :: Assertion) const;
	const forest :: Proposition* getPropositionTree() const;

	index :: Assertion axioms() const;
	index :: Assertion definitions() const;
	index :: Assertion theorems() const;
	index :: Assertion problems() const;
	index :: Assertion axiomatic() const;
	index :: Assertion provable() const;

	// object :: Object interface
	virtual void commitSuicide();
	virtual Size_t getVolume() const;
	virtual Size_t getSizeOf() const;
	virtual void show (String&) const;
	
private :
	typedef 
		Container<mdl :: Assertion, index :: Assertion>
		Ancestor_;

	forest :: Proposition* propositionTree_;

	index :: Assertion axiomCount_;
	index :: Assertion definitionCount_;
	index :: Assertion theoremCount_;
	index :: Assertion problemCount_;
	enum { 
		INITIAL_ASSERTION_VECTOR_CAPACITY = nstd :: KILOBYTE * 64
	};
};

}
}

#include "mdl/math/mdl_math_Assertions.ipp"

#endif /*MDL_MATH_ASSERTIONS_HPP_*/
