/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File Name:       mdl_math_Assertions.ipp                                  */
/* Description:     global container for assertions                          */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_MATH_ASSERTIONS_IPP_
#define MDL_MATH_ASSERTIONS_IPP_

#include "mdl/lexer/mdl_lexer.hpp"

namespace mdl {
namespace math {

	/****************************
	 *		Public members
	 ****************************/

	inline index :: Assertion
	Assertions :: getSize() const {
		return Ancestor_ :: getSize();
	}
	inline const mdl :: Assertion*
	Assertions :: get (const index :: Assertion index) const {
		return Ancestor_ :: getForm (index);
	}
	inline index :: Assertion
	Assertions :: add (const mdl :: Assertion* const assertion)
	{
		switch (assertion->getKind()) {
		case Token :: AXIOM :
			++ axiomCount_;      break;
		case Token :: DEFINITION :
			++ definitionCount_; break;
		case Token :: THEOREM :
			++ theoremCount_;    break;
		case Token :: PROBLEM :
			++ problemCount_;    break;
		default :                break;
		}
		if (Config :: useProver()) {
			for (index :: Arity i = 0; i < assertion->getPropArity(); ++ i) {
				const mdl :: Proposition*
					proposition = assertion->getProposition (i);
				const index :: Assertion
					sourceIndex = proposition->getSourceIndex();
				propositionTree_->addBranch
				(
					proposition->getExpression(),
					proposition,
					sourceIndex
				);
			}
		}
		return Ancestor_ :: addForm (assertion);
	}
	inline const forest :: Proposition*
	Assertions :: getPropositionTree() const {
		return propositionTree_;
	}

	inline index :: Assertion
	Assertions :: axioms() const {
		return axiomCount_;
	}
	inline index :: Assertion
	Assertions :: definitions() const {
		return definitionCount_;
	}
	inline index :: Assertion
	Assertions :: theorems() const {
		return theoremCount_;
	}
	inline index :: Assertion
	Assertions :: problems() const {
		return problemCount_;
	}
	inline index :: Assertion
	Assertions :: axiomatic() const {
		return axiomCount_ + definitionCount_;
	}
	inline index :: Assertion
	Assertions :: provable() const {
		return theoremCount_ + problemCount_;
	}
}
}

#endif /*MDL_MATH_ASSERTIONS_IPP_*/

