/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File Name:       mdl_math_Container.hpp                                   */
/* Description:     global container                                         */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_MATH_CONTAINER_HPP_
#define MDL_MATH_CONTAINER_HPP_

#include "mdl/interface/mdl_interface.hpp"

namespace mdl {
namespace math {

template<class T, class R = value :: Integer>
class Container : public object :: Object {
protected :
	typedef T Form_;
	typedef R Index_;
	typedef
		vector :: Vector
		<
			const Form_*,
			storage :: ByValue,
			allocator :: Heap
		>
		FormVector_;
	
	enum {
		INITIAL_FORM_VECTOR_CAPACITY = nstd :: KILOBYTE
	};
	
public :
	Container (const Index_ = INITIAL_FORM_VECTOR_CAPACITY);
	virtual ~ Container();
	
	Size_t getContentVolume() const;
	
	// object :: Object interface
	virtual void commitSuicide() = 0;
	virtual Size_t getVolume() const;
	virtual Size_t getSizeOf() const;
	virtual void show (String&) const;

protected :
	Index_ getSize() const;
	const Form_* getForm (const Index_) const;
	Index_ addForm (const Form_* const);

	FormVector_ formVector_;
};

}
}

#include "mdl/math/mdl_math_Container.ipp"

#endif /*MDL_MATH_CONTAINER_HPP_*/
