/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File Name:       mdl_math_Defiendums.cpp                                  */
/* Description:     global container for defiendum terms                     */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_MATH_DEFIENDUMS_CPP_
#define MDL_MATH_DEFIENDUMS_CPP_

namespace mdl {
namespace math {

	/****************************
	 *		Public members
	 ****************************/

	inline
	Defiendums :: Defiendums () :
	tree_ (new tree :: Term()),
	count_ (0) {
	}
	Defiendums :: ~ Defiendums () {
		deleteAggregateObject (tree_);
	}
	
	void
	Defiendums :: add (const mdl :: term :: Definition* const term) {
		tree_->addBranch (term->getExpression(), term);
		++ count_;
	}
	const mdl :: Term*
	Defiendums :: unifiesWith (const mdl :: term :: Definition* const term) const
	{
		Memory :: stack().push();
		Unify_ :: Result_
			result = tree_->unifyDirect<array :: Expression> (term->getExpression());
		Memory :: stack().pop();
		if (result.isEmpty()) {
			return NULL;
		} else {
			return result.getFirst()->getValue().getFirst()->getData().getValue();
		}
	}

	// object :: Object interface
	void
	Defiendums :: commitSuicide() {
		delete this;
	}
	Size_t
	Defiendums :: getVolume() const {
		return getAggregateVolume (tree_);
	}
	Size_t
	Defiendums :: getSizeOf() const {
		return sizeof (Defiendums);
	}
	void
	Defiendums :: show (String& string) const
	{
		//string << tab << "defiendums:    " << tab;
		//string << count_ << endLine;
	}
}
}

#endif /*MDL_MATH_DEFIENDUMS_CPP_*/
