/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File Name:       mdl_math_Math.cpp                                        */
/* Description:     all global containers for mdl language                   */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_MATH_MATH_CPP_
#define MDL_MATH_MATH_CPP_

#include "mdl/math/mdl_math.hpp"

namespace mdl {
namespace math {

	/****************************
	 *		Public members
	 ****************************/

	void
	Math :: release()
	{
		deleteAggregateObject (assertions_);
		deleteAggregateObject (constants_);
		deleteAggregateObject (defiendums_);
		deleteAggregateObject (rules_);
		deleteAggregateObject (theories_);
		deleteAggregateObject (types_);
	}

using manipulator :: endLine;
	
	// nstd :: Static implementation
	void
	Math :: initRaw()
	{
		assertions_ = new Assertions();
		constants_ = new Constants();
		defiendums_ = new Defiendums();
		rules_ = new Rules();
		theories_ = new Theories();
		types_ = new Types();
	}
	void
	Math :: showRaw (String& string)
	{
		if (isInitialized()) {
			string << "math:" << endLine;
			string << "-------------------------" << endLine;
			string << *constants_;
			string << *rules_;
			string << *theories_;
			string << *types_;
			string << *defiendums_;
			string << *assertions_;
		}
	}
	Size_t
	Math :: getRawVolume()
	{
		Size_t result = 0;
		result += getAggregateVolume (assertions_);
		result += getAggregateVolume (constants_);
		result += getAggregateVolume (defiendums_);
		result += getAggregateVolume (rules_);
		result += getAggregateVolume (theories_);
		result += getAggregateVolume (types_);
		return result;
	}

	/********************************
	 *		Private static attributes
	 ********************************/
	
	Assertions* 
		Math :: assertions_ = NULL;
	Constants* 
		Math :: constants_ = NULL;
	Defiendums*
		Math :: defiendums_ = NULL;
	Rules* 
		Math :: rules_ = NULL;
	Theories* 
		Math :: theories_ = NULL;
	Types* 
		Math :: types_ = NULL;
}
}

#endif /*MDL_MATH_MATH_CPP_*/
