/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File Name:       mdl_math_Math.hpp                                        */
/* Description:     all global containers for mdl language                   */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_MATH_MATH_HPP_
#define MDL_MATH_MATH_HPP_

#include "mdl/math/mdl_math.dpp"
#include "mdl/math/mdl_math_Assertions.hpp"
#include "mdl/math/mdl_math_Constants.hpp"
#include "mdl/math/mdl_math_Defiendums.hpp"
#include "mdl/math/mdl_math_Rules.hpp"
#include "mdl/math/mdl_math_Theories.hpp"
#include "mdl/math/mdl_math_Types.hpp"

namespace mdl {
namespace math {

class Math : public nstd :: Static<Math> {
public :
	static Assertions* assertions();
	static Constants* constants();
	static Defiendums* defiendums();
	static Rules* rules();
	static Theories* theories();
	static Types* types();

	static void release();

	// nstd :: Static interface
	static void initRaw ();
	static void showRaw (String&);
	static Size_t getRawVolume ();

private :
	static Assertions* assertions_;
	static Constants* constants_;
	static Defiendums* defiendums_;
	static Rules* rules_;
	static Theories* theories_;
	static Types* types_;
};

}
}

#include "mdl/math/mdl_math_Math.ipp"

#endif /*MDL_MATH_MATH_HPP_*/
