/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File Name:       mdl_math_Theories.cpp                                    */
/* Description:     global container for theories                            */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_MATH_THEORIES_CPP_
#define MDL_MATH_THEORIES_CPP_

#include "mdl/form/mdl_form.hpp"
#include "mdl/math/mdl_math_Theories.hpp"

namespace mdl {
namespace math {

	/****************************
	 *		Public members
	 ****************************/

	inline
	Theories :: Theories () :
	Ancestor_ (INITIAL_THEORY_VECTOR_CAPACITY),
	root_ (new form :: Theory()) {
		add (root_);
	}
	Theories :: ~ Theories () {
		deleteAggregateObject (root_);
	}

using manipulator :: endLine;
using manipulator :: tab;
using manipulator :: space;

	// object :: Object implementation
	void
	Theories :: commitSuicide() {
		delete this;
	}
	Size_t
	Theories :: getVolume() const
	{
		Size_t volume = 0;
		volume += Ancestor_ :: getVolume();
		volume += getAggregateVolume (root_);
		return volume;
	}
	Size_t
	Theories :: getSizeOf() const {
		return sizeof (Theories);
	}
	void
	Theories :: show (String& string) const
	{
		string << tab << "theories:      " << tab;
		string << Ancestor_ :: formVector_.getSize() - 1 << endLine;
	}
}
}

#endif /*MDL_MATH_THEORIES_CPP_*/
