/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File Name:       mdl_Mdl.hpp                                              */
/* Description:     mdl translator/prover                                    */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_MDL_HPP_
#define MDL_MDL_HPP_

#include "mdl/parser/mdl_parser.hpp"

namespace mdl {

class Mdl :
	public object :: Object,
	public Scalar<Mdl> {
public :
	Mdl();
	virtual ~ Mdl();

	enum Result_ {
		SUCCESS          = Error :: NONE,
		LEXER_ERROR      = Error :: LEXER,
		SYNTAX_ERROR     = Error :: PARSER,
		SEMANTIC_ERROR   = Error :: SEMANTIC,
		CHECKER_ERROR    = Error :: CHECKER,
		TRANSLATOR_ERROR = Error :: TRANSLATOR,
		PROVER_ERROR     = Error :: PROVER,
		INPUT_ERROR      = Error :: INPUT,
		OUTPUT_ERROR     = Error :: OUTPUT,
		INIT_ERROR,
		RELEASE_ERROR,
		OUT_OF_MEMORY,
		GENERAL_FAILURE
	};

	Result_ run ();
	const Timers& getTimers() const;

	static const Mdl* getMdl();
	static Result_ initMemory();
	static Result_ releaseMemory();
	static void showMemory();

	// object :: Object interface
	virtual void commitSuicide();
	virtual Size_t getVolume() const;
	virtual Size_t getSizeOf() const;
	virtual void show (String&) const;

private :
	void process ();

	void read();
	void build();
	void prove();
	void compress();
	void check();
	void write();

	static bool releaseHeap();
	static bool releaseStack();
	static bool releaseBoxed();
	static bool releaseStandard();

	const String file_;
	const Path* path_;
	parser :: Source* parser_;
	Source* source_;

	Timers timers_;
	Volume volume_;

	friend class auxiliary :: Volume;
	static Mdl* mdl_;
};

}

#endif /*MDL_MDL_HPP_*/
