/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File Name:       mdl_main.cpp                                             */
/* Description:     main function for mdl                                    */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

namespace mdl {
namespace key {
	const bool TREE_BACK_UNIFICATION = true;
	const bool VERIFY_BACK_UNIFICATION = true;
}
}
#ifdef DEBUG
#define DEBUG_BACK
#endif

#include "mdl/mdl.dpp"
#include "mdl/mdl.hpp"
#include "mdl/mdl.cpp"

using mdl :: Config;
using mdl :: Memory;
using mdl :: Size_t;
using mdl :: Path;
using mdl :: Mdl;

int main (const int argCount, const char* argValue[])
{
	if (!Config :: proceed (argCount, argValue)) {
		return Mdl :: SUCCESS;
	}
	if (Mdl :: initMemory() != Mdl :: SUCCESS) {
		return Mdl :: INIT_ERROR;
	}

	Mdl* const mdl = new Mdl();

	Mdl :: Result_
		result = Mdl :: GENERAL_FAILURE;

	result = mdl->run();

	if (Config :: verbose () || (result != Mdl :: SUCCESS)) {
		std :: cout << *mdl << std :: endl;
	}
	mdl->commitSuicide();

	if (result == Mdl :: SUCCESS) {
		result = Mdl :: releaseMemory();
	} else {
		Mdl :: releaseMemory();
	}
	return result;
}
