/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File Name:       mdl_parser_header_Line.ipp                               */
/* Description:     line parser                                              */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_PARSER_HEADER_LINE_IPP_
#define MDL_PARSER_HEADER_LINE_IPP_

#include "mdl/lexer/mdl_lexer.hpp"
#include "mdl/form/mdl_form.hpp"
#include "mdl/parser/mdl_parser_Identificator.hpp"
#include "mdl/parser/mdl_parser_Expression.hpp"

namespace mdl {
namespace parser {
namespace header {

	/****************************
	 *		Public members
	 ****************************/

	template<class T, class TF, Token :: Type token>
	const T*
	Line :: parse (Lexer* lexer)
	{
		location_ = lexer->getLocation();
		if (lexer->scan<Token :: NUMBER, Token :: COLON>() == Token :: NUMBER) {
			index_ = lexer->getToken().getIntegerValue<index :: Arity>();
			lexer->scan<Token :: COLON>();
		} else {
			index_ = 1;
		}
		type_ = identificatorParser_->parse (lexer);
		lexer->scan<Token :: EQUALITY>();

		lexer->template scan<token> ();
		expression_ = expressionParser_->parse (lexer);
		return new TF (location_, index_ - 1, type_, expression_);
	}
}
}
}

#endif /*MDL_PARSER_HEADER_DEFLINE_IPP_*/
