/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File Name:       mdl_parser_header_Variable.cpp                           */
/* Description:     variable declaration parser                              */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_PARSER_HEADER_VARIABLE_CPP_
#define MDL_PARSER_HEADER_VARIABLE_CPP_

#include "mdl/lexer/mdl_lexer.hpp"
#include "mdl/form/mdl_form.hpp"
#include "mdl/parser/mdl_parser.hpp"

namespace mdl {
namespace parser {
namespace header {

	/****************************
	 *		Public members
	 ****************************/

	inline
	Variable :: Variable() :
	location_ (),
	identificatorParser_ (new parser :: Identificator()),
	literal_ (value :: undefined :: LITERAL),
	type_ (NULL) {
	}
	Variable :: ~ Variable() {
		deleteAggregateObject (identificatorParser_);
	}

	// object :: Object implementation
	void
	Variable :: commitSuicide() {
		delete this;
	}
	Size_t
	Variable :: getVolume() const
	{
		Size_t volume = 0;
		volume += location_.getVolume();
		volume += getAggregateVolume (identificatorParser_);
		return volume;
	}
	Size_t
	Variable :: getSizeOf() const {
		return sizeof (Variable);
	}
	void
	Variable :: show (String&) const {
	}
}
}
}

#endif /*MDL_PARSER_HEADER_VARIABLES_CPP_*/
