/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File Name:       mdl_parser_proof_Notation.cpp                            */
/* Description:     proof notation parser                                    */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_PARSER_PROOF_NOTATION_CPP_
#define MDL_PARSER_PROOF_NOTATION_CPP_

#include "mdl/lexer/mdl_lexer.hpp"
#include "mdl/form/mdl_form.hpp"
#include "mdl/parser/mdl_parser.hpp"

namespace mdl {
namespace parser {
namespace proof {

	/****************************
	 *		Public members
	 ****************************/

	inline
	Notation :: Notation() :
	variablesParser_ (new parser :: header :: Variables()),
	lineParser_ (new parser :: header :: Line()),

	location_(),
	literal_ (value :: undefined :: LITERAL),
	variables_ (NULL),
	abbreviation_ (NULL),
	meaning_ (NULL) {
	}
	Notation :: ~ Notation()
	{
		deleteAggregateObject (variablesParser_);
		deleteAggregateObject (lineParser_);
	}

	const mdl :: proof :: Notation*
	Notation :: parse (Lexer* lexer, const mdl :: proof :: Line* const line)
	{
		location_ = lexer->getLocation();
		literal_ = lexer->scanLiteral();
		variables_ = variablesParser_->parse (lexer);

		lexer->scan<Token :: OPEN_BRACE>();
		lexer->scan<Token :: ABBREVIATION>();
		abbreviation_ = lineParser_->parse
		<
			mdl :: proof :: Term,
			mdl :: form :: proof :: notation :: Abbreviation,
			Token :: SHARP
		> (lexer);
		lexer->scan<Token :: MEANING>();
		meaning_ = lineParser_->parse
		<
			mdl :: proof :: Term,
			mdl :: form :: proof :: notation :: Meaning,
			Token :: SHARP
		>(lexer);
		lexer->scan<Token :: CLOSE_BRACE>();
		return create (line);
	}

	// object :: Object implementation
	void
	Notation :: commitSuicide() {
		delete this;
	}
	Size_t
	Notation :: getVolume() const
	{
		Size_t volume = 0;
		volume += getAggregateVolume (variablesParser_);
		volume += getAggregateVolume (lineParser_);
		return volume;
	}
	Size_t
	Notation :: getSizeOf() const {
		return sizeof (Notation);
	}
	void
	Notation :: show (String&) const {
	}

	/****************************
	 *		Private members
	 ****************************/

	const mdl :: proof :: Notation*
	Notation :: create (const mdl :: proof :: Line* const line)
	{
		return new form :: proof :: Notation
		(
			line,
			location_,
			literal_,
			variables_,
			abbreviation_,
			meaning_
		);
	}
}
}
}

#endif /*MDL_PARSER_PROOF_NOTATION_CPP_*/
