/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File Name:       mdl_prover_evidence_Hyp.hpp                              */
/* Description:     hyp evidence element                                     */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_PROVER_EVIDENCE_HYP_HPP_
#define MDL_PROVER_EVIDENCE_HYP_HPP_

#include "mdl/prover/mdl_prover.dpp"

namespace mdl {
namespace prover {
namespace evidence {

class Hyp :
	public Evidence,
	public Scalar<Hyp, allocator :: Prover> {
public :
	Hyp (const Hyp&);
	Hyp (const Node*, const Substitution*, const Evidence*);
	Hyp (const Node*, const Evidence*, const Evidence*);
	virtual ~ Hyp();

	const Evidence* getUp() const;

	// prover :: Evidence interface
	virtual value :: Kind getKind() const;
	virtual const Node* getNode() const;
	virtual void verifyIntegrity() const;
	virtual const Substitution* getSubstitution() const;
	virtual index :: Step getDepth() const;
	virtual value :: Integer getCardinality() const;
	virtual const Proof* constructProof (const Substitution* = NULL) const;
	virtual vector :: Proof constructProofs (const Substitution* = NULL) const;

	// object :: Cloneable interface
	virtual const Evidence* clone() const;

	// object :: Object interface
	virtual void commitSuicide();
	virtual Size_t getVolume() const;
	virtual Size_t getSizeOf() const;
	virtual void show (String&) const;

private :
	friend class Prop;
	friend class proof :: Hyp;

	const Node* node_;
	const Substitution* substitution_;
	const Evidence* up_;
	const Evidence* down_;

	index :: Step depth_;
	value :: Integer cardinality_;

	// clone
	mutable const Evidence* clone_;
};

}
}
}

#endif /*MDL_PROVER_EVIDENCE_HYP_HPP_*/
