/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File Name:       mdl_prover_evidence_Prop.hpp                             */
/* Description:     prop proof evidence element                              */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_PROVER_EVIDENCE_PROP_HPP_
#define MDL_PROVER_EVIDENCE_PROP_HPP_

#include "mdl/prover/mdl_prover.dpp"

namespace mdl {
namespace prover {
namespace evidence {

class Prop :
	public Evidence,
	public Scalar<Prop, allocator :: Prover> {
public :
	Prop (const Prop&);
	Prop
	(
		const Node*,
		const Substitution*,
		const index :: Arity
	);
	Prop
	(
		const Node*,
		const Evidence*,
		const index :: Arity
	);
	virtual ~ Prop();

	index :: Arity getArity() const;
	const Evidence* getUp (const index :: Arity) const;
	void addUp (const index :: Arity, const Evidence*);

	// prover :: Evidence interface
	virtual value :: Kind getKind() const;
	virtual const Node* getNode() const;
	virtual void verifyIntegrity() const;
	virtual const Substitution* getSubstitution() const;
	virtual index :: Step getDepth() const;
	virtual value :: Integer getCardinality() const;
	virtual const Proof* constructProof (const Substitution* = NULL) const;
	virtual vector :: Proof constructProofs (const Substitution* = NULL) const;

	// object :: Cloneable interface
	virtual const Evidence* clone() const;

	// object :: Object interface
	virtual void commitSuicide();
	virtual Size_t getVolume() const;
	virtual Size_t getSizeOf() const;
	virtual void show (String&) const;

	class Multyindex;

private :
	friend class proof :: Prop;
	typedef
		mdl :: array :: Array
		<
			const Evidence*,
			storage :: ByValue,
			allocator :: Prover
		>
		Array_;

	void verifyUp() const;
	void verifySubstitutions() const;
	void verifyVariables (const Substitution*) const;
	void verifyVariable (const Symbol&, const Substitution*) const;
	void showVerification (String&) const;

	const Node* node_;
	const Substitution* substitution_;
	Array_ up_;
	const Evidence* down_;

	index :: Step depth_;
	value :: Integer cardinality_;

	// clone
	mutable const Evidence* clone_;
};

}
}
}

#endif /*MDL_PROVER_EVIDENCE_PROP_HPP_*/
