/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File Name:       mdl_prover_evidence_Prop_Multyindex.cpp                  */
/* Description:     vector of indexes for proofs                             */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_PROVER_EVIDENCE_PROP_MULTYINDEX_CPP_
#define MDL_PROVER_EVIDENCE_PROP_MULTYINDEX_CPP_

namespace mdl {
namespace prover {
namespace evidence {

	/****************************
	 *		Public members
	 ****************************/

	inline
	Prop :: Multyindex :: Multyindex
	(
		const index :: Arity dimension,
		const vector :: Proof* matrix
	) :
	dimension_ (dimension),
	matrix_ (matrix),
	indexVector_ (dimension)
	{
		for (index :: Arity d = 0; d < dimension_; ++ d) {
			indexVector_.push() = 0;
		}
	}

	inline const Proof*
	Prop :: Multyindex :: getValue (const index :: Arity d) const
	{
		const vector :: Proof&
			proofs = matrix_ [d];
		const value :: Integer
			index = indexVector_.getValue (d);
		return proofs.getValue (index);
	}
	inline bool
	Prop :: Multyindex :: operator ++() {
		return increment();
	}
	inline value :: Integer
	Prop :: Multyindex :: getVolume() const
	{
		if (dimension_ == 0) {
			return 0;
		}
		value :: Integer volume = 1;
		for (index :: Arity d = 0; d < dimension_; ++ d) {
			const vector :: Proof&
				proofs = matrix_ [d];
			volume *= proofs.getSize();
		}
		return volume;
	}

using manipulator :: endLine;
using manipulator :: tab;
using manipulator :: space;

	template<class S>
	inline void
	Prop :: Multyindex :: show (S& string) const
	{
		string << "multyindex:" << endLine;
		string << "-----------" << endLine;
		string << tab << "dimension: " << dimension_ << endLine;
		string << tab << "volume: " << getVolume() << endLine;
		string << endLine;
		string << tab << "proofs:" << endLine;
		string << tab << "----------" << endLine;
		for (index :: Arity d = 0; d < dimension_; ++ d) {
			const vector :: Proof&
				proofs = matrix_ [d];
			string << tab << tab << d << " size: " << proofs.getSize() << endLine;
			string << endLine;
		}
		string << endLine;
	}

	/****************************
	 *		Private members
	 ****************************/

	bool
	Prop :: Multyindex :: increment (const index :: Arity d)
	{
		if (d == dimension_) {
			return true;
		}
		const value :: Integer
			index = indexVector_.getValue (d);
		const value :: Integer
			boundary = matrix_ [d].getSize();

		if (index < boundary - 1) {
			++ indexVector_.getReference (d);
			return false;
		} else {
			indexVector_.getReference (d) = 0;
			return increment (d + 1);
		}
	}

	/********************************
	 *		Overloaded operators
	 ********************************/

	template<class S>
	inline S&
	operator << (S& string, const Prop :: Multyindex& multyindex)
	{
		multyindex.show (string);
		return string;
	}
	inline std :: ostream&
	operator << (std :: ostream& os, const Prop :: Multyindex& multyindex)
	{
		static nstd :: String<> string (1024);
		return os << (string << multyindex);
	}
}
}
}

#endif /*MDL_PROVER_EVIDENCE_PROP_MULTYINDEX_CPP_*/
