/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File Name:       mdl_prover_evidence_Ref.hpp                              */
/* Description:     reference proof evidence element                         */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_PROVER_EVIDENCE_REF_HPP_
#define MDL_PROVER_EVIDENCE_REF_HPP_

#include "mdl/prover/mdl_prover.dpp"

namespace mdl {
namespace prover {
namespace evidence {

class Ref :
	public Evidence,
	public Scalar<Ref, allocator :: Prover> {
public :
	Ref
	(
		const Node*,
		const Substitution*,
		const Substitution*,
		const Evidence*
	);
	Ref
	(
		const Node*,
		const Substitution*,
		const Evidence*,
		const Evidence*
	);
	virtual ~ Ref();

	const Evidence* getUp() const;

	// prover :: Evidence interface
	virtual value :: Kind getKind() const;
	virtual const Node* getNode() const;
	virtual void verifyIntegrity() const;
	virtual const Substitution* getSubstitution() const;
	virtual index :: Step getDepth() const;
	virtual value :: Integer getCardinality() const;
	virtual const Proof* constructProof (const Substitution* = NULL) const;
	virtual vector :: Proof constructProofs (const Substitution* = NULL) const;

	// object :: Cloneable interface
	virtual const Evidence* clone() const;

	// object :: Object interface
	virtual void commitSuicide();
	virtual Size_t getVolume() const;
	virtual Size_t getSizeOf() const;
	virtual void show (String&) const;

private :
	friend class proof :: Ref;
	bool isEqual (const Ref*) const;
	void showVerification (String&) const;

	friend class Prop;
	const Node* node_;
	const Substitution* replacement_;
	const Substitution* substitution_;
	const Evidence* up_;
	const Evidence* down_;

	index :: Step depth_;
	value :: Integer cardinality_;

	// clone
	mutable const Ref* clone_;
};

}
}
}

#endif /*MDL_PROVER_EVIDENCE_REF_HPP_*/
