/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File Name:       mdl_prover_evidence_Sub.cpp                              */
/* Description:     common substitution proof evidence element               */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_PROVER_EVIDENCE_SUB_CPP_
#define MDL_PROVER_EVIDENCE_SUB_CPP_

namespace mdl {
namespace prover {
namespace evidence {

	/****************************
	 *		Public members
	 ****************************/

	inline
	Sub :: Sub (const Node* node, const Substitution* substitution) :
	node_ (node),
	substitution_ (substitution),
	up_ (1),
	depth_ (index :: undefined :: STEP),
	cardinality_ (value :: undefined :: INTEGER),
	clone_ (NULL) {
	}
	Sub :: ~ Sub() {
	}

	inline value :: Integer
	Sub :: getSize() const {
		return up_.getSize();
	}
	inline const Evidence*
	Sub :: getUp (const value :: Integer i) const {
		return up_.getValue (i);
	}
	inline void
	Sub :: addUp (const Evidence* evidence)
	{
		up_.add (evidence);
		const index :: Step
			upDepth = evidence->getDepth();
		const value :: Integer
			upCardinality = evidence->getCardinality();
		depth_ = (upDepth < depth_) ? upDepth : depth_;
		cardinality_ = (upCardinality < cardinality_) ? upCardinality : cardinality_;
	}

using manipulator :: space;
using manipulator :: tab;
using manipulator :: endLine;

	// prover :: Evidence implementation
	inline value :: Kind
	Sub :: getKind() const {
		return SUB;
	}
	inline const Node*
	Sub :: getNode() const {
		return node_;
	}
	inline void
	Sub :: verifyIntegrity() const
	{
		for (index :: Arity i = 0; i < up_.getSize(); ++ i) {
			up_.getValue (i)->verifyIntegrity();
		}
	}
	const mdl :: prover :: Substitution*
	Sub :: getSubstitution() const {
		return substitution_;
	}
	inline index :: Step
	Sub :: getDepth() const {
		return depth_;
	}
	inline value :: Integer
	Sub :: getCardinality() const {
		return cardinality_;
	}
	const Proof*
	Sub :: constructProof (const Substitution* substitution) const
	{
		return NULL;
		Substitution*
			global = Substitution :: create (substitution_);
		global = global->compose (substitution);

		const Evidence*	const
			evidence = up_.getValue (1);
		return evidence->constructProof (global);
	}
	vector :: Proof
	Sub :: constructProofs (const Substitution* substitution) const
	{
		Substitution*
			global = Substitution :: create (substitution_);
		global = global->compose (substitution);

		value :: Integer size = 1;
		vector :: Proof upProofs [up_.getSize()];
		for (value :: Integer i = 0; i < up_.getSize(); ++ i) {
			const Evidence* evidence = up_.getValue (i);
			upProofs [i] = evidence->constructProofs (global);
			size += upProofs [i].getSize();
		}

		vector :: Proof proofs (size);
		for (value :: Integer i = 0; i < up_.getSize(); ++ i) {
			for (value :: Integer j = 0; j < upProofs [i].getSize(); ++ j) {
				const Proof* const
					proof = upProofs [i].getValue (j);
				proofs.add (proof);
			}
		}
		return proofs;
	}

	// object :: Cloneable implementation
	const Evidence*
	Sub :: clone() const {
		return clone_;
	}

	// object :: Object interface
	void
	Sub :: commitSuicide() {
		delete this;
	}
	Size_t
	Sub :: getVolume() const
	{
		Size_t volume = 0;
		volume += getAggregateVolume (substitution_);
		volume += up_.getVolume();
		return volume;
	}
	Size_t
	Sub :: getSizeOf() const {
		return sizeof (Prop);
	}
	void
	Sub :: show (String& string) const
	{
		string << "############################### - Sub Node - ################################################" << endLine;
		string << "this: " << this << endLine;
		string << "node: " << node_ << endLine;
		string << "substitution: " << *substitution_ << endLine;
		string << "up:" << space;
		for (index :: Arity i = 0; i < up_.getSize(); ++ i) {
			string << up_.getValue (i);
			if (i < up_.getSize() - 1) {
				string << "," << space;
			}
		}
		string << endLine;
		for (index :: Arity i = 0; i < up_.getSize(); ++ i) {
			string << *up_.getValue (i) << endLine;
		}
		string << endLine;
		string << "****************************************************" << endLine;
		string << endLine;;
		string << endLine;
		string << endLine;
		string << endLine;
		string << "############################### - Sub Node - ################################################" << endLine;
		string << endLine;
		string << endLine;
		string << endLine;
		string << endLine;
		string << endLine;
		string << endLine;
		string << endLine;
		string << endLine;
		string << endLine;
		string << endLine;
		string << endLine;
		string << endLine;
		string << endLine;
		string << endLine;
		string << endLine;
	}
}
}
}

#endif /*MDL_PROVER_EVIDENCE_SUB_CPP_*/
