/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File Name:       mdl_prover_evidence_Sub.hpp                              */
/* Description:     common substitution proof evidence element               */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_PROVER_EVIDENCE_SUB_HPP_
#define MDL_PROVER_EVIDENCE_SUB_HPP_

#include "mdl/prover/mdl_prover.dpp"

namespace mdl {
namespace prover {
namespace evidence {

class Sub :
	public Evidence,
	public Scalar<Sub, allocator :: Prover> {
public :
	Sub (const Node*, const Substitution*);
	virtual ~ Sub();

	value :: Integer getSize() const;
	const Evidence* getUp (const value :: Integer) const;
	void addUp (const Evidence*);

	// prover :: Evidence interface
	virtual value :: Kind getKind() const;
	virtual const Node* getNode() const;
	virtual void verifyIntegrity() const;
	virtual const Substitution* getSubstitution() const;
	virtual index :: Step getDepth() const;
	virtual value :: Integer getCardinality() const;
	virtual const Proof* constructProof (const Substitution* = NULL) const;
	virtual vector :: Proof constructProofs (const Substitution* = NULL) const;

	// object :: Cloneable interface
	virtual const Evidence* clone() const;

	// object :: Object interface
	virtual void commitSuicide();
	virtual Size_t getVolume() const;
	virtual Size_t getSizeOf() const;
	virtual void show (String&) const;

private :
	friend class proof :: Prop;
	typedef
		mdl :: vector :: Vector
		<
			const Evidence*,
			storage :: ByValue,
			allocator :: Prover
		>
		Vector_;

	const Node* node_;
	const Substitution* substitution_;
	Vector_ up_;

	index :: Step depth_;
	value :: Integer cardinality_;

	// clone
	mutable const Evidence* clone_;
};

}
}
}

#endif /*MDL_PROVER_EVIDENCE_SUB_HPP_*/
