/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File Name:       mdl_prover_evidence_Top.hpp                              */
/* Description:     top (premise) proof evidence element                     */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_PROVER_EVIDENCE_TOP_HPP_
#define MDL_PROVER_EVIDENCE_TOP_HPP_

#include "mdl/prover/mdl_prover.dpp"

namespace mdl {
namespace prover {
namespace evidence {

class Top :
	public Evidence,
	public Scalar<Top, allocator :: Prover> {
public :
	Top (const Top&);
	Top (const Node*, const Premise*, const Substitution*);
	Top (const Node*, const Evidence*, const Premise*);
	virtual ~ Top();

	// prover :: Evidence interface
	virtual value :: Kind getKind() const;
	virtual const Node* getNode() const;
	virtual void verifyIntegrity() const;
	virtual const Substitution* getSubstitution() const;
	virtual index :: Step getDepth() const;
	virtual value :: Integer getCardinality() const;
	virtual const Proof* constructProof (const Substitution* = NULL) const;
	virtual vector :: Proof constructProofs (const Substitution* = NULL) const;

	// object :: Cloneable interface
	virtual const Evidence* clone() const;

	// object :: Object interface
	virtual void commitSuicide();
	virtual Size_t getVolume() const;
	virtual Size_t getSizeOf() const;
	virtual void show (String&) const;

private :
	friend class proof :: Top;
	void showVerification (String&) const;

	const Node* node_;
	const Premise* premise_;
	const Evidence* down_;
	const Substitution* substitution_;

	// clone
	mutable const Evidence* clone_;
};

}
}
}

#endif /*MDL_PROVER_EVIDENCE_TOP_HPP_*/
