/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File Name:       mdl_prover_evidence_Vector.hpp                           */
/* Description:     confirmation: vector of evidences                        */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_PROVER_EVIDENCE_VECTOR_HPP_
#define MDL_PROVER_EVIDENCE_VECTOR_HPP_

#include "mdl/prover/mdl_prover.dpp"

namespace mdl {
namespace prover {
namespace evidence {

class Vector :
	public Confirmation,
	public Scalar<Vector, allocator :: Prover> {
private :
	enum { INITIAL_PROOFS_CAPACITY = 0 };
public :
	Vector (const Node*, const value :: Integer = INITIAL_PROOFS_CAPACITY);
	Vector (const Vector&);
	virtual ~ Vector();

	void clear();
	bool isEqual (const Vector*) const;
	void showDifference (String&, const Vector*) const;
	void showSubstitutions (String&, const bool = false) const;

	// prover :: Confirmation interface
	virtual void verifyIntegrity() const;
	virtual bool boundaryMoved() const;
	virtual void markBoundary();
	virtual value :: Integer getBoundary() const;
	virtual value :: Integer getSize() const;
	virtual value :: Integer getDelta() const;
	virtual Evidence* supply (const stack :: Substitution*);
	virtual void add (const Evidence*);
	virtual void replace (const Evidence*, const value :: Integer);
	virtual const Evidence* get (const value :: Integer) const;
	virtual const Evidence* getNew (const value :: Integer) const;

	// statistics routines
	virtual Size_t getEvidenceVolume() const;
	virtual Size_t getSubstitutionVolume() const;
	virtual Size_t getEvidenceNodeVolume() const;
	virtual Size_t getGarbageVolume() const;

	// object :: Object interface
	virtual void commitSuicide();
	virtual Size_t getVolume() const;
	virtual Size_t getSizeOf() const;
	virtual void show (String&) const;

private :
	typedef
		mdl :: vector :: Vector
		<
			const Evidence*,
			storage :: ByValue,
			allocator :: Prover
		>
		EvidenceVector_;

	bool isSubset (const Vector*) const;
	bool contains (const Evidence*) const;
	void showDiffSubset (String&, const Vector*) const;

	bool disjointedsAreNew (const Evidence*, const Evidence*) const;
	bool disjointedsAreNew (const mdl :: Proposition*, const Evidence*) const;

	EvidenceVector_ evidenceVector_;
	EvidenceVector_ garbageVector_;
	value :: Integer boundary_;
	const Node* node_;
};

}
}
}

#endif /*MDL_PROVER_EVIDENCE_VECTOR_HPP_*/
