/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File Name:       mdl_prover_Prover.hpp                                    */
/* Description:     main static prover class                                 */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_PROVER_PROVER_HPP_
#define MDL_PROVER_PROVER_HPP_

#include "mdl/interface/mdl_interface.hpp"
#include "mdl/prover/mdl_prover_Top.hpp"
#include "mdl/prover/mdl_prover_Timers.hpp"
#include "mdl/prover/mdl_prover_Volume.hpp"

namespace mdl {
namespace prover {

class Prover {
public :
	// prover :: Prover interface
	static bool prove
	(
		const mdl :: Assertion*,
		const mdl :: proof :: Step*,
		const index :: Step = index :: undefined :: STEP
	);
	static const mdl :: Assertion* getProblem();
	static const tree :: Premise* getPremises();
	static index :: Assertion getPropositionBoundary();
	static index :: Step getStepBoundary();
	static const mdl :: proof :: Step* getQestionStep();
	static const mdl :: proof :: Step* getAnswerStep();
	static index :: Step getDepth();
	static Symbol createVariable (const Symbol&);

	static bool maxDepth (const index :: Step);
	static bool fitsDepth (const index :: Step);
	static bool fitsSubstitutionComplexity (const value :: Integer);
	static bool fitsExpressionComplexity (const value :: Integer);

	static void refresh (const char* = "", const bool = false);
	static void show();
	static const Log& getLog();
	static const Timers& getTimers();
	static const Volume& getVolumes();
	static Log& log();
	static Timers& timers();

	static void verifyIntegrity();

	static void init();
	static void release();

	static Size_t getVolume();
	static Size_t getSizeOf();
	static void show (String&);

private :
	static void init
	(
		const mdl :: Assertion*,
		const mdl :: proof :: Step*,
		const index :: Step = index :: undefined :: STEP
	);
	static void showStart();
	static void showResult();
	static void showStatus (const char*);

	friend class Timers;
	friend class Volume;

	enum {
		INITIAL_VARIABLES_CAPACITY = nstd :: KILOBYTE * 64
	};
	typedef
		map :: Scalar
		<
			value :: Literal,
			value :: Integer,
			allocator :: Prover
		>
		Variables_;

	static const mdl :: Assertion* problem_;
	static const mdl :: proof :: Step* question_;
	static const mdl :: proof :: Step* answer_;

	static tree :: Premise* premises_;
	static Variables_* variables_;
	static Root*  root_;
	static Top    top_;
	static Log    log_;
	static Timers timers_;
	static Volume volume_;
};

}
}

#endif /*MDL_PROVER_PROVER_HPP_*/
