/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File Name:       mdl_prover_Timers.cpp                                    */
/* Description:     prover timers class                                      */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_PROVER_TIMERS_CPP_
#define MDL_PROVER_TIMERS_CPP_

namespace mdl {
namespace prover {

	/****************************
	 *		Public members
	 ****************************/

	inline
	Timers :: Timers() :
	limitPrime_ (Config :: getQuantumClock()),
	limitSecond_ (Config :: getQuantumClock()),
	prove_ (),
	refresh_(),
	recounted_ (false) {
	}

	inline bool
	Timers :: refresh()
	{
		if (prove_.getClock() > limitSecond_) {
			throw Exception ("prover has ran out of time. ");
		}
		if (refresh_.getClock() > Config :: getRefreshClock()) {
			refresh_.start();
			return true;
		} else {
			return false;
		}
	}
	inline void
	Timers :: start()
	{
		prove_.start();
		refresh_.start();
		limitSecond_ = limitPrime_;
		recounted_ = false;
	}
	inline void
	Timers :: mark() {
		prove_.mark();
	}
	inline void
	Timers :: stop()
	{
		prove_.stop();
		refresh_.stop();
		limitSecond_ = limitPrime_;
	}
	inline void
	Timers :: recount()
	{
		if (recounted_) {
			return;
		}
		recounted_ = true;

		prove_.mark();
		const Time_t t_0 = prove_.getClock();
		const Time_t T_0 = limitPrime_;
		const Time_t t_1 = t_0 * (1 - static_cast<double>(t_0) / static_cast<double>(T_0));
		limitSecond_ = t_0 + t_1;

		if (limitSecond_ == 0) {
			limitSecond_ = Config :: getQuantumClock();
		}
	}

	inline Time_t
	Timers :: limit() const {
		return limitSecond_;
	}
	inline Timer
	Timers :: prove() const {
		return prove_;
	}
	inline Timer
	Timers :: refresh() const {
		return refresh_;
	}
	inline void
	Timers :: setLimit (const Clock_t limit)
	{
		limitPrime_ = limit;
		limitSecond_ = limitPrime_;
	}

using manipulator :: endLine;
using manipulator :: iterate;
using manipulator :: mode;

	// object :: Object implementation
	inline Size_t
	Timers :: getVolume() const
	{
		Size_t volume = 0;
		volume += prove_.getVolume();
		volume += refresh_.getVolume();
		return volume;
	}
	inline Size_t
	Timers :: getSizeOf() const {
		return sizeof (Timers);
	}
	inline void
	Timers :: show (String& string) const
	{
		if (Config :: verbosity() > Config :: VERBOSE_STANDARD) {
			string << "running: " << prove_ << endLine;
		} else {
			string << "running: " << prove_;
		}
	}

	/********************************
	 *		Overloaded operators
	 ********************************/

	inline String&
	operator << (String& string, const Timers& timers)
	{
		timers.show (string);
		return string;
	}
}
}

#endif /*MDL_PROVER_PROVER_TIMERS_CPP_*/
