/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File Name:       mdl_prover_Top.cpp                                       */
/* Description:     top level in proof variant tree                          */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_PROVER_TOP_CPP_
#define MDL_PROVER_TOP_CPP_

namespace mdl {
namespace prover {

	/****************************
	 *		Public members
	 ****************************/

	Top :: Top () :
	limitDepth_ (index :: undefined :: STEP),
	currentDepth_ (index :: undefined :: STEP),
	nextLayer_ (NULL),
	prevLayer_ (NULL),
	nodeIndex_ (value :: undefined :: INTEGER) {
	}
	Top :: ~ Top() {
	}

	void
	Top :: init (const index :: Step limitDepth)
	{
		limitDepth_ = limitDepth;
		currentDepth_ = 0;
		nodeIndex_ = 0;
		nextLayer_ = new set :: Node (INITIAL_LAYER_CAPACITY);
		prevLayer_ = new set :: Node (INITIAL_LAYER_CAPACITY);
	}
	void
	Top :: operator + ()
	{
		/*if (Memory :: getFreeVolume() < Memory :: getAllocatedVolume()) {
			operator -- ();
		}*/
		if (nextLayer_->getSize() == 0) {
			throw Exception ("no way to continue proof search. ");
		}
		prevLayer_->clear();
		set :: Node* const layer = prevLayer_;
		prevLayer_ = nextLayer_;
		nextLayer_ = layer;

		++ currentDepth_;
		nodeIndex_ = 0;
		if (currentDepth_ > limitDepth_) {
			throw Exception ("prover has ran out of depth limit. ");
		}
	}
	void
	Top :: operator ++ ()
	{
		const Node* const
			node = prevLayer_->getValue (nodeIndex_);
		const_cast<Node*>(node)->buildUp (this);
		++ nodeIndex_;
	}
	void
	Top :: operator -- ()
	{
		setRate();
		cloneThere();
		cloneBack();
	}
	bool
	Top :: finished() const {
		return (nodeIndex_ == prevLayer_->getSize());
	}
	index :: Step
	Top :: getDepth() const {
		return currentDepth_;
	}

	// prover :: Top implementation
	void
	Top :: add (const Node* node) {
		nextLayer_->add (node);
	}

using manipulator :: space;
using manipulator :: tab;
using manipulator :: endLine;

	// object :: Object implementation
	void
	Top :: commitSuicide() {
		delete this;
	}
	Size_t
	Top :: getVolume() const
	{
		Size_t volume = 0;
		volume += getAggregateVolume (prevLayer_);
		volume += getAggregateVolume (nextLayer_);
		return volume;
	}
	Size_t
	Top :: getSizeOf() const {
		return sizeof (Top);
	}
	void
	Top :: show (String& string) const
	{
		const Size_t size = prevLayer_->getSize();

		if (Config :: verbosity() > Config :: VERBOSE_STANDARD) {
			string << "top tree level"<< endLine;
			string << "--------------"<< endLine;
			string << "depth: " << currentDepth_ << endLine;
			string << "nodes: " << nextLayer_->getSize() << endLine;
			string << "percent: " << ((size == 0) ? 100 : (nodeIndex_ * 100) / size) << " % ";
			string << "(" << nodeIndex_<< " of " << prevLayer_->getSize() << "nodes)" << endLine;
			string << endLine;
		} else {
			string << "depth: " << currentDepth_ << ", ";
			string << "done of level: " << ((size == 0) ? 100 : (nodeIndex_ * 100) / size) << " %";
		}
		//string << endLine;
	}

	/****************************
	 *		Private members
	 ****************************/

	void
	Top :: setRate ()
	{
		for (value :: Integer rate = 0; rate < nextLayer_->getSize(); ++ rate) {
			const Node* const
				node = nextLayer_->getValue (rate);
			node->setRate (rate);
		}
	}
	void
	Top :: cloneThere ()
	{
		prevLayer_ = new set :: Node (INITIAL_LAYER_CAPACITY);
		try {
			for (value :: Integer i = 0; i < nextLayer_->getSize(); ++ i) {
				const Node* const
					clone = nextLayer_->getValue (i)->clone();
				prevLayer_->add (clone);
			}
		} catch (OutOfMemory&) {
		}
	}
	void
	Top :: cloneBack ()
	{
		nextLayer_ = new set :: Node (INITIAL_LAYER_CAPACITY);
		for (value :: Integer i = 0; i < prevLayer_->getSize(); ++ i) {
			const Node* const
				clone = prevLayer_->getValue (i)->clone();
			nextLayer_->add (clone);
		}
		prevLayer_ = new set :: Node (INITIAL_LAYER_CAPACITY);
	}
}
}

#endif /*MDL_PROVER_TOP_CPP_*/
