/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File Name:       mdl_prover_Top.hpp                                       */
/* Description:     top level in proof variant tree                          */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_PROVER_TOP_HPP_
#define MDL_PROVER_TOP_HPP_

#include "mdl/interface/mdl_interface.hpp"

namespace mdl {
namespace prover {

class Top :
	public Level,
	public Scalar<Top, allocator :: Prover> {
public :
	Top ();
	virtual ~ Top();

	void init (const index :: Step = index :: undefined :: STEP);
	void operator + ();
	void operator ++ ();
	void operator -- ();
	bool finished() const;
	index :: Step getDepth() const;

	// prover :: Level interface
	virtual void add (const Node*);

	// object :: Object interface
	virtual void commitSuicide();
	virtual Size_t getVolume() const;
	virtual Size_t getSizeOf() const;
	virtual void show (String&) const;

private :
	void setRate();
	void cloneThere();
	void cloneBack();

	index :: Step limitDepth_;
	index :: Step currentDepth_;
	set :: Node* nextLayer_;
	set :: Node* prevLayer_;
	value :: Integer nodeIndex_;

	enum {
		INITIAL_LAYER_CAPACITY = nstd :: KILOBYTE * 16
	};
};

}
}

#endif /*MDL_PROVER_TOP_HPP_*/
