/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File Name:       mdl_prover_Volume.hpp                                    */
/* Description:     volume counter for prover class                          */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_PROVER_VOLUME_HPP_
#define MDL_PROVER_VOLUME_HPP_

#include "mdl/types/mdl_types.hpp"

namespace mdl {
namespace prover {

class Volume {
public :
	Volume();

	void verifyIntegrity();
	void calculate();

	Size_t getVolume() const;
	Size_t getSizeOf() const;
	void show (String&) const;

private :
	void calculateData();
	void calculateBalance();
	void calculateMemoryBalance();
	void calculateSubstitutionBalance();
	void calculateNodeCountBalance();

	void showVolume (String&) const;
	void showBalance (String&) const;

	value :: Integer treeNodeCount_;
	value :: Integer evidenceNodeCount_;
	value :: Integer expressionCount_;
	value :: Integer hypNodeCount_;
	value :: Integer propNodeCount_;
	value :: Integer refNodeCount_;
	value :: Integer nodeCountSum_;

	Size_t tree_;
	Size_t evidences_;
	Size_t top_;
	Size_t expression_;
	Size_t garbage_;
	Size_t treeNode_;
	Size_t evidenceNode_;
	Size_t evidenceContainer_;
	Size_t expressions_;

	Size_t substitutionAll_;
	Size_t substitutionInt_;
	Size_t substitutionExt_;
	Size_t substitution_;
	Size_t substitutionSum_;

	Size_t allocatedBoxed_;
	Size_t usedBoxed_;
	Size_t spaceBoxed_;

	Size_t maxStack_;
	Size_t usedStack_;
	Size_t spaceStack_;

	Ptrdiff_t deltaBoxed_;
	Ptrdiff_t deltaSubstitutions_;
	Ptrdiff_t deltaNodeCount_;
};

}
}

#endif /*MDL_PROVER_VOLUME_HPP_*/
