/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File Name:       mdl_prover_node_Hyp_Statistics.hpp                       */
/* Description:     statistics for hypothesis node in proof variant tree     */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_PROVER_NODE_HYP_STATISTICS_HPP_
#define MDL_PROVER_NODE_HYP_STATISTICS_HPP_

#include "mdl/prover/mdl_prover.dpp"

namespace mdl {
namespace prover {
namespace node {

class Hyp :: Statistics {
public :
	// prover :: Statistics interface
	static Size_t getEvidenceVolume (const Hyp*);
	static Size_t getTreeVolume (const Hyp*);
	static Size_t getExpressionVolume (const Hyp*);
	static Size_t getTreeNodeVolume (const Hyp*);
	static Size_t getEvidenceNodeVolume (const Hyp*);
	static Size_t getEvidenceContainerVolume (const Hyp*);
	static Size_t getGarbageVolume (const Hyp*);

	// substitutions statistics
	static Size_t getSubstitutionAllVolume (const Hyp*);
	static Size_t getSubstitutionIntVolume (const Hyp*);
	static Size_t getSubstitutionExtVolume (const Hyp*);
	static Size_t getSubstitutionVolume (const Hyp*);

	// node count statistics
	static value :: Integer getTreeNodeCount (const Hyp*);
	static value :: Integer getEvidenceNodeCount (const Hyp*);
	static value :: Integer getExpressionCount (const Hyp*);
	static value :: Integer getHypCount (const Hyp*);
	static value :: Integer getPropCount (const Hyp*);
	static value :: Integer getRefCount (const Hyp*);

private :
	template<class T>
	static Size_t getAggregateVolume (const T*);
};


}
}
}

#endif /*MDL_PROVER_NODE_HYP_STATISTICS_HPP_*/
