/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File Name:       mdl_prover_node_Prop_Prod.cpp                            */
/* Description:     decart product matrix unification                        */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_PROVER_NODE_PROP_PROD_CPP_
#define MDL_PROVER_NODE_PROP_PROD_CPP_

namespace mdl {
namespace prover {
namespace node {

	/****************************
	 *		Public members
	 ****************************/

	inline value :: Integer
	Prop :: Prod :: getMatrixSearchVolume (const Prop* prop)
	{
		const index :: Arity
			arity = prop->up_.getSize();

		Memory :: stack().push();
		proof :: Matrix matrix (arity);
		fillMatrix (prop, arity, matrix);
		MatrixUnifier :: Multyindex_ multyindex (arity, matrix);
		const value :: Integer
			searchVolume = multyindex.getSearchVolume();
		Memory :: stack().pop();
		return searchVolume;
	}
	inline void
	Prop :: Prod :: unify (const Prop* prop, evidence :: Vector& proofs)
	{
		const index :: Arity
			arity = prop->up_.getSize();
		if (arity == 0) {
			addEvidence (prop, proofs);
			return;
		}
		Memory :: stack().push();
		proof :: Matrix matrix (arity);
		fillMatrix (prop, arity, matrix);
		MatrixUnifier unify
		(
			arity,
			prop->substitutionInt_,
			proofs
		);
		unify (matrix);
		Memory :: stack().pop();
	}

	/****************************
	 *		Private members
	 ****************************/

	inline void
	Prop :: Prod :: fillMatrix
	(
		const Prop* prop,
		const index :: Arity arity,
		proof :: Matrix& matrix
	)
	{
		for (index :: Arity i = 0; i < prop->up_.getSize(); ++ i) {
			const Node* const
				hyp = prop->up_.getValue (i);
			const Confirmation&
				confirmation = hyp->getConfirmation();
			matrix.getReference (i) = &confirmation;
		}
	}
	inline void
	Prop :: Prod :: addEvidence (const Prop* prop, evidence :: Vector& confirmation)
	{
		Evidence* evidenceSub = confirmation.supply
		(
			prop->substitutionInt_->castAway<allocator :: Stack>()
		);
		if (evidenceSub == NULL) {
			return;
		}
		const Evidence* evidenceProp = new evidence :: Prop (prop, evidenceSub, 0);
		dynamic_cast<evidence :: Sub*>(evidenceSub)->addUp (evidenceProp);
	}
}
}
}

#endif /*MDL_PROVER_NODE_PROP_PROD_CPP_*/
