/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File Name:       mdl_prover_node_Prop_Statistics.cpp                      */
/* Description:     statistics for proposition node in proof variant tree    */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_PROVER_NODE_PROP_STATISTICS_CPP_
#define MDL_PROVER_NODE_PROP_STATISTICS_CPP_

namespace mdl {
namespace prover {
namespace node {

	/****************************
	 *		Public members
	 ****************************/

	// prover :: Statistics implementation
	Size_t
	Prop :: Statistics :: getEvidenceVolume (const Prop* node)
	{
		Size_t volume = 0;
		volume += node->confirmation_.getVolume();
		for (value :: Integer i = 0; i < node->confirmation_.getSize(); ++ i) {
			const Evidence* const
				proof = node->confirmation_.get (i);
			volume += getAggregateVolume (proof);
		}
		for (index :: Arity i = 0; i < node->up_.getSize(); ++ i) {
			const Node* const
				hyp = node->up_.getValue (i);
			volume += hyp->getEvidenceVolume();
		}
		return volume;
	}
	Size_t
	Prop :: Statistics :: getTreeVolume (const Prop* node)
	{
		Size_t volume = 0;
		volume += getAggregateVolume (node->substitutionInt_);
		volume += getAggregateVolume (node->substitutionExt_);
		volume += node->up_.getVolume();
		for (index :: Arity i = 0; i < node->up_.getSize(); ++ i) {
			const Node* const
				hyp = node->up_.getValue (i);
			volume += hyp->getTreeVolume();
			volume += hyp->getSizeOf();
		}
		return volume;
	}
	Size_t
	Prop :: Statistics :: getExpressionVolume (const Prop* node)
	{
		Size_t volume = 0;
		for (index :: Arity i = 0; i < node->up_.getSize(); ++ i) {
			const Node* const
				hyp = node->up_.getValue (i);
			volume += hyp->getExpressionVolume();
		}
		return volume;
	}
	Size_t
	Prop :: Statistics :: getTreeNodeVolume (const Prop* node)
	{
		Size_t volume = 0;
		volume += node->up_.getVolume();
		for (index :: Arity i = 0; i < node->up_.getSize(); ++ i) {
			const Node* const
				hyp = node->up_.getValue (i);
			volume += hyp->getTreeNodeVolume();
			volume += hyp->getSizeOf();
		}
		return volume;
	}
	Size_t
	Prop :: Statistics :: getEvidenceNodeVolume (const Prop* node)
	{
		Size_t volume = 0;
		volume += node->confirmation_.getEvidenceNodeVolume();
		for (index :: Arity i = 0; i < node->up_.getSize(); ++ i) {
			const Node* const
				hyp = node->up_.getValue (i);
			volume += hyp->getEvidenceNodeVolume();
		}
		return volume;
	}
	Size_t
	Prop :: Statistics :: getEvidenceContainerVolume (const Prop* node)
	{
		Size_t volume = 0;
		volume += node->confirmation_.getVolume();
		for (index :: Arity i = 0; i < node->up_.getSize(); ++ i) {
			const Node* const
				hyp = node->up_.getValue (i);
			volume += hyp->getEvidenceContainerVolume();
		}
		return volume;
	}
	Size_t
	Prop :: Statistics :: getGarbageVolume (const Prop* node)
	{
		Size_t volume = 0;
		volume += node->confirmation_.getGarbageVolume();
		for (index :: Arity i = 0; i < node->up_.getSize(); ++ i) {
			const Node* const
				hyp = node->up_.getValue (i);
			volume += hyp->getGarbageVolume();
		}
		return volume;
	}

	// substitution statistics
	Size_t
	Prop :: Statistics :: getSubstitutionAllVolume (const Prop* node)
	{
		Size_t volume = 0;
		volume += node->confirmation_.getSubstitutionVolume();
		volume += getAggregateVolume (node->substitutionInt_);
		volume += getAggregateVolume (node->substitutionExt_);
		for (index :: Arity i = 0; i < node->up_.getSize(); ++ i) {
			const Node* const
				hyp = node->up_.getValue (i);
			volume += hyp->getSubstitutionAllVolume();
		}
		return volume;
	}
	Size_t
	Prop :: Statistics :: getSubstitutionIntVolume (const Prop* node)
	{
		Size_t volume = 0;
		volume += getAggregateVolume (node->substitutionInt_);
		for (index :: Arity i = 0; i < node->up_.getSize(); ++ i) {
			const Node* const
				hyp = node->up_.getValue (i);
			volume += hyp->getSubstitutionIntVolume();
		}
		return volume;
	}
	Size_t
	Prop :: Statistics :: getSubstitutionExtVolume (const Prop* node)
	{
		Size_t volume = 0;
		volume += getAggregateVolume (node->substitutionExt_);
		for (index :: Arity i = 0; i < node->up_.getSize(); ++ i) {
			const Node* const
				hyp = node->up_.getValue (i);
			volume += hyp->getSubstitutionExtVolume();
		}
		return volume;
	}
	Size_t
	Prop :: Statistics :: getSubstitutionVolume (const Prop* node)
	{
		Size_t volume = 0;
		for (value :: Integer i = 0; i < node->confirmation_.getSize(); ++ i) {
			const Substitution* const
				substitution = node->confirmation_.get (i)->getSubstitution();
			volume += getAggregateVolume (substitution);
		}
		for (index :: Arity i = 0; i < node->up_.getSize(); ++ i) {
			const Node* const
				hyp = node->up_.getValue (i);
			volume += hyp->getSubstitutionVolume();
		}
		return volume;
	}

	// node count statistics
	value :: Integer
	Prop :: Statistics :: getTreeNodeCount (const Prop* node)
	{
		Size_t count = 1;
		for (index :: Arity i = 0; i < node->up_.getSize(); ++ i) {
			const Node* const
				hyp = node->up_.getValue (i);
			count += hyp->getTreeNodeCount();
		}
		return count;
	}
	value :: Integer
	Prop :: Statistics :: getEvidenceNodeCount (const Prop* node)
	{
		Size_t count = 0;
		count += node->confirmation_.getSize();
		for (index :: Arity i = 0; i < node->up_.getSize(); ++ i) {
			const Node* const
				hyp = node->up_.getValue (i);
			count += hyp->getEvidenceNodeCount();
		}
		return count;
	}
	value :: Integer
	Prop :: Statistics :: getExpressionCount (const Prop* node)
	{
		Size_t count = 0;
		for (index :: Arity i = 0; i < node->up_.getSize(); ++ i) {
			const Node* const
				hyp = node->up_.getValue (i);
			count += hyp->getExpressionCount();
		}
		return count;
	}
	value :: Integer
	Prop :: Statistics :: getHypCount (const Prop* node)
	{
		Size_t count = 0;
		for (index :: Arity i = 0; i < node->up_.getSize(); ++ i) {
			const Node* const
				hyp = node->up_.getValue (i);
			count += hyp->getHypCount();
		}
		return count;
	}
	value :: Integer
	Prop :: Statistics :: getPropCount (const Prop* node)
	{
		Size_t count = 1;
		for (index :: Arity i = 0; i < node->up_.getSize(); ++ i) {
			const Node* const
				hyp = node->up_.getValue (i);
			count += hyp->getPropCount();
		}
		return count;
	}
	value :: Integer
	Prop :: Statistics :: getRefCount (const Prop* node)
	{
		Size_t count = 0;
		for (index :: Arity i = 0; i < node->up_.getSize(); ++ i) {
			const Node* const
				hyp = node->up_.getValue (i);
			count += hyp->getRefCount();
		}
		return count;
	}

	/****************************
	 *		Private members
	 ****************************/

	template<class T>
	Size_t
	Prop :: Statistics :: getAggregateVolume (const T* object)
	{
		Size_t volume = 0;
		if (object != NULL) {
			volume += object->getVolume();
			volume += object->getSizeOf();
		}
		return volume;
	}
}
}
}

#endif /*MDL_PROVER_NODE_PROP_STATISTICS_CPP_*/
