/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File Name:       mdl_prover_node_Prop_Statistics.hpp                      */
/* Description:     statistics for proposition node in proof variant tree    */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_PROVER_NODE_PROP_STATISTICS_HPP_
#define MDL_PROVER_NODE_PROP_STATISTICS_HPP_

#include "mdl/prover/mdl_prover.dpp"

namespace mdl {
namespace prover {
namespace node {

class Prop :: Statistics {
public :
	// prover :: Statistics interface
	static Size_t getEvidenceVolume (const Prop*);
	static Size_t getTreeVolume (const Prop*);
	static Size_t getExpressionVolume (const Prop*);
	static Size_t getTreeNodeVolume (const Prop*);
	static Size_t getEvidenceNodeVolume (const Prop*);
	static Size_t getEvidenceContainerVolume (const Prop*);
	static Size_t getGarbageVolume (const Prop*);

	// substitutions statistics
	static Size_t getSubstitutionAllVolume (const Prop*);
	static Size_t getSubstitutionIntVolume (const Prop*);
	static Size_t getSubstitutionExtVolume (const Prop*);
	static Size_t getSubstitutionVolume (const Prop*);

	// node count statistics
	static value :: Integer getTreeNodeCount (const Prop*);
	static value :: Integer getEvidenceNodeCount (const Prop*);
	static value :: Integer getExpressionCount (const Prop*);
	static value :: Integer getHypCount (const Prop*);
	static value :: Integer getPropCount (const Prop*);
	static value :: Integer getRefCount (const Prop*);

private :
	template<class T>
	static Size_t getAggregateVolume (const T*);
};

}
}
}

#endif /*MDL_PROVER_NODE_PROP_STATISTICS_HPP_*/
