/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File Name:       mdl_prover_node_Ref.hpp                                  */
/* Description:     reference node in proof variant tree                     */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_PROVER_NODE_REF_HPP_
#define MDL_PROVER_NODE_REF_HPP_

#include "mdl/prover/mdl_prover.dpp"
#include "mdl/prover/evidence/mdl_prover_evidence_Vector.hpp"

namespace mdl {
namespace prover {
namespace node {

class Ref :
	public Node,
	public Scalar<Ref, allocator :: Prover> {
public :
	Ref
	(
		const Substitution* const,
		const Node* const,
		const Node* const
	);
	virtual ~ Ref();

	// prover :: Node interface
	virtual const Expression* getExpression() const;
	virtual const mdl :: Proposition* getProposition() const;
	virtual const Confirmation& getConfirmation() const;
	virtual value :: Weight getWeight() const;
	virtual void calculateWeightUp() const;
	virtual void calculateWeightDown() const;
	virtual bool operator < (const Node&) const;
	virtual value :: Rate getRate() const;
	virtual void setRate (const value :: Rate) const;
	virtual void searchProof();
	virtual bool isProved() const;
	virtual void buildUp (Level* const);
	virtual void buildDown (const bool = true);
	virtual bool isRoot() const;
	virtual void addDown (const Node*);
	virtual bool liesDown (const Node*) const;

	virtual void showNode
	(
		String&,
		const value :: Integer = 0
	) const;
	virtual void showTree
	(
		String&,
		const value :: Integer = 0,
		const value :: Integer = value :: undefined :: INTEGER
	) const;

	// prover :: Statistics interface
	virtual Size_t getEvidenceVolume() const;
	virtual Size_t getTreeVolume() const;
	virtual Size_t getExpressionVolume() const;
	virtual Size_t getTreeNodeVolume() const;
	virtual Size_t getEvidenceNodeVolume() const;
	virtual Size_t getEvidenceContainerVolume() const;
	virtual Size_t getGarbageVolume() const;

	virtual Size_t getSubstitutionAllVolume() const;
	virtual Size_t getSubstitutionIntVolume() const;
	virtual Size_t getSubstitutionExtVolume() const;
	virtual Size_t getSubstitutionVolume() const;

	virtual value :: Integer getTreeNodeCount() const;
	virtual value :: Integer getEvidenceNodeCount() const;
	virtual value :: Integer getExpressionCount() const;
	virtual value :: Integer getHypCount() const;
	virtual value :: Integer getPropCount() const;
	virtual value :: Integer getRefCount() const;

	// object :: Cloneable interface
	virtual const Node* clone() const;

	// object :: Object interface
	virtual void commitSuicide();
	virtual Size_t getVolume() const;
	virtual Size_t getSizeOf() const;
	virtual void show (String&) const;

private :
	// weight routines
	void calculateWeight() const;

	// build down routines
	void collectConfirmation (const Confirmation&, const bool);
	void addEvidence (const Evidence*);

	void verify() const;

	friend class Prop;
	friend class Hyp;

	class Statistics;
	friend class Statistics;

	// base
	const Substitution* const replacement_;

	// variant tree
	const Node* up_;
	const Node* down_;

	// clone
	mutable const Ref* clone_;

	// weight & rate
	mutable value :: Rate   rate_;
	mutable value :: Weight weight_;

	// all proof evidences (confirmation)
	evidence :: Vector  confirmation_;
};

}
}
}

#endif /*MDL_PROVER_NODE_REF_HPP_*/
