/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File Name:       mdl_prover_node_Ref_Statistics.cpp                       */
/* Description:     statistics for reference node in proof variant tree      */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_PROVER_NODE_REF_STATISTICS_CPP_
#define MDL_PROVER_NODE_REF_STATISTICS_CPP_

#include "mdl/prover/node/mdl_prover_node.hpp"

namespace mdl {
namespace prover {
namespace node {

	/****************************
	 *		Public members
	 ****************************/

	// prover :: Statistics implementation
	Size_t
	Ref :: Statistics :: getEvidenceVolume (const Ref* ref)
	{
		Size_t volume = 0;
		volume += ref->confirmation_.getVolume();
		volume += ref->confirmation_.getEvidenceVolume();
		return volume;
	}
	Size_t
	Ref :: Statistics :: getTreeVolume (const Ref* ref) {
		return getAggregateVolume (ref->replacement_);
	}
	Size_t
	Ref :: Statistics :: getExpressionVolume (const Ref*) {
		return 0;
	}
	Size_t
	Ref :: Statistics :: getTreeNodeVolume (const Ref*) {
		return 0;
	}
	Size_t
	Ref :: Statistics :: getEvidenceNodeVolume (const Ref* ref) {
		return ref->confirmation_.getEvidenceNodeVolume();
	}
	Size_t
	Ref :: Statistics :: getEvidenceContainerVolume (const Ref* ref) {
		return ref->confirmation_.getVolume();
	}
	Size_t
	Ref :: Statistics :: getGarbageVolume (const Ref* node) {
		return node->confirmation_.getGarbageVolume();
	}

	// substitution statistics
	Size_t
	Ref :: Statistics :: getSubstitutionAllVolume (const Ref* node) {
		return node->confirmation_.getSubstitutionVolume();
	}
	Size_t
	Ref :: Statistics :: getSubstitutionIntVolume (const Ref*) {
		return 0;
	}
	Size_t
	Ref :: Statistics :: getSubstitutionExtVolume (const Ref*) {
		return 0;
	}
	Size_t
	Ref :: Statistics :: getSubstitutionVolume (const Ref* node)
	{
		Size_t volume = 0;
		for (value :: Integer i = 0; i < node->confirmation_.getSize(); ++ i) {
			const Substitution* const
				substitution = node->confirmation_.get (i)->getSubstitution();
			volume += getAggregateVolume (substitution);
		}
		return volume;
	}

	// node count statistics
	value :: Integer
	Ref :: Statistics :: getTreeNodeCount (const Ref*) {
		return 1;
	}
	value :: Integer
	Ref :: Statistics :: getEvidenceNodeCount (const Ref* ref) {
		return ref->confirmation_.getSize();
	}
	value :: Integer
	Ref :: Statistics :: getExpressionCount (const Ref*) {
		return 0;
	}
	value :: Integer
	Ref :: Statistics :: getHypCount (const Ref*) {
		return 0;
	}
	value :: Integer
	Ref :: Statistics :: getPropCount (const Ref*) {
		return 0;
	}
	value :: Integer
	Ref :: Statistics :: getRefCount (const Ref*) {
		return 1;
	}

	/****************************
	 *		Private members
	 ****************************/

	template<class T>
	Size_t
	Ref :: Statistics :: getAggregateVolume (const T* object)
	{
		Size_t volume = 0;
		if (object != NULL) {
			volume += object->getVolume();
			volume += object->getSizeOf();
		}
		return volume;
	}
}
}
}

#endif /*MDL_PROVER_NODE_REF_STATISTICS_CPP_*/
