/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File Name:       mdl_prover_node_Ref_Statistics.hpp                       */
/* Description:     statistics for reference node in proof variant tree      */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_PROVER_NODE_REF_STATISTICS_HPP_
#define MDL_PROVER_NODE_REF_STATISTICS_HPP_

#include "mdl/prover/mdl_prover.dpp"

namespace mdl {
namespace prover {
namespace node {

class Ref :: Statistics {
public :
	// prover :: Statistics interface
	static Size_t getEvidenceVolume (const Ref*);
	static Size_t getTreeVolume (const Ref*);
	static Size_t getExpressionVolume (const Ref*);
	static Size_t getTreeNodeVolume (const Ref*);
	static Size_t getEvidenceNodeVolume (const Ref*);
	static Size_t getEvidenceContainerVolume (const Ref*);
	static Size_t getGarbageVolume (const Ref*);

	// substitutions statistics
	static Size_t getSubstitutionAllVolume (const Ref*);
	static Size_t getSubstitutionIntVolume (const Ref*);
	static Size_t getSubstitutionExtVolume (const Ref*);
	static Size_t getSubstitutionVolume (const Ref*);

	// node count statistics
	static value :: Integer getTreeNodeCount (const Ref*);
	static value :: Integer getEvidenceNodeCount (const Ref*);
	static value :: Integer getExpressionCount (const Ref*);
	static value :: Integer getHypCount (const Ref*);
	static value :: Integer getPropCount (const Ref*);
	static value :: Integer getRefCount (const Ref*);

private :
	template<class T>
	static Size_t getAggregateVolume (const T*);
};

}
}
}

#endif /*MDL_PROVER_NODE_REF_STATISTICS_HPP_*/
