/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File Name:       mdl_prover_node_Root.hpp                                 */
/* Description:     root node in proof variant tree                          */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_PROVER_ROOT_HYP_HPP_
#define MDL_PROVER_ROOT_HYP_HPP_

#include "mdl/prover/mdl_prover.dpp"
#include "mdl/prover/node/mdl_prover_node_Hyp.hpp"
#include "mdl/prover/proof/mdl_prover_proof_Vector.hpp"

namespace mdl {
namespace prover {
namespace node {

class Root :
	public Node,
	public Scalar<Root, allocator :: Prover> {
public :
	Root (const Expression* const);
	virtual ~ Root();

	const proof :: Vector& getProofs() const;

	// prover :: Node interface
	virtual const Expression* getExpression() const;
	virtual const mdl :: Proposition* getProposition() const;
	virtual const Confirmation& getConfirmation() const;
	virtual value :: Weight getWeight() const;
	virtual void calculateWeightUp() const;
	virtual void calculateWeightDown() const;
	virtual bool operator < (const Node&) const;
	virtual value :: Rate getRate() const;
	virtual void setRate (const value :: Rate) const;
	virtual void searchProof();
	virtual bool isProved() const;
	virtual void buildUp (Level* const);
	virtual void buildDown (const bool = true);
	virtual bool isRoot() const;
	virtual void addDown (const Node*);
	virtual bool liesDown (const Node*) const;
	virtual void showNode
	(
		String&,
		const value :: Integer = 0
	) const;
	virtual void showTree
	(
		String&,
		const value :: Integer = 0,
		const value :: Integer = value :: undefined :: INTEGER
	) const;

	// prover :: Statistics interface
	virtual Size_t getEvidenceVolume() const;
	virtual Size_t getTreeVolume() const;
	virtual Size_t getExpressionVolume() const;
	virtual Size_t getTreeNodeVolume() const;
	virtual Size_t getEvidenceNodeVolume() const;
	virtual Size_t getEvidenceContainerVolume() const;
	virtual Size_t getGarbageVolume() const;

	virtual Size_t getSubstitutionAllVolume() const;
	virtual Size_t getSubstitutionIntVolume() const;
	virtual Size_t getSubstitutionExtVolume() const;
	virtual Size_t getSubstitutionVolume() const;

	virtual value :: Integer getTreeNodeCount() const;
	virtual value :: Integer getEvidenceNodeCount() const;
	virtual value :: Integer getExpressionCount() const;
	virtual value :: Integer getHypCount() const;
	virtual value :: Integer getPropCount() const;
	virtual value :: Integer getRefCount() const;

	// object :: Cloneable interface
	virtual const Node* clone() const;

	// object :: Object interface
	virtual void commitSuicide();
	virtual Size_t getVolume() const;
	virtual Size_t getSizeOf() const;
	virtual void show (String&) const;

private :
	void collectProofs();

	Hyp hyp_;
	proof :: Vector  proofVector_;
};

}
}
}

#endif /*MDL_PROVER_ROOT_HYP_HPP_*/
