/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File Name:       mdl_prover_proof_Hyp.cpp                                 */
/* Description:     hyp proof element                                        */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_PROVER_PROOF_HYP_CPP_
#define MDL_PROVER_PROOF_HYP_CPP_

namespace mdl {
namespace prover {
namespace proof {

	/****************************
	 *		Public members
	 ****************************/

	inline
	Hyp :: Hyp (const Hyp& hyp) :
	evidence_ (NULL),
	up_ (NULL),
	reference_ (NULL) {
	}
	inline
	Hyp :: Hyp
	(
		const Evidence* evidence,
		const Proof* up
	) :
	evidence_ (evidence),
	up_ (up),
	reference_ (NULL) {
	}
	Hyp :: ~ Hyp() {
	}

	// prover :: Proof implementation
	bool
	Hyp :: isValid() const {
		return up_->isValid();
	}
	void
	Hyp :: verifyIntegrity() const {
		evidence_->verifyIntegrity();
	}
	inline const Evidence*
	Hyp :: getEvidence() const {
		return evidence_;
	}
	index :: Step
	Hyp :: getDepth() const {
		return up_->getDepth();
	}
	value :: Integer
	Hyp :: getCardinality() const {
		return up_->getCardinality();
	}
	const mdl :: proof :: Step*
	Hyp :: compileStep (mdl :: proof :: Step* const question) const
	{
		up_->compileStep (question);
		if (!evidence_->getNode()->isRoot()) {
			reference_ = new form :: proof :: Reference
			(
				question->getLocation(),
				Token :: STEP,
				up_->getStep()->getIndex()
			);
			const_cast<mdl :: proof :: Reference*>(reference_)->
				build (question->getScope());
		}
		return up_->getStep();
	}
	void
	Hyp :: compileClaim (mdl :: proof :: Claim* const claim) const
	{
		#ifndef DEBUG
		if (reference_ != NULL) {
			return;
		}
		#endif
		up_->compileClaim (claim);
		if (!evidence_->getNode()->isRoot()) {
			if (up_->getEvidence()->getKind() == mdl :: prover :: Evidence :: PROP) {
				reference_ = new form :: proof :: Reference
				(
					claim->getLocation(),
					Token :: STEP,
					up_->getStep()->getIndex()
				);
				const_cast<mdl :: proof :: Reference*>(reference_)->
					build (claim->getProof()->getScope());
			} else {
				reference_ = up_->getReference();
			}
		} else {
			const mdl :: proof :: Qed* const
				qed = new form :: proof :: Qed (claim, up_->getStep());
			const_cast<mdl :: Proof*>(claim->getProof())->addQed (qed);
		}
	}
	const mdl :: proof :: Reference*
	Hyp :: getReference() const {
		return reference_;
	}
	const mdl :: proof :: Step*
	Hyp :: getStep() const {
		return up_->getStep();
	}

using manipulator :: space;
using manipulator :: tab;
using manipulator :: endLine;

	// object :: Object interface
	void
	Hyp :: commitSuicide() {
		delete this;
	}
	Size_t
	Hyp :: getVolume() const {
		return 0;
	}
	Size_t
	Hyp :: getSizeOf() const {
		return sizeof (Hyp);
	}
	void
	Hyp :: show (String& string) const
	{
		string << "############################### - Hyp Node - ################################################" << endLine;
		string << "this: " << this << endLine;
		string << "node: " << evidence_ << endLine;
		string << "expression: " << *(evidence_->getNode()->getExpression()) << endLine;
		string << "up: " << up_ << endLine;
		string << "############################### - Hyp Node - ################################################" << endLine;
		string << endLine;
		string << endLine;
		string << endLine;
		string << endLine;
		string << endLine;
		string << endLine;
		string << endLine;
		string << endLine;
		string << endLine;
		string << endLine;
		string << endLine;
		string << endLine;
		string << endLine;
		string << endLine;
		string << endLine;
	}
}
}
}

#endif /*MDL_PROVER_PROOF_HYP_CPP_*/
