/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File Name:       mdl_prover_proof_Hyp.hpp                                 */
/* Description:     hyp proof element                                        */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_PROVER_PROOF_HYP_HPP_
#define MDL_PROVER_PROOF_HYP_HPP_

#include "mdl/prover/mdl_prover.dpp"

namespace mdl {
namespace prover {
namespace proof {

class Hyp :
	public Proof,
	public Scalar<Hyp, allocator :: Prover> {
public :
	Hyp (const Hyp&);
	Hyp (const Evidence*, const Proof*);
	virtual ~ Hyp();

	// prover :: Proof interface
	virtual bool isValid() const;
	virtual void verifyIntegrity() const;
	virtual const Evidence* getEvidence() const;
	virtual index :: Step getDepth() const;
	virtual value :: Integer getCardinality() const;
	virtual const mdl :: proof :: Step* compileStep (mdl :: proof :: Step* const) const;
	virtual void compileClaim (mdl :: proof :: Claim* const) const;
	virtual const mdl :: proof :: Reference* getReference() const;
	virtual const mdl :: proof :: Step* getStep() const;

	// object :: Object interface
	virtual void commitSuicide();
	virtual Size_t getVolume() const;
	virtual Size_t getSizeOf() const;
	virtual void show (String&) const;

private :
	friend class Prop;
	const Evidence* evidence_;
	const Proof* up_;
	mutable const mdl :: proof :: Reference* reference_;
};

}
}
}

#endif /*MDL_PROVER_PROOF_HYP_HPP_*/
