/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File Name:       mdl_prover_proof_Prop.hpp                                */
/* Description:     prop proof element                                       */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_PROVER_PROOF_PROP_HPP_
#define MDL_PROVER_PROOF_PROP_HPP_

#include "mdl/prover/mdl_prover.dpp"

namespace mdl {
namespace prover {
namespace proof {

class Prop :
	public Proof,
	public Scalar<Prop, allocator :: Prover> {
public :
	Prop (const Prop&);
	Prop
	(
		const Evidence*,
		const Substitution*,
		const index :: Arity
	);
	virtual ~ Prop();

	void addUp (const index :: Arity, const Proof*);

	// prover :: Proof interface
	virtual bool isValid() const;
	virtual void verifyIntegrity() const;
	virtual const Evidence* getEvidence() const;
	virtual index :: Step getDepth() const;
	virtual value :: Integer getCardinality() const;
	virtual const mdl :: proof :: Step* compileStep (mdl :: proof :: Step* const) const;
	virtual void compileClaim (mdl :: proof :: Claim* const) const;
	virtual const mdl :: proof :: Reference* getReference() const;
	virtual const mdl :: proof :: Step* getStep() const;

	// object :: Object interface
	virtual void commitSuicide();
	virtual Size_t getVolume() const;
	virtual Size_t getSizeOf() const;
	virtual void show (String&) const;

private :
	typedef
		mdl :: array :: Array
		<
			const Proof*,
			storage :: ByValue,
			allocator :: Prover
		>
		Array_;

	void checkValidity();
	void compileStepUp (mdl :: proof :: Step* const) const;
	void compileClaimUp (mdl :: proof :: Claim* const) const;
	void createStep (mdl :: proof :: Step* const, const index :: Step) const;
	const mdl :: proof :: References* createReferences() const;
	void addVariables (mdl :: proof :: Claim* const) const;

	const Evidence* evidence_;
	const Substitution* substitutionInner_;
	const Substitution* substitutionOuter_;
	Array_ up_;

	index :: Step depth_;
	value :: Integer cardinality_;

	bool isValid_;
	mutable const mdl :: proof :: Step* step_;
};

}
}
}

#endif /*MDL_PROVER_PROOF_PROP_HPP_*/
