/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File Name:       mdl_prover_proof_Ref.cpp                                 */
/* Description:     reference proof element                                  */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_PROVER_PROOF_REF_CPP_
#define MDL_PROVER_PROOF_REF_CPP_

namespace mdl {
namespace prover {
namespace proof {

	/****************************
	 *		Public members
	 ****************************/

	inline
	Ref :: Ref
	(
		const Evidence* evidence,
		const Substitution* substitution,
		const Proof* up
	) :
	evidence_ (evidence),
	substitution_ (substitution),
	up_ (up),
	reference_ (NULL) {
	}
	Ref :: ~ Ref() {
	}

	// prover :: Proof implementation
	bool
	Ref :: isValid() const {
		return up_->isValid();
	}
	void
	Ref :: verifyIntegrity() const {
		evidence_->verifyIntegrity();
	}
	inline const Evidence*
	Ref :: getEvidence() const {
		return evidence_;
	}
	index :: Step
	Ref :: getDepth() const {
		return up_->getDepth();
	}
	value :: Integer
	Ref :: getCardinality() const {
		return up_->getCardinality();
	}
	const mdl :: proof :: Step*
	Ref :: compileStep (mdl :: proof :: Step* const question) const
	{
		up_->compileStep (question);
		const mdl :: proof :: Reference* const
			reference = up_->getReference();
		reference_ = new form :: proof :: Reference (reference);

		return up_->getStep();
	}
	void
	Ref :: compileClaim (mdl :: proof :: Claim* const claim) const
	{
		#ifndef DEBUG
		if (reference_ != NULL) {
			return;
		}
		#endif

		if (up_->getReference() == NULL) {
			up_->compileClaim (claim);
			reference_ = up_->getReference();
		} else {
			const mdl :: proof :: Reference* const
				reference = up_->getReference();
			reference_ = new form :: proof :: Reference (reference);
		}
	}
	const mdl :: proof :: Reference*
	Ref :: getReference() const {
		return reference_;
	}
	const mdl :: proof :: Step*
	Ref :: getStep() const {
		return up_->getStep();
	}

using manipulator :: space;
using manipulator :: tab;
using manipulator :: endLine;

	// object :: Object interface
	void
	Ref :: commitSuicide() {
		delete this;
	}
	Size_t
	Ref :: getVolume() const {
		return getAggregateVolume (substitution_);
	}
	Size_t
	Ref :: getSizeOf() const {
		return sizeof (Ref);
	}
	void
	Ref :: show (String& string) const
	{
		string << "############################### - Ref Node - ################################################" << endLine;
		string << "this: " << this << endLine;
		string << "evidence: " << evidence_ << endLine;
		string << "up: " << up_ << endLine;
		string << "****************************************************" << endLine;
		string << endLine;
		string << endLine;
		string << endLine;
		string << "############################### - Ref Node - ################################################" << endLine;
		string << endLine;
		string << endLine;
		string << endLine;
		string << endLine;
		string << endLine;
		string << endLine;
		string << endLine;
		string << endLine;
		string << endLine;
		string << endLine;
		string << endLine;
		string << endLine;
		string << endLine;
		string << endLine;
		string << endLine;
	}
}
}
}

#endif /*MDL_PROVER_PROOF_REF_CPP_*/
