/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File Name:       mdl_prover_proof_Top.cpp                                 */
/* Description:     top (premise) proof element                              */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_PROVER_PROOF_TOP_CPP_
#define MDL_PROVER_PROOF_TOP_CPP_

namespace mdl {
namespace prover {
namespace proof {

	/****************************
	 *		Public members
	 ****************************/

	inline
	Top :: Top (const Top& top) :
	evidence_ (top.evidence_),
	hypothesis_ (NULL),
	reference_ (NULL) {
	}
	inline
	Top :: Top (const Evidence* evidence) :
	evidence_ (dynamic_cast<const evidence :: Top*>(evidence)),
	hypothesis_ (NULL),
	reference_ (NULL) {
	}
	Top :: ~ Top() {
	}

	// prover :: Proof implementation
	bool
	Top :: isValid() const {
		return true;
	}
	void
	Top :: verifyIntegrity() const {
		evidence_->verifyIntegrity();
	}
	inline const Evidence*
	Top :: getEvidence() const {
		return evidence_;
	}
	index :: Step
	Top :: getDepth() const {
		return 0;
	}
	value :: Integer
	Top :: getCardinality() const {
		return 0;
	}
	const mdl :: proof :: Step*
	Top :: compileStep (mdl :: proof :: Step* const question) const
	{
		const index :: Step
			index = evidence_->premise_->getIndex();
		reference_ = new form :: proof :: Reference
		(
			question->getLocation(),
			evidence_->premise_->getKind(),
			index,
			evidence_->premise_
		);
		const_cast<mdl :: proof :: Reference*>(reference_)->
			build (question->getScope());
		return question;
	}
	void
	Top :: compileClaim (mdl :: proof :: Claim* const claim) const
	{
		#ifndef DEBUG
		if (reference_ != NULL) {
			return;
		}
		#endif

		index :: Arity
			refIndex = claim->getLink()->getReferences()->find (evidence_->premise_);

		if (refIndex == index :: undefined :: ARITY) {
			const index :: Step
				hypIndex = evidence_->premise_->getIndex();
			hypothesis_ = new form :: proof :: Reference
			(
				claim->getLocation(),
				evidence_->premise_->getKind(),
				hypIndex,
				evidence_->premise_
			);
			const_cast<mdl :: proof :: Reference*>(hypothesis_)->
				build (claim->getProof()->getScope());
			refIndex = claim->getLink()->getReferences()->getArity();
			claim->link()->references()->add (hypothesis_);
		} else {
			hypothesis_ = claim->getLink()->getReferences()->get (refIndex);
		}

		reference_ = new form :: proof :: Reference
		(
			claim->getLocation(),
			Token :: HYP,
			refIndex,
			evidence_->premise_
		);
		const_cast<mdl :: proof :: Reference*>(reference_)->
			build (claim->getScope());
	}
	const mdl :: proof :: Reference*
	Top :: getReference() const {
		return reference_;
	}
	const mdl :: proof :: Step*
	Top :: getStep() const {
		return NULL;
	}

using manipulator :: space;
using manipulator :: tab;
using manipulator :: endLine;

	// object :: Object implementation
	void
	Top :: commitSuicide() {
		delete this;
	}
	Size_t
	Top :: getVolume() const {
		return 0;
	}
	Size_t
	Top :: getSizeOf() const {
		return sizeof (Top);
	}
	void
	Top :: show (String& string) const
	{
		string << "############################### - Top Node - ################################################" << endLine;
		string << "this: " << this << endLine;
		string << "evidence: " << evidence_ << endLine;
		string << "premise: " << evidence_->premise_ << endLine;
		string << "****************************************************" << endLine;
		string << endLine;
		string << endLine;
		string << endLine;
		string << "############################### - Top Node - ################################################" << endLine;
		string << endLine;
		string << endLine;
		string << endLine;
		string << endLine;
		string << endLine;
		string << endLine;
		string << endLine;
		string << endLine;
		string << endLine;
		string << endLine;
		string << endLine;
		string << endLine;
		string << endLine;
		string << endLine;
		string << endLine;
	}
}
}
}

#endif /*MDL_PROVER_PROOF_TOP_CPP_*/
