/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File Name:       mdl_prover_proof_Vector.hpp                              */
/* Description:     proof vector: vector of proofs                           */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_PROVER_PROOF_VECTOR_HPP_
#define MDL_PROVER_PROOF_VECTOR_HPP_

#include "mdl/prover/mdl_prover.dpp"

namespace mdl {
namespace prover {
namespace proof {

class Vector :
	public object :: Object,
	public Scalar<Vector, allocator :: Prover> {
public :
	Vector ();
	Vector (const Vector&);
	Vector (const value :: Integer);
	virtual ~ Vector();

	bool hasValidProofs() const;
	void verifyIntegrity() const;
	bool boundaryMoved() const;
	void markBoundary();
	value :: Integer getBoundary() const;
	value :: Integer getSize() const;
	value :: Integer getDelta() const;
	void add (const Proof*);
	const Proof* get (const value :: Integer) const;
	const Proof* getNew (const value :: Integer) const;
	const mdl :: proof :: Step* compileProof (const mdl :: proof :: Step*) const;
	Size_t getProofVolume() const;

	// object :: Object interface
	virtual void commitSuicide();
	virtual Size_t getVolume() const;
	virtual Size_t getSizeOf() const;
	virtual void show (String&) const;

private :
	typedef
		mdl :: vector :: Vector
		<
			const Proof*,
			storage :: ByValue,
			allocator :: Prover
		>
		ProofVector_;
	enum {
		INITIAL_PROOFS_CAPACITY = 0  //8
	};

	const Proof* getLeastProof() const;
	const mdl :: proof :: Step* createStep
	(
		const Proof*,
		const mdl :: proof :: Step*
	) const;
	const mdl :: proof :: Step* createClaim
	(
		const Proof*,
		const mdl :: proof :: Step*
	) const;

	void verifyCompilation (const mdl :: proof :: Step*) const;
	const mdl :: proof :: Step* verifyClaim
	(
		const Proof*,
		const mdl :: proof :: Step*
	) const;

	ProofVector_ proofVector_;
	value :: Integer boundary_;
};

}
}
}

#endif /*MDL_PROVER_PROOF_VECTOR_HPP_*/
