/*****************************************************************************/
/* Project name:    mdl - mathematics development language                   */
/* File Name:       mdl_types_value.hpp                                      */
/* Description:     basic types for values                                   */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MDL_TYPES_VALUE_HPP_
#define MDL_TYPES_VALUE_HPP_

namespace mdl {
namespace value {

	// Basic type for any integer
	typedef
		nstd :: U_int32_t
		Integer;
	// type for 64 bit integer values
	typedef
		nstd :: U_int64_t
		Long;
	// Value type for identificator names
	typedef
		nstd :: U_int32_t
		Name;
	// Value type for literals - symbols in expressions
	typedef
		nstd :: U_int32_t
		Literal;
	// Weight type for estimation of fitness for the nodes of a tree.
	typedef
		double
		Weight;
	// Type for different factors.
	typedef
		double
		Factor;
	// Rate type for sorting. 0 is the highest rate
	typedef
		nstd :: U_int32_t
		Rate;
	// kind is a small type for distinction of objects
	typedef
		nstd :: U_int8_t
		Kind;
}
}

#include "types/value/undefined/mdl_types_value_undefined.hpp"

namespace mdl {
namespace value {

	const Rate
		HIGHEST_RATE = 0;
	const Rate
		LOWEST_RATE = undefined :: RATE - 1;
}
}

#endif /*MDL_TYPES_VALUE_HPP_*/
