/*****************************************************************************/
/* Project name:    mm - decompiler from metamath to mdl                     */
/* File Name:       mm_cut_lexer_Token.cpp                                   */
/* Description:     mm source file cutting lexer tokens                      */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MM_CUT_LEXER_TOKEN_CPP_
#define MM_CUT_LEXER_TOKEN_CPP_

namespace mm {
namespace cut {
namespace lexer {

	/********************************
	 *		Public static members
	 ********************************/

	const Token Token :: unknown_ (UNKNOWN, "unknown");
	const Token Token :: content_ (CONTENT, "<content>");
	const Token Token :: header_ (HEADER, "<header>");
	const Token Token :: chapterBegin_
		(CHAPTER_BEGIN, "$(\n#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#");
	const Token Token :: chapterEnd_
		(CHAPTER_END, "#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#\n$)");
	const Token Token :: sectionBegin_
		(SECTION_BEGIN, "$(\n=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=");
	const Token Token :: sectionEnd_
		(SECTION_END, "=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=\n$)");

	const Token Token :: eof_ (EOF_, "<eof>");

	const Token Token :: tokens_[NUMBER_OF_TOKENS] =
	{
		unknown_, content_, header_,
		chapterBegin_, chapterEnd_,
		sectionBegin_, sectionEnd_,
		eof_
	};

	/****************************
	 *		Public members
	 ****************************/

	const char*
	Token :: getString (const Type type)
	{
		for (int i = 0 ; i < NUMBER_OF_TOKENS; i ++) {
			if (tokens_[i].getType() == type) {
				return tokens_[i].getString();
			}
		}
		return unknown_.getString();
	}

	// nstd :: Object implementation
	void
	Token :: commitSuicide() {
		delete this;
	}
	nstd :: Size_t
	Token :: getSizeOf() const {
		return sizeof (Token);
	}
	nstd :: Size_t
	Token :: getVolume() const {
		return 0;
	}
}
}
}

#endif /* MM_CUT_LEXER_TOKEN_CPP_ */
