/*****************************************************************************/
/* Project name:    mm - decompiler from metamath to mdl                     */
/* File Name:       mm_cut_lexer_Token.hpp                                   */
/* Description:     mm source file cutting lexer tokens                      */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef MM_CUT_LEXER_TOKEN_HPP_
#define MM_CUT_LEXER_TOKEN_HPP_

#include <nstl.hpp>

namespace mm {
namespace cut {
namespace lexer {

class Token : public nstd :: Object {
public :
	enum Type
	{
		UNKNOWN,       ///< unknown token
		CONTENT,       /// meaningful blocks of source
		HEADER,        /// header of the block
		CHAPTER_BEGIN, ///< "$(\n#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#"
		CHAPTER_END,   ///< "#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#\n$)"
		SECTION_BEGIN, ///< "$(\n=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-="
		SECTION_END,   ///< "=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=\n$)"
		EOF_           ///< end of file
	};
	enum {
		NUMBER_OF_TOKENS = EOF_ + 1
	};

	static const Token unknown_;
	static const Token content_;
	static const Token header_;
	static const Token chapterBegin_;
	static const Token chapterEnd_;
	static const Token sectionBegin_;
	static const Token sectionEnd_;
	static const Token eof_;

	Token (const Type, const char* const);
	Token (const Token&);
	virtual ~ Token();

	static const char* getString (const Type);
	const char* getString() const {
		return string_;
	}
	Type getType() const {
		return type_;
	}

	// nstd :: Object interface
	virtual void commitSuicide();
	virtual nstd :: Size_t getSizeOf() const;
	virtual nstd :: Size_t getVolume() const;
	virtual void show (nstd :: String<>&) const;

private :
	static const Token tokens_[NUMBER_OF_TOKENS];
	const Type type_;
	const char* const string_;
};

	template<class A>
	nstd :: String<A>&
	operator << (nstd :: String<A>&, const Token&);
}
}
}

#include "mm/cut/lexer/mm_cut_lexer_Token.ipp"

#endif /*MM_CUT_LEXER_TOKEN_HPP_*/
