/*****************************************************************************/
/* Project name:    nsgl - non-standard general library                      */
/* File name:       nsgl_config_Config.cpp                                   */
/* Description:     resulting config class                                   */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef NSGL_CONFIG_CONFIG_CPP_
#define NSGL_CONFIG_CONFIG_CPP_

#include "config/nsgl_config_Config.hpp"

namespace nstd {
namespace config {

	/****************************
	 *		Protected members
	 ****************************/

	void 
	Config :: showOptions()
	{
		General :: showOptions();
		Memory :: showOptions();
		Program :: showOptions();
		Paths :: showOptions();
	}
	void 
	Config :: init()
	{
		General :: init();
		Memory :: init();
		Paths :: init();
		Program :: init();
	}
	void
	Config :: release()
	{
		General :: release();
		Memory :: release();
		Paths :: release();
		Program :: release();
	}
	Size_t
	Config :: getVolume()
	{
		Size_t volume = 0;
		volume += General :: getVolume();
		volume += Memory :: getVolume();
		volume += Paths :: getVolume();
		volume += Program :: getVolume();
		return volume;
	}

	bool
	Config :: proceedStringOption (const char* opt, const int argCount, const char* argValues[], int& i)
	{
		if (General :: proceedStringOption (opt)) {
			return true;
		}
		if (Memory :: proceedStringOption (opt)) {
			return true;
		}
		if (Program :: proceedStringOption (opt)) {
			return true;
		}
		if (Paths :: proceedStringOption (opt, argCount, argValues, i)) {
			return true;
		}
		return false;
	}

using std :: cerr;
using std :: endl;

	bool
	Config :: proceedOneCharOption (const char* opt, const int argCount, const char* argValues[], int& i)
	{
		if (opt [1] != '\0') {
			cerr << "option which starts with a single \"-\" must be one character." << endl;
			cerr << "option: \"" << opt << "\" consists of more then one character." << endl;
			abortParsing_ = true;
			return false;
		}
		if (General :: proceedOneCharOption (opt)) {
			return true;
		}
		if (Memory :: proceedOneCharOption (opt)) {
			return true;
		}
		if (Program :: proceedOneCharOption (opt)) {
			return true;
		}
		if (Paths :: proceedOneCharOption (opt, argCount, argValues, i)) {
			return true;
		}
		return false;
	}
}
}

#endif /*NSGL_CONFIG_CONFIG_CPP_*/
