/*****************************************************************************/
/* Project name:    nsgl - non-standard general library                      */
/* File name:       nsgl_config_General.cpp                                  */
/* Description:     general purpose config flags                             */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef NSGL_CONFIG_GENERAL_CPP_
#define NSGL_CONFIG_GENERAL_CPP_

#include "config/nsgl_config_General.hpp"

namespace nstd {
namespace config {

	/****************************
	 *		Public members
	 ****************************/

	inline bool
	General :: showHelp() {
		return showHelp_;
	}
	inline bool
	General :: translate() {
		return translate_;
	}
	inline bool
	General :: removeComments() {
		return removeComments_;
	}
	inline bool
	General :: verbose() {
		return verbose_;
	}
	inline bool
	General :: warnings() {
		return warnings_;
	}
	inline General :: Verbosity_
	General :: verbosity() {
		return verbosity_;
	}
	inline General :: Waringity_
	General :: warningity() {
		return warningity_;
	}

	/****************************
	 *		Protected members
	 ****************************/

using std :: cout;
using std :: endl;

	void
	General :: showOptions()
	{
		cout << "  -h,  --help                   display basic usage/options information" << endl;
		cout << "  -t,  --translate              translate source to target language" << endl;
		cout << "  -r,  --remove-comments        remove comments" << endl;
		cout << endl;
		cout << "  -v,  --verbose                verbose mode (by default - minimal)" << endl;
		cout << "       --verbose-minimal        minimal verbose mode" << endl;
		cout << "       --verbose-standard       standard verbose mode" << endl;
		cout << "       --verbose-extended       extended verbose mode" << endl;
		cout << "       --verbose-total          total verbose mode" << endl;
		cout << endl;
		cout << "  -W,  --warnings               show warnings (by default - standard)" << endl;
		cout << "       --warnings-standard      show general warnings" << endl;
		cout << "       --warnings-extended      show extended warnings" << endl;
		cout << "       --warnings-all           show all warnings" << endl;
		cout << endl;
	}
	inline void
	General :: init() {
	}
	inline void
	General :: release() {
	}
	inline Size_t
	General :: getVolume() {
		return 0;
	}

	bool
	General :: proceedStringOption (const char* option)
	{
		if (equalStrings (option, "help")) {
			showHelp_ = true;
			abortParsing_ = true;
			return true;
		}
		if (equalStrings (option, "translate")) {
			translate_ = true;
			return true;
		}
		if (equalStrings (option, "remove-comments")) {
			removeComments_ = true;
			return true;
		}
		if (proceedVerboseOption (option)) {
			return true;
		}
		if (proceedWarningsOption (option)) {
			return true;
		}
		return false;
	}

	bool
	General :: proceedOneCharOption (const char* option)
	{
		switch (*option) {
		case 'h' :
			showHelp_ = true;
			abortParsing_ = true;
			return true;
		case 't' :
			translate_ = true;
			return true;
		case 'v' :
			if (!verbose_) {
				verbose_ = true;
				verbosity_ = VERBOSE_MINIMAL;
			}
			return true;
		case 'W' :
			if (!warnings_) {
				warnings_ = true;
				warningity_ = WARNINGS_DEFAULT;
			}
			return true;
		case 'r' :
			removeComments_ = true;
			return true;
		default :
			return false;
		}
	}

	/****************************
	 *		Private members
	 ****************************/

	bool
	General :: proceedVerboseOption (const char* option)
	{
		if (equalStrings (option, "verbose")) {
			verbose_ = true;
			verbosity_ = VERBOSE_MINIMAL;
			return true;
		}
		if (equalStrings (option, "verbose-minimal")) {
			verbose_ = true;
			verbosity_ = VERBOSE_MINIMAL;
			return true;
		}
		if (equalStrings (option, "verbose-standard")) {
			verbose_ = true;
			verbosity_ = VERBOSE_STANDARD;
			return true;
		}
		if (equalStrings (option, "verbose-extended")) {
			verbose_ = true;
			verbosity_ = VERBOSE_EXTENDED;
			return true;
		}
		if (equalStrings (option, "verbose-total")) {
			verbose_ = true;
			verbosity_ = VERBOSE_TOTAL;
			return true;
		}
		return false;
	}
	bool
	General :: proceedWarningsOption (const char* option)
	{
		if (equalStrings (option, "warnings")) {
			warnings_ = true;
			warningity_ = WARNINGS_DEFAULT;
			return true;
		}
		if (equalStrings (option, "warnings-standard")) {
			warnings_ = true;
			warningity_ = WARNINGS_STANDARD;
			return true;
		}
		if (equalStrings (option, "warnings-extended")) {
			warnings_ = true;
			warningity_ = WARNINGS_EXTENDED;
			return true;
		}
		if (equalStrings (option, "warnings-all")) {
			warnings_ = true;
			warningity_ = WARNINGS_ALL;
			return true;
		}
		return false;
	}

	/************************************
	 *		Static private attributes
	 ************************************/

	bool General :: showHelp_ = false;
	bool General :: translate_ = false;
	bool General :: removeComments_ = false;
	bool General :: verbose_ = false;
	bool General :: warnings_ = false;

	General :: Verbosity_
		General :: verbosity_ = General :: VERBOSE_DEFAULT;
	General :: Waringity_
		General :: warningity_ = General :: WARNINGS_DEFAULT;
}
}

#endif /*NSGL_CONFIG_GENERAL_CPP_*/
