/*****************************************************************************/
/* Project name:    nsgl - non-standard general library                      */
/* File name:       nsgl_config_General.hpp                                  */
/* Description:     general purpose config flags                             */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef NSGL_CONFIG_GENERAL_HPP_
#define NSGL_CONFIG_GENERAL_HPP_

#include "config/nsgl_config_Base.hpp"

namespace nstd {
namespace config {

class General : public config :: Base {
public :
	enum Verbosity_ {
		VERBOSE_NONE = 0,
		VERBOSE_MINIMAL  = 1,
		VERBOSE_STANDARD = 2,
		VERBOSE_EXTENDED = 3,
		VERBOSE_TOTAL    = 4,
		VERBOSE_DEFAULT = VERBOSE_NONE
	};
	enum Waringity_ {
		WARNINGS_NONE = 0,
		WARNINGS_STANDARD = 1,
		WARNINGS_EXTENDED = 2,
		WARNINGS_ALL      = 3,
		WARNINGS_DEFAULT = WARNINGS_STANDARD
	};
	static bool showHelp();
	static bool translate();
	static bool removeComments();
	static bool verbose();
	static bool warnings();
	static Verbosity_ verbosity();
	static Waringity_ warningity();

protected :
	// Shows the usage message.
	static void showOptions();
	static void init();
	static void release();
	static Size_t getVolume();

	static bool proceedStringOption (const char*);
	static bool proceedOneCharOption (const char*);

private :
	static bool proceedVerboseOption (const char*);
	static bool proceedWarningsOption (const char*);

	// The help message is shown iff it is set to \c true.
	static bool showHelp_;
	// The source is translated iff it is set to \c true.
	static bool translate_;
	// The translation/checking is verbose, iff set to \c true.
	static bool verbose_;
	// Show warnings, iff set to \c true.
	static bool warnings_;
	// Comments are removed at translation, iff set to \c true.
	static bool removeComments_;
	// Verbosity level
	static Verbosity_  verbosity_;
	// Warningity level
	static Waringity_  warningity_;
};

}
}

#endif /*NSGL_CONFIG_GENERAL_HPP_*/
