/*****************************************************************************/
/* Project name:    nsgl - non-standard general library                      */
/* File name:       nsgl_config_Memory.cpp                                   */
/* Description:     memory management config flags                           */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef NSGL_CONFIG_MEMORY_CPP_
#define NSGL_CONFIG_MEMORY_CPP_

#include "config/nsgl_config_Memory.hpp"

namespace nstd {
namespace config {

	/****************************
	 *		Public members
	 ****************************/

	inline bool
	Memory :: showMemory()
	{
		return
			(showMemoryUsage_ || showMemoryAlloc_ || showMemoryBalance_) ||
			(showMemoryDisposed_ || showMemoryFragmentation_ || showMemoryFragmentation_);
	}
	inline bool
	Memory :: showMemoryUsage() {
		return showMemoryUsage_;
	}
	inline bool
	Memory :: showMemoryAlloc() {
		return showMemoryAlloc_;
	}
	inline bool
	Memory :: showMemoryBalance() {
		return showMemoryBalance_;
	}
	inline bool
	Memory :: showMemoryDisposed() {
		return showMemoryDisposed_;
	}
	inline bool
	Memory :: showMemoryDetails() {
		return showMemoryDetails_;
	}
	inline bool
	Memory :: showMemoryFragmentation() {
		return showMemoryFragmentation_;
	}

	/****************************
	 *		Protected members
	 ****************************/

using std :: cout;
using std :: endl;

	inline void
	Memory :: showOptions()
	{
		cout << "       --mem-all                show all memory info (except for --mem-fragm)" << endl;
		cout << "       --mem-alloc              show memory allocation info" << endl;
		cout << "       --mem-usage              show memory usage info" << endl;
		cout << "       --mem-detail             show memory detailed info" << endl;
		cout << "       --mem-delta              show memory balance info" << endl;
		cout << "       --mem-disp               show disposed memory info" << endl;
		cout << "       --mem-fragm              show memory fragmentation info" << endl;
		cout << endl;
	}
	inline void
	Memory :: init() {
	}
	inline void
	Memory :: release() {
	}
	inline Size_t
	Memory :: getVolume() {
		return 0;
	}

	bool
	Memory :: proceedStringOption (const char* opt)
	{
		if (equalStrings (opt, "mem-all")) {
			showMemoryAlloc_ = true;
			showMemoryUsage_ = true;
			showMemoryBalance_ = true;
			showMemoryDisposed_ = true;
			showMemoryDetails_ = true;
			return true;
		}
		if (equalStrings (opt, "mem-alloc")) {
			showMemoryAlloc_ = true;
			return true;
		}
		if (equalStrings (opt, "mem-usage")) {
			showMemoryUsage_ = true;
			return true;
		}
		if (equalStrings (opt, "mem-detail")) {
			showMemoryDetails_ = true;
			return true;
		}
		if (equalStrings (opt, "mem-delta")) {
			showMemoryBalance_ = true;
			return true;
		}
		if (equalStrings (opt, "mem-disp")) {
			showMemoryDisposed_ = true;
			return true;
		}
		if (equalStrings (opt, "mem-fragm")) {
			showMemoryFragmentation_ = true;
			return true;
		}
		return false;
	}

	bool
	Memory :: proceedOneCharOption (const char*) {
		return false;
	}

	/************************************
	 *		Static private attributes
	 ************************************/

	bool Memory :: showMemoryUsage_ = false;
	bool Memory :: showMemoryAlloc_ = false;
	bool Memory :: showMemoryBalance_ = false;
	bool Memory :: showMemoryDisposed_ = false;
	bool Memory :: showMemoryDetails_ = false;
	bool Memory :: showMemoryFragmentation_ = false;
}
}

#endif /*NSGL_CONFIG_MEMORY_CPP_*/
