/*****************************************************************************/
/* Project name:    nsgl - non-standard general library                      */
/* File name:       nsgl_config_Paths.hpp                                    */
/* Description:     paths config flags                                       */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef NSGL_CONFIG_PATHS_HPP_
#define NSGL_CONFIG_PATHS_HPP_

#include "nsgl_Path.hpp"
#include "config/nsgl_config_Base.hpp"

namespace nstd {
namespace config {

class Paths : public Base {
public :
	typedef Path<> Path_;

	static Path_*& source();
	static Path_*& target();
	static Path_*& log();
	static const Path_* getSource();
	static const Path_* getTarget();
	static const Path_* getLog();
	static bool overwrite();

protected :
	static void showOptions();
	static void init();
	static void release();
	static Size_t getVolume();
	static bool check();

	static bool proceedStringOption (const char*, const int, const char* [], int&);
	static bool proceedOneCharOption (const char*, const int, const char* [], int&);

private :
	static bool checkSource();
	static bool checkTarget();
	static bool checkLog();
	static bool checkBoth();

	static Path_* source_;
	static Path_* target_;
	static Path_* log_;

	// Overwrite output file without warning
	static bool overwrite_;

	enum {
		INITIAL_EXTENSION_CAPACITY = 16
	};
};

}
}

#endif /*NSGL_CONFIG_PATHS_HPP_*/
