/*****************************************************************************/
/* Project name:    nsgl - non-standard general library                      */
/* File name:       nsgl_config_Program.cpp                                  */
/* Description:     program description config flags                         */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef NSGL_CONFIG_PROGRAM_CPP_
#define NSGL_CONFIG_PROGRAM_CPP_

#include <config.h>

#include "config/nsgl_config_Program.hpp"

namespace nstd {
namespace config {

	/****************************
	 *		Public members
	 ****************************/

	inline String<>&
	Program :: version() {
		return *version_;
	}
	inline String<>&
	Program :: programName() {
		return *programName_;
	}
	inline String<>&
	Program :: programDescription() {
		return *programDescription_;
	}
	inline const String<>&
	Program :: getVersion() {
		return *version_;
	}
	inline const String<>&
	Program :: getProgramName() {
		return *programName_;
	}
	inline const String<>&
	Program :: getProgramDescription() {
		return *programDescription_;
	}
	inline bool
	Program :: showVersion() {
		return showVersion_;
	}

	/****************************
	 *		Protected members
	 ****************************/

using std :: cout;
using std :: endl;

	inline void
	Program :: showProgramUsage() {
		cout << "Usage: " << getProgramName() << " [options] file..." << endl;
	}
	inline void
	Program :: showProgramInfo()
	{
		cout << getProgramDescription() << endl;
		cout << "Version: " << getVersion() << endl;
		cout << "Copyleft (L) 2006-2010 Dmitri Yu. Vlasov" << endl;
	}
	inline void
	Program :: showOptions() {
		cout << "  -V,  --version                version of the " << getProgramName() << endl;
	}
	inline void
	Program :: init()
	{
		version_ = new String<> (INITIAL_VERSION_CAPACITY);
		*version_ << VERSION;
		programName_ = new String<> (INITIAL_PROGRAM_NAME_CAPACITY);
		programDescription_ = new String<> (INITIAL_PROGRAM_DESCRIPTION_CAPACITY);
	}
	inline void
	Program :: release()
	{
		if (version_ != NULL) {
			delete version_;
			version_ = NULL;
		}
		if (programName_ != NULL) {
			delete programName_;
			programName_ = NULL;
		}
		if (programDescription_ != NULL) {
			delete programDescription_;
			programDescription_ = NULL;
		}
	}
	inline Size_t
	Program :: getVolume()
	{
		Size_t volume = 0;
		if (version_ != NULL) {
			volume += version_->getVolume();
			volume += version_->getSizeOf();
		}
		if (programName_ != NULL) {
			volume += programName_->getVolume();
			volume += programName_->getSizeOf();
		}
		if (programDescription_ != NULL) {
			volume += programDescription_->getVolume();
			volume += programDescription_->getSizeOf();
		}
		return volume;
	}

	bool
	Program :: proceedStringOption (const char* opt)
	{
		if (equalStrings (opt, "version")) {
			showProgramInfo();
			abortParsing_ = true;
			return true;
		}
		return false;
	}

	bool
	Program :: proceedOneCharOption (const char* opt)
	{
		if (*opt == 'V') {
			showProgramInfo();
			abortParsing_ = true;
			return true;
		}
		return false;
	}

	/************************************
	 *		Static private attributes
	 ************************************/

	String<>* Program :: version_ = NULL;
	String<>* Program :: programName_ = NULL;
	String<>* Program :: programDescription_ = NULL;

	bool Program :: showVersion_ = false;
}
}

#endif /*NSGL_CONFIG_PROGRAM_CPP_*/
