/*****************************************************************************/
/* Project name:    nsgl - non-standard general library                      */
/* File name:       nsgl_lexer_Buffer.cpp                                    */
/* Description:     lexer buffer for input                                   */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef NSGL_LEXER_BUFFER_CPP_
#define NSGL_LEXER_BUFFER_CPP_

#include "system/nsgl_system.hpp"
#include "lexer/nsgl_lexer_Position.hpp"
#include "lexer/nsgl_lexer_Buffer.hpp"

namespace nstd {
namespace lexer {

	/****************************
	 *		Public members
	 ****************************/

	Buffer :: Buffer () :
	fileName_ (""),
	inputFileStream_ (),

	fileSize_ (0),
	unreadSize_ (0),
	lastReadSize_ (0),

	array_ (),
	buffer1_ (array_),
	buffer2_ (array_ + BUFFER_SIZE) {
		clear();
	}
	Buffer :: ~ Buffer () {
	}

	bool
	Buffer :: open (const char* const fileName)
	{
		fileName_ = fileName;
		fileSize_ = system :: getFileSize (fileName);
		if (fileSize_ == Undefined<Size_t> :: getValue()) {
			return false;
		}
		unreadSize_ = fileSize_;
		lastReadSize_ = 0;
		inputFileStream_.open (fileName_, std :: ios :: binary);
		fill();
		return true;
	}
	void
	Buffer :: close()
	{
		clear();
		unreadSize_ = fileSize_;
		lastReadSize_ = 0;
		inputFileStream_.close();
	}

using manipulator :: endLine;

	// nstd :: Object interface
	void
	Buffer :: commitSuicide() {
		delete this;
	}
	Size_t
	Buffer :: getSizeOf() const {
		return sizeof (Buffer);
	}
	Size_t
	Buffer :: getVolume() const {
		return 0;
	}
	void
	Buffer :: show (String<>& string) const {
		showWhole (string);
	}

	/****************************
	 *		Private members
	 ****************************/

	void
	Buffer :: shift ()
	{
		for (Size_t i = 0; i < BUFFER_SIZE; i ++) {
			const_cast<char*>(buffer1_)[i] = array_ [i + BUFFER_SIZE];
		}
		for (Size_t i = 0; i < BUFFER_SIZE + BUFFER_DELTA; i ++) {
			const_cast<char*>(buffer2_)[i] = chars :: NULL_CHAR;
		}
	}
	void
	Buffer :: clear ()
	{
		for (Size_t i = 0; i < ARRAY_SIZE; i++) {
			const_cast<char*>(array_) [i] = chars :: NULL_CHAR;
		}
	}
	void
	Buffer :: fill ()
	{
		lastReadSize_ =
			(unreadSize_ < BUFFER_SIZE) ?
			unreadSize_ :
			BUFFER_SIZE;
		char*
			buffer = const_cast<char*> (buffer2_);

		inputFileStream_.read (buffer, lastReadSize_);
		unreadSize_ -= lastReadSize_;
	}
}
}

	/*void
	Buffer :: rewind (const Position& position)
	{
		close();
		open();

		const Position :: Index_
			index_1 = position.getIndex();
		Size_t index_2 = 0;
		while (index_2 + BUFFER_SIZE < index_1) {
			index_2 += operator ++ ();
		}
		const Size_t index = index_1 - index_2;
		shift (index);
		fill ();
	}*/

#endif /*NSGL_LEXER_BUFFER_CPP_*/
