/*****************************************************************************/
/* Project name:    nsgl - non-standard general library                      */
/* File name:       nsgl_lexer_Buffer.hpp                                    */
/* Description:     lexer buffer for input                                   */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef NSGL_LEXER_BUFFER_HPP_
#define NSGL_LEXER_BUFFER_HPP_

#include <nstl.hpp>
#include "lexer/nsgl_lexer.dpp"

namespace nstd {
namespace lexer {

class Buffer : public Object {
public :
	Buffer();
	~ Buffer();

	void operator ++ ();

	bool checkEnd (const char*, const Size_t = 1) const;
	void shiftBack (const char*&) const;
	void setInitial (const char*&) const;
	Size_t getIndex (const char*) const;
	Size_t getFileSize() const;

	bool open (const char* const);
	void close();

	template<class S>
	void show (S&) const;

	template<class S>
	void showHead (S&) const;
	template<class S>
	void showWhole (S&) const;
	template<class S>
	void showTail (S&) const;
	template<class S>
	void showLocal (S&, const Size_t) const;

	// nstd :: Object interface
	virtual void commitSuicide();
	virtual Size_t getSizeOf() const;
	virtual Size_t getVolume() const;
	virtual void show (String<>&) const;

private :
	enum {
		BUFFER_SIZE = MEGABYTE * 8,
		BUFFER_DELTA = 16,
		ARRAY_SIZE = BUFFER_SIZE * 2 + BUFFER_DELTA,
		LOCAL_CODE_SIZE = 4096
	};

	const char* fileName_;
	InputFileStream inputFileStream_;
	Size_t fileSize_;
	Size_t unreadSize_;
	Size_t lastReadSize_;

	void clear();
	void fill();
	void shift();

	const char array_[ARRAY_SIZE];
	const char* const buffer1_;
	const char* const buffer2_;
};

}
}

#endif /*NSGL_LEXER_BUFFER_HPP_*/
