/*****************************************************************************/
/* Project name:    nsgl - non-standard general library                      */
/* File name:       nsgl_lexer_Buffer.ipp                                    */
/* Description:     lexer buffer for input                                   */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef NSGL_LEXER_BUFFER_IPP_
#define NSGL_LEXER_BUFFER_IPP_

namespace nstd {
namespace lexer {

	/****************************
	 *		Public members
	 ****************************/

	inline void
	Buffer :: operator ++ ()
	{
		shift();
		fill();
	}

	inline bool
	Buffer :: checkEnd  (const char* ch, const Size_t dist) const {
		return  (dist + (ch - buffer2_) >= BUFFER_SIZE);
	}
	inline void
	Buffer :: shiftBack (const char*& ch) const {
		ch -= BUFFER_SIZE;
	}
	inline void
	Buffer :: setInitial (const char*& ch) const {
		ch = buffer2_;
	}
	inline Size_t
	Buffer :: getIndex (const char* ch) const {
		return ch - buffer2_;
	}
	inline Size_t
	Buffer :: getFileSize() const {
		return fileSize_;
	}

using manipulator :: endLine;

	template<class S>
	void
	Buffer :: show (S& string) const
	{
		string << "code:" << endLine;
		showLocal (string);
	}

	template<class S>
	void
	Buffer :: showHead (S& string) const
	{
		int shift = - (LOCAL_CODE_SIZE / 2);
		while (buffer2_ [shift] == chars :: NULL_CHAR) {
			++ shift;
		}

		char localCode [LOCAL_CODE_SIZE];
		bool empty = true;
		for (Size_t i = 0; i < LOCAL_CODE_SIZE; i++) {
			const char ch = buffer2_ [i + shift];
			localCode [i] = ch;
			if (ch != chars :: NULL_CHAR) {
				empty = false;
			}
		}
		if (!empty) {
			string << "local buffer:" << endLine;
			string << "--------------------------------------------------------------" << endLine;
			string << localCode << endLine;
			string << "--------------------------------------------------------------" << endLine;
		}
	}
	template<class S>
	void
	Buffer :: showWhole (S& string) const
	{
		string << "whole buffer:" << endLine;
		string << "--------------------------------------------------------------" << endLine;
		for (Size_t i = 0; i < ARRAY_SIZE; i++) {
			const char ch = array_ [i];
			switch (ch) {
			case chars :: NULL_CHAR :
				string << "\\0";
				break;
			default :
				string << ch;
			}
		}
		string << endLine;
		string << "--------------------------------------------------------------" << endLine;
	}
	template<class S>
	void
	Buffer :: showTail (S& string) const
	{
		string << "buffer tail:" << endLine;
		string << "string end: " << (int)array_ + BUFFER_SIZE + lastReadSize_ << endLine;
		string << "----------------------------" << endLine;
		for (int i = - LOCAL_CODE_SIZE / 2; i < LOCAL_CODE_SIZE /2; ++ i) {
			const Size_t index = BUFFER_SIZE + lastReadSize_ + i;
			if (!(index < ARRAY_SIZE)) {
				break;
			}
			const char ch = array_ [index];
			if (ch != chars :: NULL_CHAR) {
				string << ch;
			} else {
				string << "\\0";
			}
		}
		string << "----------------------------" << endLine;
	}
	template<class S>
	void
	Buffer :: showLocal (S& string, const Size_t locality) const
	{
		string << "local buffer:" << endLine;
		string << "----------------------------" << endLine;
		for (int i = - LOCAL_CODE_SIZE / 2; i < LOCAL_CODE_SIZE /2; ++ i) {
			const Size_t index = BUFFER_SIZE + locality + i;
			if (!(index < ARRAY_SIZE)) {
				break;
			}
			const char ch = array_ [index];
			if (ch != chars :: NULL_CHAR) {
				string << ch;
			} else {
				string << "\\0";
			}
		}
		string << "----------------------------" << endLine;
	}
}
}

#endif /*NSGL_SOURCE_BUFFER_IPP_*/
