/*****************************************************************************/
/* Project name:    nsgl - non-standard general library                      */
/* File name:       nsgl_lexer_Char.ipp                                      */
/* Description:     lexer char wrapper                                       */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef NSGL_LEXER_CHAR_IPP_
#define NSGL_LEXER_CHAR_IPP_

namespace nstd {
namespace lexer {

	/****************************
	 *		Public members
	 ****************************/

	inline
	Char :: Char () :
	char_ (NULL) {
	}
	inline
	Char :: Char (const char* const ch) :
	char_ (ch) {
	}
	inline
	Char :: Char (const Char& ch) :
	char_ (ch.char_) {
	}

	inline char
	Char :: getValue (const Ptrdiff_t dist) const {
		return *(char_ + dist);
	}
	inline const char*
	Char :: getString() const {
		return char_;
	}

	inline bool
	Char :: isAlphabet() const
	{
		const char ch = getValue();
		const bool upperCaseLiteral = ('A' <= ch) && (ch <= 'Z');
		const bool lowerCaseLiteral = ('a' <= ch) && (ch <= 'z');
		const bool underline = (ch == chars :: UNDERLINE);
		return (lowerCaseLiteral || upperCaseLiteral || underline);
	}
	inline bool
	Char :: isAlphaNumeric () const {
		return isNumber() || isAlphabet ();
	}
	inline bool
	Char :: isWhitespace() const {
		const char ch = getValue();
		return (0 <= ch) && (ch <= chars :: SPACE);
	}
	inline bool
	Char :: isNumber() const {
		const char ch = getValue();
		return ('0' <= ch) && (ch <= '9');
	}
	inline bool
	Char :: isEOF() const {
		return (getValue() == chars :: NULL_CHAR);
	}

	inline void
	Char :: operator ++ () {
		++ char_;
	}
	inline void
	Char :: operator = (const Char& ch) {
		char_ = ch.char_;
	}
	inline bool
	Char :: operator == (const Char& ch) const {
		return (char_ == ch.char_);
	}
	inline bool
	Char :: operator != (const Char& ch) const {
		return !(operator == (ch));
	}
	inline bool
	Char :: operator == (const char ch) const {
		return (ch == *char_);
	}
	inline bool
	Char :: operator != (const char ch) const {
		return !(operator == (ch));
	}
}
}

#endif /*NSGL_LEXER_CHAR_IPP_*/
