/*****************************************************************************/
/* Project name:    nsgl - non-standard general library                      */
/* File name:       nsgl_lexer_Indent.ipp                                    */
/* Description:     indent                                                   */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef NSGL_LEXER_INDENT_IPP_
#define NSGL_LEXER_INDENT_IPP_

namespace nstd {
namespace lexer {

	/****************************
	 *		Public members
	 ****************************/

	inline
	Indent :: Indent() :
	spaces_ (0),
	tabs_ (0) {
	}
	inline Indent :: Indent (const Indent& indent) :
	spaces_ (indent.spaces_),
	tabs_ (indent.tabs_) {
	}

	inline bool
	Indent :: operator == (const Indent& indent) const
	{
		return
			spaces_ == indent.spaces_ &&
			tabs_ == indent.tabs_;
	}
	inline bool
	Indent :: operator != (const Indent& indent) const {
		return !(operator == (indent));
	}
	inline void
	Indent :: operator = (const Indent& indent)
	{
		spaces_ = indent.spaces_;
		tabs_ = indent.tabs_;
	}

	inline void
	Indent :: incSpaces() {
		spaces_ ++;
	}
	inline void
	Indent :: incTabs() {
		tabs_ ++;
	}
	inline void
	Indent :: newLine() {
		spaces_ = 0;
		tabs_ = 0;
	}

	inline void
	Indent :: clear() {
		spaces_ = 0;
		tabs_ = 0;
	}
	
	template<class A>
	void 
	Indent :: print (String<A>& string) const
	{
		for (Index_ i = 0; i < tabs_; i++) {
			string << manipulator :: tab;
		}
		for (Index_ i = 0; i < spaces_; i++) {
			string << manipulator :: space;
		}
	}
	
	/****************************
	 *		Operators
	 ****************************/
	
	template<class A>
	String<A>& 
	operator << (String<A>& string, const Indent& indent) 
	{
		indent.print (string);
		return string;
	}
}
}

#endif /* NSGL_LEXER_INDENT_IPP_ */
