/*****************************************************************************/
/* Project name:    nsgl - non-standard general library                      */
/* File name:       nsgl_lexer_Location.hpp                                  */
/* Description:     location in input                                        */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef NSGL_LEXER_LOCATION_HPP_
#define NSGL_LEXER_LOCATION_HPP_

#include "lexer/nsgl_lexer.dpp"
#include "lexer/nsgl_lexer_Indent.hpp"
#include "lexer/nsgl_lexer_Position.hpp"

namespace nstd {
namespace lexer {

class Location : public Object {
public :
	Location();
	Location (const Source* const);
	Location (const Location&);

	Position& position();
	Indent& indent();
	const Position& position() const;
	const Indent& indent() const;

	bool operator == (const Location&) const;
	bool operator != (const Location&) const;
	void operator = (const Location&);

	template<class S>
	void show (S&) const;

	// nstd :: Object interface
	virtual void commitSuicide();
	virtual Size_t getVolume() const;
	virtual Size_t getSizeOf() const;
	virtual void show (String<>&) const;

private :
	void clear();
	void newLine();

	friend class Source;

	const Source* source_;
	Position position_;
	Indent indent_;
};

}
}

#endif /*NSGL_LEXER_LOCATION_HPP_*/
