/*****************************************************************************/
/* Project name:    nsgl - non-standard general library                      */
/* File name:       nsgl_lexer_Location.ipp                                  */
/* Description:     location in input                                        */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef NSGL_LEXER_LOCATION_IPP_
#define NSGL_LEXER_LOCATION_IPP_

#include "lexer/nsgl_lexer_Source.hpp"

namespace nstd {
namespace lexer {

	/****************************
	 *		Public members
	 ****************************/

	inline
	Location :: Location () :
	source_ (NULL),
	position_ (),
	indent_ () {
	}
	inline
	Location :: Location (const Source* const source) :
	source_ (source),
	position_ (),
	indent_ () {
	}
	inline
	Location :: Location (const Location& location) :
	source_ (location.source_),
	position_ (location.position_),
	indent_ (location.indent_) {
	}

	inline Position&
	Location :: position() {
		return position_;
	}
	inline Indent&
	Location :: indent() {
		return indent_;
	}
	inline const Position&
	Location :: position() const {
		return position_;
	}
	inline const Indent&
	Location :: indent() const {
		return indent_;
	}

	inline bool
	Location :: operator == (const Location& location) const
	{
		return
			(source_ == location.source_) &&
			(position_ == location.position_) &&
			(indent_ == location.indent_);
	}
	inline bool
	Location :: operator != (const Location& location) const {
		return !(operator == (location));
	}
	inline void
	Location :: operator = (const Location& location)
	{
		source_ = location.source_;
		position_ = location.position_;
		indent_ = location.indent_;
	}

	template<class S>
	inline void
	Location :: show (S& string) const
	{
		if (source_ != NULL) {
			source_->show (string);
		}
		position_.show (string);
		if (source_ != NULL) {
			//source_->buffer_.rewind (position_);
			//source_->buffer_.show (string);
		}
	}

	/****************************
	 *		Private members
	 ****************************/

	inline void
	Location :: clear()
	{
		position_.clear();
		indent_.clear();
	}
	inline void
	Location :: newLine()
	{
		position_.incLine();
		indent_.newLine();
	}
}
}

#endif /* NSGL_LEXER_LOCATION_IPP_ */
