/*****************************************************************************/
/* Project name:    nsgl - non-standard general library                      */
/* File name:       nsgl_lexer_Position.hpp                                  */
/* Description:     position in input stream                                 */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef NSGL_LEXER_POSITION_HPP_
#define NSGL_LEXER_POSITION_HPP_

#include <nstl.hpp>

namespace nstd {
namespace lexer {

class Position : public Object {
public :
	typedef U_int16_t Column_;
	typedef U_int32_t Line_;
	typedef U_int32_t Index_;

	Position ();
	Position (const Position&);

	Line_ getLine() const;
	Column_ getColumn() const;
	Index_ getIndex() const;

	void incColumn();
	void incLine();
	void incIndex();

	void clear();

	void operator = (const Position&);
	bool operator == (const Position&) const;
	bool operator != (const Position&) const;

	template<class S>
	void show (S&) const;

	// nstd :: Object interface
	virtual void commitSuicide();
	virtual Size_t getVolume() const;
	virtual Size_t getSizeOf() const;
	virtual void show (String<>&) const;

private :
	bool isUndefined() const;

	Column_ column_;
	Line_ line_;
	Index_ index_;
};

}
}

#endif /*NSGL_LEXER_POSITION_HPP_*/
