/*****************************************************************************/
/* Project name:    nsgl - non-standard general library                      */
/* File name:       nsgl_lexer_Position.ipp                                  */
/* Description:     position in input stream                                 */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef NSGL_LEXER_POSITION_IPP_
#define NSGL_LEXER_POSITION_IPP_

namespace nstd {
namespace lexer {

	/****************************
	 *		Public members
	 ****************************/

	inline
	Position :: Position () :
	column_ (0),
	line_ (0),
	index_(0) {
	}
	inline Position :: Position (const Position& position) :
	column_ (position.column_),
	line_ (position.line_),
	index_(position.index_) {
	}

	inline Position :: Line_
	Position :: getLine() const {
		return line_;
	}
	inline Position :: Column_
	Position :: getColumn() const {
		return column_;
	}
	inline Position :: Index_
	Position :: getIndex() const {
		return index_;
	}

	inline void
	Position :: incColumn() {
		++column_;
	}
	inline void
	Position :: incLine() {
		++ line_;
		column_ = 0;
	}
	inline void
	Position :: incIndex() {
		++ index_;
	}

	inline void
	Position :: clear()
	{
		column_ = 0;
		line_ = 0;
		index_ = 0;
	}

	inline void
	Position :: operator = (const Position& position)
	{
		column_ = position.column_;
		line_ = position.line_;
		index_ = position.index_;
	}
	inline bool
	Position :: operator == (const Position& position) const
	{
		return
			(column_ == position.column_) &&
			(line_ == position.line_) &&
			(index_ == position.index_);
	}
	inline bool
	Position :: operator != (const Position& position) const {
		return !(operator == (position));
	}

	template<class S>
	inline void
	Position :: show (S& string) const
	{
		if (isUndefined()) {
			return;
		}
		string << "position:" << endLine;
		if (line_ != Undefined<Line_> :: getValue ()) {
			string << tab << "line:" << tab << line_ + 1<< endLine;
		}
		if (column_ != Undefined<Column_> :: getValue()) {
			string << tab << "column:" << tab << column_ + 1<< endLine;
		}
		if (index_ != Undefined<Index_> :: getValue()) {
			//string << tab << "index:" << tab << index_ << endLine;
		}
	}

	/****************************
	 *		Private members
	 ****************************/

	inline bool
	Position :: isUndefined() const
	{
		return
			(column_  == 0) &&
			(line_  == 0) &&
			(index_ == 0);
	}
}
}

#endif /*NSGL_LEXER_POSITION_IPP_*/
