/*****************************************************************************/
/* Project name:    nsgl - non-standard general library                      */
/* File name:       nsgl_lexer_Source.cpp                                    */
/* Description:     input source                                             */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef NSGL_LEXER_SOURCE_CPP_
#define NSGL_LEXER_SOURCE_CPP_

#include "lexer/nsgl_lexer_Source.hpp"

namespace nstd {
namespace lexer {

	/****************************
	 *		Public members
	 ****************************/

	void
	Source :: operator ++ ()
	{
		location_.position_.incColumn();
		location_.position_.incIndex();

		switch (charCurrent_.getValue()) {
		case chars :: NEW_LINE :
			location_.newLine();
			insideIndent_ = true;
			break;
		case chars :: SPACE :
			if (insideIndent_) {
				location_.indent_.incSpaces();
			}
			break;
		case chars :: TAB :
			if (insideIndent_) {
				location_.indent_.incTabs();
			}
			break;
		}
		if (!charCurrent_.isWhitespace()) {
			insideIndent_ = false;
		}

		++ charCurrent_;
		reserve();
	}

	void
	Source :: reserve (const Size_t dist)
	{
		if (buffer_.checkEnd (charCurrent_.char_, dist)) {
			++ buffer_;
			buffer_.shiftBack (charStart_);
			buffer_.shiftBack (charStop_);
			buffer_.shiftBack (charCurrent_.char_);
		}
	}

using manipulator :: endLine;
using manipulator :: space;

	// nstd :: Object implementation
	void
	Source :: commitSuicide() {
		delete this;
	}
	Size_t
	Source :: getVolume() const
	{
		Size_t volume = 0;
		volume += location_.getVolume();
		volume += charCurrent_.getVolume();
		volume += buffer_.getVolume();
		return volume;
	}
	Size_t
	Source :: getSizeOf() const {
		return sizeof (Source);
	}
	void
	Source :: show (String<>& string) const {
		string << "file: " << fileName_  << space;
	}
}
}

#endif /*NSGL_LEXER_SOURCE_CPP_*/
