/*****************************************************************************/
/* Project name:    nsgl - non-standard general library                      */
/* File name:       nsgl_lexer_Source.hpp                                    */
/* Description:     input source                                             */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef NSGL_LEXER_SOURCE_HPP_
#define NSGL_LEXER_SOURCE_HPP_

#include <nstl.hpp>

#include "lexer/nsgl_lexer_Char.hpp"
#include "lexer/nsgl_lexer_Location.hpp"
#include "lexer/nsgl_lexer_Buffer.hpp"

namespace nstd {
namespace lexer {

class Source : public Object {
public :
	enum {
		MINIMAL_RESERVE = 32
	};

	Source();
	template<class A>
	Source (const String<A>&);
	virtual ~ Source();

	bool open (const char*);
	bool open();
	void close();
	bool isEmpty() const;

	void operator ++ ();
	void reserve (const Size_t = MINIMAL_RESERVE);

	void startToken();
	void stopToken();

	Char getChar() const;
	const char* getTokenStart() const;
	const char* getTokenStop() const;
	const Location& getLocation() const;
	const Buffer& getBuffer() const;

	template<class S>
	void show (S&) const;

	// nstd :: Object interface
	virtual void commitSuicide();
	virtual Size_t getVolume() const;
	virtual Size_t getSizeOf() const;
	virtual void show (String<>&) const;

private :
	friend class Location;

	const char* fileName_;
	Location location_;
	mutable Buffer buffer_;

	const char* charStart_;
	const char* charStop_;
	Char charCurrent_;

	bool insideIndent_;
};

}
}

#endif /*NSGL_LEXER_SOURCE_HPP_*/
