/*****************************************************************************/
/* Project name:    nsgl - non-standard general library                      */
/* File name:       nsgl_lexer_Source.ipp                                    */
/* Description:     input source                                             */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef NSGL_LEXER_SOURCE_IPP_
#define NSGL_LEXER_SOURCE_IPP_

namespace nstd {
namespace lexer {

	/****************************
	 *		Public members
	 ****************************/

	inline
	Source :: Source () :
	fileName_ (NULL),
	location_ (this),
	buffer_ (),
	charStart_ (),
	charStop_ (),
	charCurrent_ (),
	insideIndent_ (true) {
	}
	template<class A>
	inline
	Source :: Source (const String<A>& fileName) :
	fileName_ (fileName.c_str()),
	location_ (this),
	buffer_ (),
	charStart_ (),
	charStop_ (),
	charCurrent_ (),
	insideIndent_ (true) {
	}
	Source :: ~ Source () {
	}

	inline bool
	Source :: open (const char* fileName) {
		fileName_ = fileName;
		return open();
	}
	bool
	Source :: open () {
		if (fileName_ == NULL) {
			return false;
		}
		if (!buffer_.open (fileName_)) {
			return false;
		}
		buffer_.setInitial (charCurrent_.char_);
		startToken();
		stopToken();
		return true;
	}
	inline void
	Source :: close()
	{
		location_.position().clear();
		location_.indent().clear();
		buffer_.close();
	}
	inline bool
	Source :: isEmpty() const {
		return (buffer_.getFileSize() == 0);
	}

	inline void
	Source :: startToken() {
		charStart_ = charCurrent_.char_;
	}
	inline void
	Source :: stopToken() {
		charStop_ = charCurrent_.char_;
	}

	inline Char
	Source :: getChar() const {
		return charCurrent_;
	}
	inline const char*
	Source :: getTokenStart() const {
		return charStart_;
	}
	inline const char*
	Source :: getTokenStop() const {
		return charStop_;
	}
	inline const Location&
	Source :: getLocation() const {
		return location_;
	}
	const Buffer&
	Source :: getBuffer() const {
		return buffer_;
	}

using manipulator :: endLine;

	template<class S>
	inline void
	Source :: show (S& string) const {
		string << "file: " << fileName_  << endLine;
	}
}
}

#endif /*NSGL_LEXER_SOURCE_IPP_*/
