/*****************************************************************************/
/* Project name:    nsgl - non-standard general library                      */
/* File name:       nsgl_Counter.ipp                                         */
/* Description:     static counter class                                     */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef NSGL_COUNTER_IPP_
#define NSGL_COUNTER_IPP_

#include <iostream>

namespace nstd {

	/****************************
	 *		Public members
	 ****************************/

	inline void
	Counter :: switchOn (const Index_ index)
	{
		if (!initialized_) {
			init();
			initialized_ = true;
		}
		isOn_ [index] = true;
		//stopCount_ [index] = 0;
		//startCount_ [index] = Undefined<Index_> :: getValue();
	}
	inline void
	Counter :: switchOff (const Index_ index)
	{
		if (!initialized_) {
			init();
			initialized_ = true;
		}
		isOn_ [index] = false;
		//stopCount_ [index] = Undefined<Index_> :: getValue();
		//startCount_ [index] = 0;
	}
	inline bool
	Counter :: stop (const Index_ index)
	{
		if (!initialized_) {
			init();
			initialized_ = true;
		}
		if (!isOn_ [index]) {
			return false;
		}
		return
			(counter_ [index] >= stopCount_ [index]) &&
			(counter_ [index] <= startCount_ [index]);
	}
	inline void
	Counter :: set (const Count_ count, const Index_ index)
	{
		if (!initialized_) {
			init();
			initialized_ = true;
		}
		stopCount_ [index] = count;
	}
	inline void
	Counter :: coset (const Count_ count, const Index_ index)
	{
		if (!initialized_) {
			init();
			initialized_ = true;
		}
		startCount_ [index] = count;
	}
	inline void
	Counter :: force (const Index_ index)
	{
		if (!initialized_) {
			init();
			initialized_ = true;
		}
		stopCount_ [index] = counter_ [index];
	}
	inline void
	Counter :: inc (const Index_ index)
	{
		if (!initialized_) {
			init();
			initialized_ = true;
		}
		++ counter_ [index];
	}
	inline Counter :: Count_
	Counter :: get (const Index_ index)
	{
		if (!initialized_) {
			init();
			initialized_ = true;
		}
		return counter_ [index];
	}
	inline void
	Counter :: show (const Index_ index)
	{
		if (!initialized_) {
			init();
			initialized_ = true;
		}
		std :: cerr << "counter " << index << " : " << counter_ [index] << std :: endl;
	}

	/****************************
	 *		Private members
	 ****************************/

	void
	Counter :: init ()
	{
		for (Index_ i = 0; i < DIMENSION; ++i) {
			counter_ [i] = 0;
			stopCount_ [i] = Undefined<Index_> :: getValue();
			startCount_ [i] = Undefined<Index_> :: getValue();
			isOn_ [i] = true;
		}
	}

	/***********************************
	 *		Private static attributes
	 ***********************************/

	Counter :: Index_
		Counter :: counter_ [DIMENSION];
	Counter :: Index_
		Counter :: stopCount_ [DIMENSION];
	Counter :: Index_
		Counter :: startCount_ [DIMENSION];
	bool
		Counter :: isOn_ [DIMENSION];
	bool
		Counter :: initialized_ = false;
}

#endif /*NSGL_COUNTER_IPP_*/
