/*****************************************************************************/
/* Project name:    nsgl - non-standard general library                      */
/* File name:       nsgl_Error.ipp                                           */
/* Description:     general class for error exceptions                       */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef NSGL_ERROR_IPP_
#define NSGL_ERROR_IPP_

namespace nstd {

	/****************************
	 *		Public members
	 ****************************/

	template<class A>
	inline
	Error<A> :: Error (const char* message) :
	program_ (INITIAL_HEADER_CAPACITY),
	header_ (INITIAL_HEADER_CAPACITY),
	locations_ (INITIAL_LOCATIONS_CAPACITY),
	messages_ (INITIAL_MESSAGES_CAPACITY) {
		add (Location_(), message);
	}
	template<class A>
	inline
	Error<A> :: Error (const Location_& location, const char* message) :
	program_ (INITIAL_HEADER_CAPACITY),
	header_ (INITIAL_HEADER_CAPACITY),
	locations_ (INITIAL_LOCATIONS_CAPACITY),
	messages_ (INITIAL_MESSAGES_CAPACITY) {
		add (location, message);
	}
	template<class A>
	Error<A> :: ~Error() throw() {
	}

using manipulator :: space;
using manipulator :: tab;
using manipulator :: endLine;

	template<class A>
	inline typename Error<A> :: Message_&
	Error<A> :: message () {
		return *(messages_.getLastValue());
	}
	template<class A>
	void
	Error<A> :: add (const Location_& location, const char* message)
	{
		locations_.add (new Location_ (location));
		messages_.add  (new Message_ (message));
	}
	template<class A>
	inline const char*
	Error<A> :: what() const throw()
	{
		Message_ message (INITIAL_MESSAGE_CAPACITY);
		show (message);
		return message.c_str();
	}
	template<class A>
	void
	Error<A> :: show (Message_& message) const
	{
		message << header_ << space;
		message << program_ << space;
		message << "error" << endLine;

		for (typename Locations_ :: Index_ i = 0; i < locations_.getSize(); ++i ) {
			if (!messages_ [i]->isEmpty()) {
				message << *messages_ [i] << endLine;
			}
			message << *locations_ [i] << endLine;
		}
	}

	// nstd :: Object implementation
	template<class A>
	void
	Error<A> :: commitSuicide() {
		delete this;
	}
	template<class A>
	Size_t
	Error<A> :: getVolume() const
	{
		Size_t volume = 0;
		volume += program_.getVolume();
		volume += header_.getVolume();
		volume += locations_.getVolume();
		volume += messages_.getVolume();
		return  volume;
	}
	template<class A>
	Size_t
	Error<A> :: getSizeOf() const {
		return sizeof (Error);
	}
	template<class A>
	void
	Error<A> :: show (String<>& string) const
	{
		Message_ message (INITIAL_MESSAGE_CAPACITY);
		show (message);
		string << message;
	}

	/****************************
	 *		Protected members
	 ****************************/

	template<class A>
	inline
	Error<A> :: Error (const Error& error) :
	program_ (error.program_),
	header_ (error.header_),
	locations_ (error.locations_),
	messages_ (error.messages_) {
	}
}

#endif /*NSGL_ERROR_IPP_*/
