/*****************************************************************************/
/* Project name:    nsgl - non-standard general library                      */
/* File name:       nsgl_NameTable.hpp                                       */
/* Description:     general class tables of identificators                   */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef NSGL_NAME_TABLE_HPP_
#define NSGL_NAME_TABLE_HPP_

#include <nstl.hpp>

namespace nstd {

/* Table of symbolic names for parsers. */
template
<
	class T, //< value type
	class A  //< allocator
>
class NameTable :
	public memory :: storage :: Scalar
	<
		NameTable<T, A>,
		A
	>,
	public Object {
public :
	enum {
		INITIAL_NAME_VECTOR_CAPACITY = 4 * KILOBYTE
	};

	typedef A Allocator_;
	typedef T Value_;
	typedef
		typename map :: String<Value_, Allocator_>
		NameMap_;
	typedef
		typename NameMap_ :: StringKey_
		NameString_;
	typedef
		typename container :: Vector
		<
			const NameString_,
			container :: storage :: ByPointer,
			Allocator_
		>
		NameVector_;

public :
	NameTable (const Size_t = INITIAL_NAME_VECTOR_CAPACITY);
	virtual ~ NameTable();

	const NameString_& get (const Value_) const;

	template<class A1>
	Value_ add (const String<A1>&);
	Value_ add (const char* const);
	template<class A1>
	Value_ find (const String<A1>&) const;
	Value_ find (const char* const) const;
	template<class A1>
	bool contains (const String<A1>&) const;
	bool contains (const char* const) const;

	bool addedNew() const;

	template<class A1>
	void showTable (String<A1>&) const;

	// nstd :: Object interface
	virtual void commitSuicide();
	virtual Size_t getVolume() const;
	virtual Size_t getSizeOf() const;
	virtual void show (String<>&) const;

protected :
	NameMap_ nameMap_;
	NameVector_ nameVector_;
	bool addedNew_;
};

}

#endif /*NSGL_NAME_TABLE_HPP_*/
