/*****************************************************************************/
/* Project name:    nsgl - non-standard general library                      */
/* File name:       nsgl_NameTable.ipp                                       */
/* Description:     general class tables of identificators                   */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef NSGL_NAME_TABLE_IPP_
#define NSGL_NAME_TABLE_IPP_

namespace nstd {

	/****************************
	 *		Public members
	 ****************************/

	template<class T, class A>
	inline
	NameTable<T, A> :: NameTable (const Size_t capacity) :
	nameMap_(),
	nameVector_ (capacity),
	addedNew_ (true) {
	}
	template<class T, class A>
	NameTable<T, A> :: ~ NameTable() {
	}

	template<class T, class A>
	inline const typename NameTable<T, A> :: NameString_&
	NameTable<T, A> :: get (const Value_ value) const {
		return *nameVector_ [value];
	}

	template<class T, class A>
	template<class A1>
	inline typename NameTable<T, A> :: Value_
	NameTable<T, A> :: add (const String<A1>& token)
	{
		const Size_t size = nameVector_.getSize();
		const Value_ result = nameMap_.addFind (token, size);
		addedNew_ = (result == size);

		if (addedNew_) {
			const NameString_* const
				name = new NameString_ (token);
			nameVector_.add (name);
		}
		return result;
	}
	template<class T, class A>
	inline typename NameTable<T, A> :: Value_
	NameTable<T, A> :: add (const char* const token)
	{
		const Size_t size = nameVector_.getSize();
		const Value_ result = nameMap_.addFind (token, size);

		if (result == size) {
			const NameString_* const
				name = new NameString_ (token);
			nameVector_.add (name);
		}
		return result;
	}
	template<class T, class A>
	template<class A1>
	inline typename NameTable<T, A> :: Value_
	NameTable<T, A> :: find (const String<A1>& token) const {
		return nameMap_.find (token);
	}
	template<class T, class A>
	inline typename NameTable<T, A> :: Value_
	NameTable<T, A> :: find (const char* const token) const {
		return nameMap_.find (token);
	}
	template<class T, class A>
	template<class A1>
	inline bool
	NameTable<T, A> :: contains (const String<A1>& token) const {
		return (nameMap_.find (token) != Undefined<Value_> :: getValue());
	}
	template<class T, class A>
	inline bool
	NameTable<T, A> :: contains (const char* const token) const {
		return (nameMap_.find (token) != Undefined<Value_> :: getValue());
	}

	template<class T, class A>
	inline bool
	NameTable<T, A> :: addedNew() const {
		return addedNew_;
	}

	template<class T, class A>
	template<class A1>
	inline void
	NameTable<T, A> :: showTable (String<A1>& string) const  {
		nameMap_.template show<String<A1> > (string);
	}

using manipulator :: endLine;
using manipulator :: tab;
using manipulator :: space;

	// nstd :: Object implementation
	template<class T, class A>
	void
	NameTable<T, A> :: commitSuicide() {
		delete this;
	}
	template<class T, class A>
	Size_t
	NameTable<T, A> :: getVolume() const
	{
		Size_t result = 0;
		result += nameMap_.getVolume();
		result += nameVector_.getVolume();
		return result;
	}
	template<class T, class A>
	Size_t
	NameTable<T, A> :: getSizeOf() const {
		return sizeof (NameTable);
	}
	template<class T, class A>
	void
	NameTable<T, A> :: show (String<>& string) const {
		string << "volume = " << space << tab << getVolume() << space;
	}
}

#endif /*NSGL_NAME_TABLE_IPP_*/
