/*****************************************************************************/
/* Project name:    nsgl - non-standard general library                      */
/* File name:       nsgl_Path.hpp                                            */
/* Description:     path in the filesystem                                   */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef NSGL_PATH_HPP_
#define NSGL_PATH_HPP_

#include <nstl.hpp>

namespace nstd {

template
<
	class A = memory :: allocator :: Standard
>
class Path :
	public Object,
	public memory :: storage :: Scalar<Path<A>, A> {
public :
	typedef A Allocator_;
	typedef String<Allocator_> String_;
	typedef Size_t Depth_;
	typedef typename String_ :: Index_ Index_;
	enum {
		NO,
		DIR,
		DOT,
		DOUBLE_DOT
	};

	Path();
	Path (const Path&);
	template<class A1>
	Path (const Path<A1>&);
	template<class A1>
	Path (const String<A1>&);
	Path (const char*);
	virtual ~ Path();

	char& prefix();
	String_& name();
	String_& directory (const Depth_);
	String_& extension();
	String_& pushDirectory();
	template<class A1>
	void addDirectoryFirst (const String<A1>&);
	template<class A1>
	void addDirectoryLast (const String<A1>&);

	char getPrefix() const;
	Depth_ getDepth() const;
	const String_& getString() const;
	const String_& getName() const;
	const String_& getExtension() const;
	const String_& getDirectory (const Depth_) const;

	bool isValid() const;
	void provideDirectory() const;

	template<class A1>
	bool operator == (const Path<A1>&);
	template<class A1>
	bool operator != (const Path<A1>&);
	template<class A1>
	void operator = (const Path<A1>&);

	// nstd :: Object interface
	virtual void commitSuicide();
	virtual Size_t getVolume() const;
	virtual Size_t getSizeOf() const;
	virtual void show (String<>&) const;

private :
	template<class>
	friend class Path;

	typedef
		container :: Vector
		<
			String_,
			container :: storage :: ByPointer,
			Allocator_
		>
		Directories_;

	const char* parsePrefix();
	const char* parseDirectories();
	const char* parseExtension();
	void parsePath();
	void buildPath() const;
	void checkPath (const char*) const;

	mutable bool changed_;
	mutable String_ path_;

	char    prefix_;
	String_ extension_;
	String_ name_;
	Directories_ directories_;
};

}

#endif /*NSGL_PATH_HPP_*/
