/*****************************************************************************/
/* Project name:    nsgl - non-standard general library                      */
/* File name:       nsgl_Timer.hpp                                           */
/* Description:     simple timer                                             */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef NSGL_TIMER_HPP_
#define NSGL_TIMER_HPP_

#include <nstl.hpp>

namespace nstd {

template
<
	class A = memory :: allocator :: Standard
>
class Timer :
	public Object,
	public memory :: storage :: Scalar<Timer<A>, A> {
public :
	enum Mode {
		START,
		STOP
	};

	Timer (const Mode = STOP);
	virtual ~ Timer();

	void start();
	void mark();
	void stop();
	
	Time_t getTime() const;
	Clock_t getClock() const;
	double getSeconds() const;
	double getMinutes() const;
	double getHours() const;

	bool isUsed() const;

	// nstd :: Object interface
	virtual void commitSuicide();
	virtual Size_t getVolume() const;
	virtual Size_t getSizeOf() const;
	virtual void show (String<>&) const;

private :
	bool   isOn_;
	Time_t  startTime_;
	Clock_t startClock_;
	mutable Time_t  stopTime_;
	mutable Clock_t stopClock_;
};

}

#endif /*NSGL_TIMER_HPP_*/
