/*****************************************************************************/
/* Project name:    nsgl - non-standard template library                     */
/* File name:       nsgl_number_Integer.hpp                                  */
/* Description:     integer number                                           */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef NSGL_NUMBER_INTEGER_HPP_
#define NSGL_NUMBER_INTEGER_HPP_

#include <nstl>

namespace nstd {
namespace number {

class Integer {
public :
	Integer() : 
	number_ (0) {
	}
	Integer (const Integer& integer) :
	number_ (integer.number_) {
	}
	Integer (const Int32_t integer) :
	number_ (integer) {
	}
	
	// arithmetic operators
	Integer operator + (const Integer& integer) const {
		return Integer (number_ + integer.number_);
	}
	Integer operator - (const Integer& integer) const {
		return Integer (number_ - integer.number_);
	}
	Integer operator * (const Integer& integer) const {
		return Integer (number_ * integer.number_);
	}
	Integer operator / (const Integer& integer) const {
		return Integer (number_ / integer.number_);
	}
	
	// arithmetic comparators
	bool operator == (const Integer& integer) const {
		return number_ == integer.number_;
	}
	bool operator != (const Integer& integer) const {
		return number_ != integer.number_;
	}
	bool operator > (const Integer& integer) const {
		return number_ > integer.number_;
	}
	bool operator < (const Integer& integer) const {
		return number_ < integer.number_;
	}
	bool operator >= (const Integer& integer) const {
		return number_ >= integer.number_;
	}
	bool operator <= (const Integer& integer) const {
		return number_ <= integer.number_;
	}
	
	void operator = (const Integer& integer) {
		number_ = integer.number_;
	}
	
	// nstd :: Object interface
	Size_t getVolume() const {
		return 0;
	}
	Size_t getSizeOf() const {
		return sizeof (Integer);
	}
	void show (String<>& string) const {
		string << number_;
	}
	
private :
	Int32_t number_;
};

}
}

#endif /*NSGL_NUMBER_INTEGER_HPP_*/
