/*****************************************************************************/
/* Project name:    nsgl - non-standard template library                     */
/* File name:       nsgl_number_Natural.hpp                                  */
/* Description:     natural number                                           */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef NSGL_NUMBER_NATURAL_HPP_
#define NSGL_NUMBER_NATURAL_HPP_

#include <nstl>

namespace nstd {
namespace number {

class Natural {
public :
	Natural() : 
	number_ (0) {
	}
	Natural (const Natural& natural) :
	number_ (natural.number_) {
	}
	Natural (const U_int32_t number) :
	number_ (number) {
	}
	
	/// arithmetic operators
	Natural operator + (const Natural& natural) const {
		return Natural (number_ + natural.number_);
	}
	Natural operator - (const Natural& natural) const {
		return Natural (number_ - natural.number_);
	}
	Natural operator * (const Natural& natural) const {
		return Natural (number_ * natural.number_);
	}
	Natural operator / (const Natural& natural) const {
		return Natural (number_ / natural.number_);
	}
	
	/// arithmetic comparators
	bool operator == (const Natural& natural) const {
		return number_ == natural.number_;
	}
	bool operator != (const Natural& natural) const {
		return number_ != natural.number_;
	}
	bool operator > (const Natural& natural) const {
		return number_ > natural.number_;
	}
	bool operator < (const Natural& natural) const {
		return number_ < natural.number_;
	}
	bool operator >= (const Natural& natural) const {
		return number_ >= natural.number_;
	}
	bool operator <= (const Natural& natural) const {
		return number_ <= natural.number_;
	}
	
	void operator = (const Natural& natural) {
		number_ = natural.number_;
	}
	
	// nstd :: Object interface
	Size_t getVolume() const {
		return 0;
	}
	Size_t getSizeOf() const {
		return sizeof (Natural);
	}
	void show (String<>& string) const {
		string << number_;
	}
	
private :
	U_int32_t number_;
};

}
}

#endif /*NSGL_NUMBER_NATURAL_HPP_*/
