/*****************************************************************************/
/* Project name:    nsgl - non-standard template library                     */
/* File name:       nsgl_number_Rational.hpp                                 */
/* Description:     rational number                                          */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef NSGL_NUMBER_RATIONAL_HPP_
#define NSGL_NUMBER_RATIONAL_HPP_

#include <nstl>

namespace nstd {
namespace number {

class Rational {
public :
	Rational() : 
	numerator_ (1),
	denumenator_ (1) {
	}
	Rational (const Rational& Rational) :
	numerator_  (Rational.numerator_ ), 
	denumenator_  (Rational.denumenator_ ) {
	}
	Rational (const Int32_t numerator, const Int32_t denumenator) :
	numerator_  (numerator), 
	denumenator_  (denumenator) {
		if (denumenator_ == Integer (0)) {
			throw std :: exception ();
		}
	}
	Rational (const Int32_t numerator) : 
	numerator_ (numerator),
	denumenator_ (1) {
	}
	Rational (const Integer numerator) : 
	numerator_ (numerator),
	denumenator_ (1) {
	}
	Rational (const Integer numerator, const Integer denumenator) :
	numerator_  (numerator), 
	denumenator_  (denumenator) {
		if (denumenator_ == Integer (0)) {
			throw std :: exception ();
		}
	}
	
	// arithmetic operators
	Rational operator + (const Rational& rational) const 
	{
		const Integer
			denumenator = denumenator_ * rational.denumenator_;
		const Integer
			numerator = 
				numerator_* rational.denumenator_ +
				rational.numerator_ * denumenator_;
		return Rational (numerator, denumenator);
	}
	Rational operator - (const Rational& rational) const 
	{
		const Integer
			denumenator = denumenator_ * rational.denumenator_;
		const Integer
			numerator = 
				numerator_* rational.denumenator_ -
				rational.numerator_ * denumenator_;
		return Rational (numerator, denumenator);
	}
	Rational operator * (const Rational& rational) const 
	{
		const Integer
			denumenator = denumenator_ * rational.denumenator_;
		const Integer
			numerator = numerator_* rational.numerator_;
		return Rational (numerator, denumenator);
	}
	Rational operator / (const Rational& rational) const 
	{
		const Integer 
			denumenator = denumenator_ * rational.numerator_;
		const Integer 
			numerator = numerator_* rational.denumenator_;
		return Rational (numerator, denumenator);
	}
	
	// arithmetic comparators
	bool operator == (const Rational& rational) const {
		return 
			numerator_ == rational.numerator_ && 
			denumenator_ == rational.denumenator_;
	}
	bool operator != (const Rational& rational) const {
		return  
			numerator_ != rational.numerator_ || 
			denumenator_ != rational.denumenator_;
	}
	bool operator > (const Rational& rational) const {
		return 
			numerator_ * rational.denumenator_ > 
			denumenator_ * rational.numerator_;
	}
	bool operator < (const Rational& rational) const {
		return 
			numerator_ * rational.denumenator_ < 
			denumenator_ * rational.numerator_;
	}
	bool operator >= (const Rational& rational) const {
		return 
			numerator_ * rational.denumenator_ >= 
			denumenator_ * rational.numerator_;
	}
	bool operator <= (const Rational& rational) const {
		return 
			numerator_ * rational.denumenator_ <= 
			denumenator_ * rational.numerator_;
	}
	
	void operator = (const Rational& rational) {
		numerator_ = rational.numerator_; 
		denumenator_ = rational.denumenator_;
	}
	
	// nstd :: Object interface
	Size_t getVolume() const {
		return 0;
	}
	Size_t getSizeOf() const {
		return sizeof (Rational);
	}
	void show (String<>& string) const
	{
		/*string << numerator_;
		string << "/";
		string << denumenator_;*/
	}
	
private :
	Integer numerator_;
	Integer denumenator_;
};

}
}

#endif /*NSGL_NUMBER_RATIONAL_HPP_*/
