/*****************************************************************************/
/* Project name:    nsgl - non-standard template library                     */
/* File name:       nsgl_number_Real.hpp                                     */
/* Description:     real number                                              */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef NSGL_NUMBER_REAL_HPP_
#define NSGL_NUMBER_REAL_HPP_

#include <nstl>

namespace nstd {
namespace number {

class Real {
public :
	Real() : 
	number_ (0) {
	}
	Real (const Real& real) :
	number_ (real.number_) {
	}
	Real (const Double number) :
	number_ (number) {
	}
	
	/// arithmetic operators
	Real operator + (const Real& real) const {
		return Real (number_ + real.number_);
	}
	Real operator - (const Real& real) const {
		return Real (number_ - real.number_);
	}
	Real operator * (const Real& real) const {
		return Real (number_ * real.number_);
	}
	Real operator / (const Real& real) const {
		return Real (number_ / real.number_);
	}
	
	/// arithmetic comparators
	bool operator == (const Real& real) const {
		return number_ == real.number_;
	}
	bool operator != (const Real& real) const {
		return number_ != real.number_;
	}
	bool operator > (const Real& real) const {
		return number_ > real.number_;
	}
	bool operator < (const Real& real) const {
		return number_ < real.number_;
	}
	bool operator >= (const Real& real) const {
		return number_ >= real.number_;
	}
	bool operator <= (const Real& real) const {
		return number_ <= real.number_;
	}
	
	void operator = (const Real& real) {
		number_ = real.number_;
	}
	
	// nstd :: Object interface
	Size_t getVolume() const {
		return 0;
	}
	Size_t getSizeOf() const {
		return sizeof (Real);
	}
	void show (String<>& string) const {
		string << number_;
	}
	
private :
	Double number_;
};

}
}

#endif /*NSGL_NUMBER_REAL_HPP_*/
