/*****************************************************************************/
/* Project name:    nsgl - non-standard general library                      */
/* File name:       nsgl_system.cpp                                          */
/* Description:     system interface module                                  */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef NSGL_SYSTEM_CPP_
#define NSGL_SYSTEM_CPP_

#include <fstream>
#include <sys/stat.h>

#include "system/nsgl_system.hpp"

namespace nstd {
namespace system {
	
	Size_t 
	getFileSize (const char* const fileName)
	{
		Size_t length = 0;
		InputFileStream file;
		file.open (fileName, std :: ios :: binary);

		// get length of file:
		file.seekg (0, std :: ios :: end);
		length = file.tellg();
		file.close();

		return length;
	}
	inline bool
	createDirectory (const char* const directory) {
		return (mkdir (directory, S_IRWXU | S_IRWXG | S_IROTH | S_IXOTH) != 0);
	}
	inline Time_t
	getTime() {
		return time (NULL);
	}
	inline Clock_t
	getClock() {
		return clock();
	}
	inline Clock_t
	getClocksPerSecond() {
		return CLOCKS_PER_SEC;
	}
	inline Clock_t
	getClocksPerMinute() {
		return getClocksPerSecond() * 60;
	}
	inline Clock_t
	getClocksPerHour() {
		return getClocksPerMinute() * 60;
	}
}
}

#endif /*NSGL_SYSTEM_CPP_*/
