/*****************************************************************************/
/* Project name:    nstl - non-standard template library                     */
/* File name:       nstl_container_LongVector.hpp                            */
/* Description:     long vector container                                    */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef NSTL_CONTAINER_LONG_VECTOR_HPP_
#define NSTL_CONTAINER_LONG_VECTOR_HPP_

#include "container/continual/nstl_container_continual_Block.hpp"
#include "container/nstl_container_Vector.hpp"

namespace nstd {
namespace container {

template
<
	class T,                 ///< stored type
	template<class> class H, ///< storage policy (by value, by pointer, etc.)
	class A                  ///< allocator
>
class LongVector :
	public memory :: storage :: Scalar
	<
		LongVector<T, H, A>,
		A
	> {
private :
	enum {
		DIGIT_CAPACITY_OF_BLOCK_SIZE = 12,
		BLOCK_SIZE = 2 << (DIGIT_CAPACITY_OF_BLOCK_SIZE - 1)
	};
	typedef A Allocator_;
	typedef typename continual :: Block<T, H, A, BLOCK_SIZE> Block_;
	typedef typename Block_ :: Index_ ShortIndex_;
	typedef U_int16_t LongIndex_;

	typedef
		typename container :: Vector
		<
			Block_,
			storage :: ByPointer,
			Allocator_
		>
		BlockVector_;

public :
	typedef U_int32_t Index_;

	typedef typename Block_ :: Storage_ Storage_;
	typedef typename Storage_ :: Type_ Type_;
	typedef typename Storage_ :: Argument_ Argument_;
	typedef typename Storage_ :: Value_ Value_;
	typedef typename Storage_ :: Reference_ Reference_;
	typedef typename Storage_ :: const_Reference_ const_Reference_;

	LongVector ();
	LongVector (const LongVector&);
	LongVector (const Index_);
	virtual ~ LongVector();

	bool contains (Argument_) const;
	Index_ find (Argument_) const;

	Reference_ operator [] (const Index_);
	Value_ getValue (const Index_) const;
	Reference_ getReference (const Index_);
	const_Reference_ getReference (const Index_) const;

	Storage_& getStorage (const Index_);
	const Storage_& getStorage (const Index_) const;

	Value_ getLastValue () const;
	Value_ getFirstValue () const;
	Reference_ getLastReference ();
	Reference_ getFirstReference ();
	const_Reference_ getLastReference () const;
	const_Reference_ getFirstReference () const;

	void set (const Index_, Argument_);

	bool operator == (const LongVector&) const;
	bool operator != (const LongVector&) const;
	bool isEqual (const LongVector&) const;

	void swap (const Index_, const Index_);

	bool isEmpty () const;
	void clear ();
	Index_ getSize() const;

	///////////////////////////////////////////////////////////////////////////////////

	void add (Argument_);
	Reference_ pushRaw ();
	Reference_ push ();

	Storage_& pushRawNode ();
	Storage_& pushNode ();

	// nstd :: Object interface
	void commitSuicide();
	Size_t getVolume () const;
	Size_t getRawVolume () const;
	Size_t getSizeOf() const;
	template<class S>
	void show (S&) const;

private :
	BlockVector_ blockVector_;
};

	template
	<
		class T,
		template<class> class H,
		class A,
		class S
	>
	S&
	operator << (S&, const LongVector<T, H, A>&);
}
}

#include "container/nstl_container_LongVector.ipp"

#endif /*NSTL_CONTAINER_LONG_VECTOR_HPP_*/
