/*****************************************************************************/
/* Project name:    nstl - non-standard template library                     */
/* File name:       nstl_container_LongVector.ipp                            */
/* Description:     long vector container                                    */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef NSTL_CONTAINER_LONG_VECTOR_IPP_
#define NSTL_CONTAINER_LONG_VECTOR_IPP_

#include "container/continual/nstl_container_continual_Block.hpp"
#include "container/nstl_container_Vector.hpp"

namespace nstd {
namespace container {

	/****************************
	 *		Public members
	 ****************************/

	template<class T, template<class> class H, class A>
	inline
	LongVector<T, H, A> :: LongVector () :
	blockVector_(1) {
		blockVector_.push ();
	}
	template<class T, template<class> class H, class A>
	inline
	LongVector<T, H, A> :: LongVector (const LongVector& longVector) :
	blockVector_ (longVector.blockVector_.getCapacity())
	{
		for (LongIndex_ i = 0; i < longVector.blockVector_.getSize(); i++) {
			blockVector_.push () = longVector.blockVector_.getReference (i);
		}
	}
	template<class T, template<class> class H, class A>
	inline
	LongVector<T, H, A> :: LongVector (const Index_ initialCapacity) :
	blockVector_ (1 + (initialCapacity >> DIGIT_CAPACITY_OF_BLOCK_SIZE)) {
		blockVector_.push ();
	}
	template<class T, template<class> class H, class A>
	LongVector<T, H, A> :: ~ LongVector() {
	}

	template<class T, template<class> class H, class A>
	inline bool
	LongVector<T, H, A> :: contains (Argument_ value) const
	{
		const Index_ ind = find (value);
		return (ind != Undefined<Index_> :: getValue());
	}

	template<class T, template<class> class H, class A>
	inline typename LongVector<T, H, A> :: Index_
	LongVector<T, H, A> :: find (Argument_ value) const
	{
		if (getSize() == 0) {
			return Undefined<Index_> :: getValue();
		}
		for (LongIndex_ i = 0; i < blockVector_.getSize(); i++) {
			const Index_ result  = blockVector_.getValue (i)->find (value);
			if (result  != Undefined<Index_> :: getValue()) {
				return result + i * BLOCK_SIZE;
			}
		}
		return Undefined<Index_> :: getValue();
	}

	template<class T, template<class> class H, class A>
	inline typename LongVector<T, H, A> :: Reference_
	LongVector<T, H, A> :: operator [] (const Index_ ind)
	{
		const LongIndex_ i = ind >> DIGIT_CAPACITY_OF_BLOCK_SIZE;
		Block_* block = blockVector_.getValue (i);
		const ShortIndex_ j = ind - i * BLOCK_SIZE;
		return block->getReference (j);
	}
	template<class T, template<class> class H, class A>
	inline typename LongVector<T, H, A> :: Value_
	LongVector<T, H, A> :: getValue (const Index_ ind) const
	{
		const LongIndex_ i = ind >> DIGIT_CAPACITY_OF_BLOCK_SIZE;
		Block_* block = blockVector_.getValue (i);
		const ShortIndex_ j = ind - i * BLOCK_SIZE;
		return block->getValue (j);
	}
	template<class T, template<class> class H, class A>
	inline typename LongVector<T, H, A> :: Reference_
	LongVector<T, H, A> :: getReference (const Index_ ind)
	{
		const LongIndex_ i = ind >> DIGIT_CAPACITY_OF_BLOCK_SIZE;
		Block_* block = blockVector_.getValue (i);
		const ShortIndex_ j = ind - i * BLOCK_SIZE;
		return block->getReference (j);
	}
	template<class T, template<class> class H, class A>
	inline typename LongVector<T, H, A> :: const_Reference_
	LongVector<T, H, A> :: getReference (const Index_ ind) const
	{
		const LongIndex_ i = ind >> DIGIT_CAPACITY_OF_BLOCK_SIZE;
		const Block_* block = blockVector_.getValue (i);
		const ShortIndex_ j = ind - i * BLOCK_SIZE;
		return block->getReference (j);
	}

	/*
	template<class T, template<class> class H, class A>
	inline typename LongVector<T, H, A> :: Storage_&
	LongVector<T, H, A> :: getNode (const Index_ ind)
	{
		const LongIndex_ i = ind >> DIGIT_CAPACITY_OF_BLOCK_SIZE;
		Block_* block = blockVector_.getValue (i);
		const ShortIndex_ j = ind - i * BLOCK_SIZE;
		return block->getNode(j);
	}
	template<class T, template<class> class H, class A>
	inline const typename LongVector<T, H, A> :: Storage_&
	LongVector<T, H, A> :: getNode (const Index_ ind) const
	{
		const LongIndex_ i = ind >> DIGIT_CAPACITY_OF_BLOCK_SIZE;
		const Block_* block = blockVector_.getValue (i);
		const ShortIndex_ j = ind - i * BLOCK_SIZE;
		return block->getNode(j);
	}
	*/

	template<class T, template<class> class H, class A>
	inline typename LongVector<T, H, A> :: Value_
	LongVector<T, H, A> :: getLastValue () const {
		const Block_* block = blockVector_.getLastValue();
		return block->getLastValue ();
	}
	template<class T, template<class> class H, class A>
	inline typename LongVector<T, H, A> :: Value_
	LongVector<T, H, A> :: getFirstValue () const {
		const Block_* block = blockVector_.getFirstValue();
		return block->getFirstValue();
	}
	template<class T, template<class> class H, class A>
	inline typename LongVector<T, H, A> :: Reference_
	LongVector<T, H, A> :: getLastReference () {
		Block_* block = blockVector_.getLastValue();
		return block->getLastReference ();
	}
	template<class T, template<class> class H, class A>
	inline typename LongVector<T, H, A> :: Reference_
	LongVector<T, H, A> :: getFirstReference () {
		Block_* block = blockVector_.getFirstValue();
		return block->getFirstReference();
	}
	template<class T, template<class> class H, class A>
	inline typename LongVector<T, H, A> :: const_Reference_
	LongVector<T, H, A> :: getLastReference () const {
		const Block_* block = blockVector_.getLastValue();
		return block->getLastReference ();
	}
	template<class T, template<class> class H, class A>
	inline typename LongVector<T, H, A> :: const_Reference_
	LongVector<T, H, A> :: getFirstReference () const {
		const Block_* block = blockVector_.getFirstValue();
		return block->getFirstReference();
	}

	template<class T, template<class> class H, class A>
	inline void
	LongVector<T, H, A> :: set (const Index_ index, Argument_ value) {
		getReference (index) = value;
	}

	template<class T, template<class> class H, class A>
	inline bool
	LongVector<T, H, A> :: operator == (const LongVector& longVector) const {
		return isEqual (longVector);
	}
	template<class T, template<class> class H, class A>
	inline bool
	LongVector<T, H, A> :: operator != (const LongVector& longVector) const {
		return !isEqual (longVector);
	}
	template<class T, template<class> class H, class A>
	bool
	LongVector<T, H, A> :: isEqual (const LongVector& longVector) const
	{
		if (getSize() != longVector.getSize()) {
			return false;
		}
		for (Index_ i = 0; i < getSize(); i++) {
			if (getValue (i) != longVector.getValue (i)) {
				return false;
			}
		}
		return true;
	}

	template<class T, template<class> class H, class A>
	inline void
	LongVector<T, H, A> :: swap (const Index_ i, const Index_ j)
	{
		const Value_ data = getValue (i);
		getReference (i) = getReference (j);
		getReference (j) = data;
	}

	template<class T, template<class> class H, class A>
	inline bool
	LongVector<T, H, A> :: isEmpty () const {
		return (getSize() == 0);
	}
	template<class T, template<class> class H, class A>
	inline void
	LongVector<T, H, A> :: clear () {
		blockVector_.clear();
	}
	template<class T, template<class> class H, class A>
	inline typename LongVector<T, H, A> :: Index_
	LongVector<T, H, A> :: getSize() const
	{
		Index_ size = 0;
		size += (blockVector_.getSize() - 1) * BLOCK_SIZE;
		const Block_* block = blockVector_.getLastValue();
		size += block->getSize();
		return size;
	}

	///////////////////////////////////////////////////////////////////////////////////

	template<class T, template<class> class H, class A>
	void
	LongVector<T, H, A> :: add (Argument_ value)
	{
		Block_* block = blockVector_.getLastValue();
		if (block->isFull()) {
			block = blockVector_.push();
			block->clear();
			block->add (value);
		}
		block->add (value);
	}

	template<class T, template<class> class H, class A>
	inline typename LongVector<T, H, A> :: Reference_
	LongVector<T, H, A> :: pushRaw ()
	{
		const Block_* block = blockVector_.getLastValue();
		if (block->isFull()) {
			block = blockVector_.push();
			block->clear();
			return block->pushRaw();
		}
		return block->pushRaw();
	}
	template<class T, template<class> class H, class A>
	inline typename LongVector<T, H, A> :: Reference_
	LongVector<T, H, A> :: push ()
	{
		const Block_* block = blockVector_.getLastValue();
		if (block->isFull()) {
			block = blockVector_.push();
			block->clear();
			return block->push();
		}
		return block->push();
	}

	template<class T, template<class> class H, class A>
	inline typename LongVector<T, H, A> :: Storage_&
	LongVector<T, H, A> :: pushRawNode ()
	{
		const Block_* block = blockVector_.getLastValue();
		if (block->isFull()) {
			block = blockVector_.push();
			block->clear();
			return block->pushRawNode();
		}
		return block->pushRawNode();
	}
	template<class T, template<class> class H, class A>
	inline typename LongVector<T, H, A> :: Storage_&
	LongVector<T, H, A> :: pushNode ()
	{
		const Block_* block = blockVector_.getLastValue();
		if (block->isFull()) {
			block = blockVector_.push();
			block->clear();
			return block->pushNode();
		}
		return block->pushNode();
	}

	// nstd :: Object implementation
	template<class T, template<class> class H, class A>
	inline void
	LongVector<T, H, A> :: commitSuicide() {
		delete this;
	}
	template<class T, template<class> class H, class A>
	inline Size_t
	LongVector<T, H, A> :: getVolume() const {
		return blockVector_.getVolume();
	}
	template<class T, template<class> class H, class A>
	Size_t
	LongVector<T, H, A> :: getRawVolume() const
	{
		Size_t result = 0;
		for (LongIndex_ i = 0; i < blockVector_.getSize(); i++) {
			const Block_* block = blockVector_.getValue (i);
			result += block->getRawVolume();
			result += block->getSizeOf();
		}
		result += blockVector_.getRawVolume();
		return result;
	}
	template<class T, template<class> class H, class A>
	inline Size_t
	LongVector<T, H, A> :: getSizeOf() const {
		return sizeof (LongVector);
	}
	template<class T, template<class> class H, class A>
	template<class S>
	void
	LongVector<T, H, A> :: show (S& s) const
	{
		if (typeid (Type_) == typeid (char)) {
			for (Index_ i = 0; i < getSize(); i++) {
				const Value_& element = getReference (i);
				s << element;
			}
		} else {
			for (Index_ i = 0; i < getSize(); i++) {
				const Value_& element = getReference (i);
				s << element << " ";
			}
		}
	}

	/********************************
	 *		Overloaded operators
	 ********************************/

	template
	<
		class T,
		template<class> class H,
		class A,
		class S
	>
	inline S&
	operator << (S& s, const LongVector<T, H, A>& vector)
	{
		vector.show (s);
		return s;
	}
}
}

#endif /*NSTL_CONTAINER_LONG_VECTOR_IPP_*/
