/*****************************************************************************/
/* Project name:    nstl - non-standard template library                     */
/* File name:       nstl_container_Set.hpp                                   */
/* Description:     set container                                            */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef NSTL_CONTAINER_SET_HPP_
#define NSTL_CONTAINER_SET_HPP_

#include "container/continual/nstl_container_continual_Resizable.hpp"

namespace nstd {
namespace container {

/* Mutable in size Set class. */
template
<
	class T,                 ///< stored type
	template<class> class H, ///< storage policy (by value, by pointer, etc.)
	class A                  ///< allocator
>
class Set : public continual :: Resizable<T, H, A> {
	typedef typename continual :: Resizable<T, H, A> Ancestor_;
public :
	typedef typename Ancestor_ :: Index_ Index_;
	typedef typename Ancestor_ :: Type_ Type_;
	typedef typename Ancestor_ :: Value_ Value_;
	typedef typename Ancestor_ :: Argument_ Argument_;
	typedef typename Ancestor_ :: Storage_ Storage_;

	Set ();
	Set (const Index_);
	Set (const Set&);
	template<class T1, template<class> class H1>
	Set (const typename continual :: Collection<T1, H1>&);
	Set (const Type_* const, const Index_);
	virtual ~ Set();

	Index_ add (Argument_);
	Index_ find (Argument_) const;
	Index_ findPlace (Argument_) const;
	bool contains (Argument_) const;

	template<class T1, template<class> class H1>
	void unite (const typename continual :: Collection<T1, H1>&);
	void unite (const Type_* const, const Index_);
	void insert (Argument_, const Index_);
	void intersect (const Set&);
	bool isDisjointed (const Set&) const;

	// nstd :: Object interface
	void commitSuicide();
	Size_t getVolume() const;
	Size_t getRawVolume () const;
	Size_t getSizeOf() const;

private :
	enum {
		LINEAR_SEARCH_THRESHOLD = 32,
		INTERSECT_THRESHOLD = 32,
		INTERSECT_FACTOR = 3
	};
	void intersectDirected (const Set&);
	bool isDisjointedDirected (const Set&) const;
};

}
}

#include "container/nstl_container_Set.ipp"

#endif /*NSTL_CONTAINER_SET_HPP_*/
