/*****************************************************************************/
/* Project name:    nstl - non-standard template library                     */
/* File name:       nstl_container_Set.ipp                                   */
/* Description:     set container                                            */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef NSTL_CONTAINER_SET_IPP_
#define NSTL_CONTAINER_SET_IPP_

namespace nstd {
namespace container {

	/****************************
	 *		Public members
	 ****************************/

	template<class T, template<class> class H, class A>
	inline
	Set<T, H, A> :: Set () :
	Ancestor_() {
	}
	template<class T, template<class> class H, class A>
	inline
	Set<T, H, A> :: Set (const Index_ initialCapacity) :
	Ancestor_ (initialCapacity) {
	}
	template<class T, template<class> class H, class A>
	inline
	Set<T, H, A> :: Set (const Set& array) :
	Ancestor_ (array) {
	}
	template<class T, template<class> class H, class A>
	template<class T1, template<class> class H1>
	inline
	Set<T, H, A> :: Set (const typename continual :: Collection<T1, H1>& array) :
	Ancestor_ (array.getSize()) {
		unite (array);
	}
	template<class T, template<class> class H, class A>
	inline
	Set<T, H, A> :: Set (const Type_* const array, const Index_ size) :
	Ancestor_ (size) {
		unite (array, size);
	}
	template<class T, template<class> class H, class A>
	inline
	Set<T, H, A> :: ~ Set() {
	}

	template<class T, template<class> class H, class A>
	inline typename Set<T, H, A> :: Index_
	Set<T, H, A> :: add (Argument_ value)
	{
		if (find (value) != Undefined<Index_> :: getValue()) {
			return Undefined<Index_> :: getValue();
		}
		const bool reallocated = Ancestor_ :: ensureOneMoreElement();
		const Index_ index = findPlace (value);
		insert (value, index);
		if (reallocated) {
			Ancestor_ :: undefineTail();
		}
		return index;
	}
	template<class T, template<class> class H, class A>
	typename Set<T, H, A> :: Index_
	Set<T, H, A> :: find (Argument_ value) const
	{
		if (Ancestor_ :: isEmpty()) {
			return Undefined<Index_> :: getValue();
		}
		if (Ancestor_ :: size_ < LINEAR_SEARCH_THRESHOLD) {
			return Ancestor_ :: find (value);
		}

		int lowerBound = 0;
		int upperBound = Ancestor_ :: getSize() - 1;

		while (true) {
			//const int medium = static_cast<int> (round ((lowerBound + upperBound)/2));
			const int medium = static_cast<int> ((lowerBound + upperBound)/2 + 0.5);
			if (value < Ancestor_ :: getValue (medium)) {
				upperBound = medium - 1;
			} else if (value > Ancestor_ :: getValue (medium)) {
				lowerBound = medium + 1;
			} else {
				return medium;
			}
			if (lowerBound > upperBound || upperBound < 0) {
				return Undefined<Index_> :: getValue();
			}
		}
	}
	template<class T, template<class> class H, class A>
	typename Set<T, H, A> :: Index_
	Set<T, H, A> :: findPlace (Argument_ value) const
	{
		if (Ancestor_ :: isEmpty()) {
			return 0;
		} else if ((Ancestor_ :: getSize() == 1) && (Ancestor_ :: getFirstValue() == value)) {
			return Undefined<Index_> :: getValue();
		}

		int lowerBound = 0;
		int upperBound = Ancestor_ :: getSize() - 1;

		if (value < Ancestor_ :: getValue (lowerBound)) {
			return 0;
		} else if (Ancestor_ :: getValue (upperBound) < value) {
			return Ancestor_ :: getSize();
		}

		while (true) {
			switch (upperBound - lowerBound) {
				case 1 :
					return upperBound;
				case 2 :
					if (Ancestor_ :: getValue (lowerBound + 1) < value) {
						return upperBound;
					} else {
						return upperBound - 1;
					}
			}

			//const int medium = static_cast<int> (round ((lowerBound + upperBound)/2));
			const int medium = static_cast<int> ((lowerBound + upperBound)/2 + 0.5);
			if (value < Ancestor_ :: getValue (medium)) {
				upperBound = medium;
			} else if (Ancestor_ :: getValue (medium + 1) < value) {
				lowerBound = medium + 1;
			} else {
				return medium + 1;
			}
		}
	}
	template<class T, template<class> class H, class A>
	inline bool
	Set<T, H, A> :: contains (Argument_ value) const
	{
		const Index_ index = find (value);
		return (index != Undefined<Index_> :: getValue());
	}

	template<class T, template<class> class H, class A>
	template<class T1, template<class> class H1>
	void
	Set<T, H, A> :: unite (const typename continual :: Collection<T1, H1>& array)
	{
		for (Index_ i = 0; i < array.getSize(); ++ i) {
			const Value_ value = array.getValue(i);
			add (value);
		}
	}
	template<class T, template<class> class H, class A>
	void
	Set<T, H, A> :: unite (const Type_* const array, const Index_ size)
	{
		for (Index_ i = 0; i < array.getSize(); ++ i) {
			const Value_ value = array [i];
			add (value);
		}
	}
	template<class T, template<class> class H, class A>
	void
	Set<T, H, A> :: insert (Argument_ value, const Index_ index) {
		Ancestor_ :: insert (value, index);
	}
	template<class T, template<class> class H, class A>
	inline void
	Set<T, H, A> :: intersect (const Set& set)
	{
		if ((Ancestor_ :: size_ < INTERSECT_THRESHOLD)||
			(Ancestor_ :: size_ < set.getSize() * INTERSECT_FACTOR)) {
			intersectDirected (set);
		} else {
			Set setCopy (set);
			setCopy.intersect (*this);
			copy (setCopy);
		}
	}
	template<class T, template<class> class H, class A>
	inline bool
	Set<T, H, A> :: isDisjointed (const Set& set) const
	{
		if ((Ancestor_ :: size_ < INTERSECT_THRESHOLD)||
			(Ancestor_ :: size_ < set.getSize() * INTERSECT_FACTOR)) {
			return isDisjointedDirected (set);
		} else {
			return set.isDisjointedDirected (*this);
		}
	}

	// nstd :: Object implementation
	template<class T, template<class> class H, class A>
	inline void
	Set<T, H, A> :: commitSuicide() {
		delete this;
	}
	template<class T, template<class> class H, class A>
	inline Size_t
	Set<T, H, A> :: getVolume() const {
		return Ancestor_ :: getVolume();
	}
	template<class T, template<class> class H, class A>
	inline Size_t
	Set<T, H, A> :: getRawVolume () const {
		return Ancestor_ :: getRawVolume();
	}
	template<class T, template<class> class H, class A>
	inline Size_t
	Set<T, H, A> :: getSizeOf() const {
		return sizeof (Set);
	}

	/****************************
	 *		Private members
	 ****************************/

	template<class T, template<class> class H, class A>
	void
	Set<T, H, A> :: intersectDirected (const Set& set)
	{
		Index_ i = 0;
		while (i < Ancestor_ :: size_) {
			const Value_ value = Ancestor_ :: getValue (i);
			if (set.contains (value)) {
				++ i;
			} else {
				Ancestor_ :: remove (i);
			}
		}
	}
	template<class T, template<class> class H, class A>
	bool
	Set<T, H, A> :: isDisjointedDirected (const Set& set) const
	{
		for (Index_ i = 0; i < Ancestor_ :: size_; ++ i) {
			const Value_ value = Ancestor_ :: getValue (i);
			if (set.contains (value)) {
				return false;
			}
		}
		return true;
	}
}
}

#endif /*NSTL_CONTAINER_SET_IPP_*/
