/*****************************************************************************/
/* Project name:    nstl - non-standard template library                     */
/* File name:       nstl_container_storage_ByValue.ipp                       */
/* Description:     storage policy by value for containers                   */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef NSTL_CONTAINER_STORAGE_BY_VALUE_IPP_
#define NSTL_CONTAINER_STORAGE_BY_VALUE_IPP_

namespace nstd {
namespace container {
namespace storage {

	/****************************
	 *		Public members
	 ****************************/

	template<class T>
	inline
	ByValue<T> :: ByValue() :
	Ancestor_ () {
	}
	template<class T>
	inline
    ByValue<T> :: ByValue (typename Ancestor_ :: Argument_ data) :
    Ancestor_ (data) {
    }
    template<class T>
    inline
    ByValue<T> :: ByValue (const ByValue& storage) :
	Ancestor_ (storage) {
	}
    template<class T>
    inline
    ByValue<T> :: ~ByValue () {
    	Ancestor_ :: data_.~ Type_();
	}

    template<class T>
	inline void
	ByValue<T> :: clear() {
		Ancestor_ :: undefine();
	}
    template<class T>
	inline void
	ByValue<T> :: erase() {
		Ancestor_ :: undefine();
	}
    template<class T>
	inline void
	ByValue<T> :: init() {
	}

    template<class T>
    inline bool
    ByValue<T> :: operator == (const ByValue& storage) const {
    	return (Ancestor_ :: operator == (storage));
    }
    template<class T>
	inline bool
	ByValue<T> :: operator != (const ByValue& storage) const {
    	return !(operator == (storage));
    }
    template<class T>
	inline void
	ByValue<T> :: operator = (const ByValue& storage) {
    	Ancestor_ :: operator = (storage);
    }

    // nstd :: Object implementation
    template<class T>
	inline void
	ByValue<T> :: commitSuicide() {
    	Ancestor_ :: data_.~ Type_();
    }
    template<class T>
	inline Size_t
	ByValue<T> :: getVolume() const {
		return 0;
	}
    template<class T>
	inline Size_t
	ByValue<T> :: getSizeOf() const {
		return sizeof (ByValue);
	}
    template<class T>
	template<class S>
	inline void
	ByValue<T> :: show (S& string) const {
    	string << Ancestor_ :: data_;
	}

    /****************************
	 *		Overloaded operators
	 ****************************/

	template<class T, class S>
	inline S&
	operator << (S& s, const ByValue<T>& storage)
	{
		storage.template show<S> (s);
		return s;
	}
}
}
}

#endif /*NSTL_CONTAINER_STORAGE_BY_VALUE_IPP_*/
