/*****************************************************************************/
/* Project name:    nstl - non-standard template library                     */
/* File name:       nstl_map_Node.ipp                                        */
/* Description:     vector mapping node                                      */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef NSTL_MAP_NODE_IPP_
#define NSTL_MAP_NODE_IPP_

namespace nstd {
namespace map {

	/****************************
	 *		Public members
	 ****************************/

	template<class V, class T>
	template<template<class> class H1, class A1>
	inline
	Node<V, T> :: Node
	(
		const container :: Vector<Key_, H1, A1>* const vector,
		const Value_ value,
		Index_ position,
		const Node* parent
	) :
	key_ (vector->getValue (position)),
	parent_ (parent),
	value_
	(
		vector->getSize() == position + 1 ?
		value :
		Undefined<Value_> :: getValue()
	),
	child_
	(
		vector->getSize() == position + 1 ?
		NULL :
		new Node (vector, value, ++ position, this)
	),
	next_ (NULL) {
	}
	template<class V, class T>
	Node<V, T> :: ~ Node()
	{
		if (child_ != NULL) {
			delete child_;
			child_ = NULL;
		}
		if (next_ != NULL) {
			delete next_;
			next_ = NULL;
		}
	}

	template<class V, class T>
	template<template<class> class H1, class A1>
	typename Node<V, T> :: Value_
	Node<V, T> :: addFind
	(
		const container :: Vector<Key_, H1, A1>* const vector,
		const Value_ value,
		Index_ position
	)
	{
		if (key_ == vector->getValue (position)) {
			if (vector->getSize() == position + 1) {
				if (!isDefined()) {
					value_ = value;
				}
				return value_;
			} else {
				if (child_ == NULL) {
					child_ = new Node (vector, value, ++ position, this);
					return value;
				} else {
					return child_->addFind (vector, value, ++ position);
				}
			}
		}
		if (next_ == NULL) {
			next_ = new Node (vector, value, position, parent_);
			return value;
		} else {
			return next_->addFind (vector, value, position);
		}
	}

	template<class V, class T>
	template<template<class> class H1, class A1>
	typename Node<V, T> :: Value_
	Node<V, T> :: find
	(
		const container :: Vector<Key_, H1, A1>* const vector,
		Index_ position
	) const
	{
		if (key_ == vector->getValue (position)) {
			if (vector->getSize() == position + 1) {
				return value_;
			} else {
				if (child_ == NULL) {
					return Undefined<Value_> :: getValue();
				} else {
					return child_->find (vector, ++ position);
				}
			}
		}
		if (next_ == NULL) {
			return Undefined<Value_> :: getValue();
		} else {
			return next_->find (vector, position);
		}
	}

	template <class V, class T>
	typename Node<V, T> :: Value_
	Node<V, T> :: find
	(
		const Key_* const vector,
		const Index_ length,
		Index_ position
	) const
	{
		if (key_ == vector [position]) {
			if (length == position + 1) {
				return value_;
			} else {
				if (child_ == NULL) {
					return Undefined<Value_> :: getValue();
				} else {
					return child_->find (vector, length, ++ position);
				}
			}
		}
		if (next_ == NULL) {
			return Undefined<Value_> :: getValue();
		} else {
			return next_->find (vector, length, position);
		}
	}

	template<class V, class T>
	inline bool
	Node<V, T> :: isDefined() const {
		return (value_ != Undefined<Value_> :: getValue());
	}

	// nstd :: Object implementation
	template<class V, class T>
	inline void
	Node<V, T> :: commitSuicide() {
		delete this;
	}
	template<class V, class T>
	Size_t
	Node<V, T> :: getVolume() const
	{
		Size_t result = 0;
		if (child_ != NULL) {
			result += child_->getVolume();
			result += child_->getSizeOf();
		}
		if (next_ != NULL) {
			result += next_->getVolume();
			result += next_->getSizeOf();
		}
		return result;
	}
	template<class V, class T>
	inline Size_t
	Node<V, T> :: getSizeOf() const {
		return sizeof (Node);
	}
	template<class V, class T>
	template<class N>
	inline void
	Node<V, T> :: show (N& n) const
	{
		if (isDefined()) {
			showBack (n);
			n << " -----> " << value_ << chars :: NEW_LINE;
		}
		if (child_ != NULL) {
			child_->show (n);
		}
		if (next_ != NULL) {
			next_->show (n);
		}
	}

	/***************************
	 *		Private members
	 ***************************/

	template<class V, class T>
	template<class N>
	inline void
	Node<V, T> :: showBack (N& n) const
	{
		if (parent_ != NULL) {
			parent_->showBack (n);
		} else {
			n << chars :: TAB;
		}
		n << key_;
	}

	/********************************
	 *		Overloaded operators
	 ********************************/

	template<class V, class T, class S>
	inline S&
	operator << (S& s, const Node<V, T>& node)
	{
		node.show (s);
		return s;
	}
}
}

#endif /*NSTL_MAP_NODE_IPP_*/

