/*****************************************************************************/
/* Project name:    nstl - non-standard template library                     */
/* File name:       nstl_map_Vector.hpp                                      */
/* Description:     vector mapping                                           */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef NSTL_MAP_VECTOR_HPP_
#define NSTL_MAP_VECTOR_HPP_

#include "container/nstl_container_Vector.hpp"
#include "map/nstl_map_Node.hpp"

namespace nstd {
namespace map {

template
<
	class V, ///< vector (key type)
	class T  ///< value type
>
class Vector :
	public memory :: storage :: Scalar
	<
		Vector<V, T>,
		typename V :: Allocator_
	> {
public :
	typedef V Vector_;
	typedef T Value_;
	typedef typename Vector_ :: Value_ Key_;
	typedef typename Vector_ :: Allocator_ Allocator_;
	typedef typename Vector_ :: Index_ Index_;

	Vector ();
	virtual ~ Vector();

	template<template<class> class H1, class A1>
	Value_ addFind (const container :: Vector<Key_, H1, A1>* const, const Value_);
	template<template<class> class H1, class A1>
	Value_ find (const container :: Vector<Key_, H1, A1>* const) const;
	template<template<class> class H1, class A1>
	Value_ find (const container :: Vector<Key_, H1, A1>&) const;
	Value_ find (const Key_* const vector, const Index_ size) const;

	// nstd :: Object interface
	void commitSuicide();
	Size_t getVolume() const;
	Size_t getSizeOf() const;
	template<class S>
	void show (S&) const;

private :
	typedef Node<Vector_, Value_> Node_;

	Node_* node_;
};

	template<class V, class T, class S>
	S&
	operator << (S&, const Vector<V, T>&);
}
}

#include "map/nstl_map_Vector.ipp"

#endif /*NSTL_MAP_VECTOR_HPP_*/
