/*****************************************************************************/
/* Project name:    nstl - non-standard template library                     */
/* File name:       nstl_memory_Interval.hpp                                 */
/* Description:     template memory interval                                 */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef NSTL_MEMORY_INTERVAL_HPP_
#define NSTL_MEMORY_INTERVAL_HPP_

#include <ostream>
#include "memory/nstl_memory.dpp"
#include "nstl.tpp"
#include "memory/exception/nstl_memory_exception.hpp"

namespace nstd {
namespace memory {

template<class T = Byte_t>
class Interval {
public :
	Interval ();
	Interval (const Interval& interval);
	Interval (T* const, T* const);
	Interval (T* const, const Size_t);

	bool isClear() const;
	bool isEmpty() const;
	bool isNull() const;
	bool isRegular() const;
	bool isValid() const;

	bool operator == (const Interval& interval) const;
	bool operator != (const Interval& interval) const;
	bool isEqual (const Interval& interval) const;
	bool isDisjointed (const Interval& interval) const;

	T* const beg() const;
	T* const end() const;

	void operator = (const Interval&);
	void copy (const Interval&);

	void shiftBeg (const Size_t);
	void shiftEnd (const Size_t);

	void setBeg (T* const);
	void setEnd (T* const);
	void setNull();

	// interval integer parameters
	Size_t getLength() const;
	Size_t getVolume() const;
	Size_t getSizeOf() const;

	template<class S>
	void show (S&) const;

private :
	template<class>
	friend class storage :: Scalar;
	friend class Space;
	friend class unit :: Block;
	friend class unit :: Boxed;
	friend class unit :: Disposed;
	friend class unit :: Heap;
	friend class unit :: Stack;
	friend class unit :: Standard;
	friend class unit :: Tank;

	Interval (const Size_t) throw (exception :: OutOfMemory);
	void allocate (const Size_t) throw (exception :: OutOfMemory);

	template<class S>
	S* const beg() const;
	template<class S>
	S* const end() const;

	template<class S>
	void operator = (const Interval<S>&);
	template<class S>
	void copy (const Interval<S>&);

	void empty();
	void clear();
	void destroy();

	Interval<> byteRepresentation() const;

	T* beg_;
	T* end_;
};

	template<class T, class S>
	S&
	operator << (S&, Interval<T>);
}
}

#include "memory/nstl_memory_Interval.ipp"

#endif /*NSTL_MEMORY_INTERVAL_HPP_*/
