/*****************************************************************************/
/* Project name:    nstl - non-standard template library                     */
/* File name:       nstl_memory_space_Fixed.hpp                              */
/* Description:     general memory space                                     */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef NSTL_MEMORY_SPACE_HPP_
#define NSTL_MEMORY_SPACE_HPP_

#include "memory/unit/nstl_memory_unit.hpp"

namespace nstd {
namespace memory {

class Space {
public :
	static Size_t getUsedVolume();
	static Size_t getOverallVolume();
	static Size_t getAllocatedVolume();
	static Size_t getDisposedVolume();
	static Size_t getFreeVolume();

	static const unit :: Heap&  getHeap();
	static const unit :: Stack& getStack();
	static const unit :: Boxed& getBoxed();
	static const unit :: Standard& getStandard();

	static unit :: Heap&  heap();
	static unit :: Stack& stack();
	static unit :: Boxed& boxed();
	static unit :: Standard& standard();

	template<class S>
	static void show
	(
		S&,
		const bool = false, // detailed
		const bool = false, // show disposed
		const bool = false  // show fragmentation
	);

private :
	friend class allocator :: Heap;
	friend class allocator :: Stack;
	friend class allocator :: Boxed;
	friend class allocator :: Standard;

	template<class T>
	static const Interval<T> allocateHeap
	(
		const Size_t,
		const allocator :: Method = allocator :: DEFAULT
	);

	template<class T>
	static const Interval<T> allocateStack (const Size_t);

	template<class T>
	static const Interval<T> allocateBoxed
	(
		const Size_t,
		const allocator :: Method = allocator :: DEFAULT
	);

	template<class T>
	static const Interval<T> allocateStandard (const Size_t);

	template<class T>
	static void disposeHeap (const Interval<T>&);
	template<class T>
	static void disposeBoxed (const Interval<T>&);
	template<class T>
	static void disposeStandard (const Interval<T>&);

	static unit :: Heap  heap_;
	static unit :: Stack stack_;
	static unit :: Boxed boxed_;
	static unit :: Standard standard_;
};

}
}

#endif /*NSTL_MEMORY_SPACE_HPP_*/
