/*****************************************************************************/
/* Project name:    nstl - non-standard template library                     */
/* File name:       nstl_memory_Space.ipp                                    */
/* Description:     general memory space                                     */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef NSTL_MEMORY_SPACE_IPP_
#define NSTL_MEMORY_SPACE_IPP_

#include "memory/nstl_memory_UsageMessage.hpp"

namespace nstd {
namespace memory {

	/****************************
	 *		Public members
	 ****************************/

	inline const unit :: Heap&
	Space :: getHeap() {
		return heap_;
	}
	inline const unit :: Stack&
	Space :: getStack() {
		return stack_;
	}
	inline const unit :: Boxed&
	Space :: getBoxed() {
		return boxed_;
	}

	inline unit :: Heap&
	Space :: heap() {
		return heap_;
	}
	inline unit :: Stack&
	Space :: stack() {
		return stack_;
	}
	inline unit :: Boxed&
	Space :: boxed() {
		return boxed_;
	}
	inline unit :: Standard&
	Space :: standard() {
		return standard_;
	}

	template<class S>
	void
	Space :: show
	(
		S& string,
		const bool detailed,
		const bool showDisposed,
		const bool showFragmentation
	)
	{
		string << "memory allocation summary:" << chars :: NEW_LINE;
		string << "--------------------------" << chars :: NEW_LINE;
		const Size_t overall = getOverallVolume();
		string << UsageMessage (overall, overall, "total memory");
		string << UsageMessage (overall, getFreeVolume(), "free memory");
		string << UsageMessage (overall, getAllocatedVolume (), "allocated memory");
		string << UsageMessage (overall, getDisposedVolume (), "disposed memory");
		string << chars :: NEW_LINE;

		if (detailed) {
			heap_.show (string, showDisposed, showFragmentation);
			stack_.show (string);
			boxed_.show (string);
			standard_.show (string);
		}
	}

	/****************************
	 *		Private members
	 ****************************/

	template<class T>
	inline const Interval<T>
	Space :: allocateHeap (const Size_t length, const allocator :: Method method) {
		return heap_.template allocate<T> (length, method);
	}
	template<class T>
	inline const Interval<T>
	Space :: allocateStack (const Size_t length) {
		return stack_.template allocate<T> (length);
	}
	template<class T>
	inline const Interval<T>
	Space :: allocateBoxed (const Size_t length, const allocator :: Method method) {
		return boxed_.template allocate<T> (length, method);
	}
	template<class T>
	inline const Interval<T>
	Space :: allocateStandard (const Size_t length) {
		return standard_.template allocate<T> (length);
	}

	template<class T>
	inline void
	Space :: disposeHeap (const Interval<T>& interval) {
		heap_.template dispose<T> (interval);
	}
	template<class T>
	inline void
	Space :: disposeBoxed (const Interval<T>& interval) {
		boxed_.template dispose<T> (interval);
	}
	template<class T>
	inline void
	Space :: disposeStandard (const Interval<T>& interval) {
		standard_.template dispose<T> (interval);
	}
}
}

#endif /*NSTL_MEMORY_SPACE_HPP_*/
