/*****************************************************************************/
/* Project name:    nstl - non-standard template library                     */
/* File name:       nstl_memory_UsageMessage.cpp                             */
/* Description:     displays memory usage message                            */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef NSTL_MEMORY_USAGE_MESSAGE_CPP_
#define NSTL_MEMORY_USAGE_MESSAGE_CPP_

#include "memory/nstl_memory_UsageMessage.hpp"

namespace nstd {
namespace memory {

	/****************************
	 *		Public members
	 ****************************/

	UsageMessage :: UsageMessage
	(
		const Size_t overallVolume,
		const Size_t currentVolume,
		const char message[],
		const Unit_ measureUnit
	) :
	measureUnit_
	(
		measureUnit == AUTO ?
		calculateUnit (currentVolume) :
		measureUnit
	),
	overallVolume_ (overallVolume),
	currentVolume_ (currentVolume),
	message_ (message) {
	}

	/****************************
	 *		Private members
	 ****************************/

	const char*
	UsageMessage :: unit() const
	{
		switch (measureUnit_) {
		case BYTE : return "b";
		case KILOBYTE : return "kb";
		case MEGABYTE : return "mb";
		case GIGABYTE : return "gb";
		default : return "";
		}
	}
	UsageMessage :: Unit_
	UsageMessage :: calculateUnit (const Size_t volume) const
	{
		if (volume < KILOBYTE) {
			return BYTE;
		}
		if (volume < MEGABYTE) {
			return KILOBYTE;
		}
		if (volume < GIGABYTE) {
			return MEGABYTE;
		}
		return GIGABYTE;
	}
}
}

#endif /*NSTL_MEMORY_USAGE_MESSAGE_CPP_*/
