/*****************************************************************************/
/* Project name:    nstl - non-standard template library                     */
/* File name:       nstl_memory_UsageMessage.ipp                             */
/* Description:     displays memory usage message                            */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef NSTL_MEMORY_USAGE_MESSAGE_IPP_
#define NSTL_MEMORY_USAGE_MESSAGE_IPP_

#include <iomanip>

namespace nstd {
namespace memory {

	/****************************
	 *		Public members
	 ****************************/

	template<class S>
	void
	UsageMessage :: show (S& s) const
	{
		const double percentVolume =
			static_cast<const double> (currentVolume_) * 100 / overallVolume_;
		const double normalizedVolume =
			static_cast<const double> (currentVolume_) / measureUnit_;

		s << message_ << ":" << chars :: TAB << normalizedVolume;
		s << chars :: SPACE << unit() << chars :: TAB;
		s << "(" << currentVolume_ << " b)" << chars :: TAB;
		s << "= " << percentVolume << " %" << chars :: NEW_LINE;
	}
	template<>
	void
	UsageMessage :: show<std :: ostream> (std :: ostream& s) const
	{
		const double percentVolume =
			static_cast<const double> (currentVolume_) * 100 / overallVolume_;
		const double normalizedVolume =
			static_cast<const double> (currentVolume_) / measureUnit_;

		s << message_ << ":" << std :: setprecision (3) << chars :: TAB << normalizedVolume;
		s << chars :: SPACE << unit() << chars :: TAB;
		s << std :: setprecision (3) << "(" << currentVolume_ << " b)" << chars :: TAB;
		s << std :: setprecision (3) << "= " << percentVolume << " %" << chars :: NEW_LINE;
	}

	/****************************
	 *		Operators
	 ****************************/

	template<class S>
	S&
	operator << (S& s, const UsageMessage& message)
	{
		message.show (s);
		return s;
	}
}
}

#endif /*NSTL_MEMORY_USAGE_MESSAGE_IPP_*/
