/*****************************************************************************/
/* Project name:    nstl - non-standard template library                     */
/* File name:       nstl_memory_unit_Block.hpp                               */
/* Description:     a chunk of memory                                        */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef NSTL_MEMORY_UNIT_BLOCK_HPP_
#define NSTL_MEMORY_UNIT_BLOCK_HPP_

#include "memory/nstl_memory_Interval.hpp"
#include "memory/allocator/nstl_memory_allocator_Method.hpp"

namespace nstd {
namespace memory {
namespace unit {

class Block {
public :
	template<class T>
	static Block* create (const Size_t);
	static Block* create ();
	virtual ~ Block();

	template<class T>
	const Interval<T> allocate
	(
		const Size_t,
		const allocator :: Method = allocator :: DEFAULT
	);

	template<class T>
	static bool canAllocate (const Size_t);

	bool isFilled() const;
	void clear();

	Size_t getOverallVolume() const;
	Size_t getAllocatedVolume() const;
	Size_t getFreeVolume() const;

private :
	enum {
		BLOCK_ARRAY_SIZE = 4096 * 64 // in bytes
	};
	Block (const Size_t = BLOCK_ARRAY_SIZE);

	const Interval<> block_;
	Interval<> free_;
};

}
}
}

#endif /*NSTL_MEMORY_UNIT_BLOCK_HPP_*/
