/*****************************************************************************/
/* Project name:    nstl - non-standard template library                     */
/* File name:       nstl_memory_unit_Block.ipp                               */
/* Description:     a chunk of memory                                        */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef NSTL_MEMORY_UNIT_BLOCK_IPP_
#define NSTL_MEMORY_UNIT_BLOCK_IPP_

#include "memory/nstl_memory_ignorableDifference.hpp"

namespace nstd {
namespace memory {
namespace unit {

	/*************************
	 *		Public members
	 *************************/

	template<class T>
	inline Block*
	Block :: create (const Size_t length) {
		return new Block (length * sizeof (T));
	}
	inline Block*
	Block :: create () {
		return new Block (BLOCK_ARRAY_SIZE);
	}
	inline
	Block :: ~ Block() {
		const_cast<Interval<>& >(block_).destroy();
	}

	template<class T>
	const Interval<T>
	Block :: allocate (const Size_t length, const allocator :: Method method)
	{
		typedef T Type;
		const Size_t bytes = length * sizeof (Type);

		if (free_.getLength() >= bytes) {
			Type* const memory = free_.beg<Type>();
			Size_t allocated = 0;
			Size_t newLength = 0;
			if (method == allocator :: GREEDY &&  ignorableDifference (bytes, free_.getLength())) {
				newLength = free_.getLength() / sizeof (Type);
				allocated = newLength * sizeof (Type);
			} else {
				newLength = length;
				allocated = bytes;
			}
			free_.shiftBeg (allocated);
			return Interval<Type> (memory, newLength);
		} else {
			return Interval<T>();
		}
	}
	template<class T>
	inline bool
	Block :: canAllocate (const Size_t length)
	{
		typedef T Type;
		const Size_t bytes = length * sizeof (Type);
		return (BLOCK_ARRAY_SIZE >= bytes);
	}

	inline bool
	Block :: isFilled() const {
		return free_.isEmpty();
	}
	inline void
	Block :: clear() {
		free_ = block_;
	}

	inline Size_t
	Block :: getOverallVolume() const {
		return block_.getVolume();
	}
	inline Size_t
	Block :: getAllocatedVolume() const {
		return block_.getVolume() - free_.getVolume();
	}
	inline Size_t
	Block :: getFreeVolume() const {
		return free_.getVolume();
	}

	/*************************
	 *		Private members
	 *************************/

	inline
	Block :: Block (const Size_t bytes):
	block_ (bytes),
	free_(block_) {
	}
}
}
}

#endif /*NSTL_MEMORY_UNIT_BLOCK_IPP_*/
