/*****************************************************************************/
/* Project name:    nstl - non-standard template library                     */
/* File name:       nstl_memory_unit_Boxed.cpp                               */
/* Description:     boxed memory manager                                     */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef NSTL_MEMORY_UNIT_BOXED_CPP_
#define NSTL_MEMORY_UNIT_BOXED_CPP_

#include "memory/unit/nstl_memory_unit.hpp"

namespace nstd {
namespace memory {
namespace unit {

	/****************************
	 *		Public members
	 ****************************/

	Boxed :: Boxed () :
	disposed_ (),
	main_ (),
	reserve_ (),
	current_ (&main_),
	maxVolume_ (0) {
	}

	void
	Boxed :: create
	(
		const Size_t mainVolume,
		const Size_t reserveVolume
	)
	{
		disposed_.create();
		main_.create (mainVolume);
		reserve_.create (reserveVolume);
	}
	void
	Boxed :: clear()
	{
		main_.clear();
		reserve_.clear();
		disposed_.clear();
		current_ = &main_;
	}
	inline void
	Boxed :: destroy ()
	{
		Ptrdiff_t allocated = 0;
		allocated += current_->getAllocatedVolume();
		allocated -= disposed_.getFreeVolume();
		allocated -= disposed_.getWastedVolume();

		disposed_.destroy();
		main_.destroy();
		reserve_.destroy();
		if (allocated != 0) {
			throw exception :: MemoryLeaks (allocated, "boxed memory leaks");
		}
	}

	inline Size_t
	Boxed :: getUsedVolume () const {
		return disposed_.getUsedVolume();
	}
	inline Size_t
	Boxed :: getOverallVolume () const
	{
		return
			main_.getOverallVolume() +
			reserve_.getOverallVolume();
	}
	inline Size_t
	Boxed :: getAllocatedVolume () const
	{
		Size_t result = 0;
		result += current_->getAllocatedVolume();
		result -= disposed_.getFreeVolume();
		result -= disposed_.getWastedVolume();
		return result;
	}
	inline Size_t
	Boxed :: getDisposedVolume () const {
		return disposed_.getFreeVolume ();
	}
	inline Size_t
	Boxed :: getFreeVolume () const
	{
		Size_t result = 0;
		result += current_->getFreeVolume();
		result += disposed_.getFreeVolume();
		return result;
	}

	inline Size_t
	Boxed :: getMainVolume() const {
		return main_.getOverallVolume();
	}
	inline Size_t
	Boxed :: getReserveVolume() const {
		return reserve_.getOverallVolume();
	}
	inline Size_t
	Boxed :: getMaxVolume() const {
		return maxVolume_;
	}
}
}
}

#endif /*NSTL_MEMORY_UNIT_BOXED_CPP_*/
