/*****************************************************************************/
/* Project name:    nstl - non-standard template library                     */
/* File name:       nstl_memory_unit_Boxed.hpp                               */
/* Description:     boxed memory manager                                     */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef NSTL_MEMORY_UNIT_BOXED_HPP_
#define NSTL_MEMORY_UNIT_BOXED_HPP_

#include "memory/unit/nstl_memory_unit_Disposed.hpp"
#include "memory/unit/nstl_memory_unit_Tank.hpp"

namespace nstd {
namespace memory {
namespace unit {

class Boxed {
public :
	Boxed ();

	void create (const Size_t, const Size_t);
 	void clear();
 	void destroy();

 	Size_t getUsedVolume() const;
	Size_t getOverallVolume () const;
	Size_t getAllocatedVolume () const;
	Size_t getDisposedVolume () const;
	Size_t getFreeVolume () const;

	Size_t getReserveVolume() const;
	Size_t getMainVolume() const;
	Size_t getMaxVolume () const;

	template<class S>
	void show (S&) const;

private :
	friend class memory :: Space;

	template<class T>
	const Interval<T> allocate
	(
		const Size_t,
		const allocator :: Method = allocator :: DEFAULT
	);
	template<class T>
	void dispose (const Interval<T>&);

	Disposed disposed_;
	Tank main_;
	Tank reserve_;
	Tank* current_;
	Size_t maxVolume_;
};

}
}
}

#endif /*NSTL_MEMORY_UNIT_BOXED_HPP_*/
