/*****************************************************************************/
/* Project name:    nstl - non-standard template library                     */
/* File name:       nstl_memory_unit_Boxed.ipp                               */
/* Description:     boxed memory manager                                     */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef NSTL_MEMORY_UNIT_BOXED_IPP_
#define NSTL_MEMORY_UNIT_BOXED_IPP_

namespace nstd {
namespace memory {
namespace unit {

	/****************************
	 *		Public members
	 ****************************/

	template<class T>
	inline const Interval<T>
	Boxed :: allocate
	(
		const Size_t length,
		const allocator :: Method method
	)
	{
		typedef T Type;
		try {
			const Interval<Type>
				interval = disposed_. template allocate<Type> (length, method);

			if (!interval.isEmpty()) {
				return interval;
			}
			const Interval<Type>
				result = current_->template allocate<Type> (length);
			maxVolume_ =
				(maxVolume_ < getAllocatedVolume()) ?
				getAllocatedVolume() :
				maxVolume_;
			return result;
		} catch (exception :: OutOfMemory& outOfMemory) {
			current_ =
				(current_ == &main_) ?
				&reserve_ :
				&main_;
			disposed_.clear();
			outOfMemory.setClass (exception :: OutOfMemory :: BOXED);
			throw;
		}
	}
	template<class T>
	inline void
	Boxed :: dispose (const Interval<T>& interval) {
		disposed_. template dispose<T> (interval);
	}

	template<class S>
	inline void
	Boxed :: show (S& string) const
	{
		string << "boxed allocation summary:" << chars :: NEW_LINE;
		string << "-------------------------" << chars :: NEW_LINE;
		main_.show (string, "main box");
		reserve_.show (string, "reserve box");
		string << UsageMessage (main_.getOverallVolume(), maxVolume_, "max boxed size");
		string << chars :: NEW_LINE;
	}
}
}
}

#endif /*NSTL_MEMORY_UNIT_BOXED_IPP_*/
