/*****************************************************************************/
/* Project name:    nstl - non-standard template library                     */
/* File name:       nstl_memory_unit_Heap.cpp                                */
/* Description:     heap memory unit                                         */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef NSTL_MEMORY_UNIT_HEAP_CPP_
#define NSTL_MEMORY_UNIT_HEAP_CPP_

#include <iostream>

#include "memory/nstl_memory_UsageMessage.hpp"
#include "memory/unit/nstl_memory_unit.hpp"

namespace nstd {
namespace memory {
namespace unit {

	/****************************
	 *		Public members
	 ****************************/

	Heap :: Heap() :
	block_ (),
	disposed_ (),
	voidBlockIndex_ (0),
	usefulBlockIndex_ (0)
	{
		for (Index_ i = 0; i < BLOCK_ARRAY_SIZE; i++) {
			block_[i] = NULL;
		}
	}

	inline void
	Heap :: create() {
		disposed_.create();
	}
	void
	Heap :: destroy()
	{
		Ptrdiff_t allocated = 0;
		for (Index_ i = 0; i < voidBlockIndex_; i ++ )  {
			allocated += block_[i]->getAllocatedVolume();
		}
		allocated -= disposed_.getFreeVolume();
		allocated -= disposed_.getWastedVolume();

		disposed_.destroy();
		for (Index_ i = 0; i < voidBlockIndex_; i ++ )  {
			delete block_[i];
		}
		if (allocated != 0) {
			throw exception :: MemoryLeaks (allocated, "heap memory leaks");
		}
	}
	inline bool
	Heap :: isEmpty() const
	{
		const Size_t
			allocatedVolume = getAllocatedVolume();
		return (allocatedVolume == 0);
	}

	Size_t
	Heap :: getUsedVolume() const {
		return disposed_.getUsedVolume();
	}
	Size_t
	Heap :: getAllocatedVolume() const
	{
		Size_t result = 0;
		for (Index_ i = 0; i < voidBlockIndex_; i ++ )  {
			result += block_[i]->getAllocatedVolume();
		}
		result -= disposed_.getFreeVolume();
		result -= disposed_.getWastedVolume();
		return result;
	}
	Size_t
	Heap :: getFreeVolume() const
	{
		Size_t result = 0;
		for (Index_ i = 0; i < voidBlockIndex_; i ++ )  {
			result += block_[i]->getFreeVolume();
		}
		result += disposed_.getFreeVolume();
		return result;
	}
	Size_t
	Heap :: getOverallVolume() const
	{
		Size_t result = 0;
		for (Index_ i = 0; i < voidBlockIndex_; i ++ )  {
			result += block_[i]->getOverallVolume();
		}
		return result;
	}
	Size_t
	Heap :: getDisposedVolume () const {
		return disposed_.getFreeVolume();
	}

	/****************************
	 *		Private members
	 ****************************/

	void
	Heap :: swapBlocks (const Index_ i, const Index_ j)
	{
		unit :: Block* b = block_ [i];
		block_[i] = block_ [j];
		block_[j] = b;
	}
}
}
}

#endif /*NSTL_MEMORY_UNIT_HEAP_CPP_*/
