/*****************************************************************************/
/* Project name:    nstl - non-standard template library                     */
/* File name:       nstl_memory_unit_Heap.hpp                                */
/* Description:     heap memory unit                                         */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef NSTL_MEMORY_UNIT_HEAP_HPP_
#define NSTL_MEMORY_UNIT_HEAP_HPP_

#include "memory/nstl_memory_Interval.hpp"
#include "memory/unit/nstl_memory_unit_Disposed.hpp"
#include "memory/unit/nstl_memory_unit_Block.hpp"

#include "memory/allocator/nstl_memory_allocator_Method.hpp"

namespace nstd {
namespace memory {
namespace unit {

class Heap {
public :
	Heap();

	void create();
	void destroy();
	bool isEmpty() const;

	Size_t getUsedVolume() const;
	Size_t getAllocatedVolume() const;
	Size_t getFreeVolume() const;
	Size_t getOverallVolume() const;
	Size_t getDisposedVolume() const;

	template<class S>
	void show
	(
		S&,
		const bool = false, // show disposed
		const bool = false  // show fragmentation
	);

private :
	friend class memory :: Space;

	template<class T>
	const Interval<T>
	allocate (const Size_t, const allocator :: Method = allocator :: DEFAULT);
	template<class T>
	void
	dispose (const Interval<T>);

	enum {
		BLOCK_ARRAY_SIZE = KILOBYTE,
		DEFAULT_BLOCK_SIZE = MEGABYTE * 16
	};

	typedef long Index_;

	Block* block_ [BLOCK_ARRAY_SIZE];
	Disposed disposed_;
	Index_ voidBlockIndex_;
	Index_ usefulBlockIndex_;

	void swapBlocks (const Index_, const Index_);
};

}
}
}

#endif /*NSTL_MEMORY_UNIT_HEAP_HPP_*/
