/*****************************************************************************/
/* Project name:    nstl - non-standard template library                     */
/* File name:       nstl_memory_unit_Heap.ipp                                */
/* Description:     heap memory unit                                         */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef NSTL_MEMORY_UNIT_HEAP_IPP_
#define NSTL_MEMORY_UNIT_HEAP_IPP_

namespace nstd {
namespace memory {
namespace unit {

	/****************************
	 *		Public members
	 ****************************/

	template<class S>
	void
	Heap :: show
	(
		S& string,
		const bool showDisposed,
		const bool showFragmentation
	)
	{
		string << "heap allocation summary:" << chars :: NEW_LINE;
		string << "------------------------" << chars :: NEW_LINE;
		const Size_t overall = getOverallVolume();
		string << UsageMessage (overall, overall, "total memory");
		string << UsageMessage (overall, getFreeVolume(), "free memory");
		string << UsageMessage (overall, getAllocatedVolume(), "allocated memory");
		string << UsageMessage (overall, getDisposedVolume(), "disposed memory");

		if (showDisposed) {
			string << chars :: NEW_LINE;
			disposed_.show (string, overall, showFragmentation);
		}
		string << chars :: NEW_LINE;
	}

	/****************************
	 *		Protected members
	 ****************************/

	template<class T>
	const Interval<T>
	Heap :: allocate (const Size_t length, const allocator :: Method method)
	{
		typedef T Type;
		const Interval<Type> result = disposed_. template allocate<Type> (length, method);
		if (!result.isEmpty()) {
			return result;
		}

		const Index_ i = voidBlockIndex_;
		if (!Block :: template canAllocate<Type> (length)) {
			block_ [voidBlockIndex_++] = Block :: template create<Type> (length);
			return block_[i]->allocate<Type> (length, method);
		}

		for (Index_ i = usefulBlockIndex_; i < voidBlockIndex_; i ++ )  {
			if (block_[i]->isFilled()) {
				swapBlocks (i, usefulBlockIndex_ ++);
				continue;
			}
			const Interval<Type> interval = block_[i]->allocate<Type> (length, method);
			if (!interval.isEmpty()) {
				if (block_[i]->isFilled()) {
					swapBlocks (i, usefulBlockIndex_ ++);
				}
				return interval;
			}
		}
		try {
			block_ [voidBlockIndex_++] = Block :: template create<Byte_t> (DEFAULT_BLOCK_SIZE);
		} catch (...) {
			exception :: OutOfMemory outOfMemory;
			outOfMemory.setClass (exception :: OutOfMemory :: HEAP);
			throw outOfMemory;
		}
		return block_[i]->allocate<Type> (length, method);
	}

	template<class T>
	void
	Heap :: dispose (const Interval<T> interval) {
		disposed_. template dispose<T> (interval);
	}
}
}
}

#endif /*NSTL_MEMORY_UNIT_HEAP_IPP_*/
