/*****************************************************************************/
/* Project name:    nstl - non-standard template library                     */
/* File name:       nstl_memory_unit_PointerStack.cpp                        */
/* Description:     stack of pointers                                        */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef NSTL_MEMORY_UNIT_POINTER_STACK_CPP_
#define NSTL_MEMORY_UNIT_POINTER_STACK_CPP_

#include "memory/unit/nstl_memory_unit_PointerStack.hpp"

namespace nstd {
namespace memory {
namespace unit {

	/****************************
	 *		Public members
	 ****************************/

	inline
	PointerStack :: PointerStack () : 
	topIndex_ (-1),
	maxIndex_ (0),
	stack_ () {
	}

	inline void
	PointerStack :: push (Byte_t* const pointer) throw (exception :: StackOverflow) 
	{
		topIndex_ ++;
		if (topIndex_ >= CAPACITY) {
			throw exception :: StackOverflow ();
		}
		stack_ [topIndex_] = pointer;
	}

	inline Byte_t* const
	PointerStack :: pop() throw (exception :: EmptyStackPop) 
	{
		if (topIndex_ == -1) {
			throw exception :: EmptyStackPop ();
		}
		if (topIndex_ > maxIndex_) {
			maxIndex_ = topIndex_;
		}
		return  stack_ [topIndex_--];
	}
}
}
}

#endif /*NSTL_MEMORY_UNIT_POINTER_STACK_CPP_*/
