/*****************************************************************************/
/* Project name:    nstl - non-standard template library                     */
/* File name:       nstl_memory_unit_PointerStack.hpp                        */
/* Description:     stack of pointers                                        */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef NSTL_MEMORY_UNIT_POINTER_STACK_HPP_
#define NSTL_MEMORY_UNIT_POINTER_STACK_HPP_

#include "memory/exception/nstl_memory_exception.hpp"
#include "memory/nstl_memory_Interval.hpp"

namespace nstd {
namespace memory {
namespace unit {

class PointerStack {
public :
	typedef int Index_;

	PointerStack ();

	void push (Byte_t* const) throw (exception :: StackOverflow);
	Byte_t* const pop() throw (exception :: EmptyStackPop);

	bool isEmpty() const;
	void clear();
	Index_ getMaxDepth() const;
	Index_ getDepth() const;

	template<class S>
	void show (S&) const;

private :
	enum {
		CAPACITY = KILOBYTE * 64
	};

	// Current index of the stack top
	Index_ topIndex_;
	Index_ maxIndex_;
	Byte_t* stack_ [CAPACITY];
};

}
}
}

#endif /*NSTL_MEMORY_UNIT_POINTER_STACK_HPP_*/
