/*****************************************************************************/
/* Project name:    nstl - non-standard template library                     */
/* File name:       nstl_memory_unit_Stack.ipp                               */
/* Description:     stack memory                                             */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef NSTL_MEMORY_UNIT_STACK_IPP_
#define NSTL_MEMORY_UNIT_STACK_IPP_

namespace nstd {
namespace memory {
namespace unit {

	/****************************
	 *		Public members
	 ****************************/

	inline void
	Stack :: push() {
		stack_.push (free_.end());
	}
	inline void
	Stack :: pop()
	{
		const Size_t
			volume = space_.getVolume() - free_.getVolume();
		if (volume > maxVolume_) {
			maxVolume_ = volume;
		}
		free_.end_ = stack_.pop();
	}
	inline void
	Stack :: clear()
	{
		stack_.clear();
		Tank :: clear();
	}
	inline bool
	Stack :: isEmpty() const {
		return stack_.isEmpty();
	}

	inline Size_t
	Stack :: getMaxVolume() {
		return maxVolume_;
	}
	inline Stack :: Index_
	Stack :: getMaxDepth() const {
		return stack_.getMaxDepth();
	}
	inline Stack :: Index_
	Stack :: getDepth() const {
		return stack_.getDepth();
	}

	template<class T>
	inline const Interval<T>
	Stack :: allocate (const Size_t length)
	{
		if (stack_.isEmpty()) {
			throw exception :: Exception ("empty stack allocation.");
		}
		typedef T Type;
		try {
			return Tank :: template allocate<Type> (length);
		} catch (exception :: OutOfMemory& error) {
			error.setClass (exception :: OutOfMemory :: STACK);
			throw;
		}
	}

using manipulator :: endLine;

	template<class S>
	inline void
	Stack :: show (S& string) const
	{
		string << "stack allocation summary:" << chars :: NEW_LINE;
		string << "-------------------------" << chars :: NEW_LINE;
		Tank :: show (string, "stack mem used");
		string << UsageMessage (space_.getVolume(), maxVolume_, "max stack size");
		stack_.show (string);
	}
}
}
}

#endif /*NSTL_MEMORY_UNIT_STACK_IPP_*/
