/*****************************************************************************/
/* Project name:    nstl - non-standard template library                     */
/* File name:       nstl_memory_unit_Standard.cpp                            */
/* Description:     standard heap memory space                               */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef NSTL_MEMORY_UNIT_STANDARD_CPP_
#define NSTL_MEMORY_UNIT_STANDARD_CPP_

#include "memory/unit/nstl_memory_unit.hpp"

namespace nstd {
namespace memory {
namespace unit {

	/****************************
	 *		Public members
	 ****************************/

	inline
	Standard :: Standard() :
	allocated_ (0),
	disposed_ (0),
	overall_ (0) {
	}

	inline void
	Standard :: create() {
	}
	inline void
	Standard :: destroy()
	{
		const Ptrdiff_t
			allocated = overall_ - disposed_;
		if (allocated != 0) {
			throw exception :: MemoryLeaks (allocated, "standard heap memory leaks");
		}
	}
	inline bool
	Standard :: isEmpty() const {
		return (allocated_ == 0);
	}

	inline Size_t
	Standard :: getUsedVolume() const {
		return 0;
	}
	inline Size_t
	Standard :: getAllocatedVolume() const {
		return allocated_;
	}
	inline Size_t
	Standard :: getDisposedVolume() const {
		return disposed_;
	}
	inline Size_t
	Standard :: getOverallVolume() const {
		return overall_;
	}
}
}
}

#endif /*NSTL_MEMORY_UNIT_STANDARD_CPP_*/
