/*****************************************************************************/
/* Project name:    nstl - non-standard template library                     */
/* File name:       nstl_memory_unit_Standard.hpp                            */
/* Description:     standard heap memory space                               */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef NSTL_MEMORY_UNIT_STANDARD_HPP_
#define NSTL_MEMORY_UNIT_STANDARD_HPP_

#include "memory/nstl_memory_Interval.hpp"
#include "memory/allocator/nstl_memory_allocator_Method.hpp"

namespace nstd {
namespace memory {
namespace unit {

class Standard {
public :
	Standard();

	void create();
	void destroy();
	bool isEmpty() const;

	Size_t getUsedVolume() const;
	Size_t getAllocatedVolume() const;
	Size_t getDisposedVolume() const;
	Size_t getOverallVolume() const;

	template<class S>
	void show (S&);

private :
	friend class memory :: Space;

	template<class T>
	const Interval<T> allocate (const Size_t size);

	template<class T>
	void dispose (const Interval<T>);

	Size_t allocated_;
	Size_t disposed_;
	Size_t overall_;
};

}
}
}

#endif /*NSTL_MEMORY_UNIT_STANDARD_HPP_*/
