/*****************************************************************************/
/* Project name:    nstl - non-standard template library                     */
/* File name:       nstl_memory_unit_Standard.ipp                            */
/* Description:     standard heap memory space                               */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef NSTL_MEMORY_UNIT_STANDARD_IPP_
#define NSTL_MEMORY_UNIT_STANDARD_IPP_

namespace nstd {
namespace memory {
namespace unit {

	/****************************
	 *		Public members
	 ****************************/

	template<class T>
	inline const Interval<T>
	Standard :: allocate (const Size_t length)
	{
		typedef T Type;
		try {
			Interval<Type> result (length);
			allocated_ += result.getVolume();
			overall_ += result.getVolume();
			return result;
		} catch (...) {
			exception :: OutOfMemory outOfMemory;
			outOfMemory.setClass (exception :: OutOfMemory :: STANDARD);
			throw outOfMemory;
		}
	}
	template<class T>
	inline void
	Standard :: dispose (const Interval<T> interval)
	{
		typedef T Type;
		if (interval.isNull()) {
			return;
		}
		allocated_ -= interval.getVolume();
		disposed_ += interval.getVolume();
		const_cast<Interval<Type>&> (interval).destroy();
	}

	template<class S>
	inline void
	Standard :: show (S& string)
	{
		string << "standard heap allocation summary:" << chars :: NEW_LINE;
		string << "---------------------------------" << chars :: NEW_LINE;
		string << "allocated: " << allocated_ << " bytes" << chars :: NEW_LINE;
		string << "disposed: " << disposed_ << " bytes" << chars :: NEW_LINE;
		string << "overall: " << overall_ << " bytes" << chars :: NEW_LINE;
		string << chars :: NEW_LINE;
	}
}
}
}

#endif /*NSTL_MEMORY_UNIT_STANDARD_IPP_*/
