/*****************************************************************************/
/* Project name:    nstl - non-standard template library                     */
/* File name:       nstl_memory_unit_Tank.cpp                                */
/* Description:     base class for memory managers                           */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef NSTL_MEMORY_UNIT_TANK_CPP_
#define NSTL_MEMORY_UNIT_TANK_CPP_

#include "memory/unit/nstl_memory_unit_Tank.hpp"
#include "memory/nstl_memory_UsageMessage.hpp"

namespace nstd {
namespace memory {
namespace unit {

	/****************************
	 *		Public members
	 ****************************/

	inline
	Tank :: Tank () :
	space_ (),
	free_ () {
	}

	inline void
	Tank :: create (const Size_t volume)
	{
		const_cast<Interval<>&> (space_).allocate (volume);
		free_ = space_;
	}
	inline void
	Tank :: clear() {
		free_ = space_;
	}
	void
	Tank :: destroy()
	{
		const_cast<Interval<>&>(space_).destroy();
		free_ = space_;
	}

	Size_t
	Tank :: getOverallVolume() const {
		return space_.getVolume();
	}
	Size_t
	Tank :: getAllocatedVolume() const
	{
		if (space_.isNull()) {
			return 0;
		}
		Byte_t* const beg = free_.end();
		Byte_t* const end = space_.end();

		const Interval<>
			used = Interval<> (beg, end);
		return used.getVolume();
	}
	Size_t
	Tank :: getFreeVolume() const {
		return free_.getVolume();
	}
}
}
}

#endif /*NSTL_MEMORY_UNIT_TANK_CPP_*/
