/*****************************************************************************/
/* Project name:    nstl - non-standard template library                     */
/* File name:       nstl_memory_unit_Tank.ipp                                */
/* Description:     base class for memory managers                           */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef NSTL_MEMORY_UNIT_TANK_IPP_
#define NSTL_MEMORY_UNIT_TANK_IPP_

#include "memory/nstl_memory_UsageMessage.hpp"
#include "memory/exception/nstl_memory_exception.hpp"

namespace nstd {
namespace memory {
namespace unit {

	/****************************
	 *		Public members
	 ****************************/

	inline bool
	Tank :: isRegular() const {
		return space_.isRegular();
	}
	inline bool
	Tank :: isEmpty() const {
		return (free_ == space_);
	}
	inline bool
	Tank :: isFull() const {
		return free_.isEmpty();
	}

	template<class T>
	const Interval<T>
	Tank :: allocate (const Size_t length)
	{
		typedef T Type;
		const Size_t size = length * sizeof (Type);

		if (free_.getVolume() < size) {
			throw exception :: OutOfMemory();
		}
		Type* memory = NULL;
		free_.shiftEnd (-size);
		memory = free_.template end<Type>();
		return Interval<T> (memory, length);
	}

	template<class S>
	void
	Tank :: show
	(
		S& string,
		const char message[]
	) const
	{
		const Size_t space = getOverallVolume();
		const Size_t used = getAllocatedVolume();
		string << UsageMessage (space, used, message);
	}
}
}
}

#endif /*NSTL_MEMORY_UNIT_TANK_IPP_*/
