/*****************************************************************************/
/* Project name:    nstl - non-standard template library                     */
/* File name:       nstl_Object.hpp                                          */
/* Description:     abstract object base class                               */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef NSTL_OBJECT_HPP_
#define NSTL_OBJECT_HPP_

#include "nstl_String.hpp"

namespace nstd {

class Object {
public :
	virtual void commitSuicide() = 0;
	virtual Size_t getVolume() const;
	virtual Size_t getSizeOf() const;
	virtual void show (String<>&) const = 0;

	static void release();

protected :
	template<class T>
	Size_t getAggregateVolume (const T*) const;
	template<class T>
	void deleteAggregateObject (T*&);
	template<class T>
	void deleteAggregateObject (T* const&);

	friend std :: ostream&
	operator << (std :: ostream&,  const Object&);
	template<class A>
	friend String<A>&
	operator << (String<A>&, const Object&);

	enum {
		INITAIL_STRING_CAPACITY = nstd :: KILOBYTE * 4
	};
	static String<>* string_;
	static bool volumeCounted_;
};
}

#include <ostream>

namespace nstd {

	std :: ostream&
	operator << (std :: ostream&, const Object&);
	std :: ostream&
	operator << (std :: ostream&, const Object* const);
	String<>&
	operator << (String<>&, const Object&);
	template<class A>
	String<A>&
	operator << (String<A>&, const Object&);
}

#endif /*NSTL_OBJECT_HPP_*/
