/*****************************************************************************/
/* Project name:    nstl - non-standard template library                     */
/* File name:       nstl_Object.ipp                                          */
/* Description:     abstract object base class                               */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef NSTL_OBJECT_IPP_
#define NSTL_OBJECT_IPP_

namespace nstd {

	/****************************
	 *		Protected members
	 ****************************/

	template<class T>
	inline Size_t
	Object :: getAggregateVolume (const T* const aggregate) const
	{
		Size_t result = 0;
		if (aggregate != NULL) {
			result += aggregate->getVolume();
			result += aggregate->getSizeOf();
		}
		return result;
	}
	template<class T>
	inline void
	Object :: deleteAggregateObject (T*& aggregate)
	{
		if (aggregate == NULL) {
			return;
		}
		typedef
			typename type :: Unconst<T> :: Result_
			Unconst;
		const_cast<Unconst*>(aggregate)->commitSuicide();
		aggregate = NULL;
	}
	template<class T>
	inline void
	Object :: deleteAggregateObject (T* const& aggregate)
	{
		if (aggregate == NULL) {
			return;
		}
		typedef
			typename type :: Unconst<T> :: Result_
			Unconst;
		const_cast<Unconst*>(aggregate)->commitSuicide();
		const_cast<T*&>(aggregate) = NULL;
	}

	/****************************
	 *		Operators
	 ****************************/

	inline std :: ostream&
	operator << (std :: ostream& os, const Object* const object) {
		return operator << (os, *object);
	}

	inline String<>&
	operator << (String<>& string, const Object& object) {
		object.show (string);
		return string;
	}

	template<class A>
	String<A>&
	operator << (String<A>& string, const Object& object)
	{
		if (Object :: string_ == NULL) {
			Object :: string_ = new String<> (Object :: INITAIL_STRING_CAPACITY);
		} else {
			Object :: string_->clear();
		}
		object.show (*(Object :: string_));
		string << *(Object :: string_);
		return string;
	}
}

#endif /*NSTL_OBJECT_IPP_*/
