/*****************************************************************************/
/* Project name:    nstl - non-standard template library                     */
/* File name:       nstl_String.hpp                                          */
/* Description:     nstl string class                                        */
/* Copyright:       (c) 2006-2009 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef NSTL_STRING_HPP_
#define NSTL_STRING_HPP_

#include "nstl/memory/nstl_memory.hpp"
#include "nstl/container/nstl_container_Vector.hpp"
#include "nstl/container/storage/nstl_container_storage_ByValue.hpp"
#include "nstl_manipulator.hpp"

namespace nstd {

template
<
	class A = memory :: allocator :: Standard
>
class String : public container :: Vector<char, container :: storage :: ByValue, A> {
	typedef
		container :: Vector<char, container :: storage :: ByValue, A>
		Ancestor_;
public :
	typedef
		typename Ancestor_ :: Allocator_
		Allocator_;
	typedef
		typename Ancestor_ :: Index_
		Index_;
	typedef
		String& (*Manipulator_)(String&);

	String ();
	String (const Index_);
	template<class A1>
	String (const String<A1>&);
	String (const char* const);
	String (const char* const, const char* const);
	String (const char* const, const Index_);
	virtual ~ String();

	static Index_ getStringLength (const char* const);
	static Index_ getStringLength (const char* const, const char* const);

	template<class T> T getIntegerValue() const;

	// c string interface
	char* c_str();
	const char* c_str() const;
	const char* getFirstChar() const;
	const char* getLastChar() const;
	const char* getTerminalChar() const;

	// assign operators
	void operator = (const char* const);
	void operator = (const String&);
	template<class A1>
	void operator = (const String<A1>&);

	// comparator operators
	template<class A1>
	bool operator == (const String<A1>&) const;
	template<class A1>
	bool operator != (const String<A1>&) const;
	bool operator == (const char* const) const;
	bool operator != (const char* const) const;

	void copyFrom (const char* const, const char* const = NULL);

	// operator << overload:
	template<class A1>
	String& operator << (const String<A1>&);
	String& operator << (Manipulator_);
	String& operator << (const char*);
	String& operator << (const std :: string&);
	String& operator << (const char);
	String& operator << (const void*);

	String& operator << (const Size_t value);
	String& operator << (const Ptrdiff_t value);
	String& operator << (const Double value);

	static Size_t getCount();
	static Size_t getMode();

	// nstd :: Object interface
	void commitSuicide();
	Size_t getVolume() const;
	Size_t getSizeOf() const;

private :
	template<class A1>
	friend String<A1>&
	manipulator :: indent (String<A1>&);
	template<class A1>
	friend String<A1>&
	manipulator :: iterate (String<A1>&);
	template<class A1>
	friend String<A1>&
	manipulator :: mode (String<A1>&);

	bool doIndent() const;
	template<class A1>
	Index_ countNewLine (const String<A1>&) const;
	Index_ countNewLine (const char*) const;
	template<class A1>
	void appendWithCharIndent (const String<A1>&);
	template<class A1>
	void appendWithManipulatorIndent (const String<A1>&);
	void appendWithCharIndent (const char*, const Index_);
	void appendWithManipulatorIndent (const char*, const Index_);

	static bool indentOn_;
	static bool iterateOn_;
	static bool modeOn_;
	static Manipulator_ indentM_;
	static char indentCh_;
	static Size_t count_;
	static Size_t mode_;
};

}

#endif /*NSTL_STRING_HPP_*/
