/*****************************************************************************/
/* Project name:    smm - verifier for the Simplified MetaMath language      */
/* File name:       smm_auxiliary_Config.hpp                                 */
/* Description:     static class for command line arguments                  */
/* Copyright:       (c) 2006-2010 Dmitri Vlasov                              */
/* Author:          Dmitri Yurievich Vlasov, Novosibirsk, Russia             */
/* Email:           vlasov at academ.org                                     */
/* URL:             http://mathdevlanguage.sourceforge.net                   */
/* Modified by:                                                              */
/* License:         GNU General Public License Version 3                     */
/*****************************************************************************/

#ifndef SMM_AUXILIARY_CONFIG_HPP_
#define SMM_AUXILIARY_CONFIG_HPP_

#include "smm/types/smm_types.hpp"
#include "smm/auxiliary/smm_auxiliary.dpp"

namespace smm {
namespace auxiliary {

// Class which contain boolean flags and string parameters for smm.
class Config : public nstd :: Config {
	typedef
		nstd :: Config
		Ancestor;
public :
	// The checker will show the content of the proof stack at proof errors.
	static bool keepChecking ();
	// The checker will show the content of the proof stack at proof errors.
	static bool showStack();

	// info output options
	static bool showTimings();
	static bool showMath();
	static bool showVolume();

	static bool proceed (const int argCount, const char* argValues[]);
	static void release();
	static Size_t getVolume();

private :
	// Shows the options usage message.
	static void showUsageMessage();
	static void init ();

	static bool proceedStringOption (const char*, const int, const char* [], int&);
	static bool proceedOneCharOption (const char*, const int, const char* [], int&);

	static bool checkPaths();
	static bool analyseArguments();

	/// The checker will show the content of the proof steck at proof errors.
	static bool keepChecking_;
	/// The checker will show the content of the proof steck at proof errors.
	static bool showStack_;

	static bool showTimings_;
	static bool showMath_;
	static bool showVolume_;

};

}
}

#endif /*SMM_AUXILIARY_CONFIG_HPP_*/
